@@ -68,8 +68,9 @@
 #include <Base/BoundBoxPy.h>
 #include <Base/PlacementPy.h>
 #include <Base/RotationPy.h>
+#include <Base/Tools.h>
+#include <Base/UnitsApi.h>
 
-#include "Feature.h"
 #include "GeoFeature.h"
 #include "FeatureTest.h"
 #include "FeaturePython.h"
@@ -96,6 +97,7 @@
 #include <boost/token_functions.hpp>
 #include <boost/signals.hpp>
 #include <boost/bind.hpp>
+#include <boost/version.hpp>
 
 using namespace App;
 using namespace std;
@@ -214,6 +216,12 @@ Application::Application(ParameterManager * /*pcSysParamMngr*/,
     PyModule_AddObject(pAppModule, "Base", pBaseModule);
     Py_INCREF(pConsoleModule);
     PyModule_AddObject(pAppModule, "Console", pConsoleModule);
+
+    //insert Units module
+    PyObject* pUnitsModule = Py_InitModule3("Units", Base::UnitsApi::Methods,
+          "The Unit API");
+    Py_INCREF(pUnitsModule);
+    PyModule_AddObject(pAppModule, "Units", pUnitsModule);
 }
 
 Application::~Application()
@@ -244,7 +252,7 @@ void Application::renameDocument(const char *OldName, const char *NewName)
 Document* Application::newDocument(const char * Name, const char * UserName)
 {
     // get anyway a valid name!
-    if (!Name)
+    if (!Name || Name[0] == '\0')
         Name = "Unnamed";
     string name = getUniqueDocumentName(Name);
 
@@ -337,18 +345,7 @@ std::string Application::getUniqueDocumentName(const char *Name) const
 {
     if (!Name || *Name == '\0')
         return std::string();
-
-    // check for first character whether it's a digit
-    std::string CleanName = Name;
-    if (!CleanName.empty() && CleanName[0] >= 48 && CleanName[0] <= 57)
-        CleanName[0] = '_';
-    // strip illegal chars
-    for (std::string::iterator it = CleanName.begin(); it != CleanName.end(); ++it) {
-        if (!((*it>=48 && *it<=57) ||  // number
-             (*it>=65 && *it<=90)  ||  // uppercase letter
-             (*it>=97 && *it<=122)))   // lowercase letter
-             *it = '_'; // it's neither number nor letter
-    }
+    std::string CleanName = Base::Tools::getIdentifier(Name);
 
     // name in use?
     std::map<string,Document*>::const_iterator pos;
@@ -359,23 +356,12 @@ std::string Application::getUniqueDocumentName(const char *Name) const
         return CleanName;
     }
     else {
-        // find highest suffix
-        int nSuff = 0;
+        std::vector<std::string> names;
+        names.reserve(DocMap.size());
         for (pos = DocMap.begin();pos != DocMap.end();++pos) {
-            const std::string &DocName = pos->first;
-            if (DocName.substr(0, CleanName.length()) == CleanName) { // same prefix
-                std::string clSuffix(DocName.substr(CleanName.length()));
-                if (clSuffix.size() > 0) {
-                    std::string::size_type nPos = clSuffix.find_first_not_of("0123456789");
-                    if (nPos==std::string::npos)
-                        nSuff = std::max<int>(nSuff, std::atol(clSuffix.c_str()));
-                }
-            }
+            names.push_back(pos->first);
         }
-
-        std::stringstream str;
-        str << CleanName << (nSuff + 1);
-        return str.str();
+        return Base::Tools::getUniqueName(CleanName, names);
     }
 }
 
@@ -452,6 +438,16 @@ const char* Application::GetHomePath(void) const
     return _mConfig["AppHomePath"].c_str();
 }
 
+const char* Application::getExecutableName(void) const
+{
+    return _mConfig["ExeName"].c_str();
+}
+
+std::string Application::getUserAppDataDir()
+{
+    return mConfig["UserAppData"];
+}
+
 std::string Application::getResourceDir()
 {
 # ifdef RESOURCEDIR
@@ -951,8 +947,10 @@ void Application::initTypes(void)
     App ::PropertyString            ::init();
     App ::PropertyStringList        ::init();
     App ::PropertyLink              ::init();
+    App ::PropertyLinkWeak          ::init();
     App ::PropertyLinkSub           ::init();
     App ::PropertyLinkList          ::init();
+    App ::PropertyLinkSubList       ::init();
     App ::PropertyMatrix            ::init();
     App ::PropertyVector            ::init();
     App ::PropertyVectorList        ::init();
@@ -968,11 +966,11 @@ void Application::initTypes(void)
     App ::PropertyPythonObject      ::init();
     // Document classes
     App ::DocumentObject            ::init();
-    App ::AbstractFeature           ::init();
     App ::GeoFeature                ::init();
     App ::FeatureTest               ::init();
     App ::FeatureTestException      ::init();
     App ::FeaturePython             ::init();
+    App ::GeometryPython            ::init();
     App ::Document                  ::init();
     App ::DocumentObjectGroup       ::init();
     App ::DocumentObjectFileIncluded::init();
@@ -1050,6 +1048,9 @@ void Application::initConfig(int argc, char ** argv)
 
     LoadParameters();
 
+    // set the default units
+    UnitsApi::setDefaults();
+
     // capture python variables
     SaveEnv("PYTHONPATH");
     SaveEnv("PYTHONHOME");
@@ -1124,7 +1125,7 @@ void Application::processCmdLineFiles(void)
                 // try to open
                 Application::_pcSingleton->openDocument(File.filePath().c_str());
             }
-            else if (File.hasExtension("fcscript")) {
+            else if (File.hasExtension("fcscript")||File.hasExtension("fcmacro")) {
                 Base::Interpreter().runFile(File.filePath().c_str());
             }
             else if (File.hasExtension("py")) {
@@ -1194,9 +1195,10 @@ void Application::LoadParameters(void)
 
     // Init parameter sets ===========================================================
     //
-
-    mConfig["UserParameter"]   = mConfig["UserAppData"] + "user.cfg";
-    mConfig["SystemParameter"] = mConfig["UserAppData"] + "system.cfg";
+    if (mConfig.find("UserParameter") == mConfig.end())
+        mConfig["UserParameter"]   = mConfig["UserAppData"] + "user.cfg";
+    if (mConfig.find("SystemParameter") == mConfig.end())
+        mConfig["SystemParameter"] = mConfig["UserAppData"] + "system.cfg";
 
 
     try {
@@ -1207,11 +1209,12 @@ void Application::LoadParameters(void)
                               "   configuration.\n");
         }
     }
-    catch (Base::Exception& e) {
-        std::stringstream str;
-        str << "Malformed parameter file " << mConfig["SystemParameter"];
-        e.setMessage(str.str());
-        throw; // re-throw this instance
+    catch (const Base::Exception& e) {
+        // try to proceed with an empty XML document
+        Base::Console().Error("%s in file %s.\n"
+                              "Continue with an empty configuration.",
+                              e.what(), mConfig["SystemParameter"].c_str());
+        _pcSysParamMngr->CreateDocument();
     }
 
     try {
@@ -1223,11 +1226,12 @@ void Application::LoadParameters(void)
 
         }
     }
-    catch (Base::Exception& e) {
-        std::stringstream str;
-        str << "Malformed parameter file " << mConfig["UserParameter"];
-        e.setMessage(str.str());
-        throw; // re-throw this instance
+    catch (const Base::Exception& e) {
+        // try to proceed with an empty XML document
+        Base::Console().Error("%s in file %s.\n"
+                              "Continue with an empty configuration.",
+                              e.what(), mConfig["UserParameter"].c_str());
+        _pcUserParamMngr->CreateDocument();
     }
 }
 
@@ -1235,6 +1239,11 @@ void Application::LoadParameters(void)
 #if (defined(_MSC_VER) && (_MSC_VER < 1600))
 // fix weird error while linking boost (all versions of VC)
 namespace boost { namespace program_options { std::string arg="arg"; } }
+#if (defined (BOOST_VERSION) && (BOOST_VERSION == 104100))
+namespace boost { namespace program_options { 
+    const unsigned options_description::m_default_line_length = 80;
+} }
+#endif
 #endif
 
 pair<string, string> customSyntax(const string& s)
@@ -1311,6 +1320,9 @@ void Application::ParseOptions(int ac, char ** av)
     config.add_options()
     //("write-log,l", value<string>(), "write a log file")
     ("write-log,l", descr.c_str())
+    ("log-file", value<string>(), "Unlike to --write-log this allows to log to an arbitrary file")
+    ("user-cfg,u", value<string>(),"User config file to load/save user settings")
+    ("system-cfg,s", value<string>(),"Systen config file to load/save system settings")
     ("run-test,t",   value<int>()   ,"Test level")
     ("module-path,M", value< vector<string> >()->composing(),"Additional module paths")
     ("python-path,P", value< vector<string> >()->composing(),"Additional python paths")
@@ -1364,7 +1376,7 @@ void Application::ParseOptions(int ac, char ** av)
         store( boost::program_options::command_line_parser(ac, av).
                options(cmdline_options).positional(p).extra_parser(customSyntax).run(), vm);
 
-        ifstream ifs("FreeCAD.cfg");
+        std::ifstream ifs("FreeCAD.cfg");
         store(parse_config_file(ifs, config_file_options), vm);
         notify(vm);
     }
@@ -1387,7 +1399,7 @@ void Application::ParseOptions(int ac, char ** av)
 
     if (vm.count("response-file")) {
         // Load the file and tokenize it
-        ifstream ifs(vm["response-file"].as<string>().c_str());
+        std::ifstream ifs(vm["response-file"].as<string>().c_str());
         if (!ifs) {
             Base::Console().Error("Could no open the response file\n");
             cerr << "Could no open the response file: '"
@@ -1456,6 +1468,19 @@ void Application::ParseOptions(int ac, char ** av)
         mConfig["LoggingFileName"] = mConfig["UserAppData"] + mConfig["ExeName"] + ".log";
     }
 
+    if (vm.count("log-file")) {
+        mConfig["LoggingFile"] = "1";
+        mConfig["LoggingFileName"] = vm["log-file"].as<string>();
+    }
+
+    if (vm.count("user-cfg")) {
+        mConfig["UserParameter"] = vm["user-cfg"].as<string>();
+    }
+
+    if (vm.count("system-cfg")) {
+        mConfig["SystemParameter"] = vm["system-cfg"].as<string>();
+    }
+
     if (vm.count("run-test")) {
         int level = vm["run-test"].as<int>();
         switch (level) {
