@@ -23,7 +23,7 @@
 
 /*! \class App::Document
 This is bisides the Application class the most importand class in FreeCAD
-It contains all the data of the opend, saved or newly created FreeCAD Document.
+It contains all the data of the opened, saved or newly created FreeCAD Document.
 The Document manage the Undo and Redo mechanism and the linking of documents.
 
 Note: the documents are not free objects. They are completly handled by the
@@ -79,6 +79,7 @@ recompute path. Also enables more complicated dependencies beyond trees.
 #include <Base/Writer.h>
 #include <Base/Stream.h>
 #include <Base/FileInfo.h>
+#include <Base/Tools.h>
 #include <Base/Uuid.h>
 
 #include <zipios++/zipios-config.h>
@@ -349,14 +350,14 @@ void Document::clearUndos()
 int Document::getAvailableUndos() const
 {
     if (d->activeUndoTransaction)
-        return mUndoTransactions.size() + 1;
+        return static_cast<int>(mUndoTransactions.size() + 1);
     else
-        return mUndoTransactions.size();
+        return static_cast<int>(mUndoTransactions.size());
 }
 
 int Document::getAvailableRedos() const
 {
-    return mRedoTransactions.size();
+    return static_cast<int>(mRedoTransactions.size());
 }
 
 void Document::setUndoMode(int iMode)
@@ -394,7 +395,7 @@ void Document::onChangedProperty(const DocumentObject *Who, const Property *What
 {
     if (d->activeTransaction && !d->rollback)
         d->activeTransaction->addObjectChange(Who,What);
-    signalChangedObject(const_cast<DocumentObject&>(*Who), const_cast<Property&>(*What));
+    signalChangedObject(*Who, *What);
 }
 
 void Document::setTransactionMode(int iMode)
@@ -484,7 +485,8 @@ Document::Document(void)
     ADD_PROPERTY_TYPE(Id,(id.UuidStr),0,Prop_None,"UUID of the document");
 
     // create transient directory
-    Base::FileInfo TransDir(Base::FileInfo::getTempPath() + id.UuidStr);
+    std::string basePath = Base::FileInfo::getTempPath() + GetApplication().getExecutableName();
+    Base::FileInfo TransDir(basePath + "_Doc_" + id.UuidStr);
     if (!TransDir.exists())
         TransDir.createDirectory();
     ADD_PROPERTY_TYPE(TransientDir,(TransDir.filePath().c_str()),0,Prop_Transient,
@@ -533,7 +535,7 @@ void Document::Save (Writer &writer) const
 {
     writer.Stream() << "<?xml version='1.0' encoding='utf-8'?>" << endl
     << "<!--" << endl
-    << " FreeCAD Document, see http://free-cad.sourceforge.net for more informations..." << endl
+    << " FreeCAD Document, see http://free-cad.sourceforge.net for more information..." << endl
     << "-->" << endl;
 
     writer.Stream() << "<Document SchemaVersion=\"4\">" << endl;
@@ -636,8 +638,9 @@ void Document::Restore(Base::XMLReader &reader)
             string name = reader.getAttribute("name");
             DocumentObject* pObj = getObject(name.c_str());
             if (pObj) { // check if this feature has been registered
-                //FIXME: We must save/restore that state of a feature
+                pObj->StatusBits.set(4);
                 pObj->Restore(reader);
+                pObj->StatusBits.reset(4);
             }
             reader.readEndElement("Feature");
         }
@@ -669,8 +672,9 @@ void Document::Restore(Base::XMLReader &reader)
             string name = reader.getAttribute("name");
             DocumentObject* pObj = getObject(name.c_str());
             if (pObj) { // check if this feature has been registered
-                //FIXME: We must save/restore that state of a feature
+                pObj->StatusBits.set(4);
                 pObj->Restore(reader);
+                pObj->StatusBits.reset(4);
             }
             reader.readEndElement("Object");
         }
@@ -680,6 +684,114 @@ void Document::Restore(Base::XMLReader &reader)
     reader.readEndElement("Document");
 }
 
+void Document::exportObjects(const std::vector<App::DocumentObject*>& obj,
+                             std::ostream& out)
+{
+    Base::ZipWriter writer(out);
+    writer.putNextEntry("Document.xml");
+    writer.Stream() << "<?xml version='1.0' encoding='utf-8'?>" << endl;
+    writer.Stream() << "<Document SchemaVersion=\"4\">" << endl;
+    // Add this block to have the same layout as for normal documents
+    writer.Stream() << "<Properties Count=\"0\">" << endl;
+    writer.Stream() << "</Properties>" << endl;
+
+    // writing the features types
+    writer.incInd(); // indention for 'Objects count'
+    writer.Stream() << writer.ind() << "<Objects Count=\"" << obj.size() <<"\">" << endl;
+
+    writer.incInd(); // indention for 'Object type'
+    std::vector<DocumentObject*>::const_iterator it;
+    for (it = obj.begin(); it != obj.end(); ++it) {
+        writer.Stream() << writer.ind() << "<Object "
+        << "type=\"" << (*it)->getTypeId().getName() << "\" "
+        << "name=\"" << (*it)->getNameInDocument() << "\" "
+        << "/>" << endl;
+    }
+
+    writer.decInd();  // indention for 'Object type'
+    writer.Stream() << writer.ind() << "</Objects>" << endl;
+
+    // writing the features itself
+    writer.Stream() << writer.ind() << "<ObjectData Count=\"" << obj.size() <<"\">" << endl;
+
+    writer.incInd(); // indention for 'Object name'
+    for (it = obj.begin(); it != obj.end(); ++it) {
+        writer.Stream() << writer.ind() << "<Object name=\"" << (*it)->getNameInDocument() << "\">" << endl;
+        (*it)->Save(writer);
+        writer.Stream() << writer.ind() << "</Object>" << endl;
+    }
+
+    writer.decInd(); // indention for 'Object name'
+    writer.Stream() << writer.ind() << "</ObjectData>" << endl;
+    writer.decInd();  // indention for 'Objects count'
+    writer.Stream() << "</Document>" << endl;
+
+    // Hook for others to add further data.
+    signalExportObjects(obj, writer);
+
+    // write additional files
+    writer.writeFiles();
+}
+
+std::vector<App::DocumentObject*>
+Document::importObjects(std::istream& input)
+{
+    std::vector<App::DocumentObject*> objs;
+    zipios::ZipInputStream zipstream(input);
+    Base::XMLReader reader("<memory>", zipstream);
+
+    int i,Cnt;
+    reader.readElement("Document");
+    long scheme = reader.getAttributeAsInteger("SchemaVersion");
+    reader.DocumentSchema = scheme;
+
+    // read the object types
+    std::map<std::string, std::string> nameMap;
+    reader.readElement("Objects");
+    Cnt = reader.getAttributeAsInteger("Count");
+    for (i=0 ;i<Cnt ;i++) {
+        reader.readElement("Object");
+        string type = reader.getAttribute("type");
+        string name = reader.getAttribute("name");
+
+        try {
+            App::DocumentObject* o = addObject(type.c_str(),name.c_str());
+            objs.push_back(o);
+            // use this name for the later access because an object with
+            // the given name may already exist
+            nameMap[name] = o->getNameInDocument();
+        }
+        catch (Base::Exception&) {
+            Base::Console().Message("Cannot create object '%s'\n", name.c_str());
+        }
+    }
+    reader.readEndElement("Objects");
+
+    // read the features itself
+    reader.readElement("ObjectData");
+    Cnt = reader.getAttributeAsInteger("Count");
+    for (i=0 ;i<Cnt ;i++) {
+        reader.readElement("Object");
+        std::string name = nameMap[reader.getAttribute("name")];
+        DocumentObject* pObj = getObject(name.c_str());
+        if (pObj) { // check if this feature has been registered
+            pObj->StatusBits.set(4);
+            pObj->Restore(reader);
+            pObj->StatusBits.reset(4);
+        }
+        reader.readEndElement("Object");
+    }
+    reader.readEndElement("ObjectData");
+
+    reader.readEndElement("Document");
+    signalImportObjects(objs, reader);
+    reader.readFiles(zipstream);
+    // reset all touched
+    for (std::vector<DocumentObject*>::iterator it= objs.begin();it!=objs.end();++it)
+        (*it)->purgeTouched();
+    return objs;
+}
+
 unsigned int Document::getMemSize (void) const
 {
     unsigned int size = 0;
@@ -906,6 +1018,11 @@ bool Document::isClosable() const
     return d->closable;
 }
 
+int Document::countObjects(void) const
+{
+   return static_cast<int>(d->objectArray.size());
+}
+
 void Document::recompute()
 {
     // delete recompute log
@@ -1282,7 +1399,7 @@ DocumentObject* Document::_copyObject(DocumentObject* obj, std::map<DocumentObje
                     static_cast<PropertyLink*>(it->second)->setValue(link_copy);
                 }
                 else if (link->getDocument() == this) {
-                    static_cast<PropertyLink*>(it->second)->setValue(link);
+                    //static_cast<PropertyLink*>(it->second)->setValue(link);
                 }
             }
             else if (prop->getTypeId() == PropertyLinkList::getClassTypeId()) {
@@ -1304,18 +1421,17 @@ DocumentObject* Document::_copyObject(DocumentObject* obj, std::map<DocumentObje
                 }
                 else {
                     std::vector<DocumentObject*> links_ref;
-                    for (std::vector<DocumentObject*>::iterator jt = links.begin(); jt != links.end(); ++jt) {
-                        if ((*jt)->getDocument() == this)
-                            links_ref.push_back(*jt);
-                    }
+                    //for (std::vector<DocumentObject*>::iterator jt = links.begin(); jt != links.end(); ++jt) {
+                    //    if ((*jt)->getDocument() == this)
+                    //        links_ref.push_back(*jt);
+                    //}
                     static_cast<PropertyLinkList*>(it->second)->setValues(links_ref);
                 }
             }
             else {
-                Property* data = prop->Copy();
-                if (data) {
+                std::auto_ptr<Property> data(prop->Copy());
+                if (data.get()) {
                     it->second->Paste(*data);
-                    delete data;
                 }
             }
         }
@@ -1407,18 +1523,7 @@ std::string Document::getUniqueObjectName(const char *Name) const
 {
     if (!Name || *Name == '\0')
         return std::string();
-
-    // check for first character whether it's a digit
-    std::string CleanName = Name;
-    if (!CleanName.empty() && CleanName[0] >= 48 && CleanName[0] <= 57)
-        CleanName[0] = '_';
-    // strip illegal chars
-    for (std::string::iterator it = CleanName.begin(); it != CleanName.end(); ++it) {
-        if (!((*it>=48 && *it<=57) ||  // number
-             (*it>=65 && *it<=90)  ||  // uppercase letter
-             (*it>=97 && *it<=122)))   // lowercase letter
-             *it = '_'; // it's neither number nor letter
-    }
+    std::string CleanName = Base::Tools::getIdentifier(Name);
 
     // name in use?
     std::map<std::string,DocumentObject*>::const_iterator pos;
@@ -1429,26 +1534,28 @@ std::string Document::getUniqueObjectName(const char *Name) const
         return CleanName;
     }
     else {
-        // find highest suffix
-        int nSuff = 0;
+        std::vector<std::string> names;
+        names.reserve(d->objectMap.size());
         for (pos = d->objectMap.begin();pos != d->objectMap.end();++pos) {
-            const std::string &ObjName = pos->first;
-            if (ObjName.substr(0, CleanName.length()) == CleanName) { // same prefix
-                std::string clSuffix(ObjName.substr(CleanName.length()));
-                if (clSuffix.size() > 0) {
-                    std::string::size_type nPos = clSuffix.find_first_not_of("0123456789");
-                    if (nPos==std::string::npos)
-                        nSuff = std::max<int>(nSuff, std::atol(clSuffix.c_str()));
-                }
-            }
+            names.push_back(pos->first);
         }
-
-        std::stringstream str;
-        str << CleanName << (nSuff + 1);
-        return str.str();
+        return Base::Tools::getUniqueName(CleanName, names, 3);
     }
 }
 
+std::string Document::getStandardObjectName(const char *Name, int d) const
+{
+    std::vector<App::DocumentObject*> mm = getObjects();
+    std::vector<std::string> labels;
+    labels.reserve(mm.size());
+
+    for (std::vector<App::DocumentObject*>::const_iterator it = mm.begin(); it != mm.end(); ++it) {
+        std::string label = (*it)->Label.getValue();
+        labels.push_back(label);
+    }
+    return Base::Tools::getUniqueName(Name, labels, d);
+}
+
 std::vector<DocumentObject*> Document::getObjects() const
 {
     std::vector<DocumentObject*> Objects;
