@@ -198,6 +198,16 @@ PropertyVectorList::~PropertyVectorList()
 //**************************************************************************
 // Base class implementer
 
+void PropertyVectorList::setSize(int newSize)
+{
+    _lValueList.resize(newSize);
+}
+
+int PropertyVectorList::getSize(void) const
+{
+    return static_cast<int>(_lValueList.size());
+}
+
 void PropertyVectorList::setValue(const Base::Vector3f& lValue)
 {
     aboutToSetValue();
@@ -319,6 +329,11 @@ void PropertyVectorList::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyVectorList::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_lValueList.size() * sizeof(Base::Vector3f));
+}
+
 //**************************************************************************
 //**************************************************************************
 // PropertyMatrix
