@@ -56,7 +56,7 @@ TYPESYSTEM_SOURCE(App::PropertyInteger , App::Property);
 
 PropertyInteger::PropertyInteger()
 {
-
+    _lValue = 0;
 }
 
 
@@ -244,6 +244,10 @@ void PropertyPath::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyPath::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_cValue.string().size());
+}
 
 //**************************************************************************
 //**************************************************************************
@@ -381,6 +385,22 @@ std::vector<std::string> PropertyEnumeration::getEnumVector(void) const
     return result;
 }
 
+void PropertyEnumeration::setEnumVector(const std::vector<std::string>& values)
+{
+    delete [] _EnumArray;
+    _EnumArray = new const char*[values.size()+1];
+    int i=0;
+    for (std::vector<std::string>::const_iterator it = values.begin(); it != values.end(); ++it) {
+#if defined (_MSC_VER)
+        _EnumArray[i++] = _strdup(it->c_str());
+#else
+        _EnumArray[i++] = strdup(it->c_str());
+#endif
+    }
+
+    _EnumArray[i] = 0; // null termination
+}
+
 const char** PropertyEnumeration::getEnums(void) const
 {
     return _EnumArray;
@@ -388,6 +408,11 @@ const char** PropertyEnumeration::getEnums(void) const
 
 PyObject *PropertyEnumeration::getPyObject(void)
 {
+    if (!_EnumArray) {
+        PyErr_SetString(PyExc_AssertionError, "The enum is empty");
+        return 0;
+    }
+
     return Py_BuildValue("s", getValueAsString());
 }
 
@@ -411,6 +436,24 @@ void PropertyEnumeration::setPyObject(PyObject *value)
         else
             throw Py::ValueError("not a member of the enum");
     }
+    else if (PyList_Check(value)) {
+        Py_ssize_t nSize = PyList_Size(value);
+        std::vector<std::string> values;
+        values.resize(nSize);
+
+        for (Py_ssize_t i=0; i<nSize;++i) {
+            PyObject* item = PyList_GetItem(value, i);
+            if (!PyString_Check(item)) {
+                std::string error = std::string("type in list must be str, not ");
+                error += item->ob_type->tp_name;
+                throw Py::TypeError(error);
+            }
+            values[i] = PyString_AsString(item);
+        }
+
+        setEnumVector(values);
+        setValue((long)0);
+    }
     else {
         std::string error = std::string("type must be int or str, not ");
         error += value->ob_type->tp_name;
@@ -455,9 +498,8 @@ void PropertyIntegerConstraint::setPyObject(PyObject *value)
 { 
     if (PyInt_Check(value)) {
         long temp = PyInt_AsLong(value);
-        if (_ConstStruct)
-        {
-            if(temp > _ConstStruct->UpperBound)
+        if (_ConstStruct) {
+            if (temp > _ConstStruct->UpperBound)
                 temp = _ConstStruct->UpperBound;
             else if(temp < _ConstStruct->LowerBound)
                 temp = _ConstStruct->LowerBound;
@@ -516,6 +558,16 @@ PropertyIntegerList::~PropertyIntegerList()
 
 }
 
+void PropertyIntegerList::setSize(int newSize)
+{
+    _lValueList.resize(newSize);
+}
+
+int PropertyIntegerList::getSize(void) const
+{
+    return static_cast<int>(_lValueList.size());
+}
+
 //**************************************************************************
 // Base class implementer
 
@@ -614,6 +666,10 @@ void PropertyIntegerList::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyIntegerList::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_lValueList.size() * sizeof(long));
+}
 
 //**************************************************************************
 //**************************************************************************
@@ -821,6 +877,16 @@ PropertyFloatList::~PropertyFloatList()
 //**************************************************************************
 // Base class implementer
 
+void PropertyFloatList::setSize(int newSize)
+{
+    _lValueList.resize(newSize);
+}
+
+int PropertyFloatList::getSize(void) const
+{
+    return static_cast<int>(_lValueList.size());
+}
+
 void PropertyFloatList::setValue(float lValue)
 {
     aboutToSetValue();
@@ -937,6 +1003,10 @@ void PropertyFloatList::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyFloatList::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_lValueList.size() * sizeof(float));
+}
 
 //**************************************************************************
 //**************************************************************************
@@ -1032,6 +1102,11 @@ void PropertyString::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyString::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_cValue.size());
+}
+
 //**************************************************************************
 // PropertyStringList
 //++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -1051,6 +1126,16 @@ PropertyStringList::~PropertyStringList()
 //**************************************************************************
 // Base class implementer
 
+void PropertyStringList::setSize(int newSize)
+{
+    _lValueList.resize(newSize);
+}
+
+int PropertyStringList::getSize(void) const
+{
+    return static_cast<int>(_lValueList.size());
+}
+
 void PropertyStringList::setValue(const std::string& lValue)
 {
     aboutToSetValue();
@@ -1118,10 +1203,10 @@ void PropertyStringList::setPyObject(PyObject *value)
 
 unsigned int PropertyStringList::getMemSize (void) const
 {
-    unsigned int size=0;
+    size_t size=0;
     for(int i = 0;i<getSize(); i++) 
         size += _lValueList[i].size();
-    return size;
+    return static_cast<unsigned int>(size);
 }
 
 void PropertyStringList::Save (Writer &writer) const
@@ -1178,10 +1263,10 @@ TYPESYSTEM_SOURCE(App::PropertyBool , App::Property);
 
 //**************************************************************************
 // Construction/Destruction
-       
+
 PropertyBool::PropertyBool()
 {
-
+    _lValue = false;
 }
 
 PropertyBool::~PropertyBool()
@@ -1432,6 +1517,16 @@ PropertyColorList::~PropertyColorList()
 //**************************************************************************
 // Base class implementer
 
+void PropertyColorList::setSize(int newSize)
+{
+    _lValueList.resize(newSize);
+}
+
+int PropertyColorList::getSize(void) const
+{
+    return static_cast<int>(_lValueList.size());
+}
+
 void PropertyColorList::setValue(const Color& lValue)
 {
     aboutToSetValue();
@@ -1558,6 +1653,11 @@ void PropertyColorList::Paste(const Property &from)
     hasSetValue();
 }
 
+unsigned int PropertyColorList::getMemSize (void) const
+{
+    return static_cast<unsigned int>(_lValueList.size() * sizeof(Color));
+}
+
 //**************************************************************************
 //**************************************************************************
 // PropertyMaterial
