@@ -113,7 +113,7 @@ public:
     virtual Property *Copy(void) const;
     virtual void Paste(const Property &from);
     
-    virtual unsigned int getMemSize (void) const{return _cValue.string().size();}
+    virtual unsigned int getMemSize (void) const;
 
 protected:
     boost::filesystem::path _cValue;
@@ -125,7 +125,7 @@ protected:
  * list is not set it act basicly like a IntegerProperty and do no checking.
  * If the list is set it checks on the range and if you set the state with
  * a string if its included in the enumarations.
- * In DEBUG the bounderies get checked, otherwise the caller of setValue()
+ * In DEBUG the boundaries get checked, otherwise the caller of setValue()
  * has the responsebility to check the correctnes.
  * This mean if you set by setValue(const char*) with an not included value
  * and not using isPartOf() before,
@@ -156,7 +156,7 @@ public:
      */
     void setValue(const char* value);
     /** set directly the enum value
-     * In DEBUG checks for bounderies.
+     * In DEBUG checks for boundaries.
      * Is faster then using setValue(const char*).
      */
     void setValue(long);
@@ -168,6 +168,8 @@ public:
     const char* getValueAsString(void) const;
     /// get all possible enum values as vector of strings
     std::vector<std::string> getEnumVector(void) const;
+    /// set all enum values as vector of strings
+    void setEnumVector(const std::vector<std::string>&);
     /// get the pointer to the enum list
     const char** getEnums(void) const;
     //@}
@@ -183,7 +185,7 @@ private:
 
 /** Constraint integer properties
  * This property fullfill the need of constraint integer. It holds basicly a 
- * state (integer) and a struct of bounderies. If the bounderies
+ * state (integer) and a struct of boundaries. If the boundaries
  * is not set it act basicly like a IntegerProperty and do no checking.
  * The constraints struct can be created on the heap or build in.
  */
@@ -200,11 +202,11 @@ public:
 
     /// Constraint methods 
     //@{
-    /// the boundery struct
+    /// the boundary struct
     struct Constraints {
         long LowerBound, UpperBound, StepSize;
     };
-    /** setting the bounderies
+    /** setting the boundaries
      * This sets the constriant struct. It can be dynamcly 
      * allocated or set as an static in the class the property
      * blongs to:
@@ -261,8 +263,8 @@ public:
      */
     ~PropertyIntegerList();
 
-    virtual void setSize(int newSize){_lValueList.resize(newSize);}   
-    virtual int getSize(void) const {return _lValueList.size();}   
+    virtual void setSize(int newSize);
+    virtual int getSize(void) const;
 
     /** Sets the property 
      */
@@ -284,8 +286,8 @@ public:
     
     virtual Property *Copy(void) const;
     virtual void Paste(const Property &from);
-    
-    virtual unsigned int getMemSize (void) const{return _lValueList.size() * sizeof(long);}
+    virtual unsigned int getMemSize (void) const;
+
 private:
     std::vector<long> _lValueList;
 };
@@ -333,7 +335,7 @@ protected:
 
 /** Constraint float properties
  * This property fullfill the need of constraint float. It holds basicly a 
- * state (float) and a struct of bounderies. If the bounderies
+ * state (float) and a struct of boundaries. If the boundaries
  * is not set it act basicly like a IntegerProperty and do no checking.
  * The constraints struct can be created on the heap or build in.
  */
@@ -357,11 +359,11 @@ public:
 
     /// Constraint methods 
     //@{
-    /// the boundery struct
+    /// the boundary struct
     struct Constraints {
         float LowerBound, UpperBound, StepSize;
     };
-    /** setting the bounderies
+    /** setting the boundaries
      * This sets the constriant struct. It can be dynamcly 
      * allocated or set as an static in the class the property
      * blongs to:
@@ -445,8 +447,8 @@ public:
      */
     virtual ~PropertyFloatList();
     
-    virtual void setSize(int newSize){_lValueList.resize(newSize);}   
-    virtual int getSize(void) const {return _lValueList.size();}   
+    virtual void setSize(int newSize);
+    virtual int getSize(void) const;
 
     /** Sets the property 
      */
@@ -472,9 +474,8 @@ public:
     
     virtual Property *Copy(void) const;
     virtual void Paste(const Property &from);
-    
-    virtual unsigned int getMemSize (void) const{return _lValueList.size() * sizeof(float);}
-    
+    virtual unsigned int getMemSize (void) const;
+
 private:
     std::vector<float> _lValueList;
 };
@@ -516,8 +517,7 @@ public:
 
     virtual Property *Copy(void) const;
     virtual void Paste(const Property &from);
-    
-    virtual unsigned int getMemSize (void) const{return _cValue.size();}
+    virtual unsigned int getMemSize (void) const;
 
 private:
     std::string _cValue;
@@ -540,9 +540,9 @@ public:
      * A more elaborate description of the destructor.
      */
     ~PropertyStringList();
-    
-    virtual void setSize(int newSize){_lValueList.resize(newSize);}   
-    virtual int getSize(void) const {return _lValueList.size();}   
+
+    virtual void setSize(int newSize);
+    virtual int getSize(void) const;
     
     /** Sets the property 
      */
@@ -682,9 +682,9 @@ public:
      * A more elaborate description of the destructor.
      */
     ~PropertyColorList();
-    
-    virtual void setSize(int newSize){_lValueList.resize(newSize);}   
-    virtual int getSize(void) const {return _lValueList.size();}   
+
+    virtual void setSize(int newSize);
+    virtual int getSize(void) const;
     
     /** Sets the property 
      */
@@ -709,8 +709,7 @@ public:
     
     virtual Property *Copy(void) const;
     virtual void Paste(const Property &from);
-    
-    virtual unsigned int getMemSize (void) const{return _lValueList.size() * sizeof(Color);}
+    virtual unsigned int getMemSize (void) const;
     
 private:
     std::vector<Color> _lValueList;
