@@ -27,7 +27,10 @@
 
 // (re-)defined in pyconfig.h
 #if defined (_POSIX_C_SOURCE)
-#   undef  _POSIX_C_SOURCE
+#   undef    _POSIX_C_SOURCE
+#endif
+#if defined (_XOPEN_SOURCE)
+#   undef    _XOPEN_SOURCE
 #endif
 
 // needed header
@@ -164,13 +167,13 @@ namespace Base
  *  need to get exported into the python namespace. This class is 
  *  very important because nearly all important classes in FreeCAD
  *  are visible in python for macro recording and automation purpose.
- *  The class App::Document is a good expample for a exported class.
+ *  The class App::Document is a good expample for an exported class.
  *  There are some conveniance macros to make it easier to inherit
  *  from this class and defining new methods exported to python.
- *  PYFUNCDEF_D defines a new exported methode.
- *  PYFUNCIMP_D defines the implementation of the new exported methode.
+ *  PYFUNCDEF_D defines a new exported method.
+ *  PYFUNCIMP_D defines the implementation of the new exported method.
  *  In the implementation you can use Py_Return, Py_Error, Py_Try and Py_Assert.
- *  PYMETHODEDEF makes the entry in the python methods tabel.
+ *  PYMETHODEDEF makes the entry in the python method tabel.
  *  @see Document
  *  @see PYFUNCDEF_D
  *  @see PYFUNCIMP_D
