@@ -30,12 +30,17 @@
 # include <sstream>
 # include <stdexcept>
 # include <QCloseEvent>
+# include <QLocale>
 # include <QMessageBox>
 # include <QPointer>
 # include <QGLFormat>
+# include <QGLPixelBuffer>
+#if QT_VERSION >= 0x040200
+# include <QGLFramebufferObject>
+#endif
+# include <QSessionManager>
 #endif
 
-#include <QSessionManager>
 
 // FreeCAD Base header
 #include <Base/Console.h>
@@ -44,7 +49,9 @@
 #include <Base/Exception.h>
 #include <Base/Factory.h>
 #include <Base/FileInfo.h>
+#include <Base/Tools.h>
 #include <App/Document.h>
+#include <App/DocumentObjectPy.h>
 
 #include "Application.h"
 #include "MainWindow.h"
@@ -65,8 +72,8 @@
 #include "BitmapFactory.h"
 #include "SoFCDB.h"
 #include "PythonConsolePy.h"
+#include "PythonDebugger.h"
 #include "View3DPy.h"
-#include "BrowserView.h"
 #include "DlgOnlineHelpImp.h"
 
 #include "View3DInventor.h"
@@ -102,9 +109,6 @@ struct ApplicationP
     _bIsClosing(false), 
     _bStartingUp(true), 
     _stderr(0)
-#if QT_VERSION >= 0x040400
-    ,pcStartPage(0)
-#endif 
     {
         // create the macro manager
         _pcMacroMngr = new MacroManager();
@@ -127,54 +131,237 @@ struct ApplicationP
     /// Handles all commands 
     CommandManager _cCommandManager;
     PyObject *_stderr;
-#if QT_VERSION >= 0x040400
-    QPointer<BrowserView> pcStartPage;
-#endif 
+};
+
+/** Observer that watches relabeled objects and make sure that the labels inside
+ * a document are unique.
+ * @note In the FreeCAD design it is explicitly allowed to have duplicate labels
+ * (i.e. the user visible text e.g. in the tree view) while the internal names
+ * are always guaranteed to be unique.
+ */
+class ObjectLabelObserver
+{
+public:
+    /// The one and only instance.
+    static ObjectLabelObserver* instance();
+    /// Destructs the sole instance.
+    static void destruct ();
+
+    /** Checks the new label of the object and relabel it if needed
+     * to make it unique document-wide
+     */
+    void slotRelabelObject(const App::DocumentObject&, const App::Property&);
+
+private:
+    static ObjectLabelObserver* _singleton;
+
+    ObjectLabelObserver();
+    ~ObjectLabelObserver();
+    const App::DocumentObject* current;
+    ParameterGrp::handle _hPGrp;
+};
+
+ObjectLabelObserver* ObjectLabelObserver::_singleton = 0;
+
+ObjectLabelObserver* ObjectLabelObserver::instance()
+{
+    if (!_singleton)
+        _singleton = new ObjectLabelObserver;
+    return _singleton;
+}
+
+void ObjectLabelObserver::destruct ()
+{
+    delete _singleton;
+    _singleton = 0;
+}
+
+void ObjectLabelObserver::slotRelabelObject(const App::DocumentObject& obj, const App::Property& prop)
+{
+    // observe only the Label property
+    if (&prop == &obj.Label) {
+        // have we processed this (or another?) object right now?
+        if (current) {
+            return;
+        }
+        // only if Label differ from internal name it can happen to have non-unique label
+        std::string label = obj.Label.getValue();
+        if (label != obj.getNameInDocument()) {
+            App::Document* doc = obj.getDocument();
+            if (doc && !_hPGrp->GetBool("DuplicateLabels")) {
+                std::vector<std::string> objectLabels;
+                std::vector<App::DocumentObject*>::const_iterator it;
+                std::vector<App::DocumentObject*> objs = doc->getObjects();
+                bool match = false;
+
+                for (it = objs.begin();it != objs.end();++it) {
+                    if (*it == &obj)
+                        continue; // don't compare object with itself
+                    std::string objLabel = (*it)->Label.getValue();
+                    if (!match && objLabel == label)
+                        match = true;
+                    objectLabels.push_back(objLabel);
+                }
+
+                // make sure that there is a name conflict otherwise we don't have to do anything
+                if (match) {
+                    label = Base::Tools::getUniqueName(label, objectLabels);
+                    this->current = &obj;
+                    const_cast<App::DocumentObject&>(obj).Label.setValue(label);
+                    this->current = 0;
+                }
+            }
+        }
+    }
+}
+
+ObjectLabelObserver::ObjectLabelObserver() : current(0)
+{
+    App::GetApplication().signalChangedObject.connect(boost::bind
+        (&ObjectLabelObserver::slotRelabelObject, this, _1, _2));
+    _hPGrp = App::GetApplication().GetUserParameter().GetGroup("BaseApp");
+    _hPGrp = _hPGrp->GetGroup("Preferences")->GetGroup("Document");
+}
+
+ObjectLabelObserver::~ObjectLabelObserver()
+{
+}
+
+static PyObject *
+FreeCADGui_subgraphFromObject(PyObject * /*self*/, PyObject *args)
+{
+    PyObject *o;
+    if (!PyArg_ParseTuple(args, "O!",&(App::DocumentObjectPy::Type), &o))
+        return NULL;
+    App::DocumentObject* obj = static_cast<App::DocumentObjectPy*>(o)->getDocumentObjectPtr();
+    std::string vp = obj->getViewProviderName();
+    SoNode* node = 0;
+    try {
+        Base::BaseClass* base = static_cast<Base::BaseClass*>(Base::Type::createInstanceByName(vp.c_str(), true));
+        if (base && base->getTypeId().isDerivedFrom(Gui::ViewProviderDocumentObject::getClassTypeId())) {
+            std::auto_ptr<Gui::ViewProviderDocumentObject> vp(static_cast<Gui::ViewProviderDocumentObject*>(base));
+            std::map<std::string, App::Property*> Map;
+            obj->getPropertyMap(Map);
+            vp->attach(obj);
+            for (std::map<std::string, App::Property*>::iterator it = Map.begin(); it != Map.end(); ++it) {
+                vp->updateData(it->second);
+            }
+
+            std::vector<std::string> modes = vp->getDisplayModes();
+            if (!modes.empty())
+                vp->setDisplayMode(modes.front().c_str());
+            node = vp->getRoot()->copy();
+            node->ref();
+            std::string type = "So";
+            type += node->getTypeId().getName().getString();
+            type += " *";
+            PyObject* proxy = 0;
+            proxy = Base::Interpreter().createSWIGPointerObj("pivy.coin", type.c_str(), (void*)node, 1);
+            return Py::new_reference_to(Py::Object(proxy, true));
+        }
+    }
+    catch (const Base::Exception& e) {
+        if (node) node->unref();
+        PyErr_SetString(PyExc_RuntimeError, e.what());
+        return 0;
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+static PyObject *
+FreeCADGui_getSoDBVersion(PyObject * /*self*/, PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+    return PyString_FromString(SoDB::getVersion());
+}
+
+static PyObject *
+FreeCADGui_getSoQtVersion(PyObject * /*self*/, PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+    return PyString_FromString(SoQt::getVersionString());
+}
+
+struct PyMethodDef FreeCADGui_methods[] = {
+    {"subgraphFromObject",FreeCADGui_subgraphFromObject,METH_VARARGS,
+     "subgraphFromObject(object) -> Node\n\n"
+     "Return the Inventor subgraph to an object"},
+    {"getSoDBVersion",FreeCADGui_getSoDBVersion,METH_VARARGS,
+     "getSoDBVersion() -> String\n\n"
+     "Return a text string containing the name\n"
+     "of the Coin library and version information"},
+    {"getSoQtVersion",FreeCADGui_getSoQtVersion,METH_VARARGS,
+     "getSoQtVersion() -> String\n\n"
+     "Return a text string containing the name\n"
+     "of the SoQt library and version information"},
+    {NULL, NULL}  /* sentinel */
 };
 
 } // namespace Gui
 
-Application::Application()
+Application::Application(bool GUIenabled)
 {
     //App::GetApplication().Attach(this);
-    App::GetApplication().signalNewDocument.connect(boost::bind(&Gui::Application::slotNewDocument, this, _1));
-    App::GetApplication().signalDeleteDocument.connect(boost::bind(&Gui::Application::slotDeleteDocument, this, _1));
-    App::GetApplication().signalRenameDocument.connect(boost::bind(&Gui::Application::slotRenameDocument, this, _1));
-    App::GetApplication().signalActiveDocument.connect(boost::bind(&Gui::Application::slotActiveDocument, this, _1));
-    App::GetApplication().signalRelabelDocument.connect(boost::bind(&Gui::Application::slotRelabelDocument, this, _1));
+    if (GUIenabled) {
+        App::GetApplication().signalNewDocument.connect(boost::bind(&Gui::Application::slotNewDocument, this, _1));
+        App::GetApplication().signalDeleteDocument.connect(boost::bind(&Gui::Application::slotDeleteDocument, this, _1));
+        App::GetApplication().signalRenameDocument.connect(boost::bind(&Gui::Application::slotRenameDocument, this, _1));
+        App::GetApplication().signalActiveDocument.connect(boost::bind(&Gui::Application::slotActiveDocument, this, _1));
+        App::GetApplication().signalRelabelDocument.connect(boost::bind(&Gui::Application::slotRelabelDocument, this, _1));
 
 
-    // install the last active language
-    ParameterGrp::handle hPGrp = App::GetApplication().GetUserParameter().GetGroup("BaseApp");
-    hPGrp = hPGrp->GetGroup("Preferences")->GetGroup("General");
-    Translator::instance()->activateLanguage(hPGrp->GetASCII("Language", "English").c_str());
-    GetWidgetFactorySupplier();
+        // install the last active language
+        ParameterGrp::handle hPGrp = App::GetApplication().GetUserParameter().GetGroup("BaseApp");
+        hPGrp = hPGrp->GetGroup("Preferences")->GetGroup("General");
+        QString lang = QLocale::languageToString(QLocale::system().language());
+        Translator::instance()->activateLanguage(hPGrp->GetASCII("Language", (const char*)lang.toAscii()).c_str());
+        GetWidgetFactorySupplier();
+
+        // setting up Python binding
+        Base::PyGILStateLocker lock;
+        PyObject* module = Py_InitModule3("FreeCADGui", Application::Methods,
+            "The functions in the FreeCADGui module allow working with GUI documents,\n"
+            "view providers, views, workbenches and much more.\n\n"
+            "The FreeCADGui instance provides a list of references of GUI documents which\n"
+            "can be addressed by a string. These documents contain the view providers for\n"
+            "objects in the associated App document. An App and GUI document can be\n"
+            "accessed with the same name.\n\n"
+            "The FreeCADGui module also provides a set of functions to work with so called\n"
+            "workbenches.");
+        Py::Module(module).setAttr(std::string("ActiveDocument"),Py::None());
+
+        //insert Selection module
+        PyObject* pSelectionModule = Py_InitModule3("Selection", SelectionSingleton::Methods,
+            "Selection module");
+        Py_INCREF(pSelectionModule);
+        PyModule_AddObject(module, "Selection", pSelectionModule);
+    }
 
-    // setting up Python binding
     Base::PyGILStateLocker lock;
-    PyObject* module = Py_InitModule3("FreeCADGui", Application::Methods,
-        "The functions in the FreeCADGui module allow working with GUI documents,\n"
-        "view providers, views, workbenches and much more.\n\n"
-        "The FreeCADGui instance provides a list of references of GUI documents which\n"
-        "can be addressed by a string. These documents contain the view providers for\n"
-        "objects in the associated App document. An App and GUI document can be\n"
-        "accessed with the same name.\n\n"
-        "The FreeCADGui module also provides a set of functions to work with so called\n"
-        "workbenches.");
-    Py::Module(module).setAttr(std::string("ActiveDocument"),Py::None());
-
-    //insert Selection module
-    PyObject* pSelectionModule = Py_InitModule3("Selection", SelectionSingleton::Methods,
-        "Selection module");
-    Py_INCREF(pSelectionModule);
-    PyModule_AddObject(module, "Selection", pSelectionModule);
+    PyObject *module = PyImport_AddModule("FreeCADGui");
+    PyMethodDef *meth = FreeCADGui_methods;
+    PyObject *dict = PyModule_GetDict(module);
+    for (; meth->ml_name != NULL; meth++) {
+        PyObject *descr;
+        descr = PyCFunction_NewEx(meth,0,0);
+        if (descr == NULL)
+            break;
+        if (PyDict_SetItemString(dict, meth->ml_name, descr) != 0)
+            break;
+        Py_DECREF(descr);
+    }
 
     // Python console binding
-    PythonStdout    ::init_type();
-    PythonStderr    ::init_type();
-    OutputStderr    ::init_type();
-    PythonStdin     ::init_type();
-    View3DInventorPy::init_type();
+    PythonDebugModule   ::init_module();
+    PythonStdout        ::init_type();
+    PythonStderr        ::init_type();
+    OutputStderr        ::init_type();
+    PythonStdin         ::init_type();
+    View3DInventorPy    ::init_type();
 
     d = new ApplicationP;
     bool redir = App::GetApplication().GetParameterGroupByPath("User parameter:BaseApp/Preferences/OutputWindow")->
@@ -192,6 +379,7 @@ Application::Application()
 
     createStandardOperations();
     MacroCommand::load();
+    ObjectLabelObserver::instance();
 }
 
 Application::~Application()
@@ -233,26 +421,6 @@ Application::~Application()
     Instance = 0;
 }
 
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-// Start page
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-
-void Application::createStartPage(const char* URL)
-{
-#if QT_VERSION < 0x040400
-
-#else
-    if (d->pcStartPage.isNull()) {
-        d->pcStartPage = new Gui::BrowserView(getMainWindow());   
-        d->pcStartPage->setWindowTitle(QObject::tr("Start page"));
-        //d->pcStartPage->setWindowIcon(FCIcon);
-        d->pcStartPage->resize(400, 300);
-        d->pcStartPage->load(URL);
-        getMainWindow()->addWindow(d->pcStartPage);
-    }
-#endif
-}
-
 
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 // creating std commands
@@ -264,18 +432,17 @@ void Application::open(const char* FileName, const char* Module)
     Base::FileInfo File(FileName);
     string te = File.extension();
 
+    // if the active document empty, close it
+    // in case of an automatically created empty document at startup
+    App::Document* act = App::GetApplication().getActiveDocument();
+    if (act && act->countObjects() == 0){
+        Command::doCommand(Command::App, "App.closeDocument('%s')", act->getName());
+        qApp->processEvents(); // an update is needed otherwise the new view isn't shown
+    }
+
     if (Module != 0) {
         // issue module loading
         Command::doCommand(Command::App, "import %s", Module);
-/*
-        // issue gui module loading
-        try {
-            Command::doCommand(Command::Gui, "import %sGui", Module);
-        }
-        catch (const Base::PyException&){
-            // ignore this type of exception (e.g. if Mod is already a Gui module)
-        }*/
-
         try {
             // load the file with the module
             Command::doCommand(Command::App, "%s.open(\"%s\")", Module, File.filePath().c_str());
@@ -321,7 +488,8 @@ void Application::importFrom(const char* FileName, const char* DocName, const ch
             else {
                 Command::doCommand(Command::App, "%s.insert(\"%s\",\"%s\")"
                                                , Module, File.filePath().c_str(), DocName);
-                getDocument(DocName)->setModified(true);
+                if (getDocument(DocName))
+                    getDocument(DocName)->setModified(true);
             }
 
             // the original file name is required
@@ -390,6 +558,7 @@ void Application::createStandardOperations()
     Gui::CreateStdCommands();
     Gui::CreateDocCommands();
     Gui::CreateFeatCommands();
+    Gui::CreateMacroCommands();
     Gui::CreateViewStdCommands();
     Gui::CreateWindowStdCommands();
     Gui::CreateTestCommands();
@@ -411,9 +580,10 @@ void Application::slotNewDocument(const App::Document& Doc)
 void Application::slotDeleteDocument(const App::Document& Doc)
 {
     std::map<const App::Document*, Gui::Document*>::iterator doc = d->lpcDocuments.find(&Doc);
-#ifdef FC_DEBUG
-    assert(doc!=d->lpcDocuments.end());
-#endif
+    if (doc == d->lpcDocuments.end()) {
+        Base::Console().Log("GUI document '%s' already deleted\n", Doc.getName());
+        return;
+    }
 
     // We must clear the selection here to notify all observers
     Gui::Selection().clearSelection(doc->second->getDocument()->getName());
@@ -421,10 +591,11 @@ void Application::slotDeleteDocument(const App::Document& Doc)
 
     // If the active document gets destructed we must set it to 0. If there are further existing documents then the 
     // view that becomes active sets the active document again. So, we needn't worry about this.
-    if ( d->_pcActiveDocument == doc->second )
+    if (d->_pcActiveDocument == doc->second)
         setActiveDocument(0);
 
-    delete doc->second; // destroy the Gui document
+    // For exception-safety use a smart pointer
+    auto_ptr<Document> delDoc (doc->second);
     d->lpcDocuments.erase(doc);
 }
 
@@ -562,28 +733,28 @@ Gui::Document* Application::getDocument(const App::Document* pDoc) const
 
 void Application::showViewProvider(App::DocumentObject* obj)
 {
-    App::Document* doc = obj->getDocument();
-    if (doc) {
-        Gui::Document* gui = getDocument(doc);
-        if (gui) {
-            ViewProvider* vp = gui->getViewProvider(obj);
-            if (vp)
-                vp->show();
-        }
-    }
+    ViewProvider* vp = getViewProvider(obj);
+    if (vp) vp->show();
 }
 
 void Application::hideViewProvider(App::DocumentObject* obj)
+{
+    ViewProvider* vp = getViewProvider(obj);
+    if (vp) vp->hide();
+}
+
+Gui::ViewProvider* Application::getViewProvider(App::DocumentObject* obj) const
 {
     App::Document* doc = obj->getDocument();
     if (doc) {
         Gui::Document* gui = getDocument(doc);
         if (gui) {
             ViewProvider* vp = gui->getViewProvider(obj);
-            if (vp)
-                vp->hide();
+            return vp;
         }
     }
+
+    return 0;
 }
 
 void Application::attachView(Gui::BaseView* pcView)
@@ -801,6 +972,7 @@ bool Application::activateWorkbench(const char* name)
 
 QPixmap Application::workbenchIcon(const QString& wb) const
 {
+    Base::PyGILStateLocker lock;
     // get the python workbench object from the dictionary
     PyObject* pcWorkbench = PyDict_GetItemString(_pcWorkbenchDictionary, wb.toAscii());
     // test if the workbench exists
@@ -855,6 +1027,7 @@ QPixmap Application::workbenchIcon(const QString& wb) const
 QString Application::workbenchToolTip(const QString& wb) const
 {
     // get the python workbench object from the dictionary
+    Base::PyGILStateLocker lock;
     PyObject* pcWorkbench = PyDict_GetItemString(_pcWorkbenchDictionary, wb.toAscii());
     // test if the workbench exists
     if (pcWorkbench) {
@@ -878,6 +1051,7 @@ QString Application::workbenchToolTip(const QString& wb) const
 QString Application::workbenchMenuText(const QString& wb) const
 {
     // get the python workbench object from the dictionary
+    Base::PyGILStateLocker lock;
     PyObject* pcWorkbench = PyDict_GetItemString(_pcWorkbenchDictionary, wb.toAscii());
     // test if the workbench exists
     if (pcWorkbench) {
@@ -1155,13 +1329,15 @@ void Application::initTypes(void)
     Gui::ViewProviderExtern                     ::init();
     Gui::ViewProviderDocumentObject             ::init();
     Gui::ViewProviderFeature                    ::init();
-    Gui::ViewProviderPythonFeature              ::init();
     Gui::ViewProviderDocumentObjectGroup        ::init();
     Gui::ViewProviderGeometryObject             ::init();
     Gui::ViewProviderInventorObject             ::init();
     Gui::ViewProviderVRMLObject                 ::init();
     Gui::ViewProviderAnnotation                 ::init();
+    Gui::ViewProviderPointMarker                ::init();
     Gui::ViewProviderMeasureDistance            ::init();
+    Gui::ViewProviderPythonFeature              ::init();
+    Gui::ViewProviderPythonGeometry             ::init();
 
     // Workbench
     Gui::Workbench                              ::init();
@@ -1262,7 +1438,7 @@ void Application::runApplication(void)
     GUIApplication mainApp(argc, App::Application::GetARGV());
     // set application icon and window title
     mainApp.setWindowIcon(Gui::BitmapFactory().pixmap(App::Application::Config()["AppIcon"].c_str()));
-    mainApp.setApplicationName(QString::fromAscii(App::Application::Config()["ExeName"].c_str()));
+    mainApp.setApplicationName(QString::fromAscii(App::GetApplication().getExecutableName()));
     QString plugin;
     plugin = QString::fromUtf8(App::GetApplication().GetHomePath());
     plugin += QLatin1String("/plugins");
@@ -1270,11 +1446,39 @@ void Application::runApplication(void)
 
     // check for OpenGL
     if (!QGLFormat::hasOpenGL()) {
-        QMessageBox::critical(0, QObject::tr("No OpenGL"), QObject::tr("This system has no OpenGL support"));
-        throw Base::Exception("This system has no OpenGL support");
+        QMessageBox::critical(0, QObject::tr("No OpenGL"), QObject::tr("This system does not support OpenGL"));
+        throw Base::Exception("This system does not support OpenGL");
+    }
+#if QT_VERSION >= 0x040200
+    if (!QGLFramebufferObject::hasOpenGLFramebufferObjects()) {
+        Base::Console().Log("This system does not support framebuffer objects");
+    }
+#endif
+    if (!QGLPixelBuffer::hasOpenGLPbuffers()) {
+        Base::Console().Log("This system does not support pbuffers");
     }
 
-    Application app;
+    QGLFormat::OpenGLVersionFlags version = QGLFormat::openGLVersionFlags ();
+    if (version & QGLFormat::OpenGL_Version_3_0)
+        Base::Console().Log("OpenGL version 3.0 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_2_1)
+        Base::Console().Log("OpenGL version 2.1 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_2_0)
+        Base::Console().Log("OpenGL version 2.0 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_1_5)
+        Base::Console().Log("OpenGL version 1.5 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_1_4)
+        Base::Console().Log("OpenGL version 1.4 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_1_3)
+        Base::Console().Log("OpenGL version 1.3 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_1_2)
+        Base::Console().Log("OpenGL version 1.2 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_1_1)
+        Base::Console().Log("OpenGL version 1.1 or higher is present\n");
+    else if (version & QGLFormat::OpenGL_Version_None)
+        Base::Console().Log("No OpenGL is present or no OpenGL context is current\n");
+
+    Application app(true);
     MainWindow mw;
     mw.setWindowTitle(mainApp.applicationName());
 
@@ -1338,12 +1542,6 @@ void Application::runApplication(void)
         App::GetApplication().newDocument();
     }
     
-    // show Startup page
-    hGrp = WindowParameter::getDefaultParameter()->GetGroup("OnlineHelp");
-    if (hGrp->GetBool("ShowStartPage",true)) {
-        QString page = Gui::Dialog::DlgOnlineHelpImp::getStartpage();
-        app.createStartPage(page.toUtf8());
-    }
 
     // run the Application event loop
     Base::Console().Log("Init: Entering event loop\n");
