@@ -252,7 +252,7 @@ PyObject* Application::sOpen(PyObject * /*self*/, PyObject *args,PyObject * /*kw
                  ext == QLatin1String("fcscript")) {
             PythonEditor* editor = new PythonEditor();
             editor->setWindowIcon(Gui::BitmapFactory().pixmap("python_small"));
-            EditorView* edit = new EditorView(editor, getMainWindow());
+            PythonEditorView* edit = new PythonEditorView(editor, getMainWindow());
             edit->open(fileName);
             edit->resize(400, 300);
             getMainWindow()->addWindow( edit );
@@ -300,7 +300,7 @@ PyObject* Application::sInsert(PyObject * /*self*/, PyObject *args,PyObject * /*
                  ext == QLatin1String("fcscript")) {
             PythonEditor* editor = new PythonEditor();
             editor->setWindowIcon(Gui::BitmapFactory().pixmap("python_small"));
-            EditorView* edit = new EditorView(editor, getMainWindow());
+            PythonEditorView* edit = new PythonEditorView(editor, getMainWindow());
             edit->open(fileName);
             edit->resize(400, 300);
             getMainWindow()->addWindow( edit );
@@ -336,7 +336,8 @@ PyObject* Application::sExport(PyObject * /*self*/, PyObject *args,PyObject * /*
             fi.setFile(fileName);
             QString ext = fi.completeSuffix().toLower();
             if (ext == QLatin1String("iv") || ext == QLatin1String("wrl") ||
-                ext == QLatin1String("vrml") || ext == QLatin1String("wrz")) {
+                ext == QLatin1String("vrml") || ext == QLatin1String("wrz") ||
+                ext == QLatin1String("svg") || ext == QLatin1String("idtf")) {
                 QString cmd = QString::fromLatin1(
                     "Gui.getDocument(\"%1\").ActiveView.dump(\"%2\")"
                     ).arg(QLatin1String(doc->getName())).arg(fi.absoluteFilePath());
@@ -674,8 +675,14 @@ PyObject* Application::sRunCommand(PyObject * /*self*/, PyObject *args,PyObject
     if (!PyArg_ParseTuple(args, "s", &pName))     // convert args: Python->C 
         return NULL;                    // NULL triggers exception 
 
-    Application::Instance->commandManager().runCommandByName(pName);
-
-    Py_INCREF(Py_None);
-    return Py_None;
+    Command* cmd = Application::Instance->commandManager().getCommandByName(pName);
+    if (cmd) {
+        cmd->invoke(0);
+        Py_INCREF(Py_None);
+        return Py_None;
+    }
+    else {
+        PyErr_Format(PyExc_Exception, "No such command '%s'", pName);
+        return 0;
+    }
 } 
