@@ -24,6 +24,7 @@
 #include "PreCompiled.h"
 #ifndef _PreComp_
 # include <QDir>
+# include <QKeySequence>
 # include <QMessageBox>
 #endif
 
@@ -39,6 +40,7 @@
 #include "BitmapFactory.h"
 #include "WhatsThis.h"
 #include "WaitCursor.h"
+#include "Control.h"
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
@@ -46,7 +48,6 @@
 #include <Base/Sequencer.h>
 
 #include <App/Document.h>
-#include <App/Feature.h>
 #include <App/DocumentObject.h>
 
 
@@ -136,22 +137,6 @@ Action * CommandBase::createAction()
     return 0;
 }
 
-void CommandBase::languageChange()
-{
-    if ( _pcAction ) {
-        _pcAction->setText       ( QObject::tr( sMenuText    ) );
-        _pcAction->setToolTip    ( QObject::tr( sToolTipText ) );
-        if ( sStatusTip )
-            _pcAction->setStatusTip ( QObject::tr( sStatusTip   ) );
-        else
-            _pcAction->setStatusTip ( QObject::tr( sToolTipText ) );
-        if ( sWhatsThis )
-            _pcAction->setWhatsThis( QObject::tr( sWhatsThis   ) );
-        else
-            _pcAction->setWhatsThis( QObject::tr( sToolTipText ) );
-    }
-}
-
 void CommandBase::setMenuText(const char* s)
 {
 #if defined (_MSC_VER)
@@ -213,6 +198,7 @@ Command::Command(const char* name)
 {
     sAppModule  = "FreeCAD";
     sGroup      = QT_TR_NOOP("Standard");
+    eType       = AlterDoc | Alter3DView | AlterSelection;
 }
 
 Command::~Command()
@@ -244,15 +230,16 @@ Application *Command::getGuiApplication(void)
     return Application::Instance;
 }
 
-Gui::Document* Command::getActiveGuiDocument(void)
+Gui::Document* Command::getActiveGuiDocument(void) const
 {
     return getGuiApplication()->activeDocument();
 }
 
-App::Document* Command::getDocument(const char* Name)
+App::Document* Command::getDocument(const char* Name) const
 {
     if (Name)
         return App::GetApplication().getDocument(Name);
+
     else {
         Gui::Document * pcDoc = getGuiApplication()->activeDocument();
         if (pcDoc)
@@ -262,16 +249,7 @@ App::Document* Command::getDocument(const char* Name)
     }
 }
 
-App::AbstractFeature* Command::getFeature(const char* Name)
-{
-    App::DocumentObject *pObj = getObject(Name);
-    if (pObj && pObj->getTypeId().isDerivedFrom(App::AbstractFeature::getClassTypeId()))
-        return dynamic_cast<App::AbstractFeature*>(pObj);
-    else
-        return 0;
-}
-
-App::DocumentObject* Command::getObject(const char* Name)
+App::DocumentObject* Command::getObject(const char* Name) const
 {
     App::Document*pDoc = getDocument();
     if (pDoc)
@@ -305,18 +283,14 @@ void Command::invoke (int i)
         QMessageBox::critical(Gui::getMainWindow(), QObject::tr("Exception"), QLatin1String(e.what()));
     }
     catch (std::exception &e) {
-        std::string str;
-        str += "C++ exception thrown (";
-        str += e.what();
-        str += ")";
-        Base::Console().Error(str.c_str());
+        Base::Console().Error("C++ exception thrown (%s)\n", e.what());
     }
     catch (const char* e) {
         Base::Console().Error("%s\n", e);
     }
 #ifndef FC_DEBUG
     catch (...) {
-        Base::Console().Error("Gui::Command::activated(%d): Unknown C++ exception in command thrown", i);
+        Base::Console().Error("Gui::Command::activated(%d): Unknown C++ exception thrown", i);
     }
 #endif
 }
@@ -325,20 +299,27 @@ void Command::testActive(void)
 {
     if (!_pcAction) return;
 
-    if ( _blockCmd ) {
-        _pcAction->setEnabled ( false );
+    if (_blockCmd) {
+        _pcAction->setEnabled(false);
+        return;
+    }
+
+    if ((!Gui::Control().isAllowedAlterDocument()  && eType & AlterDoc)    ||
+        (!Gui::Control().isAllowedAlterView()      && eType & Alter3DView) ||
+        (!Gui::Control().isAllowedAlterSelection() && eType & AlterSelection)) {
+         _pcAction->setEnabled(false);
         return;
     }
 
     bool bActive = isActive();
-    _pcAction->setEnabled ( bActive );
+    _pcAction->setEnabled(bActive);
 }
 
 //--------------------------------------------------------------------------
 // Helper methods
 //--------------------------------------------------------------------------
 
-bool Command::hasActiveDocument(void)
+bool Command::hasActiveDocument(void) const
 {
     return getActiveGuiDocument() != 0;
 }
@@ -353,7 +334,7 @@ Gui::SelectionSingleton&  Command::getSelection(void)
     return Gui::Selection();
 }
 
-std::string Command::getUniqueObjectName(const char *BaseName)
+std::string Command::getUniqueObjectName(const char *BaseName) const
 {
     assert(hasActiveDocument());
 
@@ -473,27 +454,53 @@ const char * Command::beginCmdHelp(void)
             "</head>\n"
             "<body bgcolor=\"#ffffff\">\n\n";
 }
+
 /// returns the end of a online help page
 const char * Command::endCmdHelp(void)
 {
     return "</body></html>\n\n";
 }
 
+void Command::applyCommandData(Action* action)
+{
+    action->setText(QCoreApplication::translate(
+        this->className(), sMenuText, 0,
+        QCoreApplication::CodecForTr));
+    action->setToolTip(QCoreApplication::translate(
+        this->className(), sToolTipText, 0,
+        QCoreApplication::CodecForTr));
+    if (sStatusTip)
+        action->setStatusTip(QCoreApplication::translate(
+            this->className(), sStatusTip, 0,
+            QCoreApplication::CodecForTr));
+    else
+        action->setStatusTip(QCoreApplication::translate(
+            this->className(), sToolTipText, 0,
+            QCoreApplication::CodecForTr));
+    if (sWhatsThis)
+        action->setWhatsThis(QCoreApplication::translate(
+            this->className(), sWhatsThis, 0,
+            QCoreApplication::CodecForTr));
+    else
+        action->setWhatsThis(QCoreApplication::translate(
+            this->className(), sToolTipText, 0,
+            QCoreApplication::CodecForTr));
+}
+
+int Command::keySequenceToAccel(int sk) const
+{
+    QKeySequence::StandardKey type = (QKeySequence::StandardKey)sk;
+    QKeySequence ks(type);
+    QVariant v = ks.operator QVariant();
+	return v.toInt();
+}
+
 Action * Command::createAction(void)
 {
     Action *pcAction;
 
     pcAction = new Action(this,getMainWindow());
-    pcAction->setText(QObject::tr(sMenuText));
-    pcAction->setToolTip(QObject::tr(sToolTipText));
-    if ( sStatusTip )
-        pcAction->setStatusTip(QObject::tr(sStatusTip));
-    else
-        pcAction->setStatusTip(QObject::tr(sToolTipText));
-    if ( sWhatsThis )
-        pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-    else
-        pcAction->setWhatsThis(QObject::tr(sToolTipText));
+    applyCommandData(pcAction);
     if (sPixmap)
         pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
     pcAction->setShortcut(iAccel);
@@ -501,6 +508,13 @@ Action * Command::createAction(void)
     return pcAction;
 }
 
+void Command::languageChange()
+{
+    if (_pcAction) {
+        applyCommandData(_pcAction);
+    }
+}
+
 //===========================================================================
 // MacroCommand
 //===========================================================================
@@ -521,13 +535,13 @@ void MacroCommand::activated(int iMsg)
 {
     std::string cMacroPath = App::GetApplication().GetParameterGroupByPath
                              ("User parameter:BaseApp/Preferences/Macro")->GetASCII("MacroPath",
-                                     App::GetApplication().GetHomePath());
+                                     App::Application::getUserAppDataDir().c_str());
 
     QDir d(QString::fromUtf8(cMacroPath.c_str()));
     QFileInfo fi(d, QString::fromUtf8(sScriptName));
     Application::Instance->macroManager()->run(MacroManager::File, fi.filePath().toUtf8());
     // after macro run recalculate the document
-    if ( Application::Instance->activeDocument() )
+    if (Application::Instance->activeDocument())
         Application::Instance->activeDocument()->getDocument()->recompute();
 }
 
