@@ -23,7 +23,9 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <QClipboard>
 # include <QStatusBar>
+# include <QPointer>
 #endif
 #include <algorithm>
 
@@ -32,19 +34,24 @@
 #include <Base/Sequencer.h>
 #include <App/Document.h>
 #include <App/DocumentObjectGroup.h>
-#include <App/Feature.h>
+#include <App/DocumentObject.h>
+#include <App/GeoFeature.h>
 
 #include "Action.h"
 #include "Application.h"
 #include "Document.h"
 #include "Command.h"
+#include "Control.h"
 #include "FileDialog.h"
 #include "MainWindow.h"
 #include "BitmapFactory.h"
 #include "Selection.h"
 #include "DlgProjectInformationImp.h"
+#include "Transform.h"
+#include "Placement.h"
+#include "WaitCursor.h"
+#include "ViewProvider.h"
 
-using Base::Sequencer;
 using namespace Gui;
 
 
@@ -64,7 +71,7 @@ StdCmdOpen::StdCmdOpen()
     sWhatsThis    = "Std_Open";
     sStatusTip    = QT_TR_NOOP("Open a document or import files");
     sPixmap       = "document-open";
-    iAccel        = Qt::CTRL+Qt::Key_O;
+    iAccel        = keySequenceToAccel(QKeySequence::Open);
 }
 
 void StdCmdOpen::activated(int iMsg)
@@ -183,7 +190,7 @@ void StdCmdImport::activated(int iMsg)
 
 bool StdCmdImport::isActive(void)
 {
-  return ( getActiveGuiDocument() ? true : false );
+    return (getActiveGuiDocument() ? true : false);
 }
 
 
@@ -270,7 +277,7 @@ StdCmdNew::StdCmdNew()
   sWhatsThis    = "Std_New";
   sStatusTip    = QT_TR_NOOP("Create a new empty document");
   sPixmap       = "document-new";
-  iAccel        = Qt::CTRL+Qt::Key_N;
+  iAccel        = keySequenceToAccel(QKeySequence::New);
 }
 
 void StdCmdNew::activated(int iMsg)
@@ -292,7 +299,7 @@ StdCmdSave::StdCmdSave()
   sWhatsThis    = "Std_Save";
   sStatusTip    = QT_TR_NOOP("Save the active document");
   sPixmap       = "document-save";
-  iAccel        = Qt::CTRL+Qt::Key_S;
+  iAccel        = keySequenceToAccel(QKeySequence::Save);
 }
 
 void StdCmdSave::activated(int iMsg)
@@ -395,27 +402,26 @@ DEF_STD_CMD_A(StdCmdPrint );
 StdCmdPrint::StdCmdPrint()
   :Command("Std_Print")
 {
-  sGroup        = QT_TR_NOOP("File");
-  sMenuText     = QT_TR_NOOP("&Print...");
-  sToolTipText  = QT_TR_NOOP("Print the document");
-  sWhatsThis    = "Std_Print";
-  sStatusTip    = QT_TR_NOOP("Print the document");
-  sPixmap       = "document-print";
-  iAccel        = Qt::CTRL+Qt::Key_P;;
+    sGroup        = QT_TR_NOOP("File");
+    sMenuText     = QT_TR_NOOP("&Print...");
+    sToolTipText  = QT_TR_NOOP("Print the document");
+    sWhatsThis    = "Std_Print";
+    sStatusTip    = QT_TR_NOOP("Print the document");
+    sPixmap       = "document-print";
+    iAccel        = keySequenceToAccel(QKeySequence::Print);
 }
 
 void StdCmdPrint::activated(int iMsg)
 {
-  if ( getMainWindow()->activeWindow() )
-  {
-      getMainWindow()->statusBar()->showMessage(QObject::tr("Printing..."));
-    getMainWindow()->activeWindow()->print();
-  }
+    if (getMainWindow()->activeWindow()) {
+        getMainWindow()->statusBar()->showMessage(QObject::tr("Printing..."));
+        getMainWindow()->activeWindow()->print();
+    }
 }
 
 bool StdCmdPrint::isActive(void)
 {
-  return getGuiApplication()->sendHasMsgToActiveView("Print");
+    return getGuiApplication()->sendHasMsgToActiveView("Print");
 }
 
 //===========================================================================
@@ -426,25 +432,24 @@ DEF_STD_CMD_A(StdCmdPrintPdf);
 StdCmdPrintPdf::StdCmdPrintPdf()
   :Command("Std_PrintPdf")
 {
-  sGroup        = QT_TR_NOOP("File");
-  sMenuText     = QT_TR_NOOP("&Export PDF...");
-  sToolTipText  = QT_TR_NOOP("Export the document as PDF");
-  sWhatsThis    = "Std_PrintPdf";
-  sStatusTip    = QT_TR_NOOP("Export the document as PDF");
+    sGroup        = QT_TR_NOOP("File");
+    sMenuText     = QT_TR_NOOP("&Export PDF...");
+    sToolTipText  = QT_TR_NOOP("Export the document as PDF");
+    sWhatsThis    = "Std_PrintPdf";
+    sStatusTip    = QT_TR_NOOP("Export the document as PDF");
 }
 
 void StdCmdPrintPdf::activated(int iMsg)
 {
-  if ( getMainWindow()->activeWindow() )
-  {
-    getMainWindow()->statusBar()->showMessage(QLatin1String("Exporting PDF..."));
-    getMainWindow()->activeWindow()->printPdf();
-  }
+    if (getMainWindow()->activeWindow()) {
+        getMainWindow()->statusBar()->showMessage(QObject::tr("Exporting PDF..."));
+        getMainWindow()->activeWindow()->printPdf();
+    }
 }
 
 bool StdCmdPrintPdf::isActive(void)
 {
-  return getGuiApplication()->sendHasMsgToActiveView("PrintPdf");
+    return getGuiApplication()->sendHasMsgToActiveView("PrintPdf");
 }
 
 //===========================================================================
@@ -489,7 +494,7 @@ StdCmdUndo::StdCmdUndo()
   sWhatsThis    = "Std_Undo";
   sStatusTip    = QT_TR_NOOP("Undo exactly one action");
   sPixmap       = "edit-undo";
-  iAccel        = Qt::CTRL+Qt::Key_Z;
+  iAccel        = keySequenceToAccel(QKeySequence::Undo);
 }
 
 void StdCmdUndo::activated(int iMsg)
@@ -505,18 +510,15 @@ bool StdCmdUndo::isActive(void)
 
 Action * StdCmdUndo::createAction(void)
 {
-  Action *pcAction;
+    Action *pcAction;
 
-  pcAction = new UndoAction(this,getMainWindow());
-  pcAction->setText(QObject::tr(sMenuText));
-  pcAction->setToolTip(QObject::tr(sToolTipText));
-  pcAction->setStatusTip(QObject::tr(sStatusTip));
-  pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-  if(sPixmap)
-    pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
-  pcAction->setShortcut(iAccel);
+    pcAction = new UndoAction(this,getMainWindow());
+    applyCommandData(pcAction);
+    if (sPixmap)
+        pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
+    pcAction->setShortcut(iAccel);
 
-  return pcAction;
+    return pcAction;
 }
 
 //===========================================================================
@@ -534,7 +536,7 @@ StdCmdRedo::StdCmdRedo()
   sWhatsThis    = "Std_Redo";
   sStatusTip    = QT_TR_NOOP("Redoes a previously undone action");
   sPixmap       = "edit-redo";
-  iAccel        = Qt::CTRL+Qt::Key_Y;
+  iAccel        = keySequenceToAccel(QKeySequence::Redo);
 }
 
 void StdCmdRedo::activated(int iMsg)
@@ -550,18 +552,15 @@ bool StdCmdRedo::isActive(void)
 
 Action * StdCmdRedo::createAction(void)
 {
-  Action *pcAction;
+    Action *pcAction;
 
-  pcAction = new RedoAction(this,getMainWindow());
-  pcAction->setText(QObject::tr(sMenuText));
-  pcAction->setToolTip(QObject::tr(sToolTipText));
-  pcAction->setStatusTip(QObject::tr(sStatusTip));
-  pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-  if(sPixmap)
-    pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
-  pcAction->setShortcut(iAccel);
+    pcAction = new RedoAction(this,getMainWindow());
+    applyCommandData(pcAction);
+    if (sPixmap)
+        pcAction->setIcon(Gui::BitmapFactory().pixmap(sPixmap));
+    pcAction->setShortcut(iAccel);
 
-  return pcAction;
+    return pcAction;
 }
 
 //===========================================================================
@@ -570,25 +569,25 @@ Action * StdCmdRedo::createAction(void)
 DEF_STD_CMD_A(StdCmdCut);
 
 StdCmdCut::StdCmdCut()
-  :Command("Std_Cut")
+  : Command("Std_Cut")
 {
-  sGroup        = QT_TR_NOOP("Edit");
-  sMenuText     = QT_TR_NOOP("&Cut");
-  sToolTipText  = QT_TR_NOOP("Cut out");
-  sWhatsThis    = "Std_Cut";
-  sStatusTip    = QT_TR_NOOP("Cut out");
-  sPixmap       = "edit-cut";
-  iAccel        = Qt::CTRL+Qt::Key_X;
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("&Cut");
+    sToolTipText  = QT_TR_NOOP("Cut out");
+    sWhatsThis    = "Std_Cut";
+    sStatusTip    = QT_TR_NOOP("Cut out");
+    sPixmap       = "edit-cut";
+    iAccel        = keySequenceToAccel(QKeySequence::Cut);
 }
 
 void StdCmdCut::activated(int iMsg)
 {
-  getGuiApplication()->sendMsgToActiveView("Cut");
+    getGuiApplication()->sendMsgToActiveView("Cut");
 }
 
 bool StdCmdCut::isActive(void)
 {
-  return getGuiApplication()->sendHasMsgToActiveView("Cut");
+    return getGuiApplication()->sendHasMsgToActiveView("Cut");
 }
 
 //===========================================================================
@@ -597,25 +596,33 @@ bool StdCmdCut::isActive(void)
 DEF_STD_CMD_A(StdCmdCopy);
 
 StdCmdCopy::StdCmdCopy()
-  :Command("Std_Copy")
+  : Command("Std_Copy")
 {
-  sGroup        = QT_TR_NOOP("Edit");
-  sMenuText     = QT_TR_NOOP("C&opy");
-  sToolTipText  = QT_TR_NOOP("Copy operation");
-  sWhatsThis    = "Std_Copy";
-  sStatusTip    = QT_TR_NOOP("Copy operation");
-  sPixmap       = "edit-copy";
-  iAccel        = Qt::CTRL+Qt::Key_C;
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("C&opy");
+    sToolTipText  = QT_TR_NOOP("Copy operation");
+    sWhatsThis    = "Std_Copy";
+    sStatusTip    = QT_TR_NOOP("Copy operation");
+    sPixmap       = "edit-copy";
+    iAccel        = keySequenceToAccel(QKeySequence::Copy);
 }
 
 void StdCmdCopy::activated(int iMsg)
 {
-  getGuiApplication()->sendMsgToActiveView("Copy");
+    bool done = getGuiApplication()->sendMsgToActiveView("Copy");
+    if (!done) {
+        WaitCursor wc;
+        QMimeData * mimeData = getMainWindow()->createMimeDataFromSelection();
+        QClipboard* cb = QApplication::clipboard();
+        cb->setMimeData(mimeData);
+    }
 }
 
 bool StdCmdCopy::isActive(void)
 {
-  return getGuiApplication()->sendHasMsgToActiveView("Copy");
+    if (getGuiApplication()->sendHasMsgToActiveView("Copy"))
+        return true;
+    return Selection().hasSelection();
 }
 
 //===========================================================================
@@ -624,25 +631,112 @@ bool StdCmdCopy::isActive(void)
 DEF_STD_CMD_A(StdCmdPaste);
 
 StdCmdPaste::StdCmdPaste()
-  :Command("Std_Paste")
+  : Command("Std_Paste")
 {
-  sGroup        = QT_TR_NOOP("Edit");
-  sMenuText     = QT_TR_NOOP("&Paste");
-  sToolTipText  = QT_TR_NOOP("Paste operation");
-  sWhatsThis    = "Std_Paste";
-  sStatusTip    = QT_TR_NOOP("Paste operation");
-  sPixmap       = "edit-paste";
-  iAccel        = Qt::CTRL+Qt::Key_V;
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("&Paste");
+    sToolTipText  = QT_TR_NOOP("Paste operation");
+    sWhatsThis    = "Std_Paste";
+    sStatusTip    = QT_TR_NOOP("Paste operation");
+    sPixmap       = "edit-paste";
+    iAccel        = keySequenceToAccel(QKeySequence::Paste);
 }
 
 void StdCmdPaste::activated(int iMsg)
 {
-  getGuiApplication()->sendMsgToActiveView("Paste");
+    bool done = getGuiApplication()->sendMsgToActiveView("Paste");
+    if (!done) {
+        QClipboard* cb = QApplication::clipboard();
+        const QMimeData* mimeData = cb->mimeData();
+        if (mimeData) {
+            WaitCursor wc;
+            getMainWindow()->insertFromMimeData(mimeData);
+        }
+    }
 }
 
 bool StdCmdPaste::isActive(void)
 {
-  return getGuiApplication()->sendHasMsgToActiveView("Paste");
+    if (getGuiApplication()->sendHasMsgToActiveView("Paste"))
+        return true;
+    QClipboard* cb = QApplication::clipboard();
+    const QMimeData* mime = cb->mimeData();
+    if (!mime) return false;
+    return getMainWindow()->canInsertFromMimeData(mime);
+}
+
+DEF_STD_CMD_A(StdCmdDDuplicateSelection);
+
+StdCmdDDuplicateSelection::StdCmdDDuplicateSelection()
+  :Command("Std_DuplicateSelection")
+{
+    sAppModule    = "Edit";
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("Duplicate selection");
+    sToolTipText  = QT_TR_NOOP("Put duplicates of the selected objects to the active document");
+    sWhatsThis    = QT_TR_NOOP("Put duplicates of the selected objects to the active document");
+    sStatusTip    = QT_TR_NOOP("Put duplicates of the selected objects to the active document");
+}
+
+void StdCmdDDuplicateSelection::activated(int iMsg)
+{
+    App::Document* act = App::GetApplication().getActiveDocument();
+    if (!act)
+        return; // no active document found
+    Gui::Document* doc = Gui::Application::Instance->getDocument(act);
+    std::vector<Gui::SelectionSingleton::SelObj> sel = Gui::Selection().getCompleteSelection();
+    for (std::vector<SelectionSingleton::SelObj>::iterator it = sel.begin(); it != sel.end(); ++it) {
+        if (!it->pObject)
+            continue; // should actually not happen
+        // create a copy of the object
+        App::DocumentObject* copy = act->copyObject(it->pObject, false);
+        if (!copy) // continue if no copy could be created
+            continue;
+        // mark all properties of the copy as "touched" which are touched in the original object
+        std::map<std::string,App::Property*> props;
+        it->pObject->getPropertyMap(props);
+        std::map<std::string,App::Property*> copy_props;
+        copy->getPropertyMap(copy_props);
+        for (std::map<std::string,App::Property*>::iterator jt = props.begin(); jt != props.end(); ++jt) {
+            if (jt->second->isTouched()) {
+                std::map<std::string,App::Property*>::iterator kt;
+                kt = copy_props.find(jt->first);
+                if (kt != copy_props.end()) {
+                    kt->second->touch();
+                }
+            }
+        }
+
+        Gui::Document* parent = Gui::Application::Instance->getDocument(it->pObject->getDocument());
+        if (!parent || !doc)
+            continue; // should not happen
+        // copy the properties of the associated view providers
+        Gui::ViewProvider* view = parent->getViewProvider(it->pObject);
+        Gui::ViewProvider* copy_view = doc->getViewProvider(copy);
+        if (!view || !copy_view)
+            continue; // should not happen
+
+        // get the properties of the view provider
+        props.clear();
+        view->getPropertyMap(props);
+        copy_props.clear();
+        copy_view->getPropertyMap(copy_props);
+        for (std::map<std::string,App::Property*>::iterator jt = props.begin(); jt != props.end(); ++jt) {
+            std::map<std::string,App::Property*>::iterator kt;
+            kt = copy_props.find(jt->first);
+            if (kt != copy_props.end()) {
+                std::auto_ptr<App::Property> data(jt->second->Copy());
+                if (data.get()) {
+                    kt->second->Paste(*data);
+                }
+            }
+        }
+    }
+}
+
+bool StdCmdDDuplicateSelection::isActive(void)
+{
+    return Gui::Selection().hasSelection();
 }
 
 //===========================================================================
@@ -694,7 +788,7 @@ StdCmdDelete::StdCmdDelete()
 #if QT_VERSION >= 0x040200
   sPixmap       = "edit-delete";
 #endif
-  iAccel        = Qt::Key_Delete;
+  iAccel        = keySequenceToAccel(QKeySequence::Delete);
 }
 
 void StdCmdDelete::activated(int iMsg)
@@ -737,7 +831,7 @@ StdCmdRefresh::StdCmdRefresh()
   sWhatsThis    = "Std_Refresh";
   sStatusTip    = QT_TR_NOOP("Recomputes the current active document");
   sPixmap       = "view-refresh";
-  iAccel        = Qt::Key_F5;
+  iAccel        = keySequenceToAccel(QKeySequence::Refresh);
 }
 
 void StdCmdRefresh::activated(int iMsg)
@@ -755,6 +849,63 @@ bool StdCmdRefresh::isActive(void)
   return this->getDocument() && this->getDocument()->isTouched();
 }
 
+//===========================================================================
+// Std_Transform
+//===========================================================================
+DEF_STD_CMD_A(StdCmdTransform);
+
+StdCmdTransform::StdCmdTransform()
+  : Command("Std_Transform")
+{
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("Transform...");
+    sToolTipText  = QT_TR_NOOP("Transform the geometry of selected objects");
+    sStatusTip    = QT_TR_NOOP("Transform the geometry of selected objects");
+    sWhatsThis    = "Std_Transform";
+}
+
+void StdCmdTransform::activated(int iMsg)
+{
+    Gui::Control().showDialog(new Gui::Dialog::TaskTransform());
+}
+
+bool StdCmdTransform::isActive(void)
+{
+    return (Gui::Control().activeDialog()==0);
+}
+
+//===========================================================================
+// Std_Placement
+//===========================================================================
+DEF_STD_CMD_A(StdCmdPlacement);
+
+StdCmdPlacement::StdCmdPlacement()
+  : Command("Std_Placement")
+{
+    sGroup        = QT_TR_NOOP("Edit");
+    sMenuText     = QT_TR_NOOP("Placement...");
+    sToolTipText  = QT_TR_NOOP("Place the selected objects");
+    sStatusTip    = QT_TR_NOOP("Place the selected objects");
+    sWhatsThis    = "Std_Placement";
+}
+
+void StdCmdPlacement::activated(int iMsg)
+{
+    std::vector<App::DocumentObject*> sel = Gui::Selection().getObjectsOfType(App::GeoFeature::getClassTypeId());
+    Gui::Dialog::TaskPlacement* plm = new Gui::Dialog::TaskPlacement();
+    if (!sel.empty()) {
+        App::Property* prop = sel.front()->getPropertyByName("Placement");
+        if (prop && prop->getTypeId() == App::PropertyPlacement::getClassTypeId())
+            plm->setPlacement(static_cast<App::PropertyPlacement*>(prop)->getValue());
+    }
+    Gui::Control().showDialog(plm);
+}
+
+bool StdCmdPlacement::isActive(void)
+{
+    return (Gui::Control().activeDialog()==0);
+}
+
 
 namespace Gui {
 
@@ -778,9 +929,13 @@ void CreateDocCommands(void)
     rcCmdMgr.addCommand(new StdCmdCut());
     rcCmdMgr.addCommand(new StdCmdCopy());
     rcCmdMgr.addCommand(new StdCmdPaste());
+    rcCmdMgr.addCommand(new StdCmdDDuplicateSelection());
     rcCmdMgr.addCommand(new StdCmdSelectAll());
     rcCmdMgr.addCommand(new StdCmdDelete());
     rcCmdMgr.addCommand(new StdCmdRefresh());
+    rcCmdMgr.addCommand(new StdCmdTransform());
+    rcCmdMgr.addCommand(new StdCmdPlacement());
 }
 
 } // namespace Gui
+
