@@ -51,7 +51,9 @@
 #include "View3DInventorViewer.h"
 #include "WaitCursor.h"
 #include "ViewProviderMeasureDistance.h"
+#include "ViewProviderGeometryObject.h"
 #include "SceneInspector.h"
+#include "DemoMode.h"
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
@@ -61,7 +63,7 @@
 #include <App/GeoFeature.h>
 #include <App/DocumentObjectGroup.h>
 #include <App/MeasureDistance.h>
-#include <App/Feature.h>
+#include <App/DocumentObject.h>
 
 #include <QDomDocument>
 #include <QDomElement>
@@ -76,13 +78,14 @@ DEF_STD_CMD_AC(StdOrthographicCamera);
 StdOrthographicCamera::StdOrthographicCamera()
   : Command("Std_OrthographicCamera")
 {
-  sGroup        = QT_TR_NOOP("Standard-View");
-  sMenuText     = QT_TR_NOOP("Orthographic view");
-  sToolTipText  = QT_TR_NOOP("Switches to orthographic view mode");
-  sWhatsThis    = "Std_OrthographicCamera";
-  sStatusTip    = QT_TR_NOOP("Switches to orthographic view mode");
-  sPixmap       = "view-isometric";
-  iAccel        = Qt::Key_O;
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Orthographic view");
+    sToolTipText  = QT_TR_NOOP("Switches to orthographic view mode");
+    sWhatsThis    = "Std_OrthographicCamera";
+    sStatusTip    = QT_TR_NOOP("Switches to orthographic view mode");
+    sPixmap       = "view-isometric";
+    iAccel        = Qt::Key_O;
+    eType         = Alter3DView;
 }
 
 void StdOrthographicCamera::activated(int iMsg)
@@ -96,26 +99,24 @@ void StdOrthographicCamera::activated(int iMsg)
 
 bool StdOrthographicCamera::isActive(void)
 {
-  View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-  if ( view )
-  {
-    // update the action group if needed
-    bool check = _pcAction->isChecked();
-    bool mode = view->getViewer()->getCameraType() == SoOrthographicCamera::getClassTypeId();
-    if (mode != check)
-        _pcAction->setChecked(mode);
+    View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
+    if (view) {
+        // update the action group if needed
+        bool check = _pcAction->isChecked();
+        bool mode = view->getViewer()->getCameraType() == SoOrthographicCamera::getClassTypeId();
+        if (mode != check)
+            _pcAction->setChecked(mode);
+        return true;
+    }
 
-    return true;
-  }
-
-  return false;
+    return false;
 }
 
 Action * StdOrthographicCamera::createAction(void)
 {
-  Action *pcAction = Command::createAction();
-  pcAction->setCheckable( true );
-  return pcAction;
+    Action *pcAction = Command::createAction();
+    pcAction->setCheckable(true);
+    return pcAction;
 }
 
 DEF_STD_CMD_AC(StdPerspectiveCamera);
@@ -123,13 +124,14 @@ DEF_STD_CMD_AC(StdPerspectiveCamera);
 StdPerspectiveCamera::StdPerspectiveCamera()
   : Command("Std_PerspectiveCamera")
 {
-  sGroup        = QT_TR_NOOP("Standard-View");
-  sMenuText     = QT_TR_NOOP("Perspective view");
-  sToolTipText  = QT_TR_NOOP("Switches to perspective view mode");
-  sWhatsThis    = "Std_PerspectiveCamera";
-  sStatusTip    = QT_TR_NOOP("Switches to perspective view mode");
-  sPixmap       = "view-perspective";
-  iAccel        = Qt::Key_P;
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Perspective view");
+    sToolTipText  = QT_TR_NOOP("Switches to perspective view mode");
+    sWhatsThis    = "Std_PerspectiveCamera";
+    sStatusTip    = QT_TR_NOOP("Switches to perspective view mode");
+    sPixmap       = "view-perspective";
+    iAccel        = Qt::Key_P;
+    eType         = Alter3DView;
 }
 
 void StdPerspectiveCamera::activated(int iMsg)
@@ -143,26 +145,25 @@ void StdPerspectiveCamera::activated(int iMsg)
 
 bool StdPerspectiveCamera::isActive(void)
 {
-  View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-  if ( view )
-  {
-    // update the action group if needed
-    bool check = _pcAction->isChecked();
-    bool mode = view->getViewer()->getCameraType() == SoPerspectiveCamera::getClassTypeId();
-    if (mode != check)
-        _pcAction->setChecked(mode);
+    View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
+    if (view) {
+        // update the action group if needed
+        bool check = _pcAction->isChecked();
+        bool mode = view->getViewer()->getCameraType() == SoPerspectiveCamera::getClassTypeId();
+        if (mode != check)
+            _pcAction->setChecked(mode);
 
-    return true;
-  }
+        return true;
+    }
 
-  return false;
+    return false;
 }
 
 Action * StdPerspectiveCamera::createAction(void)
 {
-  Action *pcAction = Command::createAction();
-  pcAction->setCheckable( true );
-  return pcAction;
+    Action *pcAction = Command::createAction();
+    pcAction->setCheckable(true);
+    return pcAction;
 }
 
 //===========================================================================
@@ -173,6 +174,8 @@ class StdCmdFreezeViews : public Gui::Command
 public:
     StdCmdFreezeViews();
     virtual ~StdCmdFreezeViews(){}
+    const char* className() const
+    { return "StdCmdFreezeViews"; }
 
 protected: 
     virtual void activated(int iMsg);
@@ -197,82 +200,84 @@ private:
 StdCmdFreezeViews::StdCmdFreezeViews()
   : Command("Std_FreezeViews"), maxViews(50), savedViews(0)
 {
-  sGroup        = QT_TR_NOOP("Standard-View");
-  sMenuText     = QT_TR_NOOP("Freeze display");
-  sToolTipText  = QT_TR_NOOP("Freezes the current view position");
-  sWhatsThis    = "Std_FreezeViews";
-  sStatusTip    = QT_TR_NOOP("Freezes the current view position");
-  iAccel        = Qt::SHIFT+Qt::Key_F;
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Freeze display");
+    sToolTipText  = QT_TR_NOOP("Freezes the current view position");
+    sWhatsThis    = "Std_FreezeViews";
+    sStatusTip    = QT_TR_NOOP("Freezes the current view position");
+    iAccel        = Qt::SHIFT+Qt::Key_F;
+    eType         = Alter3DView;
 }
 
 Action * StdCmdFreezeViews::createAction(void)
 {
-  ActionGroup* pcAction = new ActionGroup(this, getMainWindow());
-  pcAction->setDropDownMenu(true);
-  pcAction->setText(QObject::tr(sMenuText));
-  pcAction->setToolTip(QObject::tr(sToolTipText));
-  pcAction->setStatusTip(QObject::tr(sStatusTip));
-  pcAction->setWhatsThis(QObject::tr(sWhatsThis));
-  
-  // add the action items
-  saveView = pcAction->addAction(QObject::tr("Save views..."));
-  pcAction->addAction(QObject::tr("Load views..."));
-  pcAction->addAction(QString::fromAscii(""))->setSeparator(true);
-  freezeView = pcAction->addAction(QObject::tr("Freeze view"));
-  freezeView->setShortcut(iAccel);
-  clearView = pcAction->addAction(QObject::tr("Clear views"));
-  separator = pcAction->addAction(QString::fromAscii(""));
-  separator->setSeparator(true);
-  offset = pcAction->actions().count();
+    ActionGroup* pcAction = new ActionGroup(this, getMainWindow());
+    pcAction->setDropDownMenu(true);
+    applyCommandData(pcAction);
 
-  // allow up to 50 views
-  for (int i=0; i<maxViews; i++)
-      pcAction->addAction(QString::fromAscii(""))->setVisible(false);
+    // add the action items
+    saveView = pcAction->addAction(QObject::tr("Save views..."));
+    pcAction->addAction(QObject::tr("Load views..."));
+    pcAction->addAction(QString::fromAscii(""))->setSeparator(true);
+    freezeView = pcAction->addAction(QObject::tr("Freeze view"));
+    freezeView->setShortcut(iAccel);
+    clearView = pcAction->addAction(QObject::tr("Clear views"));
+    separator = pcAction->addAction(QString::fromAscii(""));
+    separator->setSeparator(true);
+    offset = pcAction->actions().count();
 
-  return pcAction;
+    // allow up to 50 views
+    for (int i=0; i<maxViews; i++)
+        pcAction->addAction(QString::fromAscii(""))->setVisible(false);
+
+    return pcAction;
 }
 
 void StdCmdFreezeViews::activated(int iMsg)
 {
-  ActionGroup* pcAction = qobject_cast<ActionGroup*>(_pcAction);
-  
-  if ( iMsg == 0 ) {
-      onSaveViews();
-  } else if ( iMsg == 1 ) {
-      onRestoreViews();
-  } else if ( iMsg == 3 ) {
-    // Create a new view
-    const char* ppReturn=0;
-    getGuiApplication()->sendMsgToActiveView("GetCamera",&ppReturn);
+    ActionGroup* pcAction = qobject_cast<ActionGroup*>(_pcAction);
 
-    QList<QAction*> acts = pcAction->actions();
-    int index = 0;
-    for (QList<QAction*>::ConstIterator it = acts.begin()+offset; it != acts.end(); ++it, index++) {
-      if ( !(*it)->isVisible() ) {
-        savedViews++;
-        QString viewnr = QString(QObject::tr("Restore view &%1")).arg(index+1);
-        (*it)->setText(viewnr);
-        (*it)->setToolTip(QString::fromAscii(ppReturn));
-        (*it)->setVisible(true);
-        if ( index < 9 ) {
-          int accel = Qt::CTRL+Qt::Key_1;
-          (*it)->setShortcut(accel+index);
+    if (iMsg == 0) {
+        onSaveViews();
+    }
+    else if (iMsg == 1) {
+        onRestoreViews();
+    }
+    else if (iMsg == 3) {
+        // Create a new view
+        const char* ppReturn=0;
+        getGuiApplication()->sendMsgToActiveView("GetCamera",&ppReturn);
+
+        QList<QAction*> acts = pcAction->actions();
+        int index = 0;
+        for (QList<QAction*>::ConstIterator it = acts.begin()+offset; it != acts.end(); ++it, index++) {
+            if (!(*it)->isVisible()) {
+                savedViews++;
+                QString viewnr = QString(QObject::tr("Restore view &%1")).arg(index+1);
+                (*it)->setText(viewnr);
+                (*it)->setToolTip(QString::fromAscii(ppReturn));
+                (*it)->setVisible(true);
+                if (index < 9) {
+                    int accel = Qt::CTRL+Qt::Key_1;
+                    (*it)->setShortcut(accel+index);
+                }
+                break;
+            }
         }
-        break;
-      }
+    }
+    else if (iMsg == 4) {
+        savedViews = 0;
+        QList<QAction*> acts = pcAction->actions();   
+        for (QList<QAction*>::ConstIterator it = acts.begin()+offset; it != acts.end(); ++it)
+            (*it)->setVisible(false);
+    }
+    else if (iMsg >= offset) {
+        // Activate a view
+        QList<QAction*> acts = pcAction->actions();
+        QString data = acts[iMsg]->toolTip();
+        QString send = QString::fromAscii("SetCamera %1").arg(data);
+        getGuiApplication()->sendMsgToActiveView(send.toAscii());
     }
-  } else if ( iMsg == 4 ) {
-    savedViews = 0;
-    QList<QAction*> acts = pcAction->actions();   
-    for (QList<QAction*>::ConstIterator it = acts.begin()+offset; it != acts.end(); ++it)
-      (*it)->setVisible(false);
-  } else if ( iMsg >= offset ) {
-    // Activate a view
-    QList<QAction*> acts = pcAction->actions();
-    QString data = acts[iMsg]->toolTip();
-    QString send = QString::fromAscii("SetCamera %1").arg(data);
-    getGuiApplication()->sendMsgToActiveView(send.toAscii());
-  }
 }
 
 void StdCmdFreezeViews::onSaveViews()
@@ -318,7 +323,7 @@ void StdCmdFreezeViews::onSaveViews()
 void StdCmdFreezeViews::onRestoreViews()
 {
     // Should we clear the already saved views
-    if ( savedViews > 0 ) {
+    if (savedViews > 0) {
         int ret = QMessageBox::question(getMainWindow(), QObject::tr("Restore views"), 
             QObject::tr("Importing the restored views would clear the already stored views.\n"
                         "Do you want to continue?"), QMessageBox::Yes|QMessageBox::Default,
@@ -405,39 +410,40 @@ void StdCmdFreezeViews::onRestoreViews()
 
 bool StdCmdFreezeViews::isActive(void)
 {
-  View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-  if (view) {
-    saveView->setEnabled(savedViews > 0);
-    freezeView->setEnabled(savedViews < maxViews);
-    clearView->setEnabled(savedViews > 0);
-    separator->setVisible(savedViews > 0);
-    return true;
-  } else {
-    separator->setVisible(savedViews > 0);
-  }
+    View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
+    if (view) {
+        saveView->setEnabled(savedViews > 0);
+        freezeView->setEnabled(savedViews < maxViews);
+        clearView->setEnabled(savedViews > 0);
+        separator->setVisible(savedViews > 0);
+        return true;
+    }
+    else {
+        separator->setVisible(savedViews > 0);
+    }
 
-  return false;
+    return false;
 }
 
 void StdCmdFreezeViews::languageChange()
 {
-  Command::languageChange();
+    Command::languageChange();
 
-  if (!_pcAction)
-    return;
-  ActionGroup* pcAction = qobject_cast<ActionGroup*>(_pcAction);
-  QList<QAction*> acts = pcAction->actions();
-  acts[0]->setText(QObject::tr("Save views..."));
-  acts[1]->setText(QObject::tr("Load views..."));
-  acts[3]->setText(QObject::tr("Freeze view"));
-  acts[4]->setText(QObject::tr("Clear views"));
-  int index=1;
-  for (QList<QAction*>::ConstIterator it = acts.begin()+5; it != acts.end(); ++it, index++) {
-    if ( (*it)->isVisible() ) {
-      QString viewnr = QString(QObject::tr("Restore view &%1")).arg(index);
-      (*it)->setText(viewnr);
+    if (!_pcAction)
+        return;
+    ActionGroup* pcAction = qobject_cast<ActionGroup*>(_pcAction);
+    QList<QAction*> acts = pcAction->actions();
+    acts[0]->setText(QObject::tr("Save views..."));
+    acts[1]->setText(QObject::tr("Load views..."));
+    acts[3]->setText(QObject::tr("Freeze view"));
+    acts[4]->setText(QObject::tr("Clear views"));
+    int index=1;
+    for (QList<QAction*>::ConstIterator it = acts.begin()+5; it != acts.end(); ++it, index++) {
+        if ((*it)->isVisible()) {
+            QString viewnr = QString(QObject::tr("Restore view &%1")).arg(index);
+            (*it)->setText(viewnr);
+        }
     }
-  }
 }
 
 //===========================================================================
@@ -449,45 +455,47 @@ DEF_STD_CMD_AC(StdCmdToggleClipPlane);
 StdCmdToggleClipPlane::StdCmdToggleClipPlane()
   : Command("Std_ToggleClipPlane")
 {
-  sGroup        = QT_TR_NOOP("Standard-View");
-  sMenuText     = QT_TR_NOOP("Clipping plane");
-  sToolTipText  = QT_TR_NOOP("Toggles clipping plane for active view");
-  sWhatsThis    = "Std_ToggleClipPlane";
-  sStatusTip    = QT_TR_NOOP("Toggles clipping plane for active view");
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Clipping plane");
+    sToolTipText  = QT_TR_NOOP("Toggles clipping plane for active view");
+    sWhatsThis    = "Std_ToggleClipPlane";
+    sStatusTip    = QT_TR_NOOP("Toggles clipping plane for active view");
+    eType         = Alter3DView;
 }
 
 Action * StdCmdToggleClipPlane::createAction(void)
 {
-  Action *pcAction = (Action*)Command::createAction();
-  pcAction->setCheckable( true );
-  return pcAction;
+    Action *pcAction = (Action*)Command::createAction();
+    pcAction->setCheckable(true);
+    return pcAction;
 }
 
 void StdCmdToggleClipPlane::activated(int iMsg)
 {
-  View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-  if (view) {
-    if (iMsg > 0 && !view->hasClippingPlane())
-      view->toggleClippingPlane();
-    else if (iMsg == 0 && view->hasClippingPlane())
-      view->toggleClippingPlane();
-  }
+    View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
+    if (view) {
+        if (iMsg > 0 && !view->hasClippingPlane())
+            view->toggleClippingPlane();
+        else if (iMsg == 0 && view->hasClippingPlane())
+            view->toggleClippingPlane();
+    }
 }
 
 bool StdCmdToggleClipPlane::isActive(void)
 {
-  View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-  if (view){
-    Action* action = qobject_cast<Action*>(_pcAction);
-    if (action->isChecked() != view->hasClippingPlane())
-      action->setChecked(view->hasClippingPlane());
-    return true;
-  } else {
-    Action* action = qobject_cast<Action*>(_pcAction);
-    if (action->isChecked())
-      action->setChecked(false);
-    return false;
-  }
+    View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
+    if (view) {
+        Action* action = qobject_cast<Action*>(_pcAction);
+        if (action->isChecked() != view->hasClippingPlane())
+            action->setChecked(view->hasClippingPlane());
+        return true;
+    }
+    else {
+        Action* action = qobject_cast<Action*>(_pcAction);
+        if (action->isChecked())
+            action->setChecked(false);
+        return false;
+    }
 }
 
 //===========================================================================
@@ -504,6 +512,7 @@ StdCmdToggleVisibility::StdCmdToggleVisibility()
     sStatusTip    = QT_TR_NOOP("Toggles visibility");
     sWhatsThis    = "Std_ToggleVisibility";
     iAccel        = Qt::Key_Space;
+    eType         = Alter3DView;
 }
 
 void StdCmdToggleVisibility::activated(int iMsg)
@@ -555,6 +564,52 @@ bool StdCmdToggleVisibility::isActive(void)
     return (Gui::Selection().size() != 0);
 }
 
+//===========================================================================
+// Std_ToggleSelectability
+//===========================================================================
+DEF_STD_CMD_A(StdCmdToggleSelectability);
+
+StdCmdToggleSelectability::StdCmdToggleSelectability()
+  : Command("Std_ToggleSelectability")
+{
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Toggle selectability");
+    sToolTipText  = QT_TR_NOOP("Toggles the property of the objects to get selected in the 3D-View");
+    sStatusTip    = QT_TR_NOOP("Toggles the property of the objects to get selected in the 3D-View");
+    sWhatsThis    = "Std_ToggleSelectability";
+    sPixmap       = "view-unselectable";
+    eType         = Alter3DView;
+}
+
+void StdCmdToggleSelectability::activated(int iMsg)
+{
+    // go through all documents
+    const std::vector<App::Document*> docs = App::GetApplication().getDocuments();
+    for (std::vector<App::Document*>::const_iterator it = docs.begin(); it != docs.end(); ++it) {
+        Document *pcDoc = Application::Instance->getDocument(*it);
+        std::vector<App::DocumentObject*> sel = Selection().getObjectsOfType
+            (App::DocumentObject::getClassTypeId(), (*it)->getName());
+
+ 
+        for (std::vector<App::DocumentObject*>::const_iterator ft=sel.begin();ft!=sel.end();ft++) {
+            ViewProvider *pr = pcDoc->getViewProviderByName((*ft)->getNameInDocument());
+            if(pr->isDerivedFrom(ViewProviderGeometryObject::getClassTypeId())){
+                    if (dynamic_cast<ViewProviderGeometryObject*>(pr)->Selectable.getValue())
+                        doCommand(Gui,"Gui.getDocument(\"%s\").getObject(\"%s\").Selectable=False"
+                                     , (*it)->getName(), (*ft)->getNameInDocument());
+                    else
+                        doCommand(Gui,"Gui.getDocument(\"%s\").getObject(\"%s\").Selectable=True"
+                                     , (*it)->getName(), (*ft)->getNameInDocument());
+            }
+        }
+    }
+}
+
+bool StdCmdToggleSelectability::isActive(void)
+{
+    return (Gui::Selection().size() != 0);
+}
+
 //===========================================================================
 // Std_ShowSelection
 //===========================================================================
@@ -568,6 +623,7 @@ StdCmdShowSelection::StdCmdShowSelection()
     sToolTipText  = QT_TR_NOOP("Show all selected objects");
     sStatusTip    = QT_TR_NOOP("Show all selected objects");
     sWhatsThis    = "Std_ShowSelection";
+    eType         = Alter3DView;
 }
 
 void StdCmdShowSelection::activated(int iMsg)
@@ -602,6 +658,7 @@ StdCmdHideSelection::StdCmdHideSelection()
     sToolTipText  = QT_TR_NOOP("Hide all selected objects");
     sStatusTip    = QT_TR_NOOP("Hide all selected objects");
     sWhatsThis    = "Std_HideSelection";
+    eType         = Alter3DView;
 }
 
 void StdCmdHideSelection::activated(int iMsg)
@@ -636,6 +693,7 @@ StdCmdToggleObjects::StdCmdToggleObjects()
     sToolTipText  = QT_TR_NOOP("Toggles visibility of all objects in the active document");
     sStatusTip    = QT_TR_NOOP("Toggles visibility of all objects in the active document");
     sWhatsThis    = "Std_ToggleObjects";
+    eType         = Alter3DView;
 }
 
 void StdCmdToggleObjects::activated(int iMsg)
@@ -674,6 +732,7 @@ StdCmdShowObjects::StdCmdShowObjects()
     sToolTipText  = QT_TR_NOOP("Show all objects in the document");
     sStatusTip    = QT_TR_NOOP("Show all objects in the document");
     sWhatsThis    = "Std_ShowObjects";
+    eType         = Alter3DView;
 }
 
 void StdCmdShowObjects::activated(int iMsg)
@@ -708,6 +767,7 @@ StdCmdHideObjects::StdCmdHideObjects()
     sToolTipText  = QT_TR_NOOP("Hide all objects in the document");
     sStatusTip    = QT_TR_NOOP("Hide all objects in the document");
     sWhatsThis    = "Std_HideObjects";
+    eType         = Alter3DView;
 }
 
 void StdCmdHideObjects::activated(int iMsg)
@@ -744,6 +804,7 @@ StdCmdSetAppearance::StdCmdSetAppearance()
     sStatusTip    = QT_TR_NOOP("Sets the display properties of the selected object");
     sPixmap       = "Std_Tool1";
     iAccel        = Qt::CTRL+Qt::Key_D;
+    eType         = Alter3DView;
 }
 
 void StdCmdSetAppearance::activated(int iMsg)
@@ -776,6 +837,7 @@ StdCmdViewBottom::StdCmdViewBottom()
   sStatusTip    = QT_TR_NOOP("Set to bottom view");
   sPixmap       = "view-bottom";
   iAccel        = Qt::Key_5;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewBottom::activated(int iMsg)
@@ -798,6 +860,7 @@ StdCmdViewFront::StdCmdViewFront()
   sStatusTip    = QT_TR_NOOP("Set to front view");
   sPixmap       = "view-front";
   iAccel        = Qt::Key_1;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewFront::activated(int iMsg)
@@ -820,6 +883,7 @@ StdCmdViewLeft::StdCmdViewLeft()
   sStatusTip    = QT_TR_NOOP("Set to left view");
   sPixmap       = "view-left";
   iAccel        = Qt::Key_6;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewLeft::activated(int iMsg)
@@ -842,6 +906,7 @@ StdCmdViewRear::StdCmdViewRear()
   sStatusTip    = QT_TR_NOOP("Set to rear view");
   sPixmap       = "view-rear";
   iAccel        = Qt::Key_4;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewRear::activated(int iMsg)
@@ -864,6 +929,7 @@ StdCmdViewRight::StdCmdViewRight()
   sStatusTip    = QT_TR_NOOP("Set to right view");
   sPixmap       = "view-right";
   iAccel        = Qt::Key_3;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewRight::activated(int iMsg)
@@ -886,6 +952,7 @@ StdCmdViewTop::StdCmdViewTop()
   sStatusTip    = QT_TR_NOOP("Set to top view");
   sPixmap       = "view-top";
   iAccel        = Qt::Key_2;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewTop::activated(int iMsg)
@@ -908,6 +975,7 @@ StdCmdViewAxo::StdCmdViewAxo()
   sStatusTip  = QT_TR_NOOP("Set to axometric view");
   sPixmap     = "view-axometric";
   iAccel      = Qt::Key_0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewAxo::activated(int iMsg)
@@ -930,6 +998,7 @@ StdCmdViewFitAll::StdCmdViewFitAll()
   sStatusTip    = QT_TR_NOOP("Fits the whole content on the screen");
   sPixmap       = "view-zoom-all";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewFitAll::activated(int iMsg)
@@ -961,6 +1030,7 @@ StdCmdViewFitSelection::StdCmdViewFitSelection()
     sPixmap       = "view-zoom-selection";
 #endif
     iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewFitSelection::activated(int iMsg)
@@ -983,87 +1053,103 @@ DEF_STD_CMD_AC(StdViewDockUndockFullscreen);
 StdViewDockUndockFullscreen::StdViewDockUndockFullscreen()
   : Command("Std_ViewDockUndockFullscreen")
 {
-  sGroup      = QT_TR_NOOP("Standard-View");
-  sMenuText   = QT_TR_NOOP("Display mode");
-  sToolTipText= QT_TR_NOOP("Display the active view either in fullscreen, in undocked or docked mode");
-  sWhatsThis  = "Std_ViewDockUndockFullscreen";
-  sStatusTip  = QT_TR_NOOP("Display the active view either in fullscreen, in undocked or docked mode");
+    sGroup      = QT_TR_NOOP("Standard-View");
+    sMenuText   = QT_TR_NOOP("Display mode");
+    sToolTipText= QT_TR_NOOP("Display the active view either in fullscreen, in undocked or docked mode");
+    sWhatsThis  = "Std_ViewDockUndockFullscreen";
+    sStatusTip  = QT_TR_NOOP("Display the active view either in fullscreen, in undocked or docked mode");
+    eType         = Alter3DView;
 }
 
 Action * StdViewDockUndockFullscreen::createAction(void)
 {
-  ActionGroup* pcAction = new ActionGroup(this, getMainWindow());
-  pcAction->setDropDownMenu(true);
-  pcAction->setText(QObject::tr(sMenuText));
+    ActionGroup* pcAction = new ActionGroup(this, getMainWindow());
+    pcAction->setDropDownMenu(true);
+    pcAction->setText(QCoreApplication::translate(
+        this->className(), sMenuText, 0,
+        QCoreApplication::CodecForTr));
 
-  QAction* docked = pcAction->addAction(QObject::tr(QT_TR_NOOP("Docked")));
-  docked->setToolTip(QObject::tr(sToolTipText));
-  docked->setStatusTip(QObject::tr(sStatusTip));
-  docked->setWhatsThis(QObject::tr(sWhatsThis));
-  docked->setShortcut(Qt::Key_D);
-  docked->setCheckable(true);
+    QAction* docked = pcAction->addAction(QObject::tr(QT_TR_NOOP("Docked")));
+    docked->setToolTip(QCoreApplication::translate(
+        this->className(), sToolTipText, 0,
+        QCoreApplication::CodecForTr));
+    docked->setStatusTip(QCoreApplication::translate(
+        this->className(), sStatusTip, 0,
+        QCoreApplication::CodecForTr));
+    docked->setWhatsThis(QCoreApplication::translate(
+        this->className(), sWhatsThis, 0,
+        QCoreApplication::CodecForTr));
+    docked->setShortcut(Qt::Key_D);
+    docked->setCheckable(true);
 
-  QAction* undocked = pcAction->addAction(QObject::tr(QT_TR_NOOP("Undocked")));
-  undocked->setToolTip(QObject::tr(sToolTipText));
-  undocked->setStatusTip(QObject::tr(sStatusTip));
-  undocked->setWhatsThis(QObject::tr(sWhatsThis));
-  undocked->setShortcut(Qt::Key_U);
-  undocked->setCheckable(true);
+    QAction* undocked = pcAction->addAction(QObject::tr(QT_TR_NOOP("Undocked")));
+    undocked->setToolTip(QCoreApplication::translate(
+        this->className(), sToolTipText, 0,
+        QCoreApplication::CodecForTr));
+    undocked->setStatusTip(QCoreApplication::translate(
+        this->className(), sStatusTip, 0,
+        QCoreApplication::CodecForTr));
+    undocked->setWhatsThis(QCoreApplication::translate(
+        this->className(), sWhatsThis, 0,
+        QCoreApplication::CodecForTr));
+    undocked->setShortcut(Qt::Key_U);
+    undocked->setCheckable(true);
 
-  QAction* fullscr = pcAction->addAction(QObject::tr(QT_TR_NOOP("Fullscreen")));
-  fullscr->setToolTip(QObject::tr(sToolTipText));
-  fullscr->setStatusTip(QObject::tr(sStatusTip));
-  fullscr->setWhatsThis(QObject::tr(sWhatsThis));
-  fullscr->setShortcut(Qt::Key_F11);
-  fullscr->setCheckable(true);
-  fullscr->setIcon(Gui::BitmapFactory().pixmap("view-fullscreen"));
+    QAction* fullscr = pcAction->addAction(QObject::tr(QT_TR_NOOP("Fullscreen")));
+    fullscr->setToolTip(QCoreApplication::translate(
+        this->className(), sToolTipText, 0,
+        QCoreApplication::CodecForTr));
 
-  return pcAction;
+    fullscr->setStatusTip(QCoreApplication::translate(
+        this->className(), sStatusTip, 0,
+        QCoreApplication::CodecForTr));
+    fullscr->setWhatsThis(QCoreApplication::translate(
+        this->className(), sWhatsThis, 0,
+        QCoreApplication::CodecForTr));
+    fullscr->setShortcut(Qt::Key_F11);
+    fullscr->setCheckable(true);
+    fullscr->setIcon(Gui::BitmapFactory().pixmap("view-fullscreen"));
+
+    return pcAction;
 }
 
 void StdViewDockUndockFullscreen::activated(int iMsg)
 {
-  MDIView* view = getMainWindow()->activeWindow();
-  if ( !view ) return; // no active view
-  if ( iMsg == (int)(view->currentViewMode()) ) 
-    return; // nothing to do
+    MDIView* view = getMainWindow()->activeWindow();
+    if (!view) return; // no active view
+    if (iMsg == (int)(view->currentViewMode()))
+        return; // nothing to do
 
-  if (iMsg==0)
-  {
-    view->setCurrentViewMode( MDIView::Child );
-  }
-  else if (iMsg==1)
-  {
-    view->setCurrentViewMode( MDIView::TopLevel );
-  }
-  else if (iMsg==2)
-  {
-    view->setCurrentViewMode( MDIView::FullScreen );
-  }
+    if (iMsg==0) {
+        view->setCurrentViewMode( MDIView::Child );
+    }
+    else if (iMsg==1) {
+        view->setCurrentViewMode( MDIView::TopLevel );
+    }
+    else if (iMsg==2) {
+        view->setCurrentViewMode( MDIView::FullScreen );
+    }
 }
 
 bool StdViewDockUndockFullscreen::isActive(void)
 {
-  MDIView* view = getMainWindow()->activeWindow();
-  if ( view )
-  {
-    // update the action group if needed
-    ActionGroup* pActGrp = qobject_cast<ActionGroup*>(_pcAction);
-    if ( pActGrp )
-    {
-      int index = pActGrp->checkedAction();
-      int mode = (int)(view->currentViewMode());
-      if ( index != mode )
-      {
-        // active window has changed with another view mode
-        pActGrp->setCheckedAction( mode );
-      }
+    MDIView* view = getMainWindow()->activeWindow();
+    if (qobject_cast<View3DInventor*>(view)) {
+        // update the action group if needed
+        ActionGroup* pActGrp = qobject_cast<ActionGroup*>(_pcAction);
+        if (pActGrp) {
+            int index = pActGrp->checkedAction();
+            int mode = (int)(view->currentViewMode());
+            if (index != mode) {
+                // active window has changed with another view mode
+                pActGrp->setCheckedAction(mode);
+            }
+        }
+
+        return true;
     }
 
-    return true;
-  }
-
-  return false;
+    return false;
 }
 
 //===========================================================================
@@ -1081,6 +1167,7 @@ StdViewScreenShot::StdViewScreenShot()
     sStatusTip  = QT_TR_NOOP("Creates a screenshot of the active view");
     iAccel      = 0;
     sPixmap     = "camera-photo";
+    eType         = Alter3DView;
 }
 
 void StdViewScreenShot::activated(int iMsg)
@@ -1189,6 +1276,7 @@ StdCmdViewCreate::StdCmdViewCreate()
     sStatusTip  = QT_TR_NOOP("Creates a new  view window for the active document");
     sPixmap     = "window-new";
     iAccel      = 0;
+    eType         = Alter3DView;
 }
 
 void StdCmdViewCreate::activated(int iMsg)
@@ -1216,6 +1304,7 @@ StdCmdViewExample1::StdCmdViewExample1()
   sStatusTip    = QT_TR_NOOP("Shows a 3D texture with manipulator");
   sPixmap       = "Std_Tool1";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewExample1::activated(int iMsg)
@@ -1242,6 +1331,7 @@ StdCmdViewExample2::StdCmdViewExample2()
   sStatusTip    = QT_TR_NOOP("Shows spheres and drag-lights");
   sPixmap       = "Std_Tool2";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewExample2::activated(int iMsg)
@@ -1269,6 +1359,7 @@ StdCmdViewExample3::StdCmdViewExample3()
   sStatusTip    = QT_TR_NOOP("Shows a animated texture");
   sPixmap       = "Std_Tool3";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewExample3::activated(int iMsg)
@@ -1297,6 +1388,7 @@ StdCmdViewIvStereoOff::StdCmdViewIvStereoOff()
   sStatusTip    = QT_TR_NOOP("Switch stereo viewing off");
   sPixmap       = "Std_Tool6";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvStereoOff::activated(int iMsg)
@@ -1325,6 +1417,7 @@ StdCmdViewIvStereoRedGreen::StdCmdViewIvStereoRedGreen()
   sStatusTip    = QT_TR_NOOP("Switch stereo viewing to red/green");
   sPixmap       = "Std_Tool7";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvStereoRedGreen::activated(int iMsg)
@@ -1352,6 +1445,7 @@ StdCmdViewIvStereoQuadBuff::StdCmdViewIvStereoQuadBuff()
   sStatusTip    = QT_TR_NOOP("Switch stereo viewing to quad buffer");
   sPixmap       = "Std_Tool7";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvStereoQuadBuff::activated(int iMsg)
@@ -1379,6 +1473,7 @@ StdCmdViewIvStereoInterleavedRows::StdCmdViewIvStereoInterleavedRows()
   sStatusTip    = QT_TR_NOOP("Switch stereo viewing to Interleaved Rows");
   sPixmap       = "Std_Tool7";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvStereoInterleavedRows::activated(int iMsg)
@@ -1406,6 +1501,7 @@ StdCmdViewIvStereoInterleavedColumns::StdCmdViewIvStereoInterleavedColumns()
   sStatusTip    = QT_TR_NOOP("Switch stereo viewing to Interleaved Columns");
   sPixmap       = "Std_Tool7";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvStereoInterleavedColumns::activated(int iMsg)
@@ -1434,6 +1530,7 @@ StdCmdViewIvIssueCamPos::StdCmdViewIvIssueCamPos()
   sStatusTip    = QT_TR_NOOP("Issue the camera position to the console and to a macro, to easily recall this position");
   sPixmap       = "Std_Tool8";
   iAccel        = 0;
+  eType         = Alter3DView;
 }
 
 void StdCmdViewIvIssueCamPos::activated(int iMsg)
@@ -1484,7 +1581,8 @@ StdViewZoomIn::StdViewZoomIn()
 #if QT_VERSION >= 0x040200
     sPixmap       = "view-zoom-in";
 #endif
-    iAccel        = Qt::Key_Plus;
+    iAccel        = keySequenceToAccel(QKeySequence::ZoomIn);
+    eType         = Alter3DView;
 }
 
 void StdViewZoomIn::activated(int iMsg)
@@ -1523,13 +1621,14 @@ StdViewZoomOut::StdViewZoomOut()
 #if QT_VERSION >= 0x040200
     sPixmap       = "view-zoom-out";
 #endif
-    iAccel        = Qt::Key_Minus;
+    iAccel        = keySequenceToAccel(QKeySequence::ZoomOut);
+    eType         = Alter3DView;
 }
 
 void StdViewZoomOut::activated(int iMsg)
 {
     View3DInventor* view = qobject_cast<View3DInventor*>(getMainWindow()->activeWindow());
-    if ( view ) {
+    if (view) {
         View3DInventorViewer* viewer = view->getViewer();
         // send an event to the GL widget to use the internal View3DInventorViewer::zoom() method
         // do only one step to zoom out 
@@ -1562,6 +1661,7 @@ StdViewBoxZoom::StdViewBoxZoom()
     sPixmap       = "view-zoom-border";
 #endif
     iAccel        = Qt::CTRL+Qt::Key_B;
+    eType         = Alter3DView;
 }
 
 void StdViewBoxZoom::activated(int iMsg)
@@ -1589,6 +1689,7 @@ StdCmdTreeSelection::StdCmdTreeSelection()
     sWhatsThis    = "Std_TreeSelection";
     sStatusTip    = QT_TR_NOOP("Scroll to first selected item");
     iAccel        = 0;
+    eType         = Alter3DView;
 }
 
 void StdCmdTreeSelection::activated(int iMsg)
@@ -1615,6 +1716,8 @@ StdCmdMeasureDistance::StdCmdMeasureDistance()
     sWhatsThis    = "Std_MeasureDistance";
     sStatusTip    = QT_TR_NOOP("Measure distance");
     iAccel        = 0;
+    sPixmap       = "view-measurement";
+    eType         = Alter3DView;
 }
 
 void StdCmdMeasureDistance::activated(int iMsg)
@@ -1622,17 +1725,26 @@ void StdCmdMeasureDistance::activated(int iMsg)
     Gui::Document* doc = Gui::Application::Instance->activeDocument();
     Gui::View3DInventor* view = static_cast<Gui::View3DInventor*>(doc->getActiveView());
     if (view) {
-        App::DocumentObject* obj = doc->getDocument()->addObject(App::MeasureDistance::getClassTypeId().getName(),"Distance");
         Gui::View3DInventorViewer* viewer = view->getViewer();
-        viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(), ViewProviderMeasureDistance::measureDistanceCallback, obj);
+        viewer->setEditing(true);
+        PointMarker* marker = new PointMarker(viewer);
+        viewer->addEventCallback(SoMouseButtonEvent::getClassTypeId(),
+            ViewProviderMeasureDistance::measureDistanceCallback, marker);
      }
 }
 
 bool StdCmdMeasureDistance::isActive(void)
 {
     App::Document* doc = App::GetApplication().getActiveDocument();
-    if (doc && doc->countObjectsOfType(App::GeoFeature::getClassTypeId()) > 0)
-        return true;
+    if (!doc || doc->countObjectsOfType(App::GeoFeature::getClassTypeId()) == 0)
+        return false;
+
+    Gui::MDIView* view = Gui::getMainWindow()->activeWindow();
+    if (view && view->isDerivedFrom(Gui::View3DInventor::getClassTypeId())) {
+        Gui::View3DInventorViewer* viewer = static_cast<Gui::View3DInventor*>(view)->getViewer();
+        return !viewer->isEditing();
+    }
+
     return false;
 }
 
@@ -1651,6 +1763,7 @@ StdCmdSceneInspector::StdCmdSceneInspector()
     sToolTipText  = QT_TR_NOOP("Scene inspector");
     sWhatsThis    = "Std_SceneInspector";
     sStatusTip    = QT_TR_NOOP("Scene inspector");
+    eType         = Alter3DView;
 }
 
 void StdCmdSceneInspector::activated(int iMsg)
@@ -1664,9 +1777,31 @@ void StdCmdSceneInspector::activated(int iMsg)
     }
 }
 
+DEF_STD_CMD(StdCmdDemoMode);
+
+StdCmdDemoMode::StdCmdDemoMode()
+  : Command("Std_DemoMode")
+{
+    sGroup        = QT_TR_NOOP("Standard-View");
+    sMenuText     = QT_TR_NOOP("Demo mode...");
+    sToolTipText  = QT_TR_NOOP("Demo mode");
+    sWhatsThis    = QT_TR_NOOP("Demo mode");
+    sStatusTip    = QT_TR_NOOP("Demo mode");
+    eType         = Alter3DView;
+}
+
+void StdCmdDemoMode::activated(int iMsg)
+{
+    static QPointer<QDialog> dlg = 0;
+    if (!dlg)
+        dlg = new Gui::Dialog::DemoMode(getMainWindow());
+    dlg->setAttribute(Qt::WA_DeleteOnClose);
+    dlg->show();
+}
+
 
 //===========================================================================
-// Instanciation
+// Instantiation
 //===========================================================================
 
 
@@ -1705,6 +1840,7 @@ void CreateViewStdCommands(void)
     rcCmdMgr.addCommand(new StdViewDockUndockFullscreen());
     rcCmdMgr.addCommand(new StdCmdSetAppearance());
     rcCmdMgr.addCommand(new StdCmdToggleVisibility());
+    rcCmdMgr.addCommand(new StdCmdToggleSelectability());
     rcCmdMgr.addCommand(new StdCmdShowSelection());
     rcCmdMgr.addCommand(new StdCmdHideSelection());
     rcCmdMgr.addCommand(new StdCmdToggleObjects());
@@ -1720,6 +1856,7 @@ void CreateViewStdCommands(void)
     rcCmdMgr.addCommand(new StdCmdTreeSelection());
     rcCmdMgr.addCommand(new StdCmdMeasureDistance());
     rcCmdMgr.addCommand(new StdCmdSceneInspector());
+    rcCmdMgr.addCommand(new StdCmdDemoMode());
 }
 
 } // namespace Gui
