@@ -26,6 +26,7 @@
 # include <QInputDialog>
 # include <QHeaderView>
 # include <QMessageBox>
+# include <QToolBar>
 #endif
 
 #include "DlgToolbarsImp.h"
@@ -33,6 +34,7 @@
 #include "BitmapFactory.h"
 #include "Command.h"
 #include "ToolBarManager.h"
+#include "MainWindow.h"
 #include "Widgets.h"
 #include "Workbench.h"
 #include "WorkbenchManager.h"
@@ -111,6 +113,34 @@ DlgCustomToolbars::~DlgCustomToolbars()
 {
 }
 
+void DlgCustomToolbars::addCustomToolbar(const QString&)
+{
+}
+
+void DlgCustomToolbars::removeCustomToolbar(const QString&)
+{
+}
+
+void DlgCustomToolbars::renameCustomToolbar(const QString&, const QString&)
+{
+}
+
+void DlgCustomToolbars::addCustomCommand(const QString&, const QByteArray&)
+{
+}
+
+void DlgCustomToolbars::removeCustomCommand(const QString&, const QByteArray&)
+{
+}
+
+void DlgCustomToolbars::moveUpCustomCommand(const QString&, const QByteArray&)
+{
+}
+
+void DlgCustomToolbars::moveDownCustomCommand(const QString&, const QByteArray&)
+{
+}
+
 void DlgCustomToolbars::hideEvent(QHideEvent * event)
 {
     QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
@@ -155,9 +185,6 @@ void DlgCustomToolbars::on_workbenchBox_activated(int index)
     QString workbench = data.toString();
     toolbarTreeWidget->clear();
 
-    renameButton->setEnabled(false);
-    deleteButton->setEnabled(false);
-
     QByteArray workbenchname = workbench.toAscii();
     importCustomToolbars(workbenchname);
 }
@@ -233,56 +260,6 @@ void DlgCustomToolbars::exportCustomToolbars(const QByteArray& workbench)
     }
 }
 
-/** Enables/disables buttons for change */
-void DlgCustomToolbars::on_commandTreeWidget_currentItemChanged(QTreeWidgetItem* item)
-{
-    bool canAdd = false;
-    if (item && commandTreeWidget->isItemSelected(item)) {
-        QTreeWidgetItem* current = toolbarTreeWidget->currentItem();
-        if (current && !current->parent() && toolbarTreeWidget->isItemSelected(current)) {
-            canAdd = true;
-        }
-    }
-
-    moveActionRightButton->setEnabled(canAdd);
-}
-
-/** Enables/disables buttons for change */
-void DlgCustomToolbars::on_toolbarTreeWidget_currentItemChanged(QTreeWidgetItem* item)
-{
-    bool canAdd = false;
-    bool canRemove = false;
-    bool canMoveUp = false;
-    bool canMoveDown = false;
-    bool canRename = false;
-    bool canDelete = false;
-
-    if (item && toolbarTreeWidget->isItemSelected(item)) {
-        // must not be top-level
-        QTreeWidgetItem* parent = item->parent();
-        if (parent) {
-            canRemove = true;
-            if (parent->indexOfChild(item) > 0)
-                canMoveUp = true;
-            if (parent->indexOfChild(item) < parent->childCount()-1)
-                canMoveDown = true;
-        } else {
-            canRename = true;
-            canDelete = true;
-            QTreeWidgetItem* current = commandTreeWidget->currentItem();
-            if (current && commandTreeWidget->isItemSelected(current))
-                canAdd = true;
-        }
-    }
-
-    moveActionRightButton->setEnabled(canAdd);
-    moveActionLeftButton->setEnabled(canRemove);
-    moveActionUpButton->setEnabled(canMoveUp);
-    moveActionDownButton->setEnabled(canMoveDown);
-    renameButton->setEnabled(canRename);
-    deleteButton->setEnabled(canDelete);
-}
-
 /** Adds a new action */
 void DlgCustomToolbars::on_moveActionRightButton_clicked()
 {
@@ -296,6 +273,7 @@ void DlgCustomToolbars::on_moveActionRightButton_clicked()
             QByteArray data = item->data(1, Qt::UserRole).toByteArray();
             copy->setData(0, Qt::UserRole, data);
             copy->setSizeHint(0, QSize(32, 32));
+            addCustomCommand(current->text(0), data);
         }
     }
 
@@ -312,6 +290,8 @@ void DlgCustomToolbars::on_moveActionLeftButton_clicked()
         QTreeWidgetItem* parent = item->parent();
         int index = parent->indexOfChild(item);
         parent->takeChild(index);
+        QByteArray data = item->data(0, Qt::UserRole).toByteArray();
+        removeCustomCommand(parent->text(0), data);
         delete item;
     }
 
@@ -327,9 +307,13 @@ void DlgCustomToolbars::on_moveActionUpButton_clicked()
     if (item && item->parent() && toolbarTreeWidget->isItemSelected(item)) {
         QTreeWidgetItem* parent = item->parent();
         int index = parent->indexOfChild(item);
-        parent->takeChild(index);
-        parent->insertChild(index-1, item);
-        toolbarTreeWidget->setCurrentItem(item);
+        if (index > 0) {
+            parent->takeChild(index);
+            parent->insertChild(index-1, item);
+            toolbarTreeWidget->setCurrentItem(item);
+            QByteArray data = item->data(0, Qt::UserRole).toByteArray();
+            moveUpCustomCommand(parent->text(0), data);
+        }
     }
 
     QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
@@ -344,9 +328,13 @@ void DlgCustomToolbars::on_moveActionDownButton_clicked()
     if (item && item->parent() && toolbarTreeWidget->isItemSelected(item)) {
         QTreeWidgetItem* parent = item->parent();
         int index = parent->indexOfChild(item);
-        parent->takeChild(index);
-        parent->insertChild(index+1, item);
-        toolbarTreeWidget->setCurrentItem(item);
+        if (index < parent->childCount()-1) {
+            parent->takeChild(index);
+            parent->insertChild(index+1, item);
+            toolbarTreeWidget->setCurrentItem(item);
+            QByteArray data = item->data(0, Qt::UserRole).toByteArray();
+            moveDownCustomCommand(parent->text(0), data);
+        }
     }
 
     QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
@@ -378,6 +366,7 @@ void DlgCustomToolbars::on_newButton_clicked()
         QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
         QString workbench = data.toString();
         exportCustomToolbars(workbench.toAscii());
+        addCustomToolbar(text);
     }
 }
 
@@ -387,6 +376,7 @@ void DlgCustomToolbars::on_deleteButton_clicked()
     if (item && !item->parent() && toolbarTreeWidget->isItemSelected(item)) {
         int index = toolbarTreeWidget->indexOfTopLevelItem(item);
         toolbarTreeWidget->takeTopLevelItem(index);
+        removeCustomToolbar(item->text(0));
         delete item;
     }
 
@@ -401,9 +391,10 @@ void DlgCustomToolbars::on_renameButton_clicked()
     QTreeWidgetItem* item = toolbarTreeWidget->currentItem();
     if (item && !item->parent() && toolbarTreeWidget->isItemSelected(item)) {
         bool ok;
-        QString text = item->text(0);
-        text = QInputDialog::getText(this, tr("Rename toolbar"), tr("Toolbar name:"), QLineEdit::Normal, text, &ok);
-        if (ok) {
+        QString old_text = item->text(0);
+        QString text = QInputDialog::getText(this, tr("Rename toolbar"), tr("Toolbar name:"),
+            QLineEdit::Normal, old_text, &ok);
+        if (ok && text != old_text) {
             // Check for duplicated name
             for (int i=0; i<toolbarTreeWidget->topLevelItemCount(); i++) {
                 QTreeWidgetItem* toplevel = toolbarTreeWidget->topLevelItem(i);
@@ -415,6 +406,7 @@ void DlgCustomToolbars::on_renameButton_clicked()
             }
 
             item->setText(0, text);
+            renameCustomToolbar(old_text, text);
             renamed = true;
         }
     }
@@ -524,6 +516,123 @@ DlgCustomToolbarsImp::~DlgCustomToolbarsImp()
 {
 }
 
+void DlgCustomToolbarsImp::addCustomToolbar(const QString& name)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QToolBar* bar = getMainWindow()->addToolBar(name);
+        bar->setObjectName(name);
+    }
+}
+
+void DlgCustomToolbarsImp::removeCustomToolbar(const QString& name)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(name);
+        if (bars.size() != 1) return;
+        QToolBar* tb = bars.front();
+        getMainWindow()->removeToolBar(tb);
+        delete tb;
+    }
+}
+
+void DlgCustomToolbarsImp::renameCustomToolbar(const QString& old_name, const QString& new_name)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(old_name);
+        if (bars.size() != 1) return;
+        QToolBar* tb = bars.front();
+        tb->setObjectName(new_name);
+        tb->setWindowTitle(new_name);
+    }
+}
+
+void DlgCustomToolbarsImp::addCustomCommand(const QString& name, const QByteArray& cmd)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(name);
+        if (bars.size() != 1) return;
+        CommandManager& mgr = Application::Instance->commandManager();
+        mgr.addTo(cmd, bars.front());
+    }
+}
+
+void DlgCustomToolbarsImp::removeCustomCommand(const QString& name, const QByteArray& cmd)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(name);
+        if (bars.size() != 1) return;
+        QList<QAction*> actions = bars.front()->actions();
+        for (QList<QAction*>::ConstIterator it = actions.begin(); it != actions.end(); ++it) {
+            if ((*it)->data().toByteArray() == cmd) {
+                bars.front()->removeAction(*it);
+                break;
+            }
+        }
+    }
+}
+
+void DlgCustomToolbarsImp::moveUpCustomCommand(const QString& name, const QByteArray& cmd)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(name);
+        if (bars.size() != 1) return;
+        QList<QAction*> actions = bars.front()->actions();
+        QAction* before=0;
+        for (QList<QAction*>::ConstIterator it = actions.begin(); it != actions.end(); ++it) {
+            if ((*it)->data().toByteArray() == cmd) {
+                if (before != 0) {
+                    bars.front()->removeAction(*it);
+                    bars.front()->insertAction(before, *it);
+                    break;
+                }
+            }
+
+            before = *it;
+        }
+    }
+}
+
+void DlgCustomToolbarsImp::moveDownCustomCommand(const QString& name, const QByteArray& cmd)
+{
+    QVariant data = workbenchBox->itemData(workbenchBox->currentIndex(), Qt::UserRole);
+    Workbench* w = WorkbenchManager::instance()->active();
+    if (w && w->name() == std::string((const char*)data.toByteArray())) {
+        QList<QToolBar*> bars = getMainWindow()->findChildren<QToolBar*>(name);
+        if (bars.size() != 1) return;
+        QList<QAction*> actions = bars.front()->actions();
+        for (QList<QAction*>::ConstIterator it = actions.begin(); it != actions.end(); ++it) {
+            if ((*it)->data().toByteArray() == cmd) {
+                QAction* act = *it;
+                if (*it == actions.back())
+                    break; // we're already on the last element
+                ++it;
+                // second last item
+                if (*it == actions.back()) {
+                    bars.front()->removeAction(act);
+                    bars.front()->addAction(act);
+                    break;
+                }
+                ++it;
+                bars.front()->removeAction(act);
+                bars.front()->insertAction(*it, act);
+                break;
+            }
+        }
+    }
+}
+
 
 /* TRANSLATOR Gui::Dialog::DlgCustomToolBoxbarsImp */
 
