@@ -24,6 +24,7 @@
 #include "PreCompiled.h"
 
 #ifndef _PreComp_
+# include <qapplication.h>
 # include <qdir.h>
 # include <qfileinfo.h>
 # include <qmessagebox.h>
@@ -47,6 +48,7 @@
 #include "Document.h"
 #include "DocumentPy.h"
 #include "Command.h"
+#include "Control.h"
 #include "FileDialog.h"
 #include "View3DInventor.h"
 #include "View3DInventorViewer.h"
@@ -79,6 +81,15 @@ struct DocumentP
     std::list<Gui::BaseView*> _LpcPassivViews;
     std::map<const App::DocumentObject*,ViewProviderDocumentObject*> _ViewProviderMap;
     std::map<std::string,ViewProvider*> _ViewProviderMapAnnotation;
+
+    typedef boost::signals::connection Connection;
+    Connection connectNewObject;
+    Connection connectDelObject;
+    Connection connectCngObject;
+    Connection connectRenObject;
+    Connection connectActObject;
+    Connection connectSaveDocument;
+    Connection connectRestDocument;
 };
 
 } // namespace Gui
@@ -99,19 +110,21 @@ Document::Document(App::Document* pcDocument,Application * app)
     d->_pcDocument = pcDocument;
     d->_pcInEdit = 0;
 
-    //pcDocument->m_sig.connect(boost::bind(&Gui::Document::refresh, this, _1));
-    //boost::bind(&Gui::Document::slotNewObject, this, _1)
-
-    //App::Document::connection_t  m_connection;
-    //m_connection = pcDocument->connect(boost::bind(&Document::slotNewObject, this, _1));
-    pcDocument->signalNewObject.connect(boost::bind(&Gui::Document::slotNewObject, this, _1));
-    pcDocument->signalDeletedObject.connect(boost::bind(&Gui::Document::slotDeletedObject, this, _1));
-    pcDocument->signalChangedObject.connect(boost::bind(&Gui::Document::slotChangedObject, this, _1, _2));
-    pcDocument->signalRenamedObject.connect(boost::bind(&Gui::Document::slotRenamedObject, this, _1));
-    pcDocument->signalActivatedObject.connect(boost::bind(&Gui::Document::slotActivatedObject, this, _1));
-    //pcDocument->signalNewObject.connect(&(this->slotNewObject));
-    pcDocument->signalSaveDocument.connect(boost::bind(&Gui::Document::Save, this, _1));
-    pcDocument->signalRestoreDocument.connect(boost::bind(&Gui::Document::Restore, this, _1));
+    // Setup the connections
+    d->connectNewObject = pcDocument->signalNewObject.connect
+        (boost::bind(&Gui::Document::slotNewObject, this, _1));
+    d->connectDelObject = pcDocument->signalDeletedObject.connect
+        (boost::bind(&Gui::Document::slotDeletedObject, this, _1));
+    d->connectCngObject = pcDocument->signalChangedObject.connect
+        (boost::bind(&Gui::Document::slotChangedObject, this, _1, _2));
+    d->connectRenObject = pcDocument->signalRenamedObject.connect
+        (boost::bind(&Gui::Document::slotRenamedObject, this, _1));
+    d->connectActObject = pcDocument->signalActivatedObject.connect
+        (boost::bind(&Gui::Document::slotActivatedObject, this, _1));
+    d->connectSaveDocument = pcDocument->signalSaveDocument.connect
+        (boost::bind(&Gui::Document::Save, this, _1));
+    d->connectRestDocument = pcDocument->signalRestoreDocument.connect
+        (boost::bind(&Gui::Document::Restore, this, _1));
 
     // pointer to the python class
     // NOTE: As this Python object doesn't get returned to the interpreter we
@@ -125,16 +138,26 @@ Document::Document(App::Document* pcDocument,Application * app)
 
 Document::~Document()
 {
+    // disconnect everything to avoid to be double-deleted
+    // in case an exception is raised somewhere
+    d->connectNewObject.disconnect();
+    d->connectDelObject.disconnect();
+    d->connectCngObject.disconnect();
+    d->connectRenObject.disconnect();
+    d->connectActObject.disconnect();
+    d->connectSaveDocument.disconnect();
+    d->connectRestDocument.disconnect();
+
     // e.g. if document gets closed from within a Python command
     d->_isClosing = true;
-    // because Calls Document::detachView() and alter the View List
+    // calls Document::detachView() and alter the view list
     std::list<Gui::BaseView*> temp = d->_LpcViews;
     for(std::list<Gui::BaseView*>::iterator it=temp.begin();it!=temp.end();++it)
-        delete *it;
+        (*it)->deleteSelf();
 
-    std::map<const App::DocumentObject*,ViewProviderDocumentObject*>::iterator it;
-    for (it = d->_ViewProviderMap.begin();it != d->_ViewProviderMap.end(); ++it)
-        delete it->second;
+    std::map<const App::DocumentObject*,ViewProviderDocumentObject*>::iterator jt;
+    for (jt = d->_ViewProviderMap.begin();jt != d->_ViewProviderMap.end(); ++jt)
+        delete jt->second;
     std::map<std::string,ViewProvider*>::iterator it2;
     for (it2 = d->_ViewProviderMapAnnotation.begin();it2 != d->_ViewProviderMapAnnotation.end(); ++it2)
         delete it2->second;
@@ -402,7 +425,7 @@ void Document::slotChangedObject(const App::DocumentObject& Obj, const App::Prop
         }
 
         if (viewProvider->isDerivedFrom(ViewProviderDocumentObject::getClassTypeId()))
-            signalChangedObject(*(static_cast<ViewProviderDocumentObject*>(viewProvider)));
+            signalChangedObject(static_cast<ViewProviderDocumentObject&>(*viewProvider), Prop);
     }
 
     // a property of an object has changed
@@ -465,10 +488,11 @@ bool Document::saveAs(void)
 {
     getMainWindow()->statusBar()->showMessage(QObject::tr("Save document under new filename..."));
 
-    QString exe = QString::fromUtf8(App::Application::Config()["ExeName"].c_str());
+    QString exe = QString::fromUtf8(App::GetApplication().getExecutableName());
     QString fn = QFileDialog::getSaveFileName(getMainWindow(), QObject::tr("Save %1 Document").arg(exe), 
-                 QDir::currentPath(), QObject::tr("%1 document (*.FCStd)").arg(exe));
+                 FileDialog::getWorkingDirectory(), QObject::tr("%1 document (*.FCStd)").arg(exe));
     if (!fn.isEmpty()) {
+        FileDialog::setWorkingDirectory(fn);
         QString file = fn.toLower();
         if (!file.endsWith(QLatin1String(".fcstd"))) {
             fn += QLatin1String(".fcstd");
@@ -627,7 +651,7 @@ void Document::SaveDocFile (Base::Writer &writer) const
 {
     writer.Stream() << "<?xml version='1.0' encoding='utf-8'?>" << std::endl
                     << "<!--" << std::endl
-                    << " FreeCAD Document, see http://free-cad.sourceforge.net for more informations..."
+                    << " FreeCAD Document, see http://free-cad.sourceforge.net for more information..."
                     << std::endl << "-->" << std::endl;
 
     writer.Stream() << "<Document SchemaVersion=\"1\">" << std::endl;
@@ -677,41 +701,112 @@ void Document::SaveDocFile (Base::Writer &writer) const
     writer.Stream() << "</Document>" << std::endl;
 }
 
+void Document::exportObjects(const std::vector<App::DocumentObject*>& obj, Base::Writer& writer)
+{
+    writer.Stream() << "<?xml version='1.0' encoding='utf-8'?>" << std::endl;
+    writer.Stream() << "<Document SchemaVersion=\"1\">" << std::endl;
+
+    std::map<const App::DocumentObject*,ViewProvider*> views;
+    for (std::vector<App::DocumentObject*>::const_iterator it = obj.begin(); it != obj.end(); ++it) {
+        Document* doc = Application::Instance->getDocument((*it)->getDocument());
+        if (doc) {
+            ViewProvider* vp = doc->getViewProvider(*it);
+            if (vp) views[*it] = vp;
+        }
+    }
+
+    // writing the view provider names itself
+    writer.incInd(); // indention for 'ViewProviderData Count'
+    writer.Stream() << writer.ind() << "<ViewProviderData Count=\"" 
+                    << views.size() <<"\">" << std::endl;
+
+    bool xml = writer.isForceXML();
+    writer.setForceXML(true);
+    writer.incInd(); // indention for 'ViewProvider name'
+    std::map<const App::DocumentObject*,ViewProvider*>::const_iterator jt;
+    for (jt = views.begin(); jt != views.end(); ++jt) {
+        const App::DocumentObject* doc = jt->first;
+        ViewProvider* obj = jt->second;
+        writer.Stream() << writer.ind() << "<ViewProvider name=\""
+                        << doc->getNameInDocument() << "\" type=\""
+                        << obj->getTypeId().getName()
+                        << "\">" << std::endl;
+        obj->Save(writer);
+        writer.Stream() << writer.ind() << "</ViewProvider>" << std::endl;
+    }
+    writer.setForceXML(xml);
+
+    writer.decInd(); // indention for 'ViewProvider name'
+    writer.Stream() << writer.ind() << "</ViewProviderData>" << std::endl;
+    writer.decInd();  // indention for 'ViewProviderData Count'
+    writer.incInd(); // indention for camera settings
+    writer.Stream() << writer.ind() << "<Camera settings=\"\"/>" << std::endl;
+    writer.decInd(); // indention for camera settings
+    writer.Stream() << "</Document>" << std::endl;
+}
+
+void Document::importObjects(const std::vector<App::DocumentObject*>& obj, Base::Reader& reader)
+{
+    // We must create an XML parser to read from the input stream
+    Base::XMLReader xmlReader("GuiDocument.xml", reader);
+    xmlReader.readElement("Document");
+    long scheme = xmlReader.getAttributeAsInteger("SchemaVersion");
+
+    // At this stage all the document objects and their associated view providers exist.
+    // Now we must restore the properties of the view providers only.
+    //
+    // SchemeVersion "1"
+    if (scheme == 1) {
+        // read the viewproviders itself
+        xmlReader.readElement("ViewProviderData");
+        int Cnt = xmlReader.getAttributeAsInteger("Count");
+        std::vector<App::DocumentObject*>::const_iterator it = obj.begin();
+        for (int i=0;i<Cnt&&it!=obj.end();++i,++it) {
+            // The stored name usually doesn't match with the current name anymore
+            // thus we try to match by type. This should work because the order of
+            // objects should not have changed
+            xmlReader.readElement("ViewProvider");
+            std::string type = xmlReader.getAttribute("type");
+            ViewProvider* pObj = getViewProvider(*it);
+            while (pObj && type != pObj->getTypeId().getName()) {
+                if (it != obj.end()) {
+                    ++it;
+                    pObj = getViewProvider(*it);
+                }
+            }
+            if (pObj && type == pObj->getTypeId().getName())
+                pObj->Restore(xmlReader);
+            xmlReader.readEndElement("ViewProvider");
+            if (it == obj.end())
+                break;
+        }
+        xmlReader.readEndElement("ViewProviderData");
+    }
+
+    xmlReader.readEndElement("Document");
+}
+
 void Document::createView(const char* sType) 
 {
-    QPixmap FCIcon = Gui::BitmapFactory().pixmap(App::Application::Config()["AppIcon"].c_str());
-    MDIView* pcView3D=0;
-    if (strcmp(sType,"View3DIv") == 0){
-        pcView3D = new Gui::View3DInventor(this,getMainWindow());
+    View3DInventor* view3D = new View3DInventor(this, getMainWindow());
 
-        // add the selction node of the document
-        //((View3DInventor*)pcView3D)->getViewer()->addSelectionNode(pcSelection);
-    
-        // attach the viewprovider
-        std::map<const App::DocumentObject*,ViewProviderDocumentObject*>::const_iterator It1;
-        for (It1=d->_ViewProviderMap.begin();It1!=d->_ViewProviderMap.end();++It1)
-            ((View3DInventor*)pcView3D)->getViewer()->addViewProvider(It1->second);
-        std::map<std::string,ViewProvider*>::const_iterator It2;
-        for (It2=d->_ViewProviderMapAnnotation.begin();It2!=d->_ViewProviderMapAnnotation.end();++It2)
-            ((View3DInventor*)pcView3D)->getViewer()->addViewProvider(It2->second);
-
-    } else /* if(strcmp(sType,"View3DOCC") == 0){
-        pcView3D = new MDIView3D(this,_pcAppWnd,"View3DOCC");
-    }else*/
-    {
-        Base::Console().Error("Document::createView(): Unknown view type: %s\n",sType);
-        return;
-    }
+    // attach the viewprovider
+    std::map<const App::DocumentObject*,ViewProviderDocumentObject*>::const_iterator It1;
+    for (It1=d->_ViewProviderMap.begin();It1!=d->_ViewProviderMap.end();++It1)
+        view3D->getViewer()->addViewProvider(It1->second);
+    std::map<std::string,ViewProvider*>::const_iterator It2;
+    for (It2=d->_ViewProviderMapAnnotation.begin();It2!=d->_ViewProviderMapAnnotation.end();++It2)
+        view3D->getViewer()->addViewProvider(It2->second);
 
     const char* name = getDocument()->Label.getValue();
 
-    QString aName = QString::fromAscii("%1 : %2[*]")
+    QString title = QString::fromAscii("%1 : %2[*]")
         .arg(QString::fromUtf8(name)).arg(d->_iWinCount++);
 
-    pcView3D->setWindowTitle(aName);
-    pcView3D->setWindowIcon(FCIcon);
-    pcView3D->resize( 400, 300 );
-    getMainWindow()->addWindow(pcView3D);
+    view3D->setWindowTitle(title);
+    view3D->setWindowIcon(QApplication::windowIcon());
+    view3D->resize(400, 300);
+    getMainWindow()->addWindow(view3D);
 }
 
 void Document::attachView(Gui::BaseView* pcView, bool bPassiv)
@@ -803,6 +898,12 @@ bool Document::canClose ()
             QObject::tr("The document is not closable for the moment."));
         return false;
     }
+    else if (!Gui::Control().isAllowedAlterDocument()) {
+        QMessageBox::warning(getActiveView(),
+            QObject::tr("Document not closable"),
+            QObject::tr("The document is not closable for the moment."));
+        return false;
+    }
 
     if (!isModified())
         return true;
