@@ -42,9 +42,12 @@
 
 #include "EditorView.h"
 #include "Application.h"
+#include "BitmapFactory.h"
 #include "FileDialog.h"
 #include "Macro.h"
+#include "PythonDebugger.h"
 
+#include <Base/Interpreter.h>
 #include <Base/Parameter.h>
 
 using namespace Gui;
@@ -62,6 +65,55 @@ public:
 };
 }
 
+LineMarker::LineMarker(EditorView* ev, QWidget* parent)
+    : QWidget(parent), view(ev)
+{
+    setFixedWidth(fontMetrics().width(QLatin1String("0000"))+10);
+}
+
+LineMarker::~LineMarker()
+{
+}
+
+void LineMarker::setTextEdit(QTextEdit *edit)
+{
+    this->edit = edit;
+    connect(edit->document()->documentLayout(), SIGNAL(update(const QRectF &)),
+            this, SLOT(update()));
+    connect(edit->verticalScrollBar(), SIGNAL(valueChanged(int)),
+            this, SLOT(update()));
+}
+
+void LineMarker::paintEvent(QPaintEvent*)
+{
+    QAbstractTextDocumentLayout *layout = edit->document()->documentLayout();
+    int contentsY = edit->verticalScrollBar()->value();
+    qreal pageBottom = contentsY + edit->viewport()->height();
+    const QFontMetrics fm = fontMetrics();
+    const int ascent = fontMetrics().ascent() + 1; // height = ascent + descent + 1
+    int lineCount = 1;
+
+    QPainter p(this);
+
+    for (QTextBlock block = edit->document()->begin();
+        block.isValid(); block = block.next(), ++lineCount) {
+
+        const QRectF boundingRect = layout->blockBoundingRect(block);
+
+        QPointF position = boundingRect.topLeft();
+        if (position.y() + boundingRect.height() < contentsY)
+            continue;
+        if (position.y() > pageBottom)
+            break;
+
+        const QString txt = QString::number(lineCount);
+        p.drawText(width() - fm.width(txt), qRound(position.y()) - contentsY + ascent, txt);
+        view->drawMarker(lineCount, 1, qRound(position.y()) - contentsY, &p);
+    }
+}
+
+// -------------------------------------------------------
+
 /* TRANSLATOR Gui::EditorView */
 
 /**
@@ -77,7 +129,7 @@ EditorView::EditorView(QTextEdit* editor, QWidget* parent)
     // create the editor first
     d->textEdit = editor;
     d->textEdit->setLineWrapMode(QTextEdit::NoWrap);
-    d->lineMarker = new LineMarker();
+    d->lineMarker = new LineMarker(this);
     d->lineMarker->setTextEdit(d->textEdit);
 
     // Create the layout containing the workspace and a tab bar
@@ -122,6 +174,20 @@ EditorView::~EditorView()
     getWindowParameter()->Detach( this );
 }
 
+void EditorView::drawMarker(int line, int x, int y, QPainter*)
+{
+}
+
+QTextEdit* EditorView::getEditor() const
+{
+    return d->textEdit;
+}
+
+LineMarker* EditorView::getMarker() const
+{
+    return d->lineMarker;
+}
+
 void EditorView::OnChange(Base::Subject<const char*> &rCaller,const char* rcReason)
 {
     ParameterGrp::handle hPrefGrp = getWindowParameter();
@@ -165,9 +231,6 @@ bool EditorView::onMsg(const char* pMsg,const char** ppReturn)
     if (strcmp(pMsg,"Save")==0){
         saveFile();
         return true;
-    } else if (strcmp(pMsg,"Run")==0){
-        run();
-        return true;
     } else if (strcmp(pMsg,"SaveAs")==0){
         saveAs();
         return true;
@@ -201,6 +264,8 @@ bool EditorView::onMsg(const char* pMsg,const char** ppReturn)
 bool EditorView::onHasMsg(const char* pMsg) const
 {
     if (strcmp(pMsg,"Run")==0)  return true;
+    if (strcmp(pMsg,"DebugStart")==0)  return true;
+    if (strcmp(pMsg,"DebugStop")==0)  return true;
     if (strcmp(pMsg,"SaveAs")==0)  return true;
     if (strcmp(pMsg,"Print")==0) return true;
     if (strcmp(pMsg,"PrintPdf")==0) return true;
@@ -292,14 +357,6 @@ bool EditorView::open(const QString& fileName)
     return true;
 }
 
-/**
- * Runs the opened script in the macro manager.
- */
-void EditorView::run(void)
-{
-    Application::Instance->macroManager()->run(Gui::MacroManager::File,d->fileName.toUtf8());
-}
-
 /**
  * Copies the selected text to the clipboard and deletes it from the text edit.
  * If there is no selected text nothing happens.
@@ -475,50 +532,108 @@ void EditorView::focusInEvent (QFocusEvent * e)
 
 // ---------------------------------------------------------
 
-LineMarker::LineMarker(QWidget* parent)
-    : QWidget(parent)
+PythonEditorView::PythonEditorView(QTextEdit* editor, QWidget* parent)
+  : EditorView(editor, parent), m_debugLine(-1),
+    breakpoint(QLatin1String(":/icons/breakpoint.png")),
+    debugMarker(QLatin1String(":/icons/debug-marker.png"))
 {
-    setFixedWidth(fontMetrics().width(QLatin1String("0000")));
+    _dbg = Application::Instance->macroManager()->debugger();
 }
 
-LineMarker::~LineMarker()
+PythonEditorView::~PythonEditorView()
 {
 }
 
-void LineMarker::setTextEdit(QTextEdit *edit)
+/**
+ * Runs the action specified by \a pMsg.
+ */
+bool PythonEditorView::onMsg(const char* pMsg,const char** ppReturn)
 {
-    this->edit = edit;
-    connect(edit->document()->documentLayout(), SIGNAL(update(const QRectF &)),
-            this, SLOT(update()));
-    connect(edit->verticalScrollBar(), SIGNAL(valueChanged(int)),
-            this, SLOT(update()));
+    if (strcmp(pMsg,"Run")==0) {
+        executeScript();
+        return true;
+    }
+    else if (strcmp(pMsg,"StartDebug")==0) {
+        startDebug();
+        return true;
+    }
+    else if (strcmp(pMsg,"ToggleBreakpoint")==0) {
+        toggleBreakpoint();
+        return true;
+    }
+    return EditorView::onMsg(pMsg, ppReturn);
 }
 
-void LineMarker::paintEvent(QPaintEvent*)
+/**
+ * Checks if the action \a pMsg is available. This is for enabling/disabling
+ * the corresponding buttons or menu items for this action.
+ */
+bool PythonEditorView::onHasMsg(const char* pMsg) const
 {
-    QAbstractTextDocumentLayout *layout = edit->document()->documentLayout();
-    int contentsY = edit->verticalScrollBar()->value();
-    qreal pageBottom = contentsY + edit->viewport()->height();
-    const QFontMetrics fm = fontMetrics();
-    const int ascent = fontMetrics().ascent() + 1; // height = ascent + descent + 1
-    int lineCount = 1;
+    if (strcmp(pMsg,"Run")==0)  return true;
+    if (strcmp(pMsg,"StartDebug")==0)  return true;
+    if (strcmp(pMsg,"ToggleBreakpoint")==0)  return true;
+    return EditorView::onHasMsg(pMsg);
+}
 
-    QPainter p(this);
-
-    for (QTextBlock block = edit->document()->begin();
-        block.isValid(); block = block.next(), ++lineCount) {
-
-        const QRectF boundingRect = layout->blockBoundingRect( block );
-
-        QPointF position = boundingRect.topLeft();
-        if ( position.y() + boundingRect.height() < contentsY )
-            continue;
-        if ( position.y() > pageBottom )
-            break;
-
-        const QString txt = QString::number( lineCount );
-        p.drawText( width() - fm.width(txt), qRound( position.y() ) - contentsY + ascent, txt );
+void PythonEditorView::drawMarker(int line, int x, int y, QPainter* p)
+{
+    Breakpoint bp = _dbg->getBreakpoint(fileName());
+    if (bp.checkLine(line)) {
+        p->drawPixmap(x, y, breakpoint);
+    }
+    if (m_debugLine == line) {
+        p->drawPixmap(x, y+2, debugMarker);
+        debugRect = QRect(x, y+2, debugMarker.width(), debugMarker.height());
     }
 }
 
+/**
+ * Runs the opened script in the macro manager.
+ */
+void PythonEditorView::executeScript()
+{
+    Application::Instance->macroManager()->run(Gui::MacroManager::File,fileName().toUtf8());
+}
+
+void PythonEditorView::startDebug()
+{
+    if (_dbg->start()) {
+        _dbg->runFile(fileName());
+        _dbg->stop();
+    }
+}
+
+void PythonEditorView::toggleBreakpoint()
+{
+    QTextCursor cursor = getEditor()->textCursor();
+    int line = cursor.blockNumber() + 1;
+    _dbg->toogleBreakpoint(line, fileName());
+    getMarker()->update();
+}
+
+void PythonEditorView::showDebugMarker(int line)
+{
+    m_debugLine = line;
+    getMarker()->update();
+    QTextCursor cursor = getEditor()->textCursor();
+    cursor.movePosition(QTextCursor::StartOfBlock);
+    int cur = cursor.blockNumber() + 1;
+    if (cur > line) {
+        for (int i=line; i<cur; i++)
+            cursor.movePosition(QTextCursor::Up);
+    }
+    else if (cur < line) {
+        for (int i=cur; i<line; i++)
+            cursor.movePosition(QTextCursor::Down);
+    }
+    getEditor()->setTextCursor(cursor);
+}
+
+void PythonEditorView::hideDebugMarker()
+{
+    m_debugLine = -1;
+    getMarker()->update();
+}
+
 #include "moc_EditorView.cpp"
