@@ -77,7 +77,10 @@ QString FileDialog::getSaveFileName (QWidget * parent, const QString & caption,
     QString windowTitle = caption;
     if (windowTitle.isEmpty())
         windowTitle = FileDialog::tr("Save as");
-    
+
+#if defined(FC_OS_MACOSX)
+    options |= QFileDialog::DontUseNativeDialog;
+#endif
     // NOTE: We must not change the specified file name afterwards as we may return the name of an already
     // existing file. Hence we must extract the first matching suffix from the filter list and append it 
     // before showing the file dialog.
@@ -95,6 +98,9 @@ QString FileDialog::getSaveFileName (QWidget * parent, const QString & caption,
  */
 QString FileDialog::getExistingDirectory( QWidget * parent, const QString & caption, const QString & dir, Options options )
 {
+#if defined(FC_OS_MACOSX)
+    options |= QFileDialog::DontUseNativeDialog;
+#endif
     QString path = QFileDialog::getExistingDirectory(parent, caption, dir, options);
     // valid path was selected
     if ( !path.isEmpty() ) {
@@ -120,6 +126,9 @@ QString FileDialog::getOpenFileName(QWidget * parent, const QString & caption, c
     QString windowTitle = caption;
     if (windowTitle.isEmpty())
         windowTitle = FileDialog::tr("Open");
+#if defined(FC_OS_MACOSX)
+    options |= QFileDialog::DontUseNativeDialog;
+#endif
     QString file = QFileDialog::getOpenFileName(parent, windowTitle, dirName, filter, selectedFilter, options);
     if (!file.isEmpty()) {
         setWorkingDirectory(file);
@@ -143,6 +152,9 @@ QStringList FileDialog::getOpenFileNames (QWidget * parent, const QString & capt
     QString windowTitle = caption;
     if (windowTitle.isEmpty())
         windowTitle = FileDialog::tr("Open");
+#if defined(FC_OS_MACOSX)
+    options |= QFileDialog::DontUseNativeDialog;
+#endif
     QStringList files = QFileDialog::getOpenFileNames(parent, windowTitle, dirName, filter, selectedFilter, options);
     if (!files.isEmpty()) {
         setWorkingDirectory(files.front());
@@ -161,8 +173,11 @@ QString FileDialog::getWorkingDirectory()
     std::string path = App::GetApplication().Config()["UserHomePath"];
     Base::Reference<ParameterGrp> hPath = App::GetApplication().GetUserParameter().GetGroup("BaseApp")
                                ->GetGroup("Preferences")->GetGroup("General");
-    path = hPath->GetASCII("FileOpenSavePath", path.c_str());
-    return QString::fromUtf8(path.c_str());
+    std::string dir = hPath->GetASCII("FileOpenSavePath", path.c_str());
+    QFileInfo fi(QString::fromUtf8(dir.c_str()));
+    if (!fi.exists())
+        dir = path;
+    return QString::fromUtf8(dir.c_str());
 }
 
 /**
@@ -170,7 +185,7 @@ QString FileDialog::getWorkingDirectory()
  * If \a dir is a file then the path only is taken.
  * getWorkingDirectory() returns the working directory.
  */
-void FileDialog::setWorkingDirectory( const QString& dir )
+void FileDialog::setWorkingDirectory(const QString& dir)
 {
     QString dirName = dir;
     if (!dir.isEmpty()) {
@@ -195,7 +210,11 @@ FileOptionsDialog::FileOptionsDialog( QWidget* parent, Qt::WFlags fl )
 
     //search for the grid layout and add the new button
     QGridLayout* grid = this->findChild<QGridLayout*>();
+#if QT_VERSION >= 0x040500
+    grid->addWidget(extensionButton, 4, 2, Qt::AlignLeft);
+#else
     grid->addWidget(extensionButton, 4, 5, Qt::AlignLeft);
+#endif
 
     connect(extensionButton, SIGNAL(clicked()), this, SLOT(toggleExtension()));
 
@@ -273,8 +292,9 @@ void FileOptionsDialog::setOptionsWidget(FileOptionsDialog::ExtensionPosition po
 {
     if (pos == ExtensionRight) {
         setExtension(w);
-        setOrientation( Qt::Horizontal );
-    } else if (pos == ExtensionBottom) {
+        setOrientation(Qt::Horizontal);
+    }
+    else if (pos == ExtensionBottom) {
         setExtension(w);
         setOrientation(Qt::Vertical);
     }
@@ -554,6 +574,11 @@ QString SelectModule::getModule() const
 }
 
 SelectModule::Dict SelectModule::exportHandler(const QString& fileName, const QString& filter)
+{
+    return exportHandler(QStringList() << fileName, filter);
+}
+
+SelectModule::Dict SelectModule::exportHandler(const QStringList& fileNames, const QString& filter)
 {
     // first check if there is a certain filter selected
     SelectModule::Dict dict;
@@ -563,18 +588,49 @@ SelectModule::Dict SelectModule::exportHandler(const QString& fileName, const QS
         std::map<std::string, std::string>::const_iterator it;
         it = filterList.find((const char*)filter.toUtf8());
         if (it != filterList.end()) {
-            dict[fileName] = QString::fromAscii(it->second.c_str());
+            QString module = QString::fromAscii(it->second.c_str());
+            for (QStringList::const_iterator it = fileNames.begin(); it != fileNames.end(); ++it) {
+                dict[*it] = module;
+            }
             return dict;
         }
     }
 
-    // if no export filter was set then simply use the first matching module
-    QFileInfo fi(fileName);
-    QString ext = fi.suffix().toLower();
-    std::vector<std::string> modules = App::GetApplication().getExportModules(ext.toAscii());
-    // set the default module handler
-    if (!modules.empty())
-        dict[fileName] = QString::fromAscii(modules.front().c_str());
+    // the global filter (or no filter) was selected. We now try to sort filetypes that are
+    // handled by more than one module and ask to the user to select one.
+    QMap<QString, SelectModule::Dict> filetypeHandler;
+    QMap<QString, QStringList > fileExtension;
+    for (QStringList::const_iterator it = fileNames.begin(); it != fileNames.end(); ++it) {
+        QFileInfo fi(*it);
+        QString ext = fi.completeSuffix().toLower();
+        std::map<std::string, std::string> filters = App::GetApplication().getExportFilters(ext.toAscii());
+        
+        if (filters.empty()) {
+            ext = fi.suffix().toLower();
+            filters = App::GetApplication().getExportFilters(ext.toAscii());
+        }
+
+        fileExtension[ext].push_back(*it);
+        for (std::map<std::string, std::string>::iterator jt = filters.begin(); jt != filters.end(); ++jt)
+            filetypeHandler[ext][QString::fromUtf8(jt->first.c_str())] = QString::fromAscii(jt->second.c_str());
+        // set the default module handler
+        if (!filters.empty())
+            dict[*it] = QString::fromAscii(filters.begin()->second.c_str());
+    }
+
+    for (QMap<QString, SelectModule::Dict>::const_iterator it = filetypeHandler.begin(); 
+        it != filetypeHandler.end(); ++it) {
+        if (it.value().size() > 1) {
+            SelectModule dlg(it.key(),it.value(), getMainWindow());
+            QApplication::beep();
+            if (dlg.exec()) {
+                QString mod = dlg.getModule();
+                const QStringList& files = fileExtension[it.key()];
+                for (QStringList::const_iterator jt = files.begin(); jt != files.end(); ++jt)
+                    dict[*jt] = mod;
+            }
+        }
+    }
 
     return dict;
 }
@@ -641,23 +697,6 @@ SelectModule::Dict SelectModule::importHandler(const QStringList& fileNames, con
     return dict;
 }
 
-// ----------------------------------------------------------------------
-
-ByteArrayStream::ByteArrayStream(QByteArray& buf) : buffer(buf)
-{
-}
-
-ByteArrayStream::~ByteArrayStream()
-{
-}
-
-int ByteArrayStream::overflow(int v)
-{
-    //does a re-allocation if needed
-    buffer.append((char)v);
-    return v;
-}
-
 
 #include "moc_FileDialog.cpp"
 
