@@ -23,11 +23,17 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <QApplication>
+# include <QBuffer>
+# include <QByteArray>
+# include <QClipboard>
+# include <QMimeData>
 # include <QCloseEvent>
 # include <QContextMenuEvent>
 # include <QDesktopWidget>
 # include <QDockWidget>
 # include <QLabel>
+# include <QMdiSubWindow>
 # include <QMessageBox>
 # include <QSettings>
 # include <QSignalMapper>
@@ -37,10 +43,19 @@
 # include <QWhatsThis>
 #endif
 
+#include <boost/signals.hpp>
+#include <boost/bind.hpp>
+
 // FreeCAD Base header
 #include <Base/Parameter.h>
+#include <Base/Exception.h>
 #include <Base/FileInfo.h>
+#include <Base/Persistence.h>
+#include <Base/Stream.h>
+#include <Base/Reader.h>
+#include <Base/Writer.h>
 #include <App/Application.h>
+#include <App/DocumentObject.h>
 
 #include "MainWindow.h"
 #include "Application.h"
@@ -86,6 +101,12 @@
 #include "Document.h"
 #include "ViewProviderExtern.h"
 
+#if defined(Q_OS_WIN32)
+#define slots
+#include <private/qmainwindowlayout_p.h>
+#include <private/qwidgetresizehandler_p.h>
+#endif
+
 using namespace Gui;
 using namespace Gui::DockWnd;
 using namespace std;
@@ -103,8 +124,12 @@ struct MainWindowP
     QTimer* actionTimer;
     QTimer* activityTimer;
     QTimer* visibleTimer;
+#if !defined (NO_USE_QT_MDI_AREA)
+    QMdiArea* mdiArea;
+#else
     QWorkspace* workspace;
     QTabBar* tabs;
+#endif
     QPointer<MDIView> activeView;
     QSignalMapper* windowMapper;
     QSplashScreen* splashscreen;
@@ -153,6 +178,51 @@ private:
     QMenu* menu;
 };
 
+#if defined(Q_OS_WIN32)
+class MainWindowTabBar : public QTabBar
+{
+public:
+    MainWindowTabBar(QWidget *parent) : QTabBar(parent)
+    {
+        setExpanding(false);
+    }
+protected:
+    bool event(QEvent *e)
+    {
+        // show the tooltip if tab is too small to fit label
+        if (e->type() != QEvent::ToolTip)
+            return QTabBar::event(e);
+        QSize size = this->size();
+        QSize hint = sizeHint();
+        if (shape() == QTabBar::RoundedWest || shape() == QTabBar::RoundedEast) {
+            size.transpose();
+            hint.transpose();
+        }
+        if (size.width() < hint.width())
+            return QTabBar::event(e);
+        e->accept();
+        return true;
+    }
+    void tabInserted (int index)
+    {
+        // get all dock windows
+        QList<QDockWidget*> dw = getMainWindow()->findChildren<QDockWidget*>();
+        for (QList<QDockWidget*>::iterator it = dw.begin(); it != dw.end(); ++it) {
+            // compare tab text and window title to get the right dock window
+            if (this->tabText(index) == (*it)->windowTitle()) {
+                QWidget* dock = (*it)->widget();
+                if (dock) {
+                    QIcon icon = dock->windowIcon();
+                    if (!icon.isNull())
+                        setTabIcon(index, icon);
+                }
+                break;
+            }
+        }
+    }
+};
+#endif
+
 } // namespace Gui
 
 
@@ -171,6 +241,7 @@ MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
     instance = this;
 
     // Create the layout containing the workspace and a tab bar
+#if defined(NO_USE_QT_MDI_AREA)
     QFrame* vbox = new QFrame(this);
     vbox->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
     QVBoxLayout* layout = new QVBoxLayout();
@@ -179,14 +250,36 @@ MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
 
     d->workspace = new QWorkspace();
     d->workspace->setScrollBarsEnabled( true );
-    QPixmap backgnd(( const char** ) background );
-    d->workspace->setBackground( backgnd );
+    QPixmap backgnd((const char**) background);
+    d->workspace->setBackground(backgnd);
+
     d->tabs = new MDITabbar();
-    d->tabs->setShape( QTabBar:: RoundedSouth );
+    d->tabs->setShape(QTabBar:: RoundedSouth);
 
     layout->addWidget(d->workspace);
     layout->addWidget(d->tabs);
-    setCentralWidget( vbox );
+    setCentralWidget(vbox);
+#else
+    d->mdiArea = new QMdiArea();
+#if QT_VERSION >= 0x040500
+    d->mdiArea->setTabPosition(QTabWidget::South);
+    d->mdiArea->setViewMode(QMdiArea::TabbedView);
+    QTabBar* tab = d->mdiArea->findChild<QTabBar*>();
+    if (tab) { 
+        connect(tab, SIGNAL(tabCloseRequested(int)),
+                this, SLOT(tabCloseRequested(int)));
+        tab->setTabsClosable(true);
+        // The tabs might be very wide
+        tab->setExpanding(false);
+    }
+#endif
+    d->mdiArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    d->mdiArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+    d->mdiArea->setOption(QMdiArea::DontMaximizeSubWindowOnActivation, false);
+    QPixmap backgnd((const char**) background);
+    d->mdiArea->setBackground(backgnd);
+    setCentralWidget(d->mdiArea);
+#endif
 
     // labels and progressbar
     d->status = new StatusBarObserver();
@@ -217,25 +310,43 @@ MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
     d->windowMapper = new QSignalMapper(this);
 
     // connection between workspace, window menu and tab bar
-    connect( d->windowMapper, SIGNAL( mapped(QWidget *) ), d->workspace, SLOT( setActiveWindow( QWidget* ) ) );
-    connect( d->workspace, SIGNAL( windowActivated ( QWidget * ) ), this, SLOT( onWindowActivated( QWidget* ) ) );
-    connect( d->tabs, SIGNAL( currentChanged( int) ), this, SLOT( onTabSelected(int) ) );
+#if !defined (NO_USE_QT_MDI_AREA)
+    connect(d->windowMapper, SIGNAL(mapped(QWidget *)),
+            this, SLOT(onSetActiveSubWindow(QWidget*)));
+    connect(d->mdiArea, SIGNAL(subWindowActivated(QMdiSubWindow*)),
+            this, SLOT(onWindowActivated(QMdiSubWindow* )));
+#else
+    connect(d->windowMapper, SIGNAL(mapped(QWidget *)),
+            d->workspace, SLOT(setActiveWindow(QWidget* )));
+    connect(d->workspace, SIGNAL(windowActivated(QWidget *)),
+            this, SLOT(onWindowActivated(QWidget* )));
+    connect(d->tabs, SIGNAL(currentChanged(int)),
+            this, SLOT(onTabSelected(int)));
+#endif
 
     DockWindowManager* pDockMgr = DockWindowManager::instance();
 
     // Show all dockable windows over the workbench facility
     //
+#if 0
     // Toolbox
- /*   ToolBox* toolBox = new ToolBox(this);
+    ToolBox* toolBox = new ToolBox(this);
     toolBox->setObjectName(QT_TRANSLATE_NOOP("QDockWidget","Toolbox"));
     pDockMgr->registerDockWindow("Std_ToolBox", toolBox);
     ToolBoxManager::getInstance()->setToolBox( toolBox );
-*/
 
     // Help View
-    //QString home = Gui::Dialog::DlgOnlineHelpImp::getStartpage();
-    //HelpView* pcHelpView = new HelpView( home, this );
-    //pDockMgr->registerDockWindow("Std_HelpView", pcHelpView);
+    QString home = Gui::Dialog::DlgOnlineHelpImp::getStartpage();
+    HelpView* pcHelpView = new HelpView( home, this );
+    pDockMgr->registerDockWindow("Std_HelpView", pcHelpView);
+
+    // TaskPanel view
+    TaskPanelView* pcTaskPanelView = new TaskPanelView(0, this);
+    pcTaskPanelView->setObjectName
+        (QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Task View")));
+    pcTaskPanelView->setMinimumWidth(210);
+    pDockMgr->registerDockWindow("Std_TaskPanelView", pcTaskPanelView);
+#endif
 
     // Tree view
     TreeDockWidget* tree = new TreeDockWidget(0, this);
@@ -258,33 +369,53 @@ MainWindow::MainWindow(QWidget * parent, Qt::WFlags f)
     pcSelectionView->setMinimumWidth(210);
     pDockMgr->registerDockWindow("Std_SelectionView", pcSelectionView);
 
-    // TaskPanel view
-    TaskPanelView* pcTaskPanelView = new TaskPanelView(0, this);
-    pcTaskPanelView->setObjectName
-        (QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Task View")));
-    pcTaskPanelView->setMinimumWidth(210);
-    pDockMgr->registerDockWindow("Std_TaskPanelView", pcTaskPanelView);
-
     // Combo view
     CombiView* pcCombiView = new CombiView(0, this);
     pcCombiView->setObjectName(QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Combo View")));
     pcCombiView->setMinimumWidth(150);
     pDockMgr->registerDockWindow("Std_CombiView", pcCombiView);
 
+#if QT_VERSION < 0x040500
     // Report view
     Gui::DockWnd::ReportView* pcReport = new Gui::DockWnd::ReportView(this);
     pcReport->setObjectName
         (QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Report view")));
     pDockMgr->registerDockWindow("Std_ReportView", pcReport);
+#else
+    // Report view
+    ReportOutput* pcReport = new ReportOutput(this);
+    pcReport->setWindowIcon(BitmapFactory().pixmap("MacroEditor"));
+    pcReport->setObjectName
+        (QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Report view")));
+    pDockMgr->registerDockWindow("Std_ReportView", pcReport);
 
-    // FIXME: Break into two dock windows in tabbed mode (with icon and correct tab width)
-    //QWidget* pcPython = new PythonConsole(this);
-    //pcPython->setObjectName(QT_TRANSLATE_NOOP("QDockWidget","Python view"));
-    //pDockMgr->registerDockWindow("Std_PythonView", pcPython);
+    // Python console
+    PythonConsole* pcPython = new PythonConsole(this);
+    pcPython->setWordWrapMode(QTextOption::NoWrap);
+    pcPython->setWindowIcon(Gui::BitmapFactory().pixmap("python_small"));
+    pcPython->setObjectName
+        (QString::fromAscii(QT_TRANSLATE_NOOP("QDockWidget","Python console")));
+    pDockMgr->registerDockWindow("Std_PythonView", pcPython);
+
+#if defined(Q_OS_WIN32)
+    // add our own QTabBar-derived class to the main window layout
+    // NOTE: This uses some private stuff from QMainWindow which doesn't
+    // seem to be accessible on all platforms.
+    QMainWindowLayout* l = static_cast<QMainWindowLayout*>(this->layout());
+    for (int i=0; i<5; i++) {
+        MainWindowTabBar* result = new MainWindowTabBar(this);
+        result->setDrawBase(true);
+        result->setElideMode(Qt::ElideRight);
+        //result->setDocumentMode(_documentMode);
+        connect(result, SIGNAL(currentChanged(int)), l, SLOT(tabChanged()));
+        l->unusedTabBars << result;
+    }
+#endif
+#endif
 
     // accept drops on the window, get handled in dropEvent, dragEnterEvent
     setAcceptDrops(true);
-    statusBar()->showMessage( tr("Ready"), 2001 );
+    statusBar()->showMessage(tr("Ready"), 2001);
 }
 
 MainWindow::~MainWindow()
@@ -327,37 +458,65 @@ QMenu* MainWindow::createPopupMenu ()
 
 void MainWindow::arrangeIcons()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->tileSubWindows();
+#else
     d->workspace->arrangeIcons();
+#endif
 }
 
 void MainWindow::tile()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->tileSubWindows();
+#else
     d->workspace->tile();
+#endif
 }
 
 void MainWindow::cascade()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->cascadeSubWindows();
+#else
     d->workspace->cascade();
+#endif
 }
 
 void MainWindow::closeActiveWindow ()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->closeActiveSubWindow();
+#else
     d->workspace->closeActiveWindow();
+#endif
 }
 
 void MainWindow::closeAllWindows ()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->closeAllSubWindows();
+#else
     d->workspace->closeAllWindows();
+#endif
 }
 
 void MainWindow::activateNextWindow ()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->activateNextSubWindow();
+#else
     d->workspace->activateNextWindow();
+#endif
 }
 
 void MainWindow::activatePreviousWindow ()
 {
+#if !defined (NO_USE_QT_MDI_AREA)
+    d->mdiArea->activatePreviousSubWindow();
+#else
     d->workspace->activatePreviousWindow();
+#endif
 }
 
 void MainWindow::activateWorkbench(const QString& name)
@@ -392,16 +551,24 @@ bool MainWindow::event(QEvent *e)
             qApp->installEventFilter(this);
         }
     }
-    if (e->type() == QEvent::LeaveWhatsThisMode) {
+    else if (e->type() == QEvent::LeaveWhatsThisMode) {
         // Here we can't do anything because this event is sent
         // before the WhatThisClicked event is sent. Thus, we handle
         // this in eventFilter().
     }
-
-    if (e->type() == QEvent::WhatsThisClicked) {
+    else if (e->type() == QEvent::WhatsThisClicked) {
         QWhatsThisClickedEvent* wt = static_cast<QWhatsThisClickedEvent*>(e);
         showDocumentation((const char*)wt->href().toUtf8());
     }
+    else if (e->type() == QEvent::ApplicationWindowIconChange) {
+        // if application icon changes apply it to the main window and the "About..." dialog
+        this->setWindowIcon(QApplication::windowIcon());
+        Command* about = Application::Instance->commandManager().getCommandByName("Std_About");
+        if (about) {
+            Action* action = about->getAction();
+            if (action) action->setIcon(QApplication::windowIcon());
+        }
+    }
     return QMainWindow::event(e);
 }
 
@@ -488,19 +655,47 @@ bool MainWindow::eventFilter(QObject* o, QEvent* e)
 void MainWindow::addWindow(MDIView* view)
 {
     // make workspace parent of view
+#if !defined (NO_USE_QT_MDI_AREA)
+    bool isempty = d->mdiArea->subWindowList().isEmpty();
+    QMdiSubWindow* child = new QMdiSubWindow(d->mdiArea->viewport());
+    child->setAttribute(Qt::WA_DeleteOnClose);
+    child->setWidget(view);
+    child->setWindowIcon(view->windowIcon());
+    QMenu* menu = child->systemMenu();
+    QAction* action = menu->addAction(tr("Close All"));
+    connect(action, SIGNAL(triggered()), d->mdiArea, SLOT(closeAllSubWindows()));
+    d->mdiArea->addSubWindow(child);
+#else
+    QWidget* active = d->workspace->activeWindow();
     d->workspace->addWindow(view);
-    connect( view, SIGNAL( message(const QString&, int) ), statusBar(), SLOT( showMessage(const QString&, int )) );
-    connect( this, SIGNAL( windowStateChanged(MDIView*) ), view, SLOT( windowStateChanged(MDIView*) ) );
+#if defined(Q_OS_WIN32)
+    // avoid dragging problem with not maximized mdi childs which have embedded a GL window
+    QWidget* p = view->parentWidget();
+    if (p) {
+        QWidgetResizeHandler* handler = p->findChild<QWidgetResizeHandler*>();
+        if (handler) handler->setMovingEnabled(false);
+    }
+#endif
+#endif
+    connect(view, SIGNAL(message(const QString&, int)),
+            statusBar(), SLOT(showMessage(const QString&, int)));
+    connect(this, SIGNAL(windowStateChanged(MDIView*)),
+            view, SLOT(windowStateChanged(MDIView*)));
 
     // listen to the incoming events of the view
     view->installEventFilter(this);
 
     // show the very first window in maximized mode
+#if !defined (NO_USE_QT_MDI_AREA)
+    if (isempty)
+#else
     if (d->workspace->windowList().isEmpty())
+#endif
         view->showMaximized();
     else
         view->show();
 
+#if defined(NO_USE_QT_MDI_AREA)
     // look if the window was already inserted
     for (int i=0; i < d->tabs->count(); i++) {
         if (d->tabs->tabData(i).value<QWidget*>() == view)
@@ -508,20 +703,34 @@ void MainWindow::addWindow(MDIView* view)
     }
 
     // being informed when the view is destroyed
-    connect( view, SIGNAL( destroyed() ), this, SLOT( onWindowDestroyed() ) );
+    connect(view, SIGNAL(destroyed()),
+            this, SLOT(onWindowDestroyed()));
 
     // add a new tab to our tabbar
     int index=-1;
     index = d->tabs->addTab(view->windowIcon(), view->windowTitle());
-    d->tabs->setTabToolTip( index, view->windowTitle() );
+    d->tabs->setTabToolTip(index, view->windowTitle());
     QVariant var; var.setValue<QWidget*>(view);
-    d->tabs->setTabData( index, var);
+    d->tabs->setTabData(index, var);
 
-    tabChanged( view );
-    if ( d->tabs->count() == 1 )
+    tabChanged(view);
+    if (d->tabs->count() == 1)
         d->tabs->show(); // invoke show() for the first tab
     d->tabs->update();
-    d->tabs->setCurrentIndex( index );
+    d->tabs->setCurrentIndex(index);
+#endif
+
+#if defined (NO_USE_QT_MDI_AREA)
+    // With the old QWorkspace class we have some strange update problem
+    // when adding a 3d view and the first view is a web view or text view.
+    static bool do_hack=true;
+    MDIView *active_mdi = qobject_cast<MDIView*>(active);
+    if (do_hack && active_mdi && active_mdi->getTypeId() != view->getTypeId()) {
+        d->workspace->setActiveWindow(active);
+        d->workspace->setActiveWindow(view);
+        do_hack = false; // needs to be done only once
+    }
+#endif
 }
 
 void MainWindow::removeWindow(Gui::MDIView* view)
@@ -533,6 +742,7 @@ void MainWindow::removeWindow(Gui::MDIView* view)
                view, SLOT(windowStateChanged(MDIView*)));
     view->removeEventFilter(this);
 
+#if defined(NO_USE_QT_MDI_AREA)
     for (int i = 0; i < d->tabs->count(); i++) {
         if (d->tabs->tabData(i).value<QWidget*>() == view) {
             d->tabs->removeTab(i);
@@ -541,6 +751,7 @@ void MainWindow::removeWindow(Gui::MDIView* view)
             break;
         }
     }
+#endif
 
     // check if the focus widget is a child of the view
     QWidget* foc = this->focusWidget();
@@ -555,12 +766,20 @@ void MainWindow::removeWindow(Gui::MDIView* view)
         }
     }
 
+#if defined(NO_USE_QT_MDI_AREA)
     // this view is not under control of the main window any more
-    disconnect( view, SIGNAL( destroyed() ), this, SLOT( onWindowDestroyed() ) );
+    disconnect(view, SIGNAL(destroyed()),
+               this, SLOT(onWindowDestroyed()));
+#else
+    QWidget* parent = view->parentWidget();
+    d->mdiArea->removeSubWindow(parent);
+    parent->deleteLater();
+#endif
 }
 
 void MainWindow::tabChanged(MDIView* view)
 {
+#if defined(NO_USE_QT_MDI_AREA)
     for (int i = 0; i < d->tabs->count(); i++) {
         if (d->tabs->tabData(i).value<QWidget*>() == view) {
             QString cap = view->windowTitle();
@@ -582,8 +801,10 @@ void MainWindow::tabChanged(MDIView* view)
             break;
         }
     }
+#endif
 }
 
+#if defined(NO_USE_QT_MDI_AREA)
 void MainWindow::onWindowDestroyed()
 {
     QObject* view = (QObject*)sender();
@@ -597,9 +818,64 @@ void MainWindow::onWindowDestroyed()
     }
 }
 
-void MainWindow::onWindowActivated( QWidget* w )
+void MainWindow::onTabSelected(int i)
 {
+    QVariant var = d->tabs->tabData(i);
+    if (var.isValid() && var.canConvert<QWidget*>()) {
+        QWidget* view = var.value<QWidget*>();
+        if (view && !view->hasFocus())
+            view->setFocus();
+    }
+}
+
+void MainWindow::setActiveWindow(MDIView* view)
+{
+    d->workspace->setActiveWindow(view);
+    d->activeView = view;
+    Application::Instance->viewActivated(view);
+}
+#else
+void MainWindow::tabCloseRequested(int index)
+{
+    QTabBar* tab = d->mdiArea->findChild<QTabBar*>();
+    if (index < 0 || index >= tab->count())
+        return;
+
+    const QList<QMdiSubWindow *> subWindows = d->mdiArea->subWindowList();
+    Q_ASSERT(index < subWindows.size());
+
+    QMdiSubWindow *subWindow = d->mdiArea->subWindowList().at(index);
+    Q_ASSERT(subWindow);
+    subWindow->close();
+}
+
+void MainWindow::onSetActiveSubWindow(QWidget *window)
+{
+    if (!window)
+        return;
+    d->mdiArea->setActiveSubWindow(qobject_cast<QMdiSubWindow *>(window));
+}
+
+void MainWindow::setActiveWindow(MDIView* view)
+{
+    onSetActiveSubWindow(view->parentWidget());
+    d->activeView = view;
+    Application::Instance->viewActivated(view);
+}
+#endif
+
+#if !defined (NO_USE_QT_MDI_AREA)
+void MainWindow::onWindowActivated(QMdiSubWindow* w)
+#else
+void MainWindow::onWindowActivated(QWidget* w)
+#endif
+{
+#if !defined (NO_USE_QT_MDI_AREA)
+    if (!w) return;
+    MDIView* view = dynamic_cast<MDIView*>(w->widget());
+#else
      MDIView* view = dynamic_cast<MDIView*>(w);
+#endif
 
     // Even if windowActivated() signal is emitted mdi doesn't need to be a top-level window.
     // This happens e.g. if two windows are top-level and one of them gets docked again.
@@ -618,6 +894,7 @@ void MainWindow::onWindowActivated( QWidget* w )
     d->activeView = view;
     Application::Instance->viewActivated(view);
 
+#if defined(NO_USE_QT_MDI_AREA)
     // set the appropriate tab to the new active window
     for (int i = 0; i < d->tabs->count(); i++) {
         if (d->tabs->tabData(i).value<QWidget*>() == view) {
@@ -627,22 +904,18 @@ void MainWindow::onWindowActivated( QWidget* w )
             break;
         }
     }
-}
-
-void MainWindow::onTabSelected( int i)
-{
-    QVariant var = d->tabs->tabData(i);
-    if (var.isValid() && var.canConvert<QWidget*>()) {
-        QWidget* view = var.value<QWidget*>();
-        if (view && !view->hasFocus())
-            view->setFocus();
-    }
+#endif
 }
 
 void MainWindow::onWindowsMenuAboutToShow()
 {
-    QList<QWidget*> windows = d->workspace->windowList( QWorkspace::CreationOrder );
+#if !defined (NO_USE_QT_MDI_AREA)
+    QList<QMdiSubWindow*> windows = d->mdiArea->subWindowList(QMdiArea::CreationOrder);
+    QWidget* active = d->mdiArea->activeSubWindow();
+#else
+    QList<QWidget*> windows = d->workspace->windowList(QWorkspace::CreationOrder);
     QWidget* active = d->workspace->activeWindow();
+#endif
 
     // We search for the 'Std_WindowsMenu' command that provides the list of actions
     CommandManager& cMgr = Application::Instance->commandManager();
@@ -674,9 +947,9 @@ void MainWindow::onWindowsMenuAboutToShow()
                 title = QString::fromAscii("%1*").arg(title);
         }
         if (index < 9)
-            text = QString::fromAscii("&%1 %2").arg( index+1 ).arg(title);
+            text = QString::fromAscii("&%1 %2").arg(index+1).arg(title);
         else
-            text = QString::fromAscii("%1 %2").arg( index+1 ).arg(title);
+            text = QString::fromAscii("%1 %2").arg(index+1).arg(title);
         action->setText(text);
         action->setVisible(true);
         action->setChecked(child == active);
@@ -714,17 +987,29 @@ void MainWindow::onDockWindowMenuAboutToShow()
     QList<QDockWidget*> dock = this->findChildren<QDockWidget*>();
     for (QList<QDockWidget*>::Iterator it = dock.begin(); it != dock.end(); ++it) {
         QAction* action = (*it)->toggleViewAction();
-        action->setToolTip(tr("Toogles this dockable window"));
-        action->setStatusTip(tr("Toogles this dockable window"));
-        action->setWhatsThis(tr("Toogles this dockable window"));
+        action->setToolTip(tr("Toggles this dockable window"));
+        action->setStatusTip(tr("Toggles this dockable window"));
+        action->setWhatsThis(tr("Toggles this dockable window"));
         menu->addAction(action);
     }
 }
 
-QList<QWidget*> MainWindow::windows( QWorkspace::WindowOrder order ) const
+#if !defined (NO_USE_QT_MDI_AREA)
+QList<QWidget*> MainWindow::windows(QMdiArea::WindowOrder order) const
 {
-    return d->workspace->windowList( order );
+    QList<QWidget*> mdis;
+    QList<QMdiSubWindow*> wnds = d->mdiArea->subWindowList(order);
+    for (QList<QMdiSubWindow*>::iterator it = wnds.begin(); it != wnds.end(); ++it) {
+        mdis << (*it)->widget();
+    }
+    return mdis;
 }
+#else
+QList<QWidget*> MainWindow::windows(QWorkspace::WindowOrder order) const
+{
+    return d->workspace->windowList(order);
+}
+#endif
 
 // set text to the pane
 void MainWindow::setPaneText(int i, QString text)
@@ -745,14 +1030,7 @@ MDIView* MainWindow::activeWindow(void) const
     return d->activeView;
 }
 
-void MainWindow::setActiveWindow(MDIView* view)
-{
-    d->workspace->setActiveWindow(view);
-    d->activeView = view;
-    Application::Instance->viewActivated(view);
-}
-
-void MainWindow::closeEvent ( QCloseEvent * e )
+void MainWindow::closeEvent (QCloseEvent * e)
 {
     Application::Instance->tryClose(e);
     if (e->isAccepted()) {
@@ -948,7 +1226,7 @@ QPixmap MainWindow::splashImage() const
     return splash_image;
 }
 
-void MainWindow::showTipOfTheDay( bool force )
+void MainWindow::showTipOfTheDay(bool force)
 {
     // tip of the day?
     ParameterGrp::handle
@@ -973,45 +1251,15 @@ void MainWindow::dropEvent (QDropEvent* e)
 {
     const QMimeData* data = e->mimeData();
     if (data->hasUrls()) {
-        QList<QUrl> uri = data->urls();
-        QStringList files;
-        App::Document* pDoc = 0;
-
-        // get the files which we support
-        for (QList<QUrl>::ConstIterator it = uri.begin(); it != uri.end(); ++it) {
-            QFileInfo info((*it).toLocalFile());
-            if (info.exists() && info.isFile()) {
-                if (info.isSymLink())
-                    info.setFile(info.readLink());
-                std::vector<std::string> module = App::GetApplication()
-                    .getImportModules(info.completeSuffix().toAscii());
-                if (module.empty()) {
-                    module = App::GetApplication()
-                        .getImportModules(info.suffix().toAscii());
-                }
-                if (!module.empty()) {
-                    // ok, we support files with this extension
-                    files << info.absoluteFilePath();
-                    // we load non-project files, i.e. we must create a new document
-                    if (!pDoc && info.suffix().toLower() != QLatin1String("fcstd"))
-                        pDoc = App::GetApplication().newDocument();
-                }
-            }
-        }
-
-        const char *docName = pDoc ? pDoc->getName() : "";
-        SelectModule::Dict dict = SelectModule::importHandler(files);
-        // load the files with the associated modules
-        for (SelectModule::Dict::iterator it = dict.begin(); it != dict.end(); ++it) {
-            Application::Instance->importFrom(it.key().toUtf8(), docName, it.value().toAscii());
-        }
+        // pass no document to let create a new one if needed
+        loadUrls(0, data->urls());
     }
     else {
         QMainWindow::dropEvent(e);
     }
 }
 
-void MainWindow::dragEnterEvent ( QDragEnterEvent * e )
+void MainWindow::dragEnterEvent (QDragEnterEvent * e)
 {
     // Here we must allow uri drafs and check them in dropEvent
     const QMimeData* data = e->mimeData();
@@ -1021,6 +1269,189 @@ void MainWindow::dragEnterEvent ( QDragEnterEvent * e )
         e->ignore();
 }
 
+namespace Gui {
+class MimePersistence : public Base::Persistence
+{
+public:
+    MimePersistence(App::Document* doc)
+    {
+        connectExport = doc->signalExportObjects.connect
+            (boost::bind(&MimePersistence::exportObject, this, _1, _2));
+        connectImport = doc->signalImportObjects.connect
+            (boost::bind(&MimePersistence::importObject, this, _1, _2));
+        document = Gui::Application::Instance->getDocument(doc);
+    }
+    ~MimePersistence()
+    {
+        connectExport.disconnect();
+        connectImport.disconnect();
+    }
+    unsigned int getMemSize (void) const
+    {
+        return 0;
+    }
+    void importObject(const std::vector<App::DocumentObject*>& o, Base::XMLReader & r)
+    {
+        objects = o;
+        Restore(r);
+    }
+    void exportObject(const std::vector<App::DocumentObject*>& o, Base::Writer & w)
+    {
+        objects = o;
+        Save(w);
+    }
+    void Save (Base::Writer & w) const
+    {
+        w.addFile("GuiDocument.xml", this);
+    }
+    void Restore(Base::XMLReader &r)
+    {
+        r.addFile("GuiDocument.xml", this);
+    }
+    void SaveDocFile (Base::Writer & w) const
+    {
+        document->exportObjects(objects, w);
+    }
+    void RestoreDocFile(Base::Reader & r)
+    {
+        document->importObjects(objects, r);
+    }
+
+private:
+    Gui::Document* document;
+    std::vector<App::DocumentObject*> objects;
+    typedef boost::signals::connection Connection;
+    Connection connectExport;
+    Connection connectImport;
+};
+}
+
+QMimeData * MainWindow::createMimeDataFromSelection () const
+{
+    std::vector<SelectionSingleton::SelObj> sel = Selection().getCompleteSelection();
+    unsigned int memsize=1000; // ~ for the meta-information
+    std::vector<App::DocumentObject*> obj;
+    obj.reserve(sel.size());
+    for (std::vector<SelectionSingleton::SelObj>::iterator it = sel.begin(); it != sel.end(); ++it) {
+        if (it->pObject) {
+            obj.push_back(it->pObject);
+            memsize += it->pObject->getMemSize();
+        }
+    }
+
+    // get a pointer to a document
+    if (obj.empty()) return 0;
+    App::Document* doc = obj.front()->getDocument();
+    if (!doc) return 0;
+
+    // if less than ~10 MB
+    bool use_buffer=(memsize < 0xA00000);
+    QByteArray res;
+    try {
+        res.reserve(memsize);
+    }
+    catch (const Base::MemoryException&) {
+        use_buffer = false;
+    }
+
+    QString mime;
+    if (use_buffer) {
+        mime = QLatin1String("application/x-documentobject");
+        Base::ByteArrayOStreambuf buf(res);
+        std::ostream str(&buf);
+        MimePersistence mimeView(doc);
+        doc->exportObjects(obj, str);
+    }
+    else {
+        mime = QLatin1String("application/x-documentobject-file");
+        static Base::FileInfo fi(Base::FileInfo::getTempFileName());
+        Base::ofstream str(fi, std::ios::out | std::ios::binary);
+        MimePersistence mimeView(doc);
+        doc->exportObjects(obj, str);
+        str.close();
+        res = fi.filePath().c_str();
+    }
+
+    QMimeData *mimeData = new QMimeData();
+    mimeData->setData(mime,res);
+    return mimeData;
+}
+
+bool MainWindow::canInsertFromMimeData (const QMimeData * source) const
+{
+    if (!source)
+        return false;
+    return source->hasUrls() || 
+        source->hasFormat(QLatin1String("application/x-documentobject")) ||
+        source->hasFormat(QLatin1String("application/x-documentobject-file"));
+}
+
+void MainWindow::insertFromMimeData (const QMimeData * mimeData)
+{
+    if (!mimeData)
+        return;
+    if (mimeData->hasFormat(QLatin1String("application/x-documentobject"))) {
+        QByteArray res = mimeData->data(QLatin1String("application/x-documentobject"));
+        App::Document* doc = App::GetApplication().getActiveDocument();
+        if (!doc) doc = App::GetApplication().newDocument();
+
+        Base::ByteArrayIStreambuf buf(res);
+        std::istream in(0);
+        in.rdbuf(&buf);
+        MimePersistence mimeView(doc);
+        doc->importObjects(in);
+    }
+    else if (mimeData->hasFormat(QLatin1String("application/x-documentobject-file"))) {
+        QByteArray res = mimeData->data(QLatin1String("application/x-documentobject-file"));
+        App::Document* doc = App::GetApplication().getActiveDocument();
+        if (!doc) doc = App::GetApplication().newDocument();
+
+        Base::FileInfo fi((const char*)res);
+        Base::ifstream str(fi, std::ios::in | std::ios::binary);
+        MimePersistence mimeView(doc);
+        doc->importObjects(str);
+        str.close();
+    }
+    else if (mimeData->hasUrls()) {
+        // load the files into the active document if there is one, otherwise let create one
+        loadUrls(App::GetApplication().getActiveDocument(), mimeData->urls());
+    }
+}
+
+void MainWindow::loadUrls(App::Document* doc, const QList<QUrl>& url)
+{
+    QStringList files;
+    for (QList<QUrl>::ConstIterator it = url.begin(); it != url.end(); ++it) {
+        QFileInfo info((*it).toLocalFile());
+        if (info.exists() && info.isFile()) {
+            if (info.isSymLink())
+                info.setFile(info.readLink());
+            std::vector<std::string> module = App::GetApplication()
+                .getImportModules(info.completeSuffix().toAscii());
+            if (module.empty()) {
+                module = App::GetApplication()
+                    .getImportModules(info.suffix().toAscii());
+            }
+            if (!module.empty()) {
+                // ok, we support files with this extension
+                files << info.absoluteFilePath();
+            }
+            else {
+                Base::Console().Message("No support to load file '%s'\n",
+                    (const char*)info.absoluteFilePath().toUtf8());
+            }
+        }
+    }
+
+    const char *docName = doc ? doc->getName() : "Unnamed";
+    SelectModule::Dict dict = SelectModule::importHandler(files);
+    // load the files with the associated modules
+    for (SelectModule::Dict::iterator it = dict.begin(); it != dict.end(); ++it) {
+        // if the passed document name doesn't exist the module should create it, if needed
+        Application::Instance->importFrom(it.key().toUtf8(), docName, it.value().toAscii());
+    }
+}
+
 void MainWindow::changeEvent(QEvent *e)
 {
     if (e->type() == QEvent::LanguageChange) {
@@ -1055,7 +1486,7 @@ namespace Gui {
 class CustomMessageEvent : public QEvent
 {
 public:
-     enum Type {Msg, Wrn, Err, Log};
+    enum Type {Msg, Wrn, Err, Log};
     CustomMessageEvent(Type t, const QString& s)
       : QEvent(QEvent::User), _type(t), msg(s)
     { }
@@ -1071,10 +1502,10 @@ private:
 };
 }
 
-void MainWindow::customEvent( QEvent* e )
+void MainWindow::customEvent(QEvent* e)
 {
     if (e->type() == QEvent::User) {
-        Gui::CustomMessageEvent* ce = (Gui::CustomMessageEvent*)e;
+        Gui::CustomMessageEvent* ce = static_cast<Gui::CustomMessageEvent*>(e);
         QString msg = ce->message();
         if (ce->type() == CustomMessageEvent::Log) {
             if (msg.startsWith(QLatin1String("#Inventor V2.1 ascii "))) {
