@@ -25,9 +25,15 @@
 #ifndef _PreComp_
 # include <cfloat>
 # include "InventorAll.h"
+# include <QAction>
+# include <QActionGroup>
 # include <QApplication>
+# include <QByteArray>
 # include <QCursor>
+# include <QList>
 # include <QMenu>
+# include <QMetaObject>
+# include <QRegExp>
 #endif
 #include <Inventor/sensors/SoTimerSensor.h>
 
@@ -55,6 +61,20 @@ struct NavigationStyleP {
 };
 }
 
+NavigationStyleEvent::NavigationStyleEvent(const Base::Type& s)
+  : QEvent(QEvent::User), t(s)
+{
+}
+
+NavigationStyleEvent::~NavigationStyleEvent()
+{
+}
+
+const Base::Type& NavigationStyleEvent::style() const
+{ 
+    return t;
+}
+
 #define PRIVATE(ptr) (ptr->pimpl)
 #define PUBLIC(ptr) (ptr->pub)
 
@@ -818,6 +838,46 @@ int NavigationStyle::getViewingMode() const
     return (int)this->currentmode;
 }
 
+SbBool NavigationStyle::processEvent(const SoEvent * const ev)
+{
+    // If we're in picking mode then all events must be redirected to the
+    // appropriate mouse model.
+    if (pcMouseModel) {
+        int hd=pcMouseModel->handleEvent(ev,viewer->getViewportRegion());
+        if (hd==AbstractMouseModel::Continue||hd==AbstractMouseModel::Restart) {
+            return TRUE;
+        }
+        else if (hd==AbstractMouseModel::Finish) {
+            pcPolygon = pcMouseModel->getPolygon();
+            clipInner = pcMouseModel->isInner();
+            delete pcMouseModel; pcMouseModel = 0;
+            return NavigationStyle::processSoEvent(ev);
+        }
+        else if (hd==AbstractMouseModel::Cancel) {
+            pcPolygon.clear();
+            delete pcMouseModel; pcMouseModel = 0;
+            return NavigationStyle::processSoEvent(ev);
+        }
+    }
+
+    const ViewerMode curmode = this->currentmode;
+
+    SbBool processed = FALSE;
+    processed = this->processSoEvent(ev);
+
+    // check for left click without selecting something
+    if (curmode == NavigationStyle::SELECTION && !processed) {
+        if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
+            SoMouseButtonEvent * const e = (SoMouseButtonEvent *) ev;
+            if (SoMouseButtonEvent::isButtonReleaseEvent(e,SoMouseButtonEvent::BUTTON1)) {
+                Gui::Selection().clearSelection();
+            }
+        }
+    }
+
+    return processed;
+}
+
 SbBool NavigationStyle::processSoEvent(const SoEvent * const ev)
 {
     return viewer->processSoEventBase(ev);
@@ -839,15 +899,61 @@ void NavigationStyle::openPopupMenu(const SbVec2s& position)
     MenuItem* view = new MenuItem;
     Gui::Application::Instance->setupContextMenu("View", view);
 
-    QMenu ContextMenu(viewer->getGLWidget());
-    MenuManager::getInstance()->setupContextMenu(view,ContextMenu);
+    QMenu contextMenu(viewer->getGLWidget());
+    QMenu subMenu;
+    QActionGroup subMenuGroup(&subMenu);
+    subMenuGroup.setExclusive(true);
+    subMenu.setTitle(QObject::tr("Navigation styles"));
+
+    MenuManager::getInstance()->setupContextMenu(view,contextMenu);
+    contextMenu.addMenu(&subMenu);
+
+    // add submenu at the end to select navigation style
+    QRegExp rx(QString::fromAscii("^\\w+::(\\w+)Navigation\\w+$"));
+    std::vector<Base::Type> types;
+    Base::Type::getAllDerivedFrom(UserNavigationStyle::getClassTypeId(), types);
+    for (std::vector<Base::Type>::iterator it = types.begin(); it != types.end(); ++it) {
+        if (*it != UserNavigationStyle::getClassTypeId()) {
+            QString data = QString::fromAscii(it->getName());
+            QString name = data.mid(data.indexOf(QLatin1String("::"))+2);
+            if (rx.indexIn(data) > -1) {
+                name = QObject::tr("%1 navigation").arg(rx.cap(1));
+                QAction* item = subMenuGroup.addAction(name);
+                item->setData(QByteArray(it->getName()));
+                item->setCheckable(true);
+                if (*it == this->getTypeId())
+                    item->setChecked(true);
+                subMenu.addAction(item);
+            }
+        }
+    }
+
     delete view;
-    ContextMenu.exec(QCursor::pos());
+    QAction* used = contextMenu.exec(QCursor::pos());
+    if (used && subMenuGroup.actions().indexOf(used) >= 0 && used->isChecked()) {
+        QByteArray type = used->data().toByteArray();
+        QWidget* widget = viewer->getWidget();
+        if (widget) {
+            widget = widget->parentWidget();
+            if (widget) {
+                // this is the widget where the viewer is embedded
+                Base::Type style = Base::Type::fromName((const char*)type);
+                if (style != this->getTypeId()) {
+                    QEvent* event = new NavigationStyleEvent(style);
+                    QApplication::postEvent(widget, event);
+                }
+            }
+        }
+    }
 }
 
 // ----------------------------------------------------------------------------------
 
-TYPESYSTEM_SOURCE(Gui::InventorNavigationStyle, Gui::NavigationStyle);
+TYPESYSTEM_SOURCE_ABSTRACT(Gui::UserNavigationStyle,Gui::NavigationStyle);
+
+// ----------------------------------------------------------------------------------
+
+TYPESYSTEM_SOURCE(Gui::InventorNavigationStyle, Gui::UserNavigationStyle);
 
 InventorNavigationStyle::InventorNavigationStyle()
 {
@@ -859,26 +965,6 @@ InventorNavigationStyle::~InventorNavigationStyle()
 
 SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
 {
-    // If we're in picking mode then all events must be redirected to the
-    // appropriate mouse model.
-    if (pcMouseModel) {
-        int hd=pcMouseModel->handleEvent(ev,viewer->getViewportRegion());
-        if (hd==AbstractMouseModel::Continue||hd==AbstractMouseModel::Restart) {
-            return TRUE;
-        }
-        else if (hd==AbstractMouseModel::Finish) {
-            pcPolygon = pcMouseModel->getPolygon();
-            clipInner = pcMouseModel->isInner();
-            delete pcMouseModel; pcMouseModel = 0;
-            return inherited::processSoEvent(ev);
-        }
-        else if (hd==AbstractMouseModel::Cancel) {
-            pcPolygon.clear();
-            delete pcMouseModel; pcMouseModel = 0;
-            return inherited::processSoEvent(ev);
-        }
-    }
-
     // Events when in "ready-to-seek" mode are ignored, except those
     // which influence the seek mode itself -- these are handled further
     // up the inheritance hierarchy.
@@ -901,8 +987,8 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
     // at all.
     SbBool processed = FALSE;
 
-    const ViewerMode currentmode = this->currentmode;
-    ViewerMode newmode = currentmode;
+    const ViewerMode curmode = this->currentmode;
+    ViewerMode newmode = curmode;
 
     // Mismatches in state of the modifier keys happens if the user
     // presses or releases them outside the viewer window.
@@ -968,15 +1054,26 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
         switch (button) {
         case SoMouseButtonEvent::BUTTON1:
             this->button1down = press;
-            if (!press && ev->wasAltDown()) {
+            if (press && ev->wasShiftDown() &&
+                (this->currentmode != NavigationStyle::SELECTION)) {
+                this->centerTime = ev->getTime();
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                if (!this->seekToPoint(pos)) {
-                    panToCenter(panningplane, posn);
-                    this->interactiveCountDec();
+                this->panningmove = FALSE;
+            }
+            else if (!press && ev->wasShiftDown() &&
+                (this->currentmode != NavigationStyle::SELECTION)) {
+                SbTime tmp = (ev->getTime() - this->centerTime);
+                float dci = (float)QApplication::doubleClickInterval()/1000.0f;
+                // is it just a left click?
+                if (tmp.getValue() < dci && !this->panningmove) {
+                    if (!this->seekToPoint(pos)) {
+                        panToCenter(panningplane, posn);
+                        this->interactiveCountDec();
+                    }
+                    processed = TRUE;
                 }
-                processed = TRUE;
             }
             else if (press && (this->currentmode == NavigationStyle::SEEK_WAIT_MODE)) {
                 newmode = NavigationStyle::SEEK_MODE;
@@ -1017,12 +1114,13 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
                 this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
+                this->panningmove = FALSE;
             }
             else {
                 SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
                 // is it just a middle click?
-                if (tmp.getValue() < dci) {
+                if (tmp.getValue() < dci && !this->panningmove) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
@@ -1053,6 +1151,7 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
             processed = TRUE;
         }
         else if (this->currentmode == NavigationStyle::PANNING) {
+            this->panningmove = TRUE;
             float ratio = vp.getViewportAspectRatio();
             panCamera(viewer->getCamera(), ratio, this->panningplane, posn, prevnormalized);
             processed = TRUE;
@@ -1092,10 +1191,10 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
 
     switch (combo) {
     case 0:
-        if (currentmode == NavigationStyle::SPINNING) { break; }
+        if (curmode == NavigationStyle::SPINNING) { break; }
         newmode = NavigationStyle::IDLE;
 
-        if (currentmode == NavigationStyle::DRAGGING) {
+        if (curmode == NavigationStyle::DRAGGING) {
             if (doSpin())
                 newmode = NavigationStyle::SPINNING;
         }
@@ -1128,42 +1227,32 @@ SbBool InventorNavigationStyle::processSoEvent(const SoEvent * const ev)
     default:
         // The default will make a spin stop and otherwise not do
         // anything.
-        if ((currentmode != NavigationStyle::SEEK_WAIT_MODE) &&
-            (currentmode != NavigationStyle::SEEK_MODE)) {
+        if ((curmode != NavigationStyle::SEEK_WAIT_MODE) &&
+            (curmode != NavigationStyle::SEEK_MODE)) {
             newmode = NavigationStyle::IDLE;
         }
         break;
     }
 
-    if (newmode != currentmode) {
+    if (newmode != curmode) {
         this->setViewingMode(newmode);
     }
 
     // If not handled in this class, pass on upwards in the inheritance
     // hierarchy.
-    if ((this->currentmode == NavigationStyle::SELECTION || viewer->isEditing()) && !processed)
+    if ((curmode == NavigationStyle::SELECTION || viewer->isEditing()) && !processed)
         processed = inherited::processSoEvent(ev);
     else
         return TRUE;
 
-    // check for left click without selecting something
-    if (this->currentmode == NavigationStyle::SELECTION && !processed) {
-        if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
-            SoMouseButtonEvent * const e = (SoMouseButtonEvent *) ev;
-            if (SoMouseButtonEvent::isButtonReleaseEvent(e,SoMouseButtonEvent::BUTTON1)) {
-                Gui::Selection().clearSelection();
-            }
-        }
-    }
-
-    return FALSE;
+    return processed;
 }
 
 // ----------------------------------------------------------------------------------
 
-TYPESYSTEM_SOURCE(Gui::CADNavigationStyle, Gui::NavigationStyle);
+TYPESYSTEM_SOURCE(Gui::CADNavigationStyle, Gui::UserNavigationStyle);
 
-CADNavigationStyle::CADNavigationStyle() : _bRejectSelection(false), _bSpining(false)
+CADNavigationStyle::CADNavigationStyle() : lockButton1(FALSE)
 {
 }
 
@@ -1173,27 +1262,77 @@ CADNavigationStyle::~CADNavigationStyle()
 
 SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
 {
-    SbBool processed = FALSE;
+#if 0
+    // Events when in "ready-to-seek" mode are ignored, except those
+    // which influence the seek mode itself -- these are handled further
+    // up the inheritance hierarchy.
+    if (this->isSeekMode()) { return inherited::processSoEvent(ev); }
+#else
     if (!this->isSeekMode() && this->isViewing())
         this->setViewing(false); // by default disable viewing mode to render the scene
+#endif
 
-    // Keybooard handling
-    if (ev->getTypeId().isDerivedFrom(SoKeyboardEvent::getClassTypeId())) {
-        SoKeyboardEvent * ke = (SoKeyboardEvent *)ev;
-        switch (ke->getKey()) {
-        case SoKeyboardEvent::LEFT_ALT:
-        case SoKeyboardEvent::RIGHT_ALT:
+    const SoType type(ev->getTypeId());
+
+    const SbViewportRegion & vp = viewer->getViewportRegion();
+    const SbVec2s size(vp.getViewportSizePixels());
+    const SbVec2f prevnormalized = this->lastmouseposition;
+    const SbVec2s pos(ev->getPosition());
+    const SbVec2f posn((float) pos[0] / (float) SoQtMax((int)(size[0] - 1), 1),
+                       (float) pos[1] / (float) SoQtMax((int)(size[1] - 1), 1));
+
+    this->lastmouseposition = posn;
+
+    // Set to TRUE if any event processing happened. Note that it is not
+    // necessary to restrict ourselves to only do one "action" for an
+    // event, we only need this flag to see if any processing happened
+    // at all.
+    SbBool processed = FALSE;
+
+    const ViewerMode curmode = this->currentmode;
+    ViewerMode newmode = curmode;
+
+    // Mismatches in state of the modifier keys happens if the user
+    // presses or releases them outside the viewer window.
+    if (this->ctrldown != ev->wasCtrlDown()) {
+        this->ctrldown = ev->wasCtrlDown();
+    }
+    if (this->shiftdown != ev->wasShiftDown()) {
+        this->shiftdown = ev->wasShiftDown();
+    }
+    if (this->altdown != ev->wasAltDown()) {
+        this->altdown = ev->wasAltDown();
+    }
+
+    // give the nodes in the foreground root the chance to handle events (e.g color bar)
+    if (!processed && !viewer->isEditing()) {
+        processed = handleEventInForeground(ev);
+        if (processed)
+            return TRUE;
+    }
+
+    // Keyboard handling
+    if (type.isDerivedFrom(SoKeyboardEvent::getClassTypeId())) {
+        const SoKeyboardEvent * const event = (const SoKeyboardEvent *) ev;
+        const SbBool press = event->getState() == SoButtonEvent::DOWN ? TRUE : FALSE;
+        switch (event->getKey()) {
         case SoKeyboardEvent::LEFT_CONTROL:
         case SoKeyboardEvent::RIGHT_CONTROL:
+            this->ctrldown = press;
+            break;
         case SoKeyboardEvent::LEFT_SHIFT:
         case SoKeyboardEvent::RIGHT_SHIFT:
+            this->shiftdown = press;
+            break;
+        case SoKeyboardEvent::LEFT_ALT:
+        case SoKeyboardEvent::RIGHT_ALT:
+            this->altdown = press;
             break;
         case SoKeyboardEvent::H:
-            viewer->saveHomePosition();
             processed = TRUE;
+            viewer->saveHomePosition();
             break;
         case SoKeyboardEvent::S:
-            // processSoEvent() of the base class sets the seekMode() if needed
         case SoKeyboardEvent::HOME:
         case SoKeyboardEvent::LEFT_ARROW:
         case SoKeyboardEvent::UP_ARROW:
@@ -1207,24 +1346,8 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
         }
     }
 
-    static bool MoveMode=false;
-    static bool MoveModeMoved=false;
-    static bool ZoomMode=false;
-    static bool RotMode =false;
-    static bool dCliBut3=false;
-
-    const SbViewportRegion & vp = viewer->getViewportRegion();
-    const SbVec2s size(vp.getViewportSizePixels());
-    const SbVec2f prevnormalized = lastmouseposition;
-    const SbVec2s pos(ev->getPosition());
-    const SbVec2f posn((float) pos[0] / (float) SoQtMax((int)(size[0] - 1), 1),
-                       (float) pos[1] / (float) SoQtMax((int)(size[1] - 1), 1));
-
-    lastmouseposition = posn;
-
-    // switching the mouse modes
-    if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
-
+    // Mouse Button / Spaceball Button handling
+    if (type.isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
         const SoMouseButtonEvent * const event = (const SoMouseButtonEvent *) ev;
         const int button = event->getButton();
         const SbBool press = event->getState() == SoButtonEvent::DOWN ? TRUE : FALSE;
@@ -1232,125 +1355,108 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
         // SoDebugError::postInfo("processSoEvent", "button = %d", button);
         switch (button) {
         case SoMouseButtonEvent::BUTTON1:
-            if (press) {
-                _bRejectSelection = false;
-                if (MoveMode) {
-                    RotMode = true;
-                    ZoomMode = false;
-                    MoveTime = ev->getTime();
-
-                    // Set up initial projection point for the projector object when
-                    // first starting a drag operation.
-                    spinprojector->project(lastmouseposition);
-                    //interactiveCountInc();
-                    clearLog();
-
-                    viewer->setComponentCursor(SoQtCursor::getRotateCursor());
-                    processed = TRUE;
-                }
-            }
-            else {
-                // if you come out of rotation dont deselect anything
-                if (_bRejectSelection || MoveMode) {
-                    _bRejectSelection=false;
-                    processed = TRUE;
-                }
-                if (MoveMode) {
-                    RotMode = false; 
-
-                    SbTime tmp = (ev->getTime() - MoveTime);
-                    float dci = (float)QApplication::doubleClickInterval()/1000.0f;
-                    if (tmp.getValue() < dci/*0.300*/) {
-                        ZoomMode = true;
-                        viewer->setComponentCursor(SoQtCursor::getZoomCursor());
-                    }
-                    else {
-                        ZoomMode = false;
-                        viewer->setComponentCursor(SoQtCursor::getPanCursor());
-
-                        float ratio = vp.getViewportAspectRatio();
-                        SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
-                        panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-       
-                        // check on start spining
-                        SbTime stoptime = (ev->getTime() - log.time[0]);
-                        if (this->spinanimatingallowed && stoptime.getValue() < 0.100) {
-                            const SbVec2s glsize(vp.getViewportSizePixels());
-                            SbVec3f from = spinprojector->project(SbVec2f(float(log.position[2][0]) / float(SoQtMax(glsize[0]-1, 1)),
-                                                                          float(log.position[2][1]) / float(SoQtMax(glsize[1]-1, 1))));
-                            SbVec3f to = spinprojector->project(posn);
-                            SbRotation rot = spinprojector->getRotation(from, to);
-
-                            SbTime delta = (log.time[0] - log.time[2]);
-                            double deltatime = delta.getValue();
-                            rot.invert();
-                            rot.scaleAngle(float(0.200 / deltatime));
-
-                            SbVec3f axis;
-                            float radians;
-                            rot.getValue(axis, radians);
-                            float dci = (float)QApplication::doubleClickInterval()/1000.0f;
-                            if ((radians > 0.01f) && (deltatime < dci/*0.300*/)) {
-                                _bSpining = true;
-                                spinRotation = rot;
-                                MoveMode = false;
-                                // restore the previous cursor
-                                viewer->getWidget()->setCursor(_oldCursor /*QCursor( Qt::ArrowCursor )*/);
-                            }
-                        }
-                    }
-                    processed = TRUE;
-                }
-            }
-            break;
-        case SoMouseButtonEvent::BUTTON2:
-            break;
-        case SoMouseButtonEvent::BUTTON3:
-            //if (isEditing()) // in edit mode do not do interactions 
-            //    break;
-            if (press) {
-                centerTime = ev->getTime();
-                MoveMode = true;
-                MoveModeMoved=false;
-                _bSpining = false;
-                dCliBut3 = false;
+            this->button1down = press;
+#if 0 // disable to avoid interferences where this key combination is used, too
+            if (press && ev->wasShiftDown() &&
+                (this->currentmode != NavigationStyle::SELECTION)) {
+                this->centerTime = ev->getTime();
                 float ratio = vp.getViewportAspectRatio();
                 SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
-                panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
-                // save the current cursor before overriding
-                _oldCursor = viewer->getWidget()->cursor();
-                viewer->setComponentCursor(SoQtCursor::getPanCursor());
+                this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
+                this->panningmove = FALSE;
             }
-            else {
-                SbTime tmp = (ev->getTime() - centerTime);
+            else if (!press && ev->wasShiftDown() &&
+                (this->currentmode != NavigationStyle::SELECTION)) {
+                SbTime tmp = (ev->getTime() - this->centerTime);
                 float dci = (float)QApplication::doubleClickInterval()/1000.0f;
-                // is it just a middle click?
-                if (tmp.getValue() < dci/*0.300*/ && !MoveModeMoved) {
+                // is it just a left click?
+                if (tmp.getValue() < dci && !this->panningmove) {
                     if (!this->seekToPoint(pos)) {
                         panToCenter(panningplane, posn);
                         this->interactiveCountDec();
                     }
+                    processed = TRUE;
                 }
-
-                MoveMode = false;
-                RotMode = false;
-                ZoomMode = false;
-                // restore the previous cursor
-                viewer->getWidget()->setCursor( _oldCursor /*QCursor( Qt::ArrowCursor )*/);
-                _bRejectSelection = true;
             }
-            processed = TRUE;
+            else
+#endif
+            if (press && (this->currentmode == NavigationStyle::SEEK_WAIT_MODE)) {
+                newmode = NavigationStyle::SEEK_MODE;
+                this->seekToPoint(pos); // implicitly calls interactiveCountInc()
+                processed = TRUE;
+            }
+            //else if (press && (this->currentmode == NavigationStyle::IDLE)) {
+            //    this->setViewing(true);
+            //    processed = TRUE;
+            //}
+            else if (press && (this->currentmode == NavigationStyle::PANNING ||
+                               this->currentmode == NavigationStyle::ZOOMING)) {
+                newmode = NavigationStyle::DRAGGING;
+                this->centerTime = ev->getTime();
+                processed = TRUE;
+            }
+            else if (!press && (this->currentmode == NavigationStyle::DRAGGING)) {
+                SbTime tmp = (ev->getTime() - this->centerTime);
+                float dci = (float)QApplication::doubleClickInterval()/1000.0f;
+                if (tmp.getValue() < dci) {
+                    newmode = NavigationStyle::ZOOMING;
+                }
+                processed = TRUE;
+            }
+            else if (!press && (this->currentmode == NavigationStyle::DRAGGING)) {
+                this->setViewing(false);
+                processed = TRUE;
+            }
+            else if (viewer->isEditing() && (this->currentmode == NavigationStyle::SPINNING)) {
+                processed = TRUE;
+            }
+            break;
+        case SoMouseButtonEvent::BUTTON2:
+            // If we are in edit mode then simply ignore the RMB events
+            // to pass the event to the base class.
+            if (!viewer->isEditing()) {
+                // If we are in zoom or pan mode ignore RMB events otherwise
+                // the canvas doesn't get any release events 
+                if (this->currentmode != NavigationStyle::ZOOMING && 
+                    this->currentmode != NavigationStyle::PANNING &&
+                    this->currentmode != NavigationStyle::DRAGGING) {
+                    if (this->isPopupMenuEnabled()) {
+                        if (!press) { // release right mouse button
+                            this->openPopupMenu(event->getPosition());
+                        }
+                    }
+                }
+            }
+            this->button2down = press;
+            break;
+        case SoMouseButtonEvent::BUTTON3:
+            if (press) {
+                this->centerTime = ev->getTime();
+                float ratio = vp.getViewportAspectRatio();
+                SbViewVolume vv = viewer->getCamera()->getViewVolume(ratio);
+                this->panningplane = vv.getPlane(viewer->getCamera()->focalDistance.getValue());
+                this->panningmove = FALSE;
+            }
+            else {
+                SbTime tmp = (ev->getTime() - this->centerTime);
+                float dci = (float)QApplication::doubleClickInterval()/1000.0f;
+                // is it just a middle click?
+                if (tmp.getValue() < dci && !this->panningmove) {
+                    if (!this->seekToPoint(pos)) {
+                        panToCenter(panningplane, posn);
+                        this->interactiveCountDec();
+                    }
+                    processed = TRUE;
+                }
+            }
+            this->button3down = press;
             break;
         case SoMouseButtonEvent::BUTTON4:
-            if (press) 
-                zoom(viewer->getCamera(), 0.05f);
-
+            zoom(viewer->getCamera(), 0.05f);
             processed = TRUE;
             break;
         case SoMouseButtonEvent::BUTTON5:
-            if (press) 
-                zoom(viewer->getCamera(), -0.05f);
-
+            zoom(viewer->getCamera(), -0.05f);
             processed = TRUE;
             break;
         default:
@@ -1359,32 +1465,27 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
     }
 
     // Mouse Movement handling
-    if (ev->getTypeId().isDerivedFrom(SoLocation2Event::getClassTypeId())) {
-//      const SoLocation2Event * const event = (const SoLocation2Event *) ev;
-
-        if (MoveMode && ZoomMode) {
+    if (type.isDerivedFrom(SoLocation2Event::getClassTypeId())) {
+        const SoLocation2Event * const event = (const SoLocation2Event *) ev;
+        if (this->currentmode == NavigationStyle::ZOOMING) {
             this->zoomByCursor(posn, prevnormalized);
             processed = TRUE;
         }
-        else if(MoveMode && RotMode) {
-            addToLog(ev->getPosition(), ev->getTime());
-            spin(posn);
-            processed = TRUE;
-        }
-        else if(MoveMode) {
+        else if (this->currentmode == NavigationStyle::PANNING) {
+            this->panningmove = TRUE;
             float ratio = vp.getViewportAspectRatio();
-            panCamera(viewer->getCamera(),ratio,panningplane, posn, prevnormalized);
-            MoveModeMoved=true;
+            panCamera(viewer->getCamera(), ratio, this->panningplane, posn, prevnormalized);
             processed = TRUE;
-
         }
-        else if(_bSpining) {
+        else if (this->currentmode == NavigationStyle::DRAGGING) {
+            this->addToLog(event->getPosition(), event->getTime());
+            this->spin(posn);
             processed = TRUE;
         }
     }
 
     // Spaceball & Joystick handling
-    if (ev->getTypeId().isDerivedFrom(SoMotion3Event::getClassTypeId())) {
+    if (type.isDerivedFrom(SoMotion3Event::getClassTypeId())) {
         SoMotion3Event * const event = (SoMotion3Event *) ev;
         SoCamera * const camera = viewer->getCamera();
         if (camera) {
@@ -1397,73 +1498,110 @@ SbBool CADNavigationStyle::processSoEvent(const SoEvent * const ev)
         }
     }
 
-    // give the viewprovider the chance to handle the event
-    if (!processed && !MoveMode && !RotMode) {
-        if (pcMouseModel) {
-            int hd=pcMouseModel->handleEvent(ev,viewer->getViewportRegion());
-            if (hd==AbstractMouseModel::Continue||hd==AbstractMouseModel::Restart) {
-                processed = TRUE;
-            }
-            else if (hd==AbstractMouseModel::Finish) {
-                pcPolygon = pcMouseModel->getPolygon();
-                clipInner = pcMouseModel->isInner();
-                delete pcMouseModel; pcMouseModel = 0;
-            }
-            else if (hd==AbstractMouseModel::Cancel) {
-                pcPolygon.clear();
-                delete pcMouseModel; pcMouseModel = 0;
+    enum {
+        BUTTON1DOWN = 1 << 0,
+        BUTTON3DOWN = 1 << 1,
+        CTRLDOWN =    1 << 2,
+        SHIFTDOWN =   1 << 3,
+        BUTTON2DOWN = 1 << 4
+    };
+    unsigned int combo =
+        (this->button1down ? BUTTON1DOWN : 0) |
+        (this->button2down ? BUTTON2DOWN : 0) |
+        (this->button3down ? BUTTON3DOWN : 0) |
+        (this->ctrldown ? CTRLDOWN : 0) |
+        (this->shiftdown ? SHIFTDOWN : 0);
+
+    switch (combo) {
+    case 0:
+        if (curmode == NavigationStyle::SPINNING) { break; }
+        newmode = NavigationStyle::IDLE;
+        // The left mouse button has been released right now but
+        // we want to avoid that the event is procesed elsewhere
+        if (this->lockButton1) {
+            this->lockButton1 = FALSE;
+            processed = TRUE;
+        }
+
+        //if (curmode == NavigationStyle::DRAGGING) {
+        //    if (doSpin())
+        //        newmode = NavigationStyle::SPINNING;
+        //}
+        break;
+    case BUTTON1DOWN:
+        // make sure not to change the selection when stopping spinning
+        if (curmode == NavigationStyle::SPINNING || this->lockButton1)
+            newmode = NavigationStyle::IDLE;
+        else
+            newmode = NavigationStyle::SELECTION;
+        break;
+    case BUTTON3DOWN:
+        if (curmode == NavigationStyle::SPINNING) { break; }
+        else if (newmode == NavigationStyle::ZOOMING) { break; }
+        newmode = NavigationStyle::PANNING;
+
+        if (curmode == NavigationStyle::DRAGGING) {
+            if (doSpin()) {
+                newmode = NavigationStyle::SPINNING;
+                break;
             }
         }
+        break;
+    case CTRLDOWN|BUTTON2DOWN:
+        newmode = NavigationStyle::PANNING;
+        break;
+    case SHIFTDOWN|BUTTON2DOWN:
+        newmode = NavigationStyle::DRAGGING;
+        break;
+    case CTRLDOWN|SHIFTDOWN|BUTTON2DOWN:
+        newmode = NavigationStyle::ZOOMING;
+        break;
+    //case CTRLDOWN:
+    //case CTRLDOWN|BUTTON1DOWN:
+    //case CTRLDOWN|SHIFTDOWN:
+    //case CTRLDOWN|SHIFTDOWN|BUTTON1DOWN:
+    //    newmode = NavigationStyle::SELECTION;
+    //    break;
+    //case BUTTON1DOWN|BUTTON3DOWN:
+    //case CTRLDOWN|BUTTON3DOWN:
+    //    newmode = NavigationStyle::ZOOMING;
+    //    break;
+
+        // There are many cases we don't handle that just falls through to
+        // the default case, like SHIFTDOWN, CTRLDOWN, CTRLDOWN|SHIFTDOWN,
+        // SHIFTDOWN|BUTTON3DOWN, SHIFTDOWN|CTRLDOWN|BUTTON3DOWN, etc.
+        // This is a feature, not a bug. :-)
+        //
+        // mortene.
+
+    default:
+        // The default will make a spin stop and otherwise not do
+        // anything.
+        //if ((curmode != NavigationStyle::SEEK_WAIT_MODE) &&
+        //    (curmode != NavigationStyle::SEEK_MODE)) {
+        //    newmode = NavigationStyle::IDLE;
+        //}
+        break;
     }
 
-    // give the nodes in the foreground root the chance to handle events (e.g color bar)
-    // Note: this must be done _before_ ceding to the viewer  
-    if (!processed) {
-        processed = handleEventInForeground(ev);
+    if (newmode != curmode) {
+        this->setViewingMode(newmode);
     }
 
-    // invokes the appropriate callback function when user interaction has started or finished
-    bool bInteraction = (MoveMode||ZoomMode||RotMode|_bSpining);
-    if (bInteraction && this->getInteractiveCount()==0)
-        this->interactiveCountInc();
-    // must not be in seek mode because it gets decremented in setSeekMode(FALSE)
-    else if (!bInteraction && !dCliBut3 && this->getInteractiveCount()>0 && !this->isSeekMode())
-        this->interactiveCountDec();
+    // If for dragging the buttons 1 and 3 are pressed
+    // but then button 3 is relaesed we shouldn't switch
+    // into selection mode.
+    if (this->button1down && this->button3down)
+        this->lockButton1 = TRUE;
 
-    if (!processed)
+    // If not handled in this class, pass on upwards in the inheritance
+    // hierarchy.
+    if (/*(curmode == NavigationStyle::SELECTION || viewer->isEditing()) && */!processed)
         processed = inherited::processSoEvent(ev);
-    else 
+    else
         return TRUE;
 
-    // right mouse button pressed
-    if (!processed && !MoveMode && !RotMode) {
-        if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
-            SoMouseButtonEvent * const e = (SoMouseButtonEvent *) ev;
-            if ((e->getButton() == SoMouseButtonEvent::BUTTON2) && e->getState() == SoButtonEvent::UP) {
-                if (this->isPopupMenuEnabled()) {
-                    if (e->getState() == SoButtonEvent::UP) {
-                        this->openPopupMenu(e->getPosition());
-                    }
-
-                    // Steal all RMB-events if the viewer uses the popup-menu.
-                    return TRUE;
-                }
-            }
-        }
-    }
-
-    // check for left click without selecting something
-    if (!processed) {
-        if (ev->getTypeId().isDerivedFrom(SoMouseButtonEvent::getClassTypeId())) {
-            SoMouseButtonEvent * const e = (SoMouseButtonEvent *) ev;
-            if (SoMouseButtonEvent::isButtonReleaseEvent(e,SoMouseButtonEvent::BUTTON1)) {
-                //Base::Console().Log("unhandled left button click!");
-                Gui::Selection().clearSelection();
-            }
-        }
-    }
-
-    return FALSE;
+    return processed;
 }
 
 #undef PRIVATE
