@@ -24,9 +24,14 @@
 #include "PreCompiled.h"
 #ifndef _PreComp_
 # include <QClipboard>
+# include <QDockWidget>
+# include <QGridLayout>
+# include <QHBoxLayout>
 # include <QKeyEvent>
 # include <QMenu>
 # include <QMessageBox>
+# include <QPushButton>
+# include <QSpacerItem>
 # include <QTextCursor>
 # include <QTextDocumentFragment>
 # include <QTextStream>
@@ -684,13 +689,31 @@ void PythonConsole::printStatement( const QString& cmd )
 /**
  * Shows the Python window and sets the focus to set text cursor.
  */
-void PythonConsole::showEvent ( QShowEvent * e )
+void PythonConsole::showEvent (QShowEvent * e)
 {
-    TextEdit::showEvent( e );
+    TextEdit::showEvent(e);
     // set also the text cursor to the edit field
     setFocus();
 }
 
+void PythonConsole::visibilityChanged (bool visible)
+{
+    if (visible)
+        setFocus();
+}
+
+void PythonConsole::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::ParentChange) {
+        QDockWidget* dw = qobject_cast<QDockWidget*>(this->parentWidget());
+        if (dw) {
+            connect(dw, SIGNAL(visibilityChanged(bool)),
+                    this, SLOT(visibilityChanged(bool)));
+        }
+    }
+    TextEdit::changeEvent(e);
+}
+
 /**
  * Drops the event \a e and writes the right Python command.
  */
@@ -983,7 +1006,7 @@ void PythonConsole::contextMenuEvent ( QContextMenuEvent * e )
 void PythonConsole::onSaveHistoryAs()
 {
     QString cMacroPath = QString::fromUtf8(getDefaultParameter()->GetGroup( "Macro" )->
-        GetASCII("MacroPath",App::GetApplication().GetHomePath()).c_str());
+        GetASCII("MacroPath",App::Application::getUserAppDataDir().c_str()).c_str());
     QString fn = FileDialog::getSaveFileName(this, tr("Save History"), cMacroPath,
         tr("Macro Files (*.FCMacro *.py)"));
     if (!fn.isEmpty()) {
@@ -1150,4 +1173,69 @@ const QStringList& ConsoleHistory::values() const
     return this->_history;
 }
 
+// -----------------------------------------------------
+
+PythonInputField::PythonInputField(QWidget* parent)
+  : QWidget(parent)
+{
+    QGridLayout* gridLayout = new QGridLayout(this);
+    gridLayout->setSpacing(6);
+    gridLayout->setMargin(9);
+
+    editField = new PythonEditor(this);
+    gridLayout->addWidget(editField, 0, 0, 1, 1);
+
+    QHBoxLayout* hboxLayout = new QHBoxLayout();
+    hboxLayout->setSpacing(6);
+    hboxLayout->setMargin(0);
+
+    QSpacerItem* spacerItem = new QSpacerItem(131, 31, QSizePolicy::Expanding, QSizePolicy::Minimum);
+    hboxLayout->addItem(spacerItem);
+
+    okButton = new QPushButton(this);
+    hboxLayout->addWidget(okButton);
+    clearButton = new QPushButton(this);
+    hboxLayout->addWidget(clearButton);
+    gridLayout->addLayout(hboxLayout, 1, 0, 1, 1);
+
+
+    this->setWindowTitle(Gui::PythonConsole::tr("Python Input Dialog"));
+    okButton->setText(tr("OK"));
+    clearButton->setText(tr("Clear"));
+
+    QObject::connect(okButton, SIGNAL(clicked()), this, SIGNAL(textEntered()));
+    QObject::connect(clearButton, SIGNAL(clicked()), editField, SLOT(clear()));
+}
+
+PythonInputField::~PythonInputField()
+{
+}
+
+QString PythonInputField::getText() const
+{
+    return editField->toPlainText();
+}
+
+void PythonInputField::clear()
+{
+    return editField->clear();
+}
+
+void PythonInputField::changeEvent(QEvent *e)
+{
+    if (e->type() == QEvent::LanguageChange) {
+        this->setWindowTitle(Gui::PythonConsole::tr("Python Input Dialog"));
+        okButton->setText(tr("OK"));
+        clearButton->setText(tr("Clear"));
+    }
+    else {
+        QWidget::changeEvent(e);
+    }
+}
+
+void PythonInputField::showEvent(QShowEvent* e)
+{
+    editField->setFocus();
+}
+
 #include "moc_PythonConsole.cpp"
