@@ -28,6 +28,8 @@
 # include <string>
 # include <boost/signals.hpp>
 # include <boost/bind.hpp>
+# include <QString>
+# include <QStatusBar>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
@@ -40,6 +42,8 @@
 #include <App/Document.h>
 #include <App/DocumentObject.h>
 #include <App/DocumentObjectPy.h>
+#include "MainWindow.h"
+
 
 
 using namespace Gui;
@@ -261,6 +265,11 @@ void SelectionObserverPython::removePreselection(const SelectionChanges& msg)
 
 // -------------------------------------------
 
+bool SelectionSingleton::hasSelection() const
+{
+    return !_SelList.empty();
+}
+
 std::vector<SelectionSingleton::SelObj> SelectionSingleton::getCompleteSelection() const
 {
     std::vector<SelObj> temp;
@@ -285,8 +294,6 @@ std::vector<SelectionSingleton::SelObj> SelectionSingleton::getSelection(const c
     SelObj tempSelObj;
 
     App::Document *pcDoc;
-    string DocName;
-
     pcDoc = getDocument(pDocName);
 
     if (!pcDoc)
@@ -310,12 +317,47 @@ std::vector<SelectionSingleton::SelObj> SelectionSingleton::getSelection(const c
     return temp;
 }
 
-std::vector<SelectionObject> SelectionSingleton::getSelectionEx(const char* pDocName) const
+bool SelectionSingleton::hasSelection(const char* doc) const
+{
+    App::Document *pcDoc;
+    pcDoc = getDocument(doc);
+    if (!pcDoc)
+        return false;
+    for(std::list<_SelObj>::const_iterator It = _SelList.begin();It != _SelList.end();++It) {
+        if (It->pDoc == pcDoc) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+//std::vector<SelectionObject> SelectionSingleton::getSelectionEx(const char* pDocName) const
+//{
+//    return getSelectionEx(pDocName,App::DocumentObject::getClassTypeId());
+//}
+//
+//std::vector<SelectionObject> SelectionSingleton::getSelectionEx(const char* pDocName,const char* typeName) const
+//{
+//    // search the type
+//    Base::Type typeId;
+//    if(typeName)
+//        typeId = Base::Type::fromName(typeName);
+//    else
+//        typeId = App::DocumentObject::getClassTypeId();
+//    return getSelectionEx(pDocName,typeId);
+//}
+std::vector<SelectionObject> SelectionSingleton::getSelectionEx(const char* pDocName,Base::Type typeId) const
 {
     std::vector<SelectionObject> temp;
     
     std::map<App::DocumentObject*,SelectionObject> SortMap;
 
+    // check the type
+    if (typeId == Base::Type::badType()) 
+        return temp;
+
+
     App::Document *pcDoc;
     string DocName;
 
@@ -326,21 +368,24 @@ std::vector<SelectionObject> SelectionSingleton::getSelectionEx(const char* pDoc
 
     for (std::list<_SelObj>::const_iterator It = _SelList.begin();It != _SelList.end();++It) {
         if (It->pDoc == pcDoc) {
-            // if the object has already an entry
-            if (SortMap.find(It->pObject) != SortMap.end()){
-                // only add sub-element
-                SortMap[It->pObject].SubNames.push_back(It->SubName);
-                SortMap[It->pObject].SelPoses.push_back(Base::Vector3d(It->x,It->y,It->z));
-            }
-            else{
-                // create a new entry
-                SelectionObject tempSelObj;
-                tempSelObj.DocName  = It->DocName;
-                tempSelObj.FeatName = It->FeatName;
-                tempSelObj.SubNames.push_back(It->SubName);
-                tempSelObj.TypeName = It->TypeName.c_str();
-                tempSelObj.SelPoses.push_back(Base::Vector3d(It->x,It->y,It->z));
-                SortMap.insert(std::pair<App::DocumentObject*,SelectionObject>(It->pObject,tempSelObj));
+            // right type?
+            if(It->pObject->getTypeId().isDerivedFrom(typeId)){
+                // if the object has already an entry
+                if (SortMap.find(It->pObject) != SortMap.end()){
+                    // only add sub-element
+                    SortMap[It->pObject].SubNames.push_back(It->SubName);
+                    SortMap[It->pObject].SelPoses.push_back(Base::Vector3d(It->x,It->y,It->z));
+                }
+                else{
+                    // create a new entry
+                    SelectionObject tempSelObj;
+                    tempSelObj.DocName  = It->DocName;
+                    tempSelObj.FeatName = It->FeatName;
+                    tempSelObj.SubNames.push_back(It->SubName);
+                    tempSelObj.TypeName = It->TypeName.c_str();
+                    tempSelObj.SelPoses.push_back(Base::Vector3d(It->x,It->y,It->z));
+                    SortMap.insert(std::pair<App::DocumentObject*,SelectionObject>(It->pObject,tempSelObj));
+                }
             }
         }
     }
@@ -406,6 +451,8 @@ unsigned int SelectionSingleton::countObjectsOfType(const char* typeName, const
 
 bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectName, const char* pSubName, float x, float y, float z)
 {
+    static char buf[513];
+
     if (DocName != "")
         rmvPreselect();
 
@@ -416,8 +463,8 @@ bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectN
     hy = y;
     hz = z;
 
+    // set up the change object
     SelectionChanges Chng;
-
     Chng.pDocName  = DocName.c_str();
     Chng.pObjectName = FeatName.c_str();
     Chng.pSubName  = SubName.c_str();
@@ -426,6 +473,17 @@ bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectN
     Chng.z = z;
     Chng.Type = SelectionChanges::SetPreselect;
 
+    // set the current preselection
+    CurrentPreselection = Chng;
+
+    snprintf(buf,512,"Preselected: %s.%s.%s (%f,%f,%f)",Chng.pDocName
+                                                       ,Chng.pObjectName
+                                                       ,Chng.pSubName
+                                                       ,x,y,z);
+
+    if (getMainWindow())
+        getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
+
 
     Notify(Chng);
     signalSelectionChanged(Chng);
@@ -436,6 +494,23 @@ bool SelectionSingleton::setPreselect(const char* pDocName, const char* pObjectN
     return true;
 }
 
+void SelectionSingleton::setPreselectCoord( float x, float y, float z)
+{
+    static char buf[513];
+
+    CurrentPreselection.x = x;
+    CurrentPreselection.y = y;
+    CurrentPreselection.z = z;
+
+    snprintf(buf,512,"Preselected: %s.%s.%s (%f,%f,%f)",CurrentPreselection.pDocName
+                                                       ,CurrentPreselection.pObjectName
+                                                       ,CurrentPreselection.pSubName
+                                                       ,x,y,z);
+
+    if (getMainWindow())
+        getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
+}
+
 void SelectionSingleton::rmvPreselect()
 {
     if (DocName == "")
@@ -447,6 +522,15 @@ void SelectionSingleton::rmvPreselect()
     Chng.pSubName  = SubName.c_str();
     Chng.Type = SelectionChanges::RmvPreselect;
 
+    // reset the current preselection
+    CurrentPreselection.pDocName =0;
+    CurrentPreselection.pObjectName = 0;
+    CurrentPreselection.pSubName = 0;
+    CurrentPreselection.x = 0.0;
+    CurrentPreselection.y = 0.0;
+    CurrentPreselection.z = 0.0;
+
+    // notify observing objects
     Notify(Chng);
     signalSelectionChanged(Chng);
 
@@ -460,6 +544,11 @@ void SelectionSingleton::rmvPreselect()
     //Base::Console().Log("Sel : Rmv preselect \n");
 }
 
+const SelectionChanges &SelectionSingleton::getPreselection(void) const
+{
+    return CurrentPreselection;
+}
+
 App::Document* SelectionSingleton::getDocument(const char* pDocName) const
 {
     if (pDocName)
@@ -565,17 +654,28 @@ void SelectionSingleton::setSelection(const char* pDocName, const std::vector<Ap
     if (!pcDoc)
         return;
 
+    std::set<App::DocumentObject*> cur_sel, new_sel;
+    new_sel.insert(sel.begin(), sel.end());
+
+    // Make sure to keep the order of the currently selected objects
     std::list<_SelObj> temp;
-    std::vector<App::DocumentObject*> prev_sel;
     for (std::list<_SelObj>::const_iterator it = _SelList.begin(); it != _SelList.end(); ++it) {
         if (it->pDoc != pcDoc)
             temp.push_back(*it);
-        else
-            prev_sel.push_back(it->pObject);
+        else {
+            cur_sel.insert(it->pObject);
+            if (new_sel.find(it->pObject) != new_sel.end())
+                temp.push_back(*it);
+        }
     }
 
+    // Get the objects we must add to the selection
+    std::vector<App::DocumentObject*> diff_new_cur;
+    std::back_insert_iterator< std::vector<App::DocumentObject*> > biit(diff_new_cur);
+    std::set_difference(new_sel.begin(), new_sel.end(), cur_sel.begin(), cur_sel.end(), biit);
+
     _SelObj obj;
-    for (std::vector<App::DocumentObject*>::const_iterator it = sel.begin(); it != sel.end(); ++it) {
+    for (std::vector<App::DocumentObject*>::const_iterator it = diff_new_cur.begin(); it != diff_new_cur.end(); ++it) {
         obj.pDoc = pcDoc;
         obj.pObject = *it;
         obj.DocName = pDocName;
@@ -588,10 +688,7 @@ void SelectionSingleton::setSelection(const char* pDocName, const std::vector<Ap
         temp.push_back(obj);
     }
 
-    std::vector<App::DocumentObject*> curr_sel = sel;
-    std::sort(curr_sel.begin(), curr_sel.end());
-    std::sort(prev_sel.begin(), prev_sel.end());
-    if (curr_sel == prev_sel) // nothing has changed
+    if (cur_sel == new_sel) // nothing has changed
         return;
 
     _SelList = temp;
@@ -670,13 +767,20 @@ bool SelectionSingleton::isSelected(const char* pDocName, const char* pObjectNam
     return false;
 }
 
-bool SelectionSingleton::isSelected(App::DocumentObject* obj) const
+bool SelectionSingleton::isSelected(App::DocumentObject* obj, const char* pSubName) const
 {
     if (!obj) return false;
 
     for(list<_SelObj>::const_iterator It = _SelList.begin();It != _SelList.end();++It) {
-        if (It->pObject == obj)
-            return true;
+        if (It->pObject == obj) {
+            if (pSubName) {
+                if (It->SubName == pSubName)
+                    return true;
+            }
+            else {
+                return true;
+            }
+        }
     }
 
     return false;
@@ -788,7 +892,9 @@ PyMethodDef SelectionSingleton::Methods[] = {
 PyObject *SelectionSingleton::sAddSelection(PyObject * /*self*/, PyObject *args, PyObject * /*kwd*/)
 {
     PyObject *object;
-    if (!PyArg_ParseTuple(args, "O!", &(App::DocumentObjectPy::Type), &object))     // convert args: Python->C 
+    char* subname=0;
+    float x=0,y=0,z=0;
+    if (!PyArg_ParseTuple(args, "O!|sfff", &(App::DocumentObjectPy::Type),&object,&subname,&x,&y,&z))
         return NULL;                             // NULL triggers exception 
 
     App::DocumentObjectPy* docObjPy = static_cast<App::DocumentObjectPy*>(object);
@@ -798,7 +904,9 @@ PyObject *SelectionSingleton::sAddSelection(PyObject * /*self*/, PyObject *args,
         return NULL;
     }
 
-    Selection().addSelection(docObj->getDocument()->getName(), docObj->getNameInDocument());
+    Selection().addSelection(docObj->getDocument()->getName(),
+                             docObj->getNameInDocument(),
+                             subname,x,y,z);
 
     Py_Return;
 }
@@ -806,7 +914,8 @@ PyObject *SelectionSingleton::sAddSelection(PyObject * /*self*/, PyObject *args,
 PyObject *SelectionSingleton::sRemoveSelection(PyObject * /*self*/, PyObject *args, PyObject * /*kwd*/)
 {
     PyObject *object;
-    if (!PyArg_ParseTuple(args, "O!", &(App::DocumentObjectPy::Type), &object))     // convert args: Python->C 
+    char* subname=0;
+    if (!PyArg_ParseTuple(args, "O!|s", &(App::DocumentObjectPy::Type),&object,&subname))
         return NULL;                             // NULL triggers exception 
 
     App::DocumentObjectPy* docObjPy = static_cast<App::DocumentObjectPy*>(object);
@@ -816,7 +925,9 @@ PyObject *SelectionSingleton::sRemoveSelection(PyObject * /*self*/, PyObject *ar
         return NULL;
     }
 
-    Selection().rmvSelection(docObj->getDocument()->getName(), docObj->getNameInDocument());
+    Selection().rmvSelection(docObj->getDocument()->getName(),
+                             docObj->getNameInDocument(),
+                             subname);
 
     Py_Return;
 }
@@ -833,11 +944,12 @@ PyObject *SelectionSingleton::sClearSelection(PyObject * /*self*/, PyObject *arg
 PyObject *SelectionSingleton::sIsSelected(PyObject * /*self*/, PyObject *args, PyObject * /*kwd*/)
 {
     PyObject *object;
-    if (!PyArg_ParseTuple(args, "O!", &(App::DocumentObjectPy::Type), &object))     // convert args: Python->C 
+    char* subname=0;
+    if (!PyArg_ParseTuple(args, "O!|s", &(App::DocumentObjectPy::Type), &object, &subname))
         return NULL;                             // NULL triggers exception 
 
     App::DocumentObjectPy* docObj = static_cast<App::DocumentObjectPy*>(object);
-    bool ok = Selection().isSelected(docObj->getDocumentObjectPtr());
+    bool ok = Selection().isSelected(docObj->getDocumentObjectPtr(), subname);
     return Py_BuildValue("O", (ok ? Py_True : Py_False));
 }
 
