@@ -30,6 +30,10 @@
 //#include <Inventor/nodes/SoSubNodeP.h>
 #include <Inventor/elements/SoOverrideElement.h>
 #include <Inventor/elements/SoLazyElement.h>
+#include <Inventor/elements/SoCacheElement.h>
+#include <Inventor/elements/SoOverrideElement.h>
+#include <Inventor/elements/SoWindowElement.h>
+
 #include <Inventor/SoFullPath.h>
 #include <Inventor/actions/SoGLRenderAction.h>
 #include <Inventor/actions/SoHandleEventAction.h>
@@ -40,13 +44,17 @@
 #include <Inventor/events/SoLocation2Event.h>
 #include <Inventor/SoPickedPoint.h>
 
+#include "View3DInventor.h"
+#include "View3DInventorViewer.h"
+
 #include <Base/Console.h>
 #include "SoFCSelection.h"
 #include "MainWindow.h"
 #include "Selection.h"
 #include "SoFCSelectionAction.h"
+#include "SoFCInteractiveElement.h"
 
-#define NO_FRONTBUFFER
+//#define NO_FRONTBUFFER
 
 using namespace Gui;
 
@@ -96,6 +104,8 @@ SoFCSelection::SoFCSelection()
   highlighted = FALSE;
   bShift      = false;
   bCtrl       = false;
+
+  selected = NOTSELECTED;
 }
 
 /*!
@@ -117,7 +127,7 @@ SoFCSelection::~SoFCSelection()
 void
 SoFCSelection::initClass(void)
 {
-  SO_NODE_INIT_CLASS(SoFCSelection,SoSeparator,"Separator");
+  SO_NODE_INIT_CLASS(SoFCSelection,SoGroup,"Separator");
 }
 
 void SoFCSelection::finish()
@@ -169,7 +179,10 @@ void SoFCSelection::doAction(SoAction *action)
         else {
             this->selectionMode = SoFCSelection::SEL_OFF;
             //this->style = SoFCSelection::BOX;
-            this->selected = NOTSELECTED;
+            if (selected.getValue() == SELECTED) {
+                this->selected = NOTSELECTED;
+            }
+
         }
     }
 
@@ -193,24 +206,41 @@ void SoFCSelection::doAction(SoAction *action)
                 (subElementName.getValue() == selaction->SelChange.pSubName || 
                 *(selaction->SelChange.pSubName) == '\0') ) {
                 if (selaction->SelChange.Type == SelectionChanges::AddSelection) {
-                    selected = SELECTED;
+                    if(selected.getValue() == NOTSELECTED){
+                        selected = SELECTED;
+                    }
                 }
                 else {
-                    selected = NOTSELECTED;
+                    if(selected.getValue() == SELECTED){
+                        selected = NOTSELECTED;
+                    }
                 }
+                return;
             }
         }
         else if (selaction->SelChange.Type == SelectionChanges::ClrSelection) {
             if (documentName.getValue() == selaction->SelChange.pDocName ||
-                strcmp(selaction->SelChange.pDocName,"") == 0)
-                selected = NOTSELECTED;
+                strcmp(selaction->SelChange.pDocName,"") == 0){
+                if(selected.getValue() == SELECTED){
+                    selected = NOTSELECTED;
+                }
+               
+            }
         }
         else if (selaction->SelChange.Type == SelectionChanges::SetSelection) {
             bool sel = Selection().isSelected(
                     documentName.getValue().getString(),
                     objectName.getValue().getString()/*,
                     subElementName.getValue().getString()*/);
-            selected = (sel ? SELECTED : NOTSELECTED);
+            if(sel){
+                   if(selected.getValue() == NOTSELECTED){
+                        selected = SELECTED;
+                   }
+            }else{
+                if(selected.getValue() == SELECTED){
+                     selected = NOTSELECTED;
+                }
+            }
         }
     }
 
@@ -222,10 +252,10 @@ void SoFCSelection::doAction(SoAction *action)
 void
 SoFCSelection::handleEvent(SoHandleEventAction * action)
 {
-#ifdef NO_FRONTBUFFER
     static char buf[513];
     HighlightModes mymode = (HighlightModes) this->highlightMode.getValue();
     const SoEvent * event = action->getEvent();
+#ifdef NO_FRONTBUFFER
   
     // mouse move events for preselection
     if (event->isOfType(SoLocation2Event::getClassTypeId())) {
@@ -402,56 +432,156 @@ SoFCSelection::handleEvent(SoHandleEventAction * action)
     // NOTE: we still have to pick for ON even though we don't have
     // to re-render, because the app needs to be notified as the mouse
     // goes over locate highlight nodes.
-    if ( highlightMode.getValue() == OFF ) {
-	SoSeparator::handleEvent( action );
-	return;
-    }
-
-    // get event from the action
-    const SoEvent *event = action->getEvent();
+    //if ( highlightMode.getValue() == OFF ) {
+	   // SoGroup::handleEvent( action );
+	   // return;
+    //}
 
+    
     //
     // If this is a mouseMotion event, then check for locate highlighting
     //
     if (event->isOfType(SoLocation2Event::getClassTypeId())) {
 
-	// check to see if the mouse is over our geometry...
-	SbBool underTheMouse = FALSE;
-	const SoPickedPoint *pp = action->getPickedPoint();
-	SoFullPath *pPath = (pp != NULL) ? (SoFullPath *) pp->getPath() : NULL;
-	if (pPath && pPath->containsPath(action->getCurPath())) {
-	    // Make sure I'm the lowest LocHL in the pick path!
-	    underTheMouse = TRUE;
-	    for (int i = 0; i < pPath->getLength(); i++) {
-		SoNode *node = pPath->getNodeFromTail(i);
-		if (node->isOfType(SoFCSelection::getClassTypeId())) {
-		    if (node != this)
-			underTheMouse = FALSE;
-		    break; // found the lowest LocHL - look no further
-		}
+	    // check to see if the mouse is over our geometry...
+	    SbBool underTheMouse = FALSE;
+	    const SoPickedPoint *pp = action->getPickedPoint();
+	    SoFullPath *pPath = (pp != NULL) ? (SoFullPath *) pp->getPath() : NULL;
+	    if (pPath && pPath->containsPath(action->getCurPath())) {
+	        // Make sure I'm the lowest LocHL in the pick path!
+	        underTheMouse = TRUE;
+	        for (int i = 0; i < pPath->getLength(); i++) {
+		        SoNode *node = pPath->getNodeFromTail(i);
+		        if (node->isOfType(SoFCSelection::getClassTypeId())) {
+		            if (node != this)
+			        underTheMouse = FALSE;
+		            break; // found the lowest LocHL - look no further
+		        }
+	        }
+ 
+        }
+        	// Am I currently highlighted?
+	    if (isHighlighted(action)) {
+            if ( ! underTheMouse ){
+		    // re-draw the object with it's normal color
+		        redrawHighlighted(action, FALSE);
+            }else{
+		        action->setHandled();
+                const SoPickedPoint * pp = action->getPickedPoint();
+                Gui::Selection().setPreselectCoord(pp->getPoint()[0]
+                                                 ,pp->getPoint()[1]
+                                                 ,pp->getPoint()[2]);
+            }
 	    }
-	}
+	    // Else I am not currently highlighted
+	    else {
+	        // If under the mouse, then highlight!
+            if (underTheMouse ){
+		    // draw this object highlighted
+		        redrawHighlighted(action, TRUE);
+                const SoPickedPoint * pp = action->getPickedPoint();
+                Gui::Selection().setPreselect(documentName.getValue().getString()
+                                                 ,objectName.getValue().getString()
+                                                 ,subElementName.getValue().getString()
+                                                 ,pp->getPoint()[0]
+                                                 ,pp->getPoint()[1]
+                                                 ,pp->getPoint()[2]);
+            }
+
+	    }
+        //if(selected == SELECTED){
+        //    redrawHighlighted(action, TRUE);
+        //}
+        //if(selectionCleared ){
+        //    redrawHighlighted(action, FALSE);
+        //    selectionCleared = false;
+        //}
+
+            
 
-	// Am I currently highlighted?
-	if (isHighlighted(action)) {
-	    if ( ! underTheMouse)
-		// re-draw the object with it's normal color
-		redrawHighlighted(action, FALSE);
-	    else
-		action->setHandled();
-	}
-	// Else I am not currently highlighted
-	else {
-	    // If under the mouse, then highlight!
-	    if (underTheMouse)
-		// draw this object highlighted
-		redrawHighlighted(action, TRUE);
-	}
     }
+    // key press events
+    else if (event->isOfType(SoKeyboardEvent ::getClassTypeId())) {
+        SoKeyboardEvent  * const e = (SoKeyboardEvent  *) event;
+        if (SoKeyboardEvent::isKeyPressEvent(e,SoKeyboardEvent::LEFT_SHIFT)     ||
+            SoKeyboardEvent::isKeyPressEvent(e,SoKeyboardEvent::RIGHT_SHIFT)     )
+            bShift = true;
+        if (SoKeyboardEvent::isKeyReleaseEvent(e,SoKeyboardEvent::LEFT_SHIFT)   ||
+            SoKeyboardEvent::isKeyReleaseEvent(e,SoKeyboardEvent::RIGHT_SHIFT)   )
+            bShift = false;
+        if (SoKeyboardEvent::isKeyPressEvent(e,SoKeyboardEvent::LEFT_CONTROL)   ||
+            SoKeyboardEvent::isKeyPressEvent(e,SoKeyboardEvent::RIGHT_CONTROL)   )
+            bCtrl = true;
+        if (SoKeyboardEvent::isKeyReleaseEvent(e,SoKeyboardEvent::LEFT_CONTROL) ||
+            SoKeyboardEvent::isKeyReleaseEvent(e,SoKeyboardEvent::RIGHT_CONTROL) )
+            bCtrl = false;
+    }
+    // mouse press events for (de)selection (only if selection is enabled on this node)
+    else if (event->isOfType(SoMouseButtonEvent::getClassTypeId()) && 
+             selectionMode.getValue() == SoFCSelection::SEL_ON) {
+        SoMouseButtonEvent * const e = (SoMouseButtonEvent *) event;
+        if (SoMouseButtonEvent::isButtonReleaseEvent(e,SoMouseButtonEvent::BUTTON1)) {
+            //FIXME: Shouldn't we remove the preselection for newly selected objects?
+            //       Otherwise the tree signals that an object is preselected even though it is hidden. (Werner)
+            const SoPickedPoint * pp = action->getPickedPoint();
+            if (pp && pp->getPath()->containsPath(action->getCurPath())) {
+                if (bCtrl) {
+                    if (Gui::Selection().isSelected(documentName.getValue().getString()
+                                         ,objectName.getValue().getString()
+                                         ,subElementName.getValue().getString())) {
+                        Gui::Selection().rmvSelection(documentName.getValue().getString()
+                                          ,objectName.getValue().getString()
+                                          ,subElementName.getValue().getString());
+                    } else {
+                        Gui::Selection().addSelection(documentName.getValue().getString()
+                                          ,objectName.getValue().getString()
+                                          ,subElementName.getValue().getString()
+                                          ,pp->getPoint()[0]
+                                          ,pp->getPoint()[1]
+                                          ,pp->getPoint()[2]);
+
+                        if (mymode == OFF) {
+                            snprintf(buf,512,"Selected: %s.%s.%s (%f,%f,%f)",documentName.getValue().getString()
+                                                       ,objectName.getValue().getString()
+                                                       ,subElementName.getValue().getString()
+                                                       ,pp->getPoint()[0]
+                                                       ,pp->getPoint()[1]
+                                                       ,pp->getPoint()[2]);
+
+                            getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
+                        }
+                    }
+                } else { // Ctrl
+                    Gui::Selection().clearSelection(documentName.getValue().getString());
+                    Gui::Selection().addSelection(documentName.getValue().getString()
+                                          ,objectName.getValue().getString()
+                                          ,subElementName.getValue().getString()
+                                          ,pp->getPoint()[0]
+                                          ,pp->getPoint()[1]
+                                          ,pp->getPoint()[2]);
+ 
+                    if (mymode == OFF) {
+                        snprintf(buf,512,"Selected: %s.%s.%s (%f,%f,%f)",documentName.getValue().getString()
+                                                   ,objectName.getValue().getString()
+                                                   ,subElementName.getValue().getString()
+                                                   ,pp->getPoint()[0]
+                                                   ,pp->getPoint()[1]
+                                                   ,pp->getPoint()[2]);
+
+                        getMainWindow()->statusBar()->showMessage(QString::fromAscii(buf),3000);
+                    }
+                }
+
+                action->setHandled(); 
+            } // picked point
+        } // mouse release
+    }
+
+    
 
     // Let the base class traverse the children.
     if ( action->getGrabber() != this )
-	SoSeparator::handleEvent(action);
+	    SoGroup::handleEvent(action);
 #endif
 }
 
@@ -476,7 +606,7 @@ SoFCSelection::GLRenderBelowPath(SoGLRenderAction * action)
     SbBool drawHighlighted = preRender(action, oldDepthFunc);
 
     // now invoke the parent method
-    SoSeparator::GLRenderBelowPath(action);
+    SoGroup::GLRenderBelowPath(action);
 
     // Restore old depth buffer model if needed
     if (drawHighlighted || highlighted)
@@ -488,10 +618,16 @@ SoFCSelection::GLRenderBelowPath(SoGLRenderAction * action)
 #endif
 }
 
+void SoFCSelection::GLRender(SoGLRenderAction * action)
+{
+    SoGroup::GLRender(action);
+}
+
 // doc from parent
 void
 SoFCSelection::GLRenderInPath(SoGLRenderAction * action)
 {
+    //Base::Console().Log("SoFCSelection::GLRenderInPath() (%p)\n",this);
 #ifdef NO_FRONTBUFFER
   // check if preselection is active
   HighlightModes mymode = (HighlightModes) this->highlightMode.getValue();
@@ -509,7 +645,7 @@ SoFCSelection::GLRenderInPath(SoGLRenderAction * action)
     SbBool drawHighlighted = preRender(action, oldDepthFunc);
 
     // now invoke the parent method
-    SoSeparator::GLRenderInPath(action);
+    SoGroup::GLRenderInPath(action);
 
     // Restore old depth buffer model if needed
     if (drawHighlighted || highlighted)
@@ -520,45 +656,46 @@ SoFCSelection::GLRenderInPath(SoGLRenderAction * action)
 	action->getState()->pop();
 #endif
 }
-#include <Inventor/elements/SoCacheElement.h>
-#include <Inventor/elements/SoLazyElement.h>
-#include <Inventor/elements/SoOverrideElement.h>
-#include <Inventor/elements/SoWindowElement.h>
 
 SbBool
 SoFCSelection::preRender(SoGLRenderAction *action, GLint &oldDepthFunc)
 //
 ////////////////////////////////////////////////////////////////////////
 {
+    //Base::Console().Log("SoFCSelection::preRender() (%p)\n",this);
+
     // If not performing locate highlighting, just return.
     if (highlightMode.getValue() == OFF)
-	return FALSE;
+	    return FALSE;
 
     SoState *state = action->getState();
 
     // ??? prevent caching at this level - for some reason the
     // ??? SoWindowElement::copyMatchInfo() method get called, which should
     // ??? never be called. We are not caching this node correctly yet....
-    SoCacheElement::invalidate(state);
+    //SoCacheElement::invalidate(state);
 
-    SbBool drawHighlighted = (highlightMode.getValue() == ON || isHighlighted(action));
+    SbBool drawHighlighted = (highlightMode.getValue() == ON || isHighlighted(action) || selected.getValue() == SELECTED);
 
     if (drawHighlighted) {
 
-	// prevent diffuse & emissive color from leaking out...
-	state->push();
+	    // prevent diffuse & emissive color from leaking out...
+	    state->push();
+        SbColor col;
+        if(selected.getValue() == SELECTED)
+	        col = colorSelection.getValue();
+        else
+	        col = colorHighlight.getValue();
 
-	SbColor col = colorHighlight.getValue();
+	    // Emissive Color
+	    SoLazyElement::setEmissive(state, &col);
+	    SoOverrideElement::setEmissiveColorOverride(state, this, TRUE);
 
-	// Emissive Color
-	SoOverrideElement::setEmissiveColorOverride(state, this, TRUE);
-	SoLazyElement::setEmissive(state, &col);
-
-	// Diffuse Color
-	if (style.getValue() == EMISSIVE_DIFFUSE) {
-	    SoOverrideElement::setDiffuseColorOverride(state, this, TRUE);
-	    SoLazyElement::setDiffuse(state, this, 1, &col, &colorpacker);
-	}
+	    // Diffuse Color
+	    if (style.getValue() == EMISSIVE_DIFFUSE) {
+	        SoLazyElement::setDiffuse(state, this, 1, &col, &colorpacker);
+	        SoOverrideElement::setDiffuseColorOverride(state, this, TRUE);
+	    }
     }
 
     // Draw on top of other things at same z-buffer depth if:
@@ -567,9 +704,9 @@ SoFCSelection::preRender(SoGLRenderAction *action, GLint &oldDepthFunc)
     //     non-hilit to lit OR VICE VERSA.
     // Otherwise, leave it alone...
     if (drawHighlighted || highlighted) {
-	glGetIntegerv(GL_DEPTH_FUNC, &oldDepthFunc);
-	if (oldDepthFunc != GL_LEQUAL)
-	    glDepthFunc(GL_LEQUAL);
+	    glGetIntegerv(GL_DEPTH_FUNC, &oldDepthFunc);
+	    if (oldDepthFunc != GL_LEQUAL)
+	        glDepthFunc(GL_LEQUAL);
     }
 
     return drawHighlighted;
@@ -579,12 +716,11 @@ SoFCSelection::preRender(SoGLRenderAction *action, GLint &oldDepthFunc)
   Empty method in Coin. Can be used by subclasses to be told
   when status change.
 */
-#include "MainWindow.h"
-#include "View3DInventor.h"
-#include "View3DInventorViewer.h"
 void
 SoFCSelection::redrawHighlighted(SoAction *  action , SbBool  doHighlight )
 {
+    //Base::Console().Log("SoFCSelection::redrawHighlighted() (%p) doHigh=%d \n",this,doHighlight?1:0);
+
 #ifdef NO_FRONTBUFFER
 #else
     // If we are about to highlight, and there is something else highlighted,
@@ -592,14 +728,14 @@ SoFCSelection::redrawHighlighted(SoAction *  action , SbBool  doHighlight )
     if (doHighlight && currenthighlight != NULL &&
         !(*((SoFullPath *)action->getCurPath()) == *currenthighlight)) {
 
-	SoNode *tail = currenthighlight->getTail();
-	if (tail->isOfType( SoFCSelection::getClassTypeId()))
-	    ((SoFCSelection *)tail)->redrawHighlighted(action, FALSE);
-	else {
-	    // Just get rid of the path. It's no longer valid for redraw.
-	    currenthighlight->unref();
-	    currenthighlight = NULL;
-	}
+	    SoNode *tail = currenthighlight->getTail();
+	    if (tail->isOfType( SoFCSelection::getClassTypeId()))
+	        ((SoFCSelection *)tail)->redrawHighlighted(action, FALSE);
+	    else {
+	        // Just get rid of the path. It's no longer valid for redraw.
+	        currenthighlight->unref();
+	        currenthighlight = NULL;
+	    }
     }
 
     SoPath *pathToRender;
@@ -617,37 +753,39 @@ SoFCSelection::redrawHighlighted(SoAction *  action , SbBool  doHighlight )
     }
     // delete our path if we are no longer highlighted
     else {
+        if(currenthighlight){
+	        // We will be rendering this old path to unhighlight it
+	        pathToRender = currenthighlight;
+	        pathToRender->ref();
 
-	// We will be rendering this old path to unhighlight it
-	pathToRender = currenthighlight;
-	pathToRender->ref();
-
-	currenthighlight->unref();
-	currenthighlight = NULL;
+	        currenthighlight->unref();
+	        currenthighlight = NULL;
+        }
     }
 
     // If highlighting is forced on for this node, we don't need this special render.
     if (highlightMode.getValue() != AUTO) {
-	pathToRender->unref();
-	return;
-    }
+	    pathToRender->unref();
+	    return;
+        }
 
     SoState *state = action->getState();
 
-    void* window;
-    void* context;
-    void *display;
+    //void* window;
+    //void* context;
+    //void *display;
+    QGLWidget* window;
     SoGLRenderAction *glAction;
-    SoWindowElement::get(state, window, context, display, glAction);
+    //SoWindowElement::get(state, window, context, display, glAction);
+    SoGLWidgetElement::get(state, window);
+    SoGLRenderActionElement::get(state, glAction);
 
     // If we don't have a current window, then simply return...
-    //if (window == 0 || context == NULL || display == NULL || glAction == NULL)
-	//return;
+    if (window == 0 /*|| context == NULL || display == NULL*/ || glAction == NULL)
+        return;
 
-    glAction = static_cast<View3DInventor*>(getMainWindow()->activeWindow())
-        ->getViewer()->getGLRenderAction();
-
-#ifndef WIN32  // FIXME !!!!
+    window->makeCurrent();
+#ifndef WIN32
     // set the current window
     //glXMakeCurrent(display, window, context);
 #endif
@@ -655,7 +793,7 @@ SoFCSelection::redrawHighlighted(SoAction *  action , SbBool  doHighlight )
     GLint whichBuffer;
     glGetIntegerv(GL_DRAW_BUFFER, &whichBuffer);
     if (whichBuffer != GL_FRONT)
-	glDrawBuffer(GL_FRONT);
+        glDrawBuffer(GL_FRONT);
 
     highlighted = TRUE;
     glAction->apply(pathToRender);
@@ -663,7 +801,7 @@ SoFCSelection::redrawHighlighted(SoAction *  action , SbBool  doHighlight )
 
     // restore the buffering type
     if (whichBuffer != GL_FRONT)
-	glDrawBuffer((GLenum)whichBuffer);
+        glDrawBuffer((GLenum)whichBuffer);
     glFlush();
 
     pathToRender->unref();
@@ -683,6 +821,7 @@ SoFCSelection::readInstance  (  SoInput *  in, unsigned short  flags )
 void
 SoFCSelection::setOverride(SoGLRenderAction * action)
 {
+  //Base::Console().Log("SoFCSelection::setOverride() (%p)\n",this);
   SoState * state = action->getState();
   if(this->selected.getValue() == SELECTED)
     SoLazyElement::setEmissive(state, &this->colorSelection.getValue());
@@ -704,6 +843,8 @@ SoFCSelection::setOverride(SoGLRenderAction * action)
 void
 SoFCSelection::turnoffcurrent(SoAction * action)
 {
+//    Base::Console().Log("SoFCSelection::turnoffcurrent() (%p)\n",this);
+
 #ifdef NO_FRONTBUFFER
   if (SoFCSelection::currenthighlight &&
       SoFCSelection::currenthighlight->getLength()) {
