@@ -40,7 +40,7 @@
 
 #include <Base/Console.h>
 #include <App/Document.h>
-#include <App/Feature.h>
+#include <App/DocumentObject.h>
 #include <App/DocumentObjectGroup.h>
 
 #include "Tree.h"
@@ -665,14 +665,13 @@ DocumentItem::DocumentItem(const Gui::Document* doc, QTreeWidgetItem * parent)
     : QTreeWidgetItem(parent, TreeWidget::DocumentType), pDocument(doc)
 {
     // Setup connections
-    Gui::Document* pdoc = const_cast<Gui::Document*>(doc);
-    pdoc->signalNewObject.connect(boost::bind(&DocumentItem::slotNewObject, this, _1));
-    pdoc->signalDeletedObject.connect(boost::bind(&DocumentItem::slotDeleteObject, this, _1));
-    pdoc->signalChangedObject.connect(boost::bind(&DocumentItem::slotChangeObject, this, _1));
-    pdoc->signalRenamedObject.connect(boost::bind(&DocumentItem::slotRenameObject, this, _1));
-    pdoc->signalActivatedObject.connect(boost::bind(&DocumentItem::slotActiveObject, this, _1));
-    pdoc->signalInEdit.connect(boost::bind(&DocumentItem::slotInEdit, this, _1));
-    pdoc->signalResetEdit.connect(boost::bind(&DocumentItem::slotResetEdit, this, _1));
+    doc->signalNewObject.connect(boost::bind(&DocumentItem::slotNewObject, this, _1));
+    doc->signalDeletedObject.connect(boost::bind(&DocumentItem::slotDeleteObject, this, _1));
+    doc->signalChangedObject.connect(boost::bind(&DocumentItem::slotChangeObject, this, _1));
+    doc->signalRenamedObject.connect(boost::bind(&DocumentItem::slotRenameObject, this, _1));
+    doc->signalActivatedObject.connect(boost::bind(&DocumentItem::slotActiveObject, this, _1));
+    doc->signalInEdit.connect(boost::bind(&DocumentItem::slotInEdit, this, _1));
+    doc->signalResetEdit.connect(boost::bind(&DocumentItem::slotResetEdit, this, _1));
 
     setFlags(Qt::ItemIsEnabled/*|Qt::ItemIsEditable*/);
 }
@@ -1080,12 +1079,11 @@ void DocumentObjectItem::displayStatusInfo()
 
 void DocumentObjectItem::setData (int column, int role, const QVariant & value)
 {
+    QTreeWidgetItem::setData(column, role, value);
     if (role == Qt::EditRole) {
         QString label = value.toString();
         viewObject->getObject()->Label.setValue((const char*)label.toUtf8());
     }
-
-    QTreeWidgetItem::setData(column, role, value);
 }
 
 void DocumentObjectItem::slotChangeIcon()
