@@ -31,6 +31,10 @@
 # include <QEvent>
 # include <QDropEvent>
 # include <QDragEnterEvent>
+# include <QFileDialog>
+# include <QPainter>
+# include <QPrinter>
+# include <QPrintDialog>
 # include <QTimer>
 # include <QUrl>
 # include <Inventor/actions/SoWriteAction.h>
@@ -47,6 +51,9 @@
 #endif
 
 #include <Base/Exception.h>
+#include <Base/Console.h>
+
+#include <App/DocumentObject.h>
 
 #include "View3DInventor.h"
 #include "View3DInventorViewer.h"
@@ -55,8 +62,7 @@
 #include "Application.h"
 #include "MainWindow.h"
 #include "MenuManager.h"
-#include <App/Feature.h>
-#include <Base/Console.h>
+#include "WaitCursor.h"
 
 // build in Inventor
 #include "Inventor/Qt/viewers/SoQtExaminerViewer.h"
@@ -88,17 +94,33 @@ View3DInventor::View3DInventor(Gui::Document* pcDocument, QWidget* parent, Qt::W
     hGrp->Attach(this);
 
     // create the inventor widget and set the defaults
+#if !defined (NO_USE_QT_MDI_AREA)
+    _viewer = new View3DInventorViewer(0);
+    setCentralWidget(_viewer->getWidget());
+#else
     _viewer = new View3DInventorViewer(this);
-    setViewerDefaults();
+#endif
     // apply the user settings
-    hGrp->Notify("HeadlightColor");
-    hGrp->Notify("HeadlightDirection");
-    hGrp->Notify("HeadlightIntensity");
-    hGrp->Notify("EnableBacklight");
-    hGrp->Notify("BacklightColor");
-    hGrp->Notify("BacklightDirection");
-    hGrp->Notify("BacklightIntensity");
-    hGrp->Notify("NavigationStyle");
+    OnChange(*hGrp,"EyeDistance");
+    OnChange(*hGrp,"CornerCoordSystem");
+    OnChange(*hGrp,"UseAutoRotation");
+    OnChange(*hGrp,"Gradient");
+    OnChange(*hGrp,"BackgroundColor");
+    OnChange(*hGrp,"BackgroundColor2");
+    OnChange(*hGrp,"BackgroundColor3");
+    OnChange(*hGrp,"BackgroundColor4");
+    OnChange(*hGrp,"UseBackgroundColorMid");
+    OnChange(*hGrp,"UseAntialiasing");
+    OnChange(*hGrp,"ShowFPS");
+    OnChange(*hGrp,"Orthographic");
+    OnChange(*hGrp,"HeadlightColor");
+    OnChange(*hGrp,"HeadlightDirection");
+    OnChange(*hGrp,"HeadlightIntensity");
+    OnChange(*hGrp,"EnableBacklight");
+    OnChange(*hGrp,"BacklightColor");
+    OnChange(*hGrp,"BacklightDirection");
+    OnChange(*hGrp,"BacklightIntensity");
+    OnChange(*hGrp,"NavigationStyle");
 
     stopSpinTimer = new QTimer(this);
     connect(stopSpinTimer, SIGNAL(timeout()), this, SLOT(stopAnimating()));
@@ -143,40 +165,6 @@ PyObject *View3DInventor::getPyObject(void)
     return _viewerPy;
 }
 
-void View3DInventor::setViewerDefaults(void)
-{
-    _viewer->setStereoOffset(hGrp->GetFloat("EyeDistance",65.0));
-    _viewer->setFeedbackVisibility(hGrp->GetBool("CornerCoordSystem",true));
-    _viewer->setAnimationEnabled(hGrp->GetBool("UseAutoRotation",true));
-    _viewer->setGradientBackgroud((hGrp->GetBool("Gradient",true)));
-    unsigned long col1 = hGrp->GetUnsigned("BackgroundColor",0);
-    unsigned long col2 = hGrp->GetUnsigned("BackgroundColor2",2139082239); // default color (purple)
-    unsigned long col3 = hGrp->GetUnsigned("BackgroundColor3",ULONG_MAX); // default color (white)
-    unsigned long col4 = hGrp->GetUnsigned("BackgroundColor4",ULONG_MAX); // default color (white)
-    float r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4;
-    r1 = ((col1 >> 24) & 0xff) / 255.0; g1 = ((col1 >> 16) & 0xff) / 255.0; b1 = ((col1 >> 8) & 0xff) / 255.0;
-    r2 = ((col2 >> 24) & 0xff) / 255.0; g2 = ((col2 >> 16) & 0xff) / 255.0; b2 = ((col2 >> 8) & 0xff) / 255.0;
-    r3 = ((col3 >> 24) & 0xff) / 255.0; g3 = ((col3 >> 16) & 0xff) / 255.0; b3 = ((col3 >> 8) & 0xff) / 255.0;
-    r4 = ((col4 >> 24) & 0xff) / 255.0; g4 = ((col4 >> 16) & 0xff) / 255.0; b4 = ((col4 >> 8) & 0xff) / 255.0;
-    _viewer->setBackgroundColor(SbColor(r1, g1, b1));
-    if (hGrp->GetBool("UseBackgroundColorMid",false) == false)
-        _viewer->setGradientBackgroudColor(SbColor(r2, g2, b2), SbColor(r3, g3, b3));
-    else
-        _viewer->setGradientBackgroudColor(SbColor(r2, g2, b2), SbColor(r3, g3, b3), SbColor(r4, g4, b4));
-
-    if (hGrp->GetBool("UseAntialiasing"  ,false))
-        _viewer->getGLRenderAction()->setSmoothing(true);
-    else
-        _viewer->getGLRenderAction()->setSmoothing(false);
-    _viewer->setEnabledFPSCounter(hGrp->GetBool("ShowFPS",false));
-
-    // check whether a perspective or orthogrphic camera should be set
-    if (hGrp->GetBool("Orthographic", true))
-        _viewer->setCameraType(SoOrthographicCamera::getClassTypeId());
-    else
-        _viewer->setCameraType(SoPerspectiveCamera::getClassTypeId());
-}
-
 void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::MessageType Reason)
 {
     const ParameterGrp& rGrp = static_cast<ParameterGrp&>(rCaller);
@@ -263,8 +251,46 @@ void View3DInventor::OnChange(ParameterGrp::SubjectType &rCaller,ParameterGrp::M
         Base::Type type = Base::Type::fromName(model.c_str());
         _viewer->setNavigationType(type);
     }
+    else if (strcmp(Reason,"EyeDistance") == 0) {
+        _viewer->setStereoOffset(rGrp.GetFloat("EyeDistance",65.0));
+    }
+    else if (strcmp(Reason,"CornerCoordSystem") == 0) {
+        _viewer->setFeedbackVisibility(rGrp.GetBool("CornerCoordSystem",true));
+    }
+    else if (strcmp(Reason,"UseAutoRotation") == 0) {
+        _viewer->setAnimationEnabled(rGrp.GetBool("UseAutoRotation",true));
+    }
+    else if (strcmp(Reason,"Gradient") == 0) {
+        _viewer->setGradientBackgroud((rGrp.GetBool("Gradient",true)));
+    }
+    else if (strcmp(Reason,"UseAntialiasing") == 0) {
+        _viewer->getGLRenderAction()->setSmoothing(rGrp.GetBool("UseAntialiasing",false));
+    }
+    else if (strcmp(Reason,"ShowFPS") == 0) {
+        _viewer->setEnabledFPSCounter(rGrp.GetBool("ShowFPS",false));
+    }
+    else if (strcmp(Reason,"Orthographic") == 0) {
+        // check whether a perspective or orthogrphic camera should be set
+        if (rGrp.GetBool("Orthographic", true))
+            _viewer->setCameraType(SoOrthographicCamera::getClassTypeId());
+        else
+            _viewer->setCameraType(SoPerspectiveCamera::getClassTypeId());
+    }
     else {
-        setViewerDefaults();
+        unsigned long col1 = rGrp.GetUnsigned("BackgroundColor",0);
+        unsigned long col2 = rGrp.GetUnsigned("BackgroundColor2",2139082239); // default color (purple)
+        unsigned long col3 = rGrp.GetUnsigned("BackgroundColor3",ULONG_MAX); // default color (white)
+        unsigned long col4 = rGrp.GetUnsigned("BackgroundColor4",ULONG_MAX); // default color (white)
+        float r1,g1,b1,r2,g2,b2,r3,g3,b3,r4,g4,b4;
+        r1 = ((col1 >> 24) & 0xff) / 255.0; g1 = ((col1 >> 16) & 0xff) / 255.0; b1 = ((col1 >> 8) & 0xff) / 255.0;
+        r2 = ((col2 >> 24) & 0xff) / 255.0; g2 = ((col2 >> 16) & 0xff) / 255.0; b2 = ((col2 >> 8) & 0xff) / 255.0;
+        r3 = ((col3 >> 24) & 0xff) / 255.0; g3 = ((col3 >> 16) & 0xff) / 255.0; b3 = ((col3 >> 8) & 0xff) / 255.0;
+        r4 = ((col4 >> 24) & 0xff) / 255.0; g4 = ((col4 >> 16) & 0xff) / 255.0; b4 = ((col4 >> 8) & 0xff) / 255.0;
+        _viewer->setBackgroundColor(SbColor(r1, g1, b1));
+        if (rGrp.GetBool("UseBackgroundColorMid",false) == false)
+            _viewer->setGradientBackgroudColor(SbColor(r2, g2, b2), SbColor(r3, g3, b3));
+        else
+            _viewer->setGradientBackgroudColor(SbColor(r2, g2, b2), SbColor(r3, g3, b3), SbColor(r4, g4, b4));
     }
 }
 
@@ -295,11 +321,42 @@ const char *View3DInventor::getName(void) const
     return "View3DInventor";
 }
 
+void View3DInventor::print()
+{
+    QPrinter printer(QPrinter::HighResolution);
+    printer.setFullPage(true);
+    QPrintDialog dlg(&printer, this);
+    if (dlg.exec() == QDialog::Accepted) {
+        print(&printer);
+    }
+}
+
+void View3DInventor::printPdf()
+{
+    QString filename = QFileDialog::getSaveFileName(this, tr("Export PDF"), QString(), tr("PDF file (*.pdf)"));
+    if (!filename.isEmpty()) {
+        QPrinter printer(QPrinter::HighResolution);
+        printer.setOutputFormat(QPrinter::PdfFormat);
+        printer.setOutputFileName(filename);
+        print(&printer);
+    }
+}
+
+void View3DInventor::print(QPrinter* printer)
+{
+    QImage img;
+    QPainter p(printer);
+    QRect rect = printer->pageRect();
+    _viewer->savePicture(rect.width(), rect.height(), View3DInventorViewer::White, img);
+    p.drawImage(0,0,img);
+    p.end();
+}
+
 // **********************************************************************************
 
 bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
 {
-    if (strcmp("ViewFit",pMsg) == 0 ) {
+    if (strcmp("ViewFit",pMsg) == 0) {
         _viewer->viewAll();
         return true;
     }
@@ -371,6 +428,7 @@ bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
     }
     else if(strcmp("GetCamera",pMsg) == 0 ) {
         SoCamera * Cam = _viewer->getCamera();
+        if (!Cam) return false;
         *ppReturn = SoFCDB::writeNodesToString(Cam).c_str();
         return true;
     }
@@ -447,59 +505,61 @@ bool View3DInventor::onMsg(const char* pMsg, const char** ppReturn)
         return false;
 }
 
-
 bool View3DInventor::onHasMsg(const char* pMsg) const
 {
-  if(strcmp("Save",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("SaveAs",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("Undo",pMsg) == 0 ){
-    return getAppDocument()->getAvailableUndos() > 0;
-  }else if(strcmp("Redo",pMsg) == 0 ){
-    return getAppDocument()->getAvailableRedos() > 0; 
-  }else if(strcmp("SetStereoRedGreen",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("SetStereoQuadBuff",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("SetStereoInterleavedRows",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("SetStereoInterleavedColumns",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("SetStereoOff",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("Example1",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("Example2",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("Example3",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewFit",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewSelection",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewBottom",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewFront",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewLeft",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewRear",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewRight",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewTop",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("ViewAxo",pMsg) == 0 ){
-    return true;
-  }else if(strcmp("GetCamera",pMsg) == 0 ){
-    return true;
-  }else if(strncmp("SetCamera",pMsg,9) == 0 ){
-    return true;
-  }else if(strncmp("Dump",pMsg,4) == 0 ){
-    return true;
-  }
-  return false;
+    if  (strcmp("Save",pMsg) == 0)
+        return true;
+    else if (strcmp("SaveAs",pMsg) == 0)
+        return true;
+    else if (strcmp("Undo",pMsg) == 0)
+        return getAppDocument()->getAvailableUndos() > 0;
+    else if (strcmp("Redo",pMsg) == 0)
+        return getAppDocument()->getAvailableRedos() > 0; 
+    else if (strcmp("Print",pMsg) == 0)
+        return true; 
+    else if (strcmp("PrintPdf",pMsg) == 0)
+        return true; 
+    else if(strcmp("SetStereoRedGreen",pMsg) == 0)
+        return true;
+    else if(strcmp("SetStereoQuadBuff",pMsg) == 0)
+        return true;
+    else if(strcmp("SetStereoInterleavedRows",pMsg) == 0)
+        return true;
+    else if(strcmp("SetStereoInterleavedColumns",pMsg) == 0)
+        return true;
+    else if(strcmp("SetStereoOff",pMsg) == 0)
+        return true;
+    else if(strcmp("Example1",pMsg) == 0)
+        return true;
+    else if(strcmp("Example2",pMsg) == 0)
+        return true;
+    else if(strcmp("Example3",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewFit",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewSelection",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewBottom",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewFront",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewLeft",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewRear",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewRight",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewTop",pMsg) == 0)
+        return true;
+    else if(strcmp("ViewAxo",pMsg) == 0)
+        return true;
+    else if(strcmp("GetCamera",pMsg) == 0)
+        return true;
+    else if(strncmp("SetCamera",pMsg,9) == 0)
+        return true;
+    else if(strncmp("Dump",pMsg,4) == 0)
+        return true;
+    return false;
 }
 
 bool View3DInventor::setCamera(const char* pCamera)
@@ -629,26 +689,7 @@ void View3DInventor::dropEvent (QDropEvent * e)
 {
     const QMimeData* data = e->mimeData();
     if (data->hasUrls()) {
-        QList<QUrl> uri = data->urls();
-        QStringList files;
-        App::Document* pDoc = getAppDocument();
-        if (pDoc) {
-            for (QList<QUrl>::ConstIterator it = uri.begin(); it != uri.end(); ++it) {
-                QFileInfo info((*it).toLocalFile());
-                if ( info.exists() && info.isFile() ) {
-                    if (info.isSymLink())
-                        info.setFile(info.readLink());
-                    files << info.absoluteFilePath();
-                }
-            }
-
-            const char *docName = pDoc->getName();
-            SelectModule::Dict dict = SelectModule::importHandler(files);
-            // load the files with the associated modules
-            for (SelectModule::Dict::iterator it = dict.begin(); it != dict.end(); ++it) {
-                Application::Instance->importFrom(it.key().toUtf8(), docName, it.value().toAscii());
-            }
-        }
+        getMainWindow()->loadUrls(getAppDocument(), data->urls());
     }
     else {
         MDIView::dropEvent(e);
@@ -659,7 +700,7 @@ void View3DInventor::dragEnterEvent (QDragEnterEvent * e)
 {
     // Here we must allow uri drafs and check them in dropEvent
     const QMimeData* data = e->mimeData();
-    if ( data->hasUrls() )
+    if (data->hasUrls())
         e->accept();
     else
         e->ignore();
@@ -668,6 +709,8 @@ void View3DInventor::dragEnterEvent (QDragEnterEvent * e)
 void View3DInventor::setCurrentViewMode(ViewMode newmode)
 {
     ViewMode oldmode = MDIView::currentViewMode();
+    if (oldmode == newmode)
+        return;
     MDIView::setCurrentViewMode(newmode);
 
     // This widget becomes the focus proxy of the embedded GL widget if we leave 
@@ -689,7 +732,8 @@ void View3DInventor::setCurrentViewMode(ViewMode newmode)
         _viewer->getGLWidget()->setFocusProxy(this);
         // To be notfified for new actions
         qApp->installEventFilter(this);
-    } else if (newmode == Child) {
+    }
+    else if (newmode == Child) {
         _viewer->getGLWidget()->setFocusProxy(0);
         qApp->removeEventFilter(this);
         QList<QAction*> acts = this->actions();
@@ -727,7 +771,8 @@ void View3DInventor::keyPressEvent (QKeyEvent* e)
         // by pressing the matching accelerator or ESC. 
         if (e->key() == Qt::Key_Escape) {
             setCurrentViewMode(Child);
-        } else {
+        }
+        else {
             // Note: The key events should be redirected directly to the GL widget and not to the main window
             // otherwise the first redirected key event always disappears in hyperspace.
             //
@@ -735,7 +780,8 @@ void View3DInventor::keyPressEvent (QKeyEvent* e)
             QWidget* w = _viewer->getGLWidget();
             QApplication::sendEvent(w,e);
         }
-    } else {
+    }
+    else {
         QMainWindow::keyPressEvent(e);
     }
 }
@@ -757,4 +803,17 @@ void View3DInventor::focusInEvent (QFocusEvent * e)
     _viewer->getGLWidget()->setFocus();
 }
 
+void View3DInventor::contextMenuEvent (QContextMenuEvent*e)
+{
+    MDIView::contextMenuEvent(e);
+}
+
+void View3DInventor::customEvent(QEvent * e)
+{
+    if (e->type() == QEvent::User) {
+        NavigationStyleEvent* se = static_cast<NavigationStyleEvent*>(e);
+        _viewer->setNavigationType(se->style());
+    }
+}
+
 #include "moc_View3DInventor.cpp"
