@@ -39,8 +39,6 @@
 # include <Inventor/actions/SoHandleEventAction.h> 
 # include <Inventor/actions/SoToVRML2Action.h>
 # include <Inventor/actions/SoWriteAction.h>
-# include <Inventor/annex/HardCopy/SoVectorizePSAction.h>
-# include <Inventor/annex/HardCopy/SoVectorOutput.h>
 # include <Inventor/manips/SoClipPlaneManip.h>
 # include <Inventor/nodes/SoBaseColor.h>
 # include <Inventor/nodes/SoCallback.h> 
@@ -75,15 +73,17 @@
 # include <Inventor/SoOffscreenRenderer.h>
 # include <Inventor/SoPickedPoint.h>
 # include <Inventor/VRMLnodes/SoVRMLGroup.h>
+# include <QEventLoop>
 # include <QMessageBox>
+# include <QTimer>
 #endif
 
 #include <sstream>
 #include <Base/Console.h>
-#include <Base/gzstream.h>
 #include <Base/Stream.h>
 #include <Base/FileInfo.h>
 #include <Base/Sequencer.h>
+#include <Base/Tools.h>
 #include <zipios++/gzipoutputstream.h>
 
 #include "View3DInventorViewer.h"
@@ -96,6 +96,7 @@
 #include "SoFCInteractiveElement.h"
 #include "Selection.h"
 #include "SoFCSelectionAction.h"
+#include "SoFCVectorizeU3DAction.h"
 #include "SoFCVectorizeSVGAction.h"
 #include "SoFCDB.h"
 #include "MainWindow.h"
@@ -200,6 +201,7 @@ View3DInventorViewer::View3DInventorViewer (QWidget *parent, const char *name,
     pEventCallback->setUserData(this);
     pEventCallback->ref();
     pcViewProviderRoot->addChild(pEventCallback);
+    pEventCallback->addEventCallback(SoEvent::getClassTypeId(), handleEventCB, this);
 
     // This is a callback node that logs all action that traverse the Inventor tree.
 #if defined (FC_DEBUG) && defined(FC_LOGGING_CB)
@@ -347,6 +349,15 @@ void View3DInventorViewer::setGLWidget(void * userdata, SoAction * action)
     }
 }
 
+void View3DInventorViewer::handleEventCB(void * ud, SoEventCallback * n)
+{
+    View3DInventorViewer* that = reinterpret_cast<View3DInventorViewer*>(ud);
+    SoGLRenderAction * glra = that->getGLRenderAction();
+    SoAction* action = n->getAction();
+    SoGLRenderActionElement::set(action->getState(), glra);
+    SoGLWidgetElement::set(action->getState(), qobject_cast<QGLWidget*>(that->getGLWidget()));
+}
+
 void View3DInventorViewer::setGradientBackgroud(bool on)
 {
     if (on && backgroundroot->findChild(pcBackGround) == -1)
@@ -402,6 +413,11 @@ void View3DInventorViewer::setNavigationType(Base::Type t)
     this->navigation->setViewer(this);
 }
 
+NavigationStyle* View3DInventorViewer::navigationStyle() const
+{
+    return this->navigation;
+}
+
 SoDirectionalLight* View3DInventorViewer::getBacklight(void) const
 {
     return this->backlight;
@@ -465,7 +481,7 @@ void View3DInventorViewer::savePicture(const char* filename, int w, int h,
             renderer.setComponents(SoFCOffscreenRenderer::RGB_TRANSPARENCY );
             break;
         default:
-            throw Base::Exception("View3DInventorViewer::makeScreenShot(): Unknown parameter");
+            break;
     }
 
     SoSeparator* root = new SoSeparator;
@@ -535,7 +551,7 @@ void View3DInventorViewer::savePicture(int w, int h, int eBackgroundType, QImage
             renderer.setComponents(SoFCOffscreenRenderer::RGB_TRANSPARENCY );
             break;
         default:
-            throw Base::Exception("View3DInventorViewer::makeScreenShot(): Unknown parameter");
+            break;
     }
 
     SoSeparator* root = new SoSeparator;
@@ -569,43 +585,22 @@ void View3DInventorViewer::savePicture(int w, int h, int eBackgroundType, QImage
     }
 }
 
-void View3DInventorViewer::saveGraphic(const char* filename, int pagesize,
-                                       int eBackgroundType) const
+void View3DInventorViewer::saveGraphic(int pagesize, int eBackgroundType, SoVectorizeAction* va) const
 {
-    SoVectorizeAction* ps=0;
-    Base::FileInfo fi(filename);
-    if (fi.hasExtension("ps") || fi.hasExtension("eps")) {
-        ps = new SoVectorizePSAction();
-        //ps.setGouraudThreshold(0.0f);
-    }
-    else if (fi.hasExtension("svg")) {
-        ps = new SoFCVectorizeSVGAction();
-    }
-    else
-        throw Base::Exception("Not supported vector graphic");
-
-    SbViewportRegion vp(getViewportRegion());
-    SoVectorOutput * out = ps->getOutput();
-    if (!out->openFile(filename)) {
-        throw Base::FileException("View3DInventorViewer::saveGraphic(): "
-                                  "Cannot open file", filename);
-    }
-
     switch(eBackgroundType){
         case Current:
-            ps->setBackgroundColor(true, this->getBackgroundColor());
+            va->setBackgroundColor(true, this->getBackgroundColor());
             break;
         case White:
-            ps->setBackgroundColor(true, SbColor(1.0, 1.0, 1.0));
+            va->setBackgroundColor(true, SbColor(1.0, 1.0, 1.0));
             break;
         case Black:
-            ps->setBackgroundColor(true, SbColor(0.0, 0.0, 0.0));
+            va->setBackgroundColor(true, SbColor(0.0, 0.0, 0.0));
             break;
         case Transparent:
             break; // not supported
         default:
-            throw Base::Exception("View3DInventorViewer::saveGraphic(): "
-                                  "Unknown parameter");
+            break;
     }
 
     float border = 10.0f;
@@ -613,17 +608,17 @@ void View3DInventorViewer::saveGraphic(const char* filename, int pagesize,
     float vpratio = ((float)vpsize[0]) / ((float)vpsize[1]); 
 
     if (vpratio > 1.0f) {
-      ps->setOrientation(SoVectorizeAction::LANDSCAPE);
+      va->setOrientation(SoVectorizeAction::LANDSCAPE);
       vpratio = 1.0f / vpratio;
     }
     else {
-      ps->setOrientation(SoVectorizeAction::PORTRAIT);
+      va->setOrientation(SoVectorizeAction::PORTRAIT);
     }
 
-    ps->beginStandardPage(SoVectorizeAction::PageSize(pagesize), border);
+    va->beginStandardPage(SoVectorizeAction::PageSize(pagesize), border);
 
     // try to fill as much "paper" as possible
-    SbVec2f size = ps->getPageSize();
+    SbVec2f size = va->getPageSize();
 
     float pageratio = size[0] / size[1];
     float xsize, ysize;
@@ -640,15 +635,13 @@ void View3DInventorViewer::saveGraphic(const char* filename, int pagesize,
     float offx = border + (size[0]-xsize) * 0.5f;
     float offy = border + (size[1]-ysize) * 0.5f;
 
-    ps->beginViewport(SbVec2f(offx, offy), SbVec2f(xsize, ysize));
-    ps->calibrate(this->getViewportRegion());
+    va->beginViewport(SbVec2f(offx, offy), SbVec2f(xsize, ysize));
+    va->calibrate(this->getViewportRegion());
     
-    ps->apply(this->getSceneManager()->getSceneGraph());
+    va->apply(this->getSceneManager()->getSceneGraph());
 
-    ps->endViewport();    
-    ps->endPage();
-    out->closeFile();
-    delete ps;
+    va->endViewport();    
+    va->endPage();
 }
 
 void View3DInventorViewer::startPicking(View3DInventorViewer::ePickMode mode)
@@ -745,6 +738,30 @@ bool View3DInventorViewer::dumpToFile(const char* filename, bool binary) const
             }
         }
     }
+    else if (fi.hasExtension("idtf") || fi.hasExtension("svg") ) {
+        int ps=4, t=2;
+        std::auto_ptr<SoVectorizeAction> vo;
+
+        if (fi.hasExtension("svg")) {
+            vo = std::auto_ptr<SoVectorizeAction>(new SoFCVectorizeSVGAction());
+        }
+        else if (fi.hasExtension("idtf")) {
+            vo = std::auto_ptr<SoVectorizeAction>(new SoFCVectorizeU3DAction());
+        }
+        else {
+            throw Base::Exception("Not supported vector graphic");
+        }
+
+        SoVectorOutput * out = vo->getOutput();
+        if (!out || !out->openFile(filename)) {
+            std::ostringstream a_out;
+            a_out << "Cannot open file '" << filename << "'";
+            throw Base::Exception(a_out.str());
+        }
+
+        saveGraphic(ps,t,vo.get());
+        out->closeFile();
+    }
     else {
         // Write Inventor in ASCII
         std::string buffer = SoFCDB::writeNodesToString(pcViewProviderRoot);
@@ -804,6 +821,9 @@ void View3DInventorViewer::actualRedraw(void)
     glClearColor(col[0], col[1], col[2], 0.0f);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
+    // using 90% of the z-buffer for the background and the main node
+    glDepthRange(0.1,1.0);
+
     // Render our scenegraph with the image.
     SoGLRenderAction * glra = this->getGLRenderAction();
     SoGLWidgetElement::set(glra->getState(), qobject_cast<QGLWidget*>(this->getGLWidget()));
@@ -824,11 +844,16 @@ void View3DInventorViewer::actualRedraw(void)
             QObject::tr("Not enough memory available to display the data."));
     }
 
+    // using 10% of the z-buffer for the foreground node
+    glDepthRange(0.0,0.1);
+
     // Render overlay front scenegraph.
-    glClear(GL_DEPTH_BUFFER_BIT);
     glra->apply(this->foregroundroot);
 
     if (this->axiscrossEnabled) { this->drawAxisCross(); }
+   
+    // using the main portion of z-buffer again (for frontbuffer highlighting)
+    glDepthRange(0.1,1.0);
 
     // draw lines for the flags
     int ct = _flaglayout->count();
@@ -946,7 +971,7 @@ SbBool View3DInventorViewer::processSoEvent(const SoEvent * const ev)
         }
     }
 
-    return navigation->processSoEvent(ev);
+    return navigation->processEvent(ev);
 }
 
 SbBool View3DInventorViewer::processSoEventBase(const SoEvent * const ev)
@@ -957,6 +982,7 @@ SbBool View3DInventorViewer::processSoEventBase(const SoEvent * const ev)
 SbVec3f View3DInventorViewer::getViewDirection() const
 {
     SoCamera* cam = this->getCamera();
+    if (!cam) return SbVec3f(0,0,-1); // this is the default
     SbRotation camrot = cam->orientation.getValue();
     SbVec3f lookat(0, 0, -1); // init to default view direction vector
     camrot.multVec(lookat, lookat);
@@ -966,6 +992,7 @@ SbVec3f View3DInventorViewer::getViewDirection() const
 SbVec3f View3DInventorViewer::getUpDirection() const
 {
     SoCamera* cam = this->getCamera();
+    if (!cam) return SbVec3f(0,1,0);
     SbRotation camrot = cam->orientation.getValue();
     SbVec3f upvec(0, 1, 0); // init to default up vector
     camrot.multVec(upvec, upvec);
@@ -994,6 +1021,7 @@ SbVec3f View3DInventorViewer::getPointOnScreen(const SbVec2s& pnt) const
     }
 
     SoCamera* pCam = this->getCamera();
+    if (!pCam) return SbVec3f(); // return invalid point
     SbViewVolume  vol = pCam->getViewVolume();
 
     float nearDist = pCam->nearDistance.getValue();
@@ -1013,6 +1041,7 @@ SbVec3f View3DInventorViewer::getPointOnScreen(const SbVec2s& pnt) const
 void View3DInventorViewer::getNearPlane(SbVec3f& rcPt, SbVec3f& rcNormal) const
 {
     SoCamera* pCam = getCamera();
+    if (!pCam) return; // just do nothing
     SbViewVolume vol = pCam->getViewVolume();
 
     // get the normal of the front clipping plane
@@ -1027,6 +1056,7 @@ void View3DInventorViewer::getNearPlane(SbVec3f& rcPt, SbVec3f& rcNormal) const
 void View3DInventorViewer::getFarPlane(SbVec3f& rcPt, SbVec3f& rcNormal) const
 {
     SoCamera* pCam = getCamera();
+    if (!pCam) return; // just do nothing
     SbViewVolume vol = pCam->getViewVolume();
 
     // get the normal of the back clipping plane
@@ -1042,6 +1072,7 @@ SbVec3f View3DInventorViewer::projectOnNearPlane(const SbVec2f& pt) const
 {
     SbVec3f pt1, pt2;
     SoCamera* cam = this->getCamera();
+    if (!cam) return SbVec3f(); // return invalid point
     SbViewVolume vol = cam->getViewVolume();
     vol.projectPointToLine(pt, pt1, pt2);
     return pt1;
@@ -1051,6 +1082,7 @@ SbVec3f View3DInventorViewer::projectOnFarPlane(const SbVec2f& pt) const
 {
     SbVec3f pt1, pt2;
     SoCamera* cam = this->getCamera();
+    if (!cam) return SbVec3f(); // return invalid point
     SbViewVolume vol = cam->getViewVolume();
     vol.projectPointToLine(pt, pt1, pt2);
     return pt2;
@@ -1143,6 +1175,35 @@ void View3DInventorViewer::setCameraOrientation(const SbRotation& rot)
     navigation->setCameraOrientation(rot);
 }
 
+void View3DInventorViewer::moveCameraTo(const SbRotation& rot, const SbVec3f& pos, int steps, int ms)
+{
+    SoCamera* cam = this->getCamera();
+    if (cam == 0) return;
+
+    SbVec3f campos = cam->position.getValue();
+    SbRotation camrot = cam->orientation.getValue();
+    //SbVec3f dir1, dir2;
+    //camrot.multVec(SbVec3f(0, 0, -1), dir1);
+    //rot.multVec(SbVec3f(0, 0, -1), dir2);
+
+    QEventLoop loop;
+    QTimer timer;
+    timer.setSingleShot(true);
+    QObject::connect(&timer, SIGNAL(timeout()), &loop, SLOT(quit()));
+    for (int i=0; i<steps; i++) {
+        float s = float(i)/float(steps);
+        SbVec3f curpos = campos * (1.0f-s) + pos * s;
+        SbRotation currot = SbRotation::slerp(camrot, rot, s);
+        cam->orientation.setValue(currot);
+        cam->position.setValue(curpos);
+        timer.start(Base::clamp<int>(ms,0,5000));
+        loop.exec(QEventLoop::ExcludeUserInputEvents);
+    }
+
+    cam->orientation.setValue(rot);
+    cam->position.setValue(pos);
+}
+
 void View3DInventorViewer::boxZoom(const SbBox2s& box)
 {
     navigation->boxZoom(box);
