@@ -35,12 +35,15 @@
 #include "Document.h"
 #include "SoFCSelection.h"
 #include "SoFCSelectionAction.h"
+#include "SoFCVectorizeSVGAction.h"
+#include "SoFCVectorizeU3DAction.h"
 #include "View3DInventor.h"
 #include "View3DInventorViewer.h"
 
 #include <Base/Console.h>
 #include <Base/Exception.h>
 #include <Base/Interpreter.h>
+#include <Base/PlacementPy.h>
 #include <Base/VectorPy.h>
 #include <Base/GeometryPyCXX.h>
 
@@ -68,6 +71,7 @@ void View3DInventorPy::init_type()
     add_varargs_method("viewRight",&View3DInventorPy::viewRight,"viewRight()");
     add_varargs_method("viewTop",&View3DInventorPy::viewTop,"viewTop()");
     add_varargs_method("viewAxometric",&View3DInventorPy::viewAxometric,"viewAxometric()");
+    add_varargs_method("viewPosition",&View3DInventorPy::viewPosition,"viewPosition()");
     add_varargs_method("startAnimating",&View3DInventorPy::startAnimating,"startAnimating()");
     add_varargs_method("stopAnimating",&View3DInventorPy::stopAnimating,"stopAnimating()");
     add_varargs_method("setAnimationEnabled",&View3DInventorPy::setAnimationEnabled,"setAnimationEnabled()");
@@ -379,6 +383,38 @@ Py::Object View3DInventorPy::viewAxometric(const Py::Tuple& args)
     return Py::None();
 }
 
+Py::Object View3DInventorPy::viewPosition(const Py::Tuple& args)
+{
+    PyObject* p=0;
+    int steps = 20;
+    int ms = 30;
+    if (!PyArg_ParseTuple(args.ptr(), "|O!ii",&Base::PlacementPy::Type,&p,&steps,&ms))
+        throw Py::Exception();
+
+    if (p) {
+        Base::Placement* plm = static_cast<Base::PlacementPy*>(p)->getPlacementPtr();
+        Base::Rotation rot = plm->getRotation();
+        Base::Vector3d pos = plm->getPosition();
+        double q0,q1,q2,q3;
+        rot.getValue(q0,q1,q2,q3);
+        _view->getViewer()->moveCameraTo(
+            SbRotation((float)q0, (float)q1, (float)q2, (float)q3),
+            SbVec3f((float)pos.x, (float)pos.y, (float)pos.z), steps, ms);
+    }
+
+    SoCamera* cam = _view->getViewer()->getCamera();
+    if (!cam) return Py::None();
+
+    SbRotation rot = cam->orientation.getValue();
+    SbVec3f pos = cam->position.getValue();
+    float q0,q1,q2,q3;
+    rot.getValue(q0,q1,q2,q3);
+    Base::Placement plm(
+        Base::Vector3d(pos[0], pos[1], pos[2]),
+        Base::Rotation(q0, q1, q2, q3));
+    return Py::Placement(plm);
+}
+
 Py::Object View3DInventorPy::startAnimating(const Py::Tuple& args)
 {
     float x,y,z;
@@ -470,19 +506,32 @@ Py::Object View3DInventorPy::saveVectorGraphic(const Py::Tuple& args)
     if (!PyArg_ParseTuple(args.ptr(), "s|ii",&filename,&ps,&t))
         throw Py::Exception();
 
-    try {
-        _view->getViewer()->saveGraphic(filename,ps,t);
-        return Py::None();
+    std::auto_ptr<SoVectorizeAction> vo;
+    Base::FileInfo fi(filename);
+    if (fi.hasExtension("ps") || fi.hasExtension("eps")) {
+        vo = std::auto_ptr<SoVectorizeAction>(new SoVectorizePSAction());
+        //vo->setGouraudThreshold(0.0f);
     }
-    catch (const Base::Exception& e) {
-        throw Py::Exception(e.what());
+    else if (fi.hasExtension("svg")) {
+        vo = std::auto_ptr<SoVectorizeAction>(new SoFCVectorizeSVGAction());
     }
-    catch (const std::exception& e) {
-        throw Py::Exception(e.what());
+    else if (fi.hasExtension("idtf")) {
+        vo = std::auto_ptr<SoVectorizeAction>(new SoFCVectorizeU3DAction());
     }
-    catch(...) {
-        throw Py::Exception("Unknown C++ exception");
+    else {
+        throw Py::Exception("Not supported vector graphic");
     }
+
+    SoVectorOutput * out = vo->getOutput();
+    if (!out || !out->openFile(filename)) {
+        std::ostringstream a_out;
+        a_out << "Cannot open file '" << filename << "'";
+        throw Py::Exception(a_out.str());
+    }
+
+    _view->getViewer()->saveGraphic(ps,t,vo.get());
+    out->closeFile();
+    return Py::None();
 }
 
 Py::Object View3DInventorPy::getCameraNode(const Py::Tuple& args)
@@ -497,7 +546,7 @@ Py::Object View3DInventorPy::getCameraNode(const Py::Tuple& args)
         type = "So"; // seems that So prefix is missing in camera node
         type += camera->getTypeId().getName().getString();
         type += " *";
-        proxy = Base::Interpreter().createSWIGPointerObj(type.c_str(), (void*)camera, 1);
+        proxy = Base::Interpreter().createSWIGPointerObj("pivy.coin", type.c_str(), (void*)camera, 1);
         camera->ref();
         return Py::Object(proxy, true);
     }
@@ -518,7 +567,8 @@ Py::Object View3DInventorPy::getCamera(const Py::Tuple& args)
     try {
         SoWriteAction wa(&out);
         SoCamera * cam = _view->getViewer()->getCamera();
-        wa.apply(cam);
+        if (cam) wa.apply(cam);
+        else buffer[0] = '\0';
         return Py::String(buffer);
     }
     catch (const Base::Exception& e) {
@@ -537,8 +587,8 @@ Py::Object View3DInventorPy::getViewDirection(const Py::Tuple& args)
     if (!PyArg_ParseTuple(args.ptr(), ""))
         throw Py::Exception();
     try {
-	SbVec3f dvec = _view->getViewer()->getViewDirection();
-	return Py::Vector(Base::Vector3f(dvec[0], dvec[1], dvec[2]));
+        SbVec3f dvec = _view->getViewer()->getViewDirection();
+        return Py::Vector(Base::Vector3f(dvec[0], dvec[1], dvec[2]));
     }
     catch (const Base::Exception& e) {
         throw Py::Exception(e.what());
@@ -1330,7 +1380,7 @@ Py::Object View3DInventorPy::getSceneGraph(const Py::Tuple& args)
     try {
         SoNode* scene = _view->getViewer()->getSceneGraph();
         PyObject* proxy = 0;
-        proxy = Base::Interpreter().createSWIGPointerObj("SoSelection *", (void*)scene, 1);
+        proxy = Base::Interpreter().createSWIGPointerObj("pivy.coin", "SoSelection *", (void*)scene, 1);
         scene->ref();
         return Py::Object(proxy, true);
     }
@@ -1350,7 +1400,7 @@ wrap_SoQtViewer_setViewDirection(PyObject *proxy, PyObject *args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoQtViewer *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_RuntimeError, e.what());
@@ -1368,7 +1418,8 @@ wrap_SoQtViewer_setViewDirection(PyObject *proxy, PyObject *args)
         if (dir.length() < 0.001f)
             throw Py::ValueError("Null vector cannot be used to set direction");
         SoCamera* cam = viewer->getCamera();
-        cam->orientation.setValue(SbRotation(SbVec3f(0, 0, -1), dir));
+        if (cam)
+            cam->orientation.setValue(SbRotation(SbVec3f(0, 0, -1), dir));
         return Py::new_reference_to(Py::None());
     }
     catch (Py::Exception&) {
@@ -1384,7 +1435,7 @@ wrap_SoQtViewer_getViewDirection(PyObject *proxy, PyObject *args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoQtViewer *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_RuntimeError, e.what());
@@ -1394,6 +1445,10 @@ wrap_SoQtViewer_getViewDirection(PyObject *proxy, PyObject *args)
     SoQtViewer* viewer = reinterpret_cast<SoQtViewer*>(ptr);
     try {
         SoCamera* cam = viewer->getCamera();
+        if (!cam) {
+            PyErr_SetString(PyExc_RuntimeError, "No camera set");
+            return 0;
+        }
         SbRotation camrot = cam->orientation.getValue();
         SbVec3f lookat(0, 0, -1); // init to default view direction vector
         camrot.multVec(lookat, lookat);
@@ -1418,7 +1473,7 @@ wrap_SoQtViewer_setFocalDistance(PyObject *proxy, PyObject *args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoQtViewer *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_RuntimeError, e.what());
@@ -1428,7 +1483,8 @@ wrap_SoQtViewer_setFocalDistance(PyObject *proxy, PyObject *args)
     SoQtViewer* viewer = reinterpret_cast<SoQtViewer*>(ptr);
     PY_TRY {
         SoCamera* cam = viewer->getCamera();
-        cam->focalDistance.setValue(distance);
+        if (cam)
+            cam->focalDistance.setValue(distance);
     } PY_CATCH;
 
     Py_Return;
@@ -1442,7 +1498,7 @@ wrap_SoQtViewer_getFocalDistance(PyObject *proxy, PyObject *args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoQtViewer *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_RuntimeError, e.what());
@@ -1450,8 +1506,10 @@ wrap_SoQtViewer_getFocalDistance(PyObject *proxy, PyObject *args)
     }
 
     SoQtViewer* viewer = reinterpret_cast<SoQtViewer*>(ptr);
+    double dist = 0;
     SoCamera* cam = viewer->getCamera();
-    return PyFloat_FromDouble(cam->focalDistance.getValue());
+    if (cam) dist = cam->focalDistance.getValue();
+    return PyFloat_FromDouble(dist);
 }
 
 static PyObject *
@@ -1463,7 +1521,7 @@ wrap_SoQtViewer_seekToPoint(PyObject *proxy, PyObject *args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoQtViewer *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         PyErr_SetString(PyExc_RuntimeError, e.what());
@@ -1528,7 +1586,7 @@ Py::Object View3DInventorPy::getViewer(const Py::Tuple& args)
         // Note: Once we have closed the viewer the Python object must not be used
         // anymore as it has a dangling pointer.
         SoQtViewer* view = _view->getViewer();
-        proxy = Base::Interpreter().createSWIGPointerObj("SoQtViewer *", (void*)view, 0);
+        proxy = Base::Interpreter().createSWIGPointerObj("pivy.gui.soqt", "SoQtViewer *", (void*)view, 0);
     }
     catch (const Base::Exception& e) {
         throw Py::Exception(e.what());
@@ -1563,7 +1621,7 @@ void View3DInventorPy::eventCallbackPivy(void * ud, SoEventCallback * n)
 
     PyObject* proxy = 0;
     try {
-        proxy = Base::Interpreter().createSWIGPointerObj(type.c_str(), (void*)e, 0);
+        proxy = Base::Interpreter().createSWIGPointerObj("pivy.coin", type.c_str(), (void*)e, 0);
         // now run the method
         Py::Object event(proxy,true);
         Py::Callable method(reinterpret_cast<PyObject*>(ud));
@@ -1596,7 +1654,7 @@ void View3DInventorPy::eventCallbackPivyEx(void * ud, SoEventCallback * n)
 
     PyObject* proxy = 0;
     try {
-        proxy = Base::Interpreter().createSWIGPointerObj(type.c_str(), (void*)n, 0);
+        proxy = Base::Interpreter().createSWIGPointerObj("pivy.coin", type.c_str(), (void*)n, 0);
         // now run the method
         Py::Object event(proxy,true);
         Py::Callable method(reinterpret_cast<PyObject*>(ud));
@@ -1632,7 +1690,7 @@ Py::Object View3DInventorPy::addEventCallbackPivy(const Py::Tuple& args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoType *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.coin", "SoType *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         throw Py::Exception(e.what());
@@ -1673,7 +1731,7 @@ Py::Object View3DInventorPy::removeEventCallbackPivy(const Py::Tuple& args)
 
     void* ptr = 0;
     try {
-        Base::Interpreter().convertSWIGPointerObj("SoType *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("pivy.coin", "SoType *", proxy, &ptr, 0);
     }
     catch (const Base::Exception& e) {
         throw Py::Exception(e.what());
