@@ -1,7 +1,7 @@
-
 #***************************************************************************
 #*                                                                         *
-#*   Copyright (c) 2009 Yorik van Havre <yorik@gmx.fr>                     *  
+#*   Copyright (c) 2009, 2010                                              *
+#*   Yorik van Havre <yorik@gmx.fr>, Ken Cline <cline@frii.com>            *  
 #*                                                                         *
 #*   This program is free software; you can redistribute it and/or modify  *
 #*   it under the terms of the GNU General Public License (GPL)            *
@@ -21,9 +21,9 @@
 #*                                                                         *
 #***************************************************************************
 
-__title__="FreeCAD Draft Workbench - Geometry li"
-__author__ = "Yorik van Havre <yorik@gmx.fr>"
-__url__ = ["http://yorik.orgfree.com","http://free-cad.sourceforge.net"]
+__title__="FreeCAD Draft Workbench - Geometry library"
+__author__ = "Yorik van Havre, Jacques-Antoine Gaudin, Ken Cline"
+__url__ = ["http://free-cad.sourceforge.net"]
 
 "this file contains generic geometry functions for manipulating Part shapes"
 
@@ -31,8 +31,9 @@ import FreeCAD, Part, fcvec, math, cmath
 from FreeCAD import Vector
 
 NORM = Vector(0,0,1) # provisory normal direction for all geometry ops.
-PREC = 4 # Precision of comparisions (decimal places/digits).
 
+params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
+precision = params.GetInt("precision")
 
 # Generic functions *********************************************************
 
@@ -70,8 +71,8 @@ def isPtOnEdge(pt,edge) :
 		center = edge.Curve.Center
 		axis   = edge.Curve.Axis ; axis.normalize()
 		radius = edge.Curve.Radius
-		if  round(pt.sub(center).dot(axis),PREC) == 0 \
-		and round(pt.sub(center).Length - radius,PREC) == 0 :
+		if  round(pt.sub(center).dot(axis),precision) == 0 \
+		and round(pt.sub(center).Length - radius,precision) == 0 :
 			if len(edge.Vertexes) == 1 :
 				return True # edge is a complete circle
 			else :
@@ -83,7 +84,7 @@ def isPtOnEdge(pt,edge) :
 					# newArc = Part.Arc(begin,pt,end)
 					# return fcvec.isNull(newArc.Center.sub(center)) \
 					#    and fcvec.isNull(newArc.Axis-axis) \
-					#    and round(newArc.Radius-radius,PREC) == 0
+					#    and round(newArc.Radius-radius,precision) == 0
 					angle1 = fcvec.angle(begin.sub(center))
 					angle2 = fcvec.angle(end.sub(center))
 					anglept = fcvec.angle(pt.sub(center))
@@ -158,7 +159,7 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 				int = [center.add(toLine).add(onLine)]
 				onLine  = Vector(dirVec) ; onLine.scale(-dOnLine,-dOnLine,-dOnLine)
 				int += [center.add(toLine).add(onLine)]
-			elif round(toLine.Length-arc.Curve.Radius,PREC) == 0 :
+			elif round(toLine.Length-arc.Curve.Radius,precision) == 0 :
 				int = [center.add(toLine)]
 			else :
 				return []
@@ -171,7 +172,7 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 				toPlane = Vector(vec1)
 				toPlane.scale(dToPlane/d,dToPlane/d,dToPlane/d)
 				ptOnPlane = toPlane.add(pt1)
-				if round(ptOnPlane.sub(center).Length - arc.Curve.Radius,PREC) == 0 :
+				if round(ptOnPlane.sub(center).Length - arc.Curve.Radius,precision) == 0 :
 					int = [ptOnPlane]
 				else :
 					return []
@@ -198,12 +199,12 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 		c2c          = cent2.sub(cent1)
 		
 		if fcvec.isNull(axis1.cross(axis2)) :
-			if round(c2c.dot(axis1),PREC) == 0 :
+			if round(c2c.dot(axis1),precision) == 0 :
 				# circles are on same plane
 				dc2c = c2c.Length ;
 				if not fcvec.isNull(c2c): c2c.normalize()
-				if round(rad1+rad2-dc2c,PREC) < 0 \
-				or round(rad1-dc2c-rad2,PREC) > 0 or round(rad2-dc2c-rad1,PREC) > 0 :
+				if round(rad1+rad2-dc2c,precision) < 0 \
+				or round(rad1-dc2c-rad2,precision) > 0 or round(rad2-dc2c-rad1,precision) > 0 :
 					return []
 				else :
 					norm = c2c.cross(axis1)
@@ -212,7 +213,7 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 					else: x = (dc2c**2 + rad1**2 - rad2**2)/(2*dc2c)
 					y = abs(rad1**2 - x**2)**(0.5)
 					c2c.scale(x,x,x)
-					if round(y,PREC) != 0 :
+					if round(y,precision) != 0 :
 						norm.scale(y,y,y)
 						int =  [cent1.add(c2c).add(norm)]
 						int += [cent1.add(c2c).sub(norm)]
@@ -233,7 +234,7 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 			intTemp = findIntersection(planeIntersectionVector,edge1,True,True)
 			int = []
 			for pt in intTemp :
-				if round(pt.sub(cent2).Length-rad2,PREC) == 0 :
+				if round(pt.sub(cent2).Length-rad2,precision) == 0 :
 					int += [pt]
 					
 		if infinite1 == False :
@@ -247,7 +248,7 @@ def findIntersection(edge1,edge2,infinite1=False,infinite2=False) :
 		
 		return int
 	else :
-		print "fcgeo: Unsupported curve type"
+		print "fcgeo: Unsupported curve type: (" + str(edge1.Curve) + ", " + str(edge2.Curve) + ")"
 
 
 def mirror (point, edge):
@@ -272,7 +273,7 @@ def findClosest(basepoint,pointslist):
 	if not pointslist: return None
 	smallest = 100000
 	for n in range(len(pointslist)):
-		new = fcvec.new(basepoint,pointslist[n]).Length
+		new = basepoint.sub(pointslist[n]).Length
 		if new < smallest:
 			smallest = new
 			npoint = n
@@ -313,7 +314,7 @@ def sortEdges(lEdges, aVertex=None):
 	"an alternative, more accurate version of Part.__sortEdges__"
 	
 	def isSameVertex(V1, V2):
-		''' Test if vertexes have same coordinates with precision 10E(-PREC)'''
+		''' Test if vertexes have same coordinates with precision 10E(-precision)'''
 		if round(V1.X-V2.X,1)==0 and round(V1.Y-V2.Y,1)==0 and round(V1.Z-V2.Z,1)==0 :
 			return True
 		else :
@@ -336,10 +337,11 @@ def sortEdges(lEdges, aVertex=None):
 	olEdges = [] # ol stands for ordered list 
 	if aVertex == None:
 		for i in range(len(lEdges)*2) :
-			result = lookfor(lEdges[i/2].Vertexes[i%2],lEdges)
-			if result[0] == 1 :  # Have we found an end ?
-				olEdges = sortEdges(lEdges, result[3].Vertexes[result[2]])
-				return olEdges
+			if len(lEdges[i/2].Vertexes) > 1:
+				result = lookfor(lEdges[i/2].Vertexes[i%2],lEdges)
+				if result[0] == 1 :  # Have we found an end ?
+					olEdges = sortEdges(lEdges, result[3].Vertexes[result[2]])
+					return olEdges
 		# if the wire is closed there is no end so choose 1st Vertex
 		return sortEdges(lEdges, lEdges[0].Vertexes[0]) 
 	else :
@@ -360,18 +362,18 @@ def findMidpoint(edge):
 		center = edge.Curve.Center
 		radius = edge.Curve.Radius
 		axis = edge.Curve.Axis
-		chord = fcvec.new(first,last)
-		perp = fcvec.crossproduct(chord,axis)
-		perp = fcvec.normalized(perp)
-		ray = fcvec.new(center,first)
-		apothem = fcvec.dotproduct(ray,perp)
+		chord = last.sub(first)
+		perp = chord.cross(axis)
+		perp.normalize()
+		ray = first.sub(center)
+		apothem = ray.dot(perp)
 		sagitta = radius - apothem
 		startpoint = Vector.add(first, fcvec.scale(chord,0.5))
-		endpoint = fcvec.scale(fcvec.normalized(perp),sagitta)
+		endpoint = fcvec.scaleTo(perp,sagitta)
 		return Vector.add(startpoint,endpoint)
 
 	elif isinstance(edge.Curve,Part.Line):
-		halfedge = fcvec.scale(fcvec.new(first,last),.5)
+		halfedge = fcvec.scale(last.sub(first),.5)
 		return Vector.add(first,halfedge)
 
 	else:
@@ -424,7 +426,6 @@ def findPerpendicular(point,edgeslist,force=None):
 				else:
 					if (dist.Length < valid[0].Length):
 						valid = [dist,edgeslist.index(edge)]
-
 		return valid
 	else:
 		edge = edgeslist[force]
@@ -446,7 +447,7 @@ def offset(edge,vector):
 		v2 = Vector.add(edge.Vertexes[-1].Point, vector)
 		return Part.Line(v1,v2).toShape()
 	else:
-		rad = fcvec.new(edge.Curve.Center,edge.Vertexes[0].Point)
+		rad = edge.Vertexes[0].Point.sub(edge.Curve.Center)
 		newrad = Vector.add(rad,vector).Length
 		return Part.Circle(edge.Curve.Center,NORM,newrad).toShape()
 
@@ -459,16 +460,17 @@ def findDistance(point,edge,strict=False):
 	if isinstance(point, FreeCAD.Vector):
 		if isinstance(edge.Curve, Part.Line):
 			segment = vec(edge)
-			chord = fcvec.new(point,edge.Vertexes[0].Point)
-			norm = fcvec.crossproduct(segment,chord)
-			perp = fcvec.crossproduct(segment,norm)
+			chord = edge.Vertexes[0].Point.sub(point)
+			norm = segment.cross(chord)
+			perp = segment.cross(norm)
 			dist = fcvec.project(chord,perp)
-			newpoint = Vector.add(point, dist)
+			if not dist: return None
+			newpoint = point.add(dist)
 			if (dist.Length == 0):
-				return 0
+				return None
 			if strict:
-				s1 = fcvec.new(edge.Vertexes[0].Point,newpoint)
-				s2 = fcvec.new(edge.Vertexes[-1].Point,newpoint)
+				s1 = newpoint.sub(edge.Vertexes[0].Point)
+				s2 = newpoint.sub(edge.Vertexes[-1].Point)
 				if (s1.Length <= segment.Length) and (s2.Length <= segment.Length):
 					return dist
 				else:
@@ -481,16 +483,16 @@ def findDistance(point,edge,strict=False):
 			else:
 				ve2 = None
 			center = edge.Curve.Center
-			segment = fcvec.new(point, center)
+			segment = center.sub(point)
 			ratio = (segment.Length - edge.Curve.Radius) / segment.Length
 			dist = fcvec.scale(segment,ratio)
 			newpoint = Vector.add(point, dist)
 			if (dist.Length == 0):
 				return None
 			if strict and ve2:
-				ang1 = fcvec.angle(fcvec.new(center, ve1))
-				ang2 = fcvec.angle(fcvec.new(center, ve2))
-				angpt = fcvec.angle(fcvec.new(center, newpoint))
+				ang1 = fcvec.angle(ve1.sub(center))
+				ang2 = fcvec.angle(ve2.sub(center))
+				angpt = fcvec.angle(newpoint.sub(center))
 				if ((angpt <= ang2 and angpt >= ang1) or (angpt <= ang1 and angpt >= ang2)):
 					return dist
 				else:
@@ -518,13 +520,13 @@ def angleBisection(edge1, edge2):
 			angleDiff = fcvec.angle(line1Dir, p4.sub(p3))
 			ang = angleDiff * 0.5
 			origin = int[0]
-			dir = fcvec.normalized(line1Dir)
-			dir = fcvec.rotate(dir, ang)
+			line1Dir.normalize()
+			dir = fcvec.rotate(line1Dir, ang)
 			return Part.Line(origin,origin.add(dir)).toShape()
 		else:
 			diff = p3.sub(p1)
 			origin = p1.add(fcvec.scale(diff, 0.5))
-			dir = fcvec.normalized(p2.sub(p1))
+			dir = p2.sub(p1); dir.normalize()
 			return Part.Line(origin,origin.add(dir)).toShape()
 	else:
 		return None
@@ -615,15 +617,16 @@ def circlefrom1Line2Points(edge, p1, p2):
 	s = s[0]
 	v1 = p1.sub(s)
 	v2 = p2.sub(s)
-	projectedDist = math.sqrt(abs(fcvec.dotproduct(v1, v2)))
-	edgeDir = fcvec.normalized(vec(edge))
+	projectedDist = math.sqrt(abs(v1.dot(v2)))
+	edgeDir = vec(edge); edgeDir.normailze()
 	projectedCen1 = Vector.add(s, fcvec.scale(edgeDir, projectedDist))
 	projectedCen2 = Vector.add(s, fcvec.scale(edgeDir, -projectedDist))
-	perpEdgeDir = fcvec.crossproduct(edgeDir)
+	perpEdgeDir = edgeDir.cross(Vector(0,0,1))
 	perpCen1 = Vector.add(projectedCen1, perpEdgeDir)
 	perpCen2 = Vector.add(projectedCen2, perpEdgeDir)
 	mid = findMidpoint(p1_p2)
-	perp_mid = Vector.add(mid, fcvec.normalized(fcvec.crossproduct(vec(p1_p2))))
+	x = fcvec.crossproduct(vec(p1_p2)); x.normalize()
+	perp_mid = Vector.add(mid, x)
 	cen1 = findIntersection(edg(projectedCen1, perpCen1), edg(mid, perp_mid), True, True)
 	cen2 = findIntersection(edg(projectedCen2, perpCen2), edg(mid, perp_mid), True, True)
 	circles = []
@@ -712,7 +715,7 @@ def circleFromPointLineRadius (point, edge, radius):
 	center2 = None
 	if dist.Length == 0:
 		segment = vec(edge)
-		perpVec = fcvec.normalized(fcvec.crossproduct(segment))
+		perpVec = fcvec.crossproduct(segment); perpVec.normalize()
 		normPoint_c1 = fcvec.scale(perpVec, radius)
 		normPoint_c2 = fcvec.scale(perpVec, -radius)
 		center1 = point.add(normPoint_c1)
@@ -732,8 +735,8 @@ def circleFromPointLineRadius (point, edge, radius):
 		normPoint = point.add(findDistance(point, edge, False))
 		normDist = fcvec.dist(normPoint, point)
 		dist = math.sqrt(radius**2 - (radius - normDist)**2)
-		centerNormVec = fcvec.scale(fcvec.normalized(point.sub(normPoint)), radius)
-		edgeDir = fcvec.normalized(edge.Vertexes[0].Point.sub(normPoint))
+		centerNormVec = fcvec.scaleTo(point.sub(normPoint), radius)
+		edgeDir = edge.Vertexes[0].Point.sub(normPoint); edgeDir.normalize()
 		center1 = centerNormVec.add(normPoint.add(fcvec.scale(edgeDir, dist)))
 		center2 = centerNormVec.add(normPoint.add(fcvec.scale(edgeDir, -dist)))
 	circles = []
@@ -762,8 +765,8 @@ def circleFrom2PointsRadius(p1, p2, radius):
 		circle = Part.Circle(mid, norm, radius)
 		if circle: return [circle]
 		else: return None
-	dir = fcvec.normalized(vec(p1_p2))
-	perpDir = fcvec.normalized(fcvec.crossproduct(dir))
+	dir = vec(p1_p2); dir.normalize()
+	perpDir = dir.cross(Vector(0,0,1)); perpDir.normailze()
 	dist = math.sqrt(radius**2 - (dist_p1p2 / 2.0)**2)
 	cen1 = Vector.add(mid, fcvec.scale(perpDir, dist))
 	cen2 = Vector.add(mid, fcvec.scale(perpDir, -dist))
@@ -1002,10 +1005,10 @@ def findHomotheticCenterOfCircles(circle1, circle2):
 			return None
 
 		cen1_cen2 = Part.Line(circle1.Curve.Center, circle2.Curve.Center).toShape()
-		cenDir = fcvec.normalized(vec(cen1_cen2))
+		cenDir = vec(cen1_cen2); cenDir.normalize()
 
 		# Get the perpedicular vector.
-		perpCenDir = fcvec.normalized(fcvec.crossproduct(cenDir))
+		perpCenDir = cenDir.cross(Vector(0,0,1)); perpCenDir.normalize()
 
 		# Get point on first circle
 		p1 = Vector.add(circle1.Curve.Center, fcvec.scale(perpCenDir, circle1.Curve.Radius))
@@ -1057,10 +1060,10 @@ def findRadicalAxis(circle1, circle2):
 		cen1 = circle1.Curve.Center
 		# dist .. the distance from cen1 to cen2.
 		dist = fcvec.dist(cen1, circle2.Curve.Center)
-		cenDir = fcvec.normalized(Vector.sub(cen1, circle2.Curve.Center))
+		cenDir = cen1.sub(circle2.Curve.Center); cenDir.normalize()
 
 		# Get the perpedicular vector.
-		perpCenDir = fcvec.normalized(fcvec.crossproduct(cenDir))
+		perpCenDir = cenDir.cross(Vector(0,0,1)); perpCenDir.normalize()
 
 		# J ... The radical center.
 		# K ... The point where the cadical axis crosses the line of cen1->cen2.
