@@ -41,8 +41,8 @@ texts, colors,layers (from groups)
 import FreeCAD, os, Part, math, re, string
 from draftlibs import fcvec, dxfColorMap, dxfLibrary, fcgeo
 from draftlibs.dxfReader import readDXF
-from PyQt4 import QtGui
 from Draft import Dimension, DimensionViewProvider
+from FreeCAD import Vector
 
 try: import FreeCADGui
 except: gui = False
@@ -80,12 +80,13 @@ def calcBulge(v1,bulge,v2):
 	calculates intermediary vertex for curved segments.
 	algorithm from http://www.afralisp.net/lisp/Bulges1.htm
 	'''
-	chord = fcvec.new(v1,v2)
+	chord = v2.sub(v1)
 	sagitta = (bulge * chord.Length)/2
-	startpoint = fcvec.add(v1,fcvec.scale(chord,0.5))
-	perp = fcvec.crossproduct(chord)
-	endpoint = fcvec.scale(fcvec.normalized(perp),sagitta)
-	return fcvec.add(startpoint,endpoint)
+	startpoint = v1.add(fcvec.scale(chord,0.5))
+	perp = chord.cross(Vector(1,0,0))
+	if not fcvec.isNull(perp): perp.normalize()
+	endpoint = fcvec.scale(perp,sagitta)
+	return startpoint.add(endpoint)
 
 def getGroup(ob,exportList):
 	"checks if the object is part of a group"
@@ -142,9 +143,9 @@ class fcformat:
 				b2 = float((c2>>8)&0xFF)
 				v1 = FreeCAD.Vector(r1,g1,b1)
 				v2 = FreeCAD.Vector(r2,g2,b2)
-				v = fcvec.new(v1,v2)
+				v = v2.sub(v1)
 				v = fcvec.scale(v,0.5)
-				cv = fcvec.add(v1,v)
+				cv = v1.add(v)
 			else:
 				c1 = bparams.GetUnsigned("BackgroundColor")
 				r1 = float((c1>>24)&0xFF)
@@ -268,13 +269,13 @@ def drawLine(line):
 		v1=FreeCAD.Vector(line.points[0][0],line.points[0][1],line.points[0][2])
 		v2=FreeCAD.Vector(line.points[1][0],line.points[1][1],line.points[1][2])
 		if not fcvec.equals(v1,v2):
-			return Part.Line(v1,v2).toShape()
+			try: return Part.Line(v1,v2).toShape()
+			except: warn(line)
 	return None
 
 def drawPolyline(polyline):
 	"returns a Part shape from a dxf polyline"
 	if (len(polyline.points) > 1):
-		if len(polyline.points) == 2: print "debug: 2-point polyline!"
 		edges = []
 		for p in range(len(polyline.points)-1):
 			p1 = polyline.points[p]
@@ -284,32 +285,39 @@ def drawPolyline(polyline):
 			if not fcvec.equals(v1,v2):
 				if polyline.points[p].bulge:
 					cv = calcBulge(v1,polyline.points[p].bulge,v2)
-					edges.append(Part.Arc(v1,cv,v2).toShape())
+					if fcvec.isColinear([v1,cv,v2]):
+						try: edges.append(Part.Line(v1,v2).toShape())
+						except: warn(polyline)
+					else:
+						try: edges.append(Part.Arc(v1,cv,v2).toShape())
+						except: warn(polyline)
 				else:
-					edges.append(Part.Line(v1,v2).toShape())
+					try: edges.append(Part.Line(v1,v2).toShape())
+					except: warn(polyline)
 		if polyline.closed:
 			p1 = polyline.points[len(polyline.points)-1]
 			p2 = polyline.points[0]
 			v1 = FreeCAD.Vector(p1[0],p1[1],p2[2])
 			v2 = FreeCAD.Vector(p2[0],p2[1],p2[2])
 			if not fcvec.equals(v1,v2):
-				edges.append(Part.Line(v1,v2).toShape())
-		if edges: return Part.Wire(edges)
+				try: edges.append(Part.Line(v1,v2).toShape())
+				except: warn(polyline)
+		if edges:
+			try: return Part.Wire(edges)
+			except: warn(polyline)
 	return None
 
 def drawArc(arc):
 	"returns a Part shape from a dxf arc"
 	v=FreeCAD.Vector(arc.loc[0],arc.loc[1],arc.loc[2])
-	firstangle=(arc.start_angle/180)*math.pi
+        firstangle=(arc.start_angle/180)*math.pi
 	lastangle=(arc.end_angle/180)*math.pi
-	rayvec=FreeCAD.Vector(arc.radius,0,0)
-	v1 = fcvec.add(v,fcvec.rotate(rayvec,firstangle))
-	v3 = fcvec.add(v,fcvec.rotate(rayvec,lastangle))
-	if lastangle > firstangle:
-		v2 = fcvec.add(v,fcvec.rotate(rayvec,(lastangle-firstangle)/2+firstangle))
-	else:
-		v2 = fcvec.add(v,fcvec.rotate(rayvec,(lastangle-(math.pi*2-firstangle))/2))
-	return Part.Arc(v1,v2,v3).toShape()
+	circle=Part.Circle()
+	circle.Center=v
+	circle.Radius=arc.radius
+	try: return circle.toShape(firstangle,lastangle)
+	except: warn(arc)
+	return None
 
 def drawCircle(circle):
 	"returns a Part shape from a dxf circle"
@@ -317,7 +325,9 @@ def drawCircle(circle):
 	curve = Part.Circle()
 	curve.Radius = circle.radius
 	curve.Center = v
-	return curve.toShape()
+	try: return curve.toShape()
+	except: warn(circle)
+	return None
 
 def drawBlock(blockref):
 	"returns a shape from a dxf block reference"
@@ -340,7 +350,8 @@ def drawBlock(blockref):
 	for insert in blockref.entities.get_type('insert'):
 		s = drawInsert(insert)
 		if s: shapes.append(s)
-	shape = Part.makeCompound(shapes)
+	try: shape = Part.makeCompound(shapes)
+	except: warn(blockref)
 	if shape:
 		blockshapes[blockref.name]=shape
 		return shape
@@ -378,6 +389,7 @@ def processdxf(document,filename):
 	"this does the translation of the dxf contents into FreeCAD Part objects"
 
 	global drawing # for debugging - so drawing is still accessible to python after the script
+	FreeCAD.Console.PrintMessage("opening "+filename+"...\n")
 	drawing = readDXF(filename)
 	global layers
 	layers = []
@@ -385,7 +397,8 @@ def processdxf(document,filename):
 	doc = document
 	global blockshapes
 	blockshapes = {}
-	print "dxf: parsing file ",filename
+	global badobjects
+	badobjects = []
 
 	# getting config parameters
 
@@ -401,7 +414,7 @@ def processdxf(document,filename):
 			newob = addObject(shape,"Line",line.layer)
 			if gui: fmt.formatObject(newob,line)
 						
-	# drawing polylines - at the moment arc segments get straightened...
+	# drawing polylines
 
 	polylines = drawing.entities.get_type("lwpolyline")
 	polylines.extend(drawing.entities.get_type("polyline"))
@@ -434,6 +447,8 @@ def processdxf(document,filename):
 
 	# drawing texts
 
+	print fmt.paramtext
+
 	if fmt.paramtext:
 		texts = drawing.entities.get_type("mtext")
 		texts.extend(drawing.entities.get_type("text"))
@@ -454,7 +469,7 @@ def processdxf(document,filename):
 	# drawing dims
 
 	dims = drawing.entities.get_type("dimension")
-	if dims and fmt.paramtext:
+	if fmt.paramtext:
 		FreeCAD.Console.PrintMessage("drawing "+str(len(dims))+" dimensions...\n")
 		for dim in dims:
 			try:
@@ -475,7 +490,7 @@ def processdxf(document,filename):
 				if d: angle = float(d)
 				else: angle = 0
 			except:
-				print "debug: dim data not processed: ",dim.data
+				warn(dim)
 			else:
 				lay=locateLayer(layer)
 				pt = FreeCAD.Vector(x1,y1,z1)
@@ -515,9 +530,19 @@ def processdxf(document,filename):
 				newob = addObject(shape,"Block",insert.layer)
 				if gui: fmt.formatObject(newob,insert)
 
+	# finishing
+
 	doc.recompute()
-	del fmt
 	FreeCAD.Console.PrintMessage("successfully imported "+filename+"\n")
+	if badobjects: print "dxf: ",len(badobjects)," objects were not imported"
+	del fmt
+	del doc
+	del blockshapes
+
+def warn(dxfobject):
+	"outputs a warning if a dxf object couldn't be imported"
+	print "dxf: couldn't import", dxfobject
+	badobjects.append(dxfobject)
 
 def open(filename):
 	"called when freecad opens a file"
@@ -536,89 +561,174 @@ def insert(filename,docname):
 	for l in layers:
 		importgroup.addObject(l)
 
-def export(exportList,filename):
-	"called when freecad exports a file"
-	dxf = dxfLibrary.Drawing()
-	for i in exportList:
-		if isinstance(i,Part.Feature):			
-			for wire in i.Shape.Wires:
+		
+# EXPORT ########################################################################
 
-				# polylines
-				
+
+def getSplineSegs(edge):
+	"returns an array of vectors from a bSpline edge"
+	curve = edge.Curve
+	params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
+	seglength = params.GetInt("maxsplinesegment")
+	points = []
+	if seglength == 0:
+		points.append(edge.Vertexes[0].Point)
+		points.append(edge.Vertexes[-1].Point)
+	else:
+		l = curve.toShape().Length
+		points.append(curve.value(0))
+		if l > seglength:
+			nbverts = int(math.floor(l/seglength))
+			step = seglength/l
+			for nv in range(nbverts):
+				u = step+step*nv
+				v = curve.value(u)
+				points.append(v)
+		points.append(curve.value(1))
+	return points
+
+def getWire(wire):
+	"returns an array of dxf-ready points and bulges from a wire"
+	edges = fcgeo.sortEdges(wire.Edges)
+	points = []
+	for edge in edges:
+		v1 = edge.Vertexes[0].Point
+		if (isinstance(edge.Curve,Part.Circle)):
+			v2 = edge.Vertexes[-1].Point
+			c = edge.Curve.Center
+			angle = abs(fcvec.angle(v1.sub(c),v2.sub(c)))
+			if (fcvec.angle(v2.sub(c)) < fcvec.angle(v1.sub(c))):
+				angle = -angle
+			bul = math.tan(angle/4)
+			points.append((v1.x,v1.y,v1.z,None,None,bul))
+		elif (isinstance(edge.Curve,Part.BSplineCurve)):
+			bul = 0.0
+			spline = getSplineSegs(edge)
+			spline.pop()
+			for p in spline:
+				points.append((p.x,p.y,p.z,None,None,bul))
+		else:
+			bul = 0.0
+			points.append((v1.x,v1.y,v1.z,None,None,bul))
+	if not wire.isClosed():
+		v = edges[-1].Vertexes[-1].Point
+		points.append(fcvec.tup(v))
+	return points
+
+def getBlock(obj):
+	"returns a dxf block with the contents of the object"
+	block = dxfLibrary.Block(name=obj.Name,layer=getGroup(obj,exportList))
+	writeShape(obj,block)
+	
+	return block
+
+def writeShape(ob,dxfobject):
+	"writes the object's shape contents in the given dxf object"
+	processededges = []
+	for wire in ob.Shape.Wires: # polylines
+		for e in wire.Edges: processededges.append(e.hashCode())
+		dxfobject.append(dxfLibrary.PolyLine(getWire(wire), [0.0,0.0,0.0],
+					       int(wire.isClosed()), color=getACI(ob),
+					       layer=getGroup(ob,exportList)))
+	if len(processededges) < len(ob.Shape.Edges): # lone edges
+		loneedges = []
+		for e in ob.Shape.Edges:
+			if not(e.hashCode() in processededges): loneedges.append(e)
+		print "lone edges ",loneedges
+		for edge in loneedges:
+			if (len(edge.Vertexes) == 2) and (isinstance (edge.Curve,Part.Line)): # lines
+				ve1=edge.Vertexes[0].Point
+				ve2=edge.Vertexes[1].Point
+				dxfobject.append(dxfLibrary.Line([fcvec.tup(ve1), fcvec.tup(ve2)],
+								 color=getACI(ob),
+								 layer=getGroup(ob,exportList)))
+			elif (isinstance(edge.Curve,Part.BSplineCurve)): # splines
 				points = []
-				bulges = []
-				flag= int(wire.isClosed())
-				org_point = [0.0,0.0,0.0]
-				edges = fcgeo.sortEdges(wire.Edges)
-				for edge in edges:
-					v1 = edge.Vertexes[0].Point
-					if (isinstance(edge.Curve,Part.Circle)):
-						v2 = edge.Vertexes[-1].Point
-						c = edge.Curve.Center
-						angle = abs(fcvec.angle(fcvec.new(c,v1),fcvec.new(c,v2)))
-						if (fcvec.angle(fcvec.new(c,v2)) < fcvec.angle(fcvec.new(c,v1))):
-							angle = -angle
-						bul = math.tan(angle/4)
+				spline = getSplineSegs(edge)
+				for p in spline:
+					points.append((p.x,p.y,p.z,None,None,0.0))
+				dxfobject.append(dxfLibrary.PolyLine(points, [0.0,0.0,0.0],
+								     0, color=getACI(ob),
+								     layer=getGroup(ob,exportList)))
+			elif (len(edge.Vertexes) == 1): # circles
+				center = fcvec.tup(edge.Curve.Center)
+				radius = ob.Shape.Edges[0].Curve.Radius
+				dxfobject.append(dxfLibrary.Circle(center, radius,
+								   color=getACI(ob),
+								   layer=getGroup(ob,exportList)))
+			elif (len(edge.Vertexes) == 3) or (isinstance (edge.Curve,Part.Circle)): # arcs
+				ce = ob.Shape.Edges[0].Curve.Center
+				radius = ob.Shape.Edges[0].Curve.Radius
+				ve1 = edge.Vertexes[0].Point
+				ve2 = edge.Vertexes[-1].Point
+				ang1=-math.degrees(fcvec.angle(ve1.sub(ce)))
+				ang2=-math.degrees(fcvec.angle(ve2.sub(ce)))
+				dxfobject.append(dxfLibrary.Arc(fcvec.tup(ce), radius,
+								ang1, ang2, color=getACI(ob),
+								layer=getGroup(ob,exportList)))
+
+		
+		
+def export(objectslist,filename):
+	"called when freecad exports a file"
+	global exportList
+	exportList = objectslist
+	dxf = dxfLibrary.Drawing()
+	for ob in exportList:
+		print "processing ",ob.Name
+		if isinstance(ob,Part.Feature):
+			if ob.Shape.ShapeType == 'Compound':
+				if (len(ob.Shape.Wires) == 1):
+					print "open wire found"
+					if (len(ob.Shape.Wires[0].Edges) == len(ob.Shape.Edges)):
+						wire = ob.Shape.Wires[0]
+						dxf.append(dxfLibrary.PolyLine(getWire(wire),
+									       [0.0,0.0,0.0],
+									       int(wire.isClosed()),
+									       color=getACI(ob),
+									       layer=getGroup(ob,exportList)))
 					else:
-						bul = 0.0
-					points.append((v1.x,v1.y,v1.z,None,None,bul))
-				if not wire.isClosed():
-					points.append(fcvec.tup(edges[-1].Vertexes[-1].Point))
-						
-				dxf.append(dxfLibrary.PolyLine(points, org_point, flag, color=getACI(i), layer=getGroup(i,exportList)))
-										
-			if (len(i.Shape.Wires) == 0) and (len(i.Shape.Edges) == 1):
-				edge = i.Shape.Edges[0]
-				if (len(i.Shape.Vertexes) == 2) and (isinstance (edge.Curve,Part.Line)):
-
-					# lines
-					
-					ve1=edge.Vertexes[0].Point
-					ve2=edge.Vertexes[1].Point
-					dxf.append(dxfLibrary.Line([fcvec.tup(ve1),fcvec.tup(ve2)], color=getACI(i), layer=getGroup(i,exportList)))
-				elif (len(i.Shape.Vertexes) == 1):
-
-					# circles
-					
-					center = fcvec.tup(edge.Curve.Center)
-					radius = i.Shape.Edges[0].Curve.Radius
-					dxf.append(dxfLibrary.Circle(center, radius, color=getACI(i), layer=getGroup(i,exportList)))
-
-				elif (len(i.Shape.Vertexes) == 3) or (isinstance (edge.Curve,Part.Circle)):
-
-					# arcs
-					
-					ce = i.Shape.Edges[0].Curve.Center
-					radius = i.Shape.Edges[0].Curve.Radius
-					ve1 = edge.Vertexes[0].Point
-					ve2 = edge.Vertexes[-1].Point
-					ang1=-math.degrees(fcvec.angle(fcvec.new(ce,ve1)))
-					ang2=-math.degrees(fcvec.angle(fcvec.new(ce,ve2)))
-					dxf.append(dxfLibrary.Arc(fcvec.tup(ce), radius, ang1, ang2, color=getACI(i), layer=getGroup(i,exportList)))
-
-		elif (i.Type == "App::Annotation"):
+						print "creating block - 1 wire"
+						block = getBlock(ob)
+						dxf.blocks.append(block)
+						dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
+				else:
+					print "creating block"
+					block = getBlock(ob)
+					dxf.blocks.append(block)
+					dxf.append(dxfLibrary.Insert(name=ob.Name.upper()))
+			else:
+				writeShape(ob,dxf)
+				
+		elif (ob.Type == "App::Annotation"):
 
 			# texts
 
 			# temporary - as dxfLibrary doesn't support mtexts well, we use several single-line texts
 			# well, anyway, at the moment, Draft only writes single-line texts, so...
-			for text in i.LabelText:
-				point = fcvec.tup(FreeCAD.Vector(i.Position.x,i.Position.y-i.LabelText.index(text),i.Position.z))
-				if gui: height = float(i.ViewObject.FontSize)
+			for text in ob.LabelText:
+				point = fcvec.tup(FreeCAD.Vector(ob.Position.x,
+								 ob.Position.y-ob.LabelText.index(text),
+								 ob.Position.z))
+				if gui: height = float(ob.ViewObject.FontSize)
 				else: height = 1
-				dxf.append(dxfLibrary.Text(text,point,height=height, color=getACI(i,text=True), layer=getGroup(i,exportList)))
+				dxf.append(dxfLibrary.Text(text,point,height=height,
+							   color=getACI(ob,text=True),
+							   style='STANDARD',
+							   layer=getGroup(ob,exportList)))
 
-		elif (i.Type == "App::FeaturePython"):
-			if 'Dimline' in i.PropertiesList:
-				p1 = fcvec.tup(i.Start)
-				p2 = fcvec.tup(i.End)
-				base = Part.Line(i.Start,i.End).toShape()
-				proj = fcgeo.findDistance(i.Dimline,base)
+		elif (ob.Type == "App::FeaturePython"):
+			if 'Dimline' in ob.PropertiesList:
+				p1 = fcvec.tup(ob.Start)
+				p2 = fcvec.tup(ob.End)
+				base = Part.Line(ob.Start,ob.End).toShape()
+				proj = fcgeo.findDistance(ob.Dimline,base)
 				if not proj:
-					pbase = fcvec.tup(i.End)
+					pbase = fcvec.tup(ob.End)
 				else:
-					pbase = fcvec.tup(i.End.add(fcvec.neg(proj)))
-				dxf.append(dxfLibrary.Dimension(pbase,p1,p2,color=getACI(i), layer=getGroup(i,exportList)))
+					pbase = fcvec.tup(ob.End.add(fcvec.neg(proj)))
+				dxf.append(dxfLibrary.Dimension(pbase,p1,p2,color=getACI(ob),
+								layer=getGroup(ob,exportList)))
 					
 	dxf.saveas(filename)
 	FreeCAD.Console.PrintMessage("successfully exported "+filename+"\r\n")
