@@ -39,7 +39,7 @@ try: import FreeCADGui
 except: gui = False
 else: gui = True
 try: draftui = FreeCADGui.activeWorkbench().draftToolBar.ui
-except: draftgui = None
+except: draftui = None
 
 pythonopen = open
 
@@ -238,7 +238,7 @@ class svgHandler(xml.sax.ContentHandler):
 		params = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/Draft")
 		self.style = params.GetInt("svgstyle")
 	
-		if gui and draftgui:
+		if gui and draftui:
 			r = float(draftui.color.red()/255.0)
 			g = float(draftui.color.green()/255.0)
 			b = float(draftui.color.blue()/255.0)
@@ -319,14 +319,14 @@ class svgHandler(xml.sax.ContentHandler):
 
 			path = []
 			point = []
-			lastvec = None
+			lastvec = Vector(0,0,0)
 			command = None
 			relative = False
 			firstvec = None
 
 			pathdata = []
 			for d in data['d']:
-				if (len(d) == 1):
+				if (len(d) == 1) and (d in ['m','M','l','L','h','H','v','V','a','A','c','C']):
 					pathdata.append(d)
 				else:
 					try:
@@ -385,11 +385,11 @@ class svgHandler(xml.sax.ContentHandler):
 					command = "close"
 					point = []
 				elif (d == "C"):
-					command = "connect"
+					command = "curve"
 					relative = False
 					point = []
 				elif (d == "c"):
-					command = "connect"
+					command = "curve"
 					relative = True
 					point = []
 				else:
@@ -398,6 +398,8 @@ class svgHandler(xml.sax.ContentHandler):
 					except ValueError:
 						pass
 
+				print "command: ",command, ' point: ',point
+
 				if (len(point)==2) and (command=="move"):
 					if path:
 						sh = Part.Wire(path)
@@ -409,16 +411,20 @@ class svgHandler(xml.sax.ContentHandler):
 						path = []
 					if relative:
 						lastvec = lastvec.add(Vector(point[0],-point[1],0))
+						command="line"
 					else:
 						lastvec = Vector(point[0],-point[1],0)
 					firstvec = lastvec
-					command = None
+					print "move ",lastvec
+					command = "line"
+					point = []
 				elif (len(point)==2) and (command=="line"):
 					if relative:
 						currentvec = lastvec.add(Vector(point[0],-point[1],0))
 					else:
 						currentvec = Vector(point[0],-point[1],0)
 					seg = Part.Line(lastvec,currentvec).toShape()
+					print "line ",lastvec,currentvec
 					lastvec = currentvec
 					path.append(seg)
 					point = []
@@ -474,13 +480,14 @@ class svgHandler(xml.sax.ContentHandler):
 						path = []
 					point = []
 					command = None
-				elif (len(point)==2) and (command=="connect"):
+				elif (len(point)==6) and (command=="curve"):
 					if relative:
-						currentvec = lastvec.add(Vector(point[0],-point[1],0))
+						currentvec = lastvec.add(Vector(point[4],-point[5],0))
 					else:
-						currentvec = Vector(point[0],-point[1],0)
+						currentvec = Vector(point[4],-point[5],0)
 					if not fcvec.equals(currentvec,lastvec):
 						seg = Part.Line(lastvec,currentvec).toShape()
+						print "connect ",lastvec,currentvec
 						lastvec = currentvec
 						path.append(seg)
 					point = []
@@ -686,12 +693,9 @@ def export(exportList,filename):
 					p2 = p1.add(fcvec.neg(proj))
 					p3 = p4.add(fcvec.neg(proj))
 					if dmax and (proj.Length > dmax):
-						p1 = p2.add(fcvec.scale(fcvec.normalized(proj),dmax))
-						p4 = p3.add(fcvec.scale(fcvec.normalized(proj),dmax))
+						p1 = p2.add(fcvec.scale(proj.normalize(),dmax))
+						p4 = p3.add(fcvec.scale(proj.normalize(),dmax))
 				midpoint = p2.add(fcvec.scale(p3.sub(p2),0.5))
-				if not proj:
-					ed = fcgeo.vec(base)
-					proj = fcvec.crossproduct(ed)
 				angle = fcvec.angle(p3.sub(p2))
 				if (angle > math.pi/2) or (angle < -math.pi/2): angle = angle+math.pi
 				offset = fcvec.rotate(FreeCAD.Vector(height*.2,0,0),-angle+math.pi/2)
