@@ -447,12 +447,14 @@ void MeshAlgorithm::GetFacetBorders (const std::vector<unsigned long> &raulInd,
                 ulLast = pEI->second;
                 clBorder.push_back(ulLast);
                 aclEdges.erase(pEI);
+                pEI = aclEdges.begin();
                 break;
             }
             else if (pEI->second == ulFirst) {
                 ulFirst = pEI->first;
                 clBorder.push_front(ulFirst);
                 aclEdges.erase(pEI);
+                pEI = aclEdges.begin();
                 break;
             }
             // Note: Using this might result into boundaries with wrong orientation.
@@ -462,12 +464,14 @@ void MeshAlgorithm::GetFacetBorders (const std::vector<unsigned long> &raulInd,
                 ulLast = pEI->first;
                 clBorder.push_back(ulLast);
                 aclEdges.erase(pEI);
+                pEI = aclEdges.begin();
                 break;
             }
             else if (pEI->first == ulFirst && ignoreOrientation) {
                 ulFirst = pEI->second;
                 clBorder.push_front(ulFirst);
                 aclEdges.erase(pEI);
+                pEI = aclEdges.begin();
                 break;
             }
         }
@@ -636,12 +640,12 @@ bool MeshAlgorithm::FillupHole(const std::vector<unsigned long>& boundary,
     unsigned long refPoint0 = *(boundary.begin());
     unsigned long refPoint1 = *(boundary.begin()+1);
     if (pP2FStructure) {
-        std::set<MeshFacetArray::_TConstIterator> ring = (*pP2FStructure)[refPoint0];
+        const std::set<unsigned long>& ring = (*pP2FStructure)[refPoint0];
         bool found = false;
-        for (std::set<MeshFacetArray::_TConstIterator>::iterator it = ring.begin(); it != ring.end() && !found; ++it) {
+        for (std::set<unsigned long>::const_iterator it = ring.begin(); it != ring.end() && !found; ++it) {
             for (int i=0; i<3; i++) {
-                if ((*it)->_aulPoints[i] == refPoint1) {
-                    rFace = **it;
+                if (pP2FStructure->getFacet(*it)->_aulPoints[i] == refPoint1) {
+                    rFace = *pP2FStructure->getFacet(*it);
                     rTriangle = _rclMesh.GetFacet(rFace);
                     found = true;
                     break;
@@ -693,11 +697,11 @@ bool MeshAlgorithm::FillupHole(const std::vector<unsigned long>& boundary,
     }
 
     if (cTria.TriangulatePolygon()) {
-        // get the facets and add the additional points to the array
-        rFaces.insert(rFaces.end(), cTria.GetFacets().begin(), cTria.GetFacets().end());
         // if we have enough points then we fit a surface through the points and project
         // the added points onto this surface
         cTria.ProjectOntoSurface(surf_pts);
+        // get the facets and add the additional points to the array
+        rFaces.insert(rFaces.end(), cTria.GetFacets().begin(), cTria.GetFacets().end());
         std::vector<Base::Vector3f> newVertices = cTria.AddedPoints();
         for (std::vector<Base::Vector3f>::iterator pt = newVertices.begin(); pt != newVertices.end(); ++pt) {
             rPoints.push_back((*pt));
@@ -801,6 +805,28 @@ void MeshAlgorithm::SetPointsFlag (const std::vector<unsigned long> &raulInds, M
     _rclMesh._aclPointArray[*i].SetFlag(tF);
 }
 
+void MeshAlgorithm::GetFacetsFlag (std::vector<unsigned long> &raulInds, MeshFacet::TFlagType tF) const
+{
+    raulInds.reserve(raulInds.size() + CountFacetFlag(tF));
+    MeshFacetArray::_TConstIterator beg = _rclMesh._aclFacetArray.begin();
+    MeshFacetArray::_TConstIterator end = _rclMesh._aclFacetArray.end();
+    for (MeshFacetArray::_TConstIterator it = beg; it != end; ++it) {
+        if (it->IsFlag(tF))
+            raulInds.push_back(it-beg);
+    }
+}
+
+void MeshAlgorithm::GetPointsFlag (std::vector<unsigned long> &raulInds, MeshPoint::TFlagType tF) const
+{
+    raulInds.reserve(raulInds.size() + CountPointFlag(tF));
+    MeshPointArray::_TConstIterator beg = _rclMesh._aclPointArray.begin();
+    MeshPointArray::_TConstIterator end = _rclMesh._aclPointArray.end();
+    for (MeshPointArray::_TConstIterator it = beg; it != end; ++it) {
+        if (it->IsFlag(tF))
+            raulInds.push_back(it-beg);
+    }
+}
+
 void MeshAlgorithm::ResetFacetsFlag (const std::vector<unsigned long> &raulInds, MeshFacet::TFlagType tF) const
 {
   for (std::vector<unsigned long>::const_iterator i = raulInds.begin(); i != raulInds.end(); i++)
@@ -1227,6 +1253,21 @@ void MeshAlgorithm::CutBorderFacets (std::vector<unsigned long> &raclFacetIndice
   raclFacetIndices = aclResult;
 }
 
+unsigned long MeshAlgorithm::CountBorderEdges() const
+{
+    unsigned long cnt=0;
+    const MeshFacetArray &rclFAry = _rclMesh._aclFacetArray;
+    MeshFacetArray::_TConstIterator end = rclFAry.end();
+    for (MeshFacetArray::_TConstIterator it = rclFAry.begin(); it != end; ++it) {
+        for (int i=0; i<3; i++) {
+            if (it->_aulNeighbours[i] == ULONG_MAX)
+                cnt++;
+        }
+    }
+
+    return cnt;
+}
+
 void MeshAlgorithm::CheckBorderFacets (const std::vector<unsigned long> &raclFacetIndices, std::vector<unsigned long> &raclResultIndices, unsigned short usLevel) const
 {
   ResetFacetFlag(MeshFacet::TMP0);
@@ -1676,13 +1717,14 @@ std::set<unsigned long> MeshRefPointToFacets::NeighbourPoints(const std::vector<
     std::set<unsigned long> cp,nb,lp;
     cp.insert(pt.begin(), pt.end());
     lp.insert(pt.begin(), pt.end());
+    MeshFacetArray::_TConstIterator f_it = _rclMesh.GetFacets().begin();
     for (int i=0; i < level; i++) {
         std::set<unsigned long> cur;
         for (std::set<unsigned long>::iterator it = lp.begin(); it != lp.end(); ++it) {
-            const std::set<MeshFacetArray::_TConstIterator>& ft = (*this)[*it];
-            for (std::set<MeshFacetArray::_TConstIterator>::const_iterator jt = ft.begin(); jt != ft.end(); ++jt) {
+            const std::set<unsigned long>& ft = (*this)[*it];
+            for (std::set<unsigned long>::const_iterator jt = ft.begin(); jt != ft.end(); ++jt) {
                 for (int j = 0; j < 3; j++) {
-                    unsigned long index = (*jt)->_aulPoints[j];
+                    unsigned long index = f_it[*jt]._aulPoints[j];
                     if (cp.find(index) == cp.end() && nb.find(index) == nb.end()) {
                         nb.insert(index);
                         cur.insert(index);
@@ -1714,43 +1756,39 @@ void MeshRefPointToFacets::Neighbours (unsigned long ulFacetInd, float fMpxDist,
 }
 
 /// @todo
-void MeshRefPointToFacets::SearchNeighbours(MeshFacetArray::_TConstIterator pFIter, const Base::Vector3f &rclCenter, float fMpxDist, std::vector<MeshFacetArray::_TConstIterator> &rclNb)
+void MeshRefPointToFacets::SearchNeighbours(MeshFacetArray::_TConstIterator f_it, const Base::Vector3f &rclCenter, float fMpxDist, std::vector<MeshFacetArray::_TConstIterator> &rclNb)
 {
-    if (pFIter->IsFlag(MeshFacet::VISIT) == true)
+    if (f_it->IsFlag(MeshFacet::VISIT) == true)
         return;
 
-    if (Base::DistanceP2(rclCenter, _rclMesh.GetFacet(*pFIter).GetGravityPoint()) > fMpxDist)
+    if (Base::DistanceP2(rclCenter, _rclMesh.GetFacet(*f_it).GetGravityPoint()) > fMpxDist)
         return;
 
-    rclNb.push_back(pFIter);
-    pFIter->SetFlag(MeshFacet::VISIT);
+    rclNb.push_back(f_it);
+    f_it->SetFlag(MeshFacet::VISIT);
 
-    const MeshPointArray& rPoints = _rclMesh.GetPoints();
-    MeshPointArray::_TConstIterator pPBegin = rPoints.begin();
+    MeshPointArray::_TConstIterator p_beg = _rclMesh.GetPoints().begin();
+    MeshFacetArray::_TConstIterator f_beg = _rclMesh.GetFacets().begin();
 
     for (int i = 0; i < 3; i++) {
-        MeshPointArray::_TConstIterator pPIter = rPoints.begin() + pFIter->_aulPoints[i];
-        const std::set<MeshFacetArray::_TConstIterator> &f = operator[](pPIter - pPBegin);
+        const std::set<unsigned long> &f = (*this)[f_it->_aulPoints[i]];
 
-        for (std::set<MeshFacetArray::_TConstIterator>::const_iterator j = f.begin(); j != f.end(); ++j) {
-            SearchNeighbours(*j, rclCenter, fMpxDist, rclNb);
+        for (std::set<unsigned long>::const_iterator j = f.begin(); j != f.end(); ++j) {
+            SearchNeighbours(f_beg+*j, rclCenter, fMpxDist, rclNb);
         }
     }
 }
 
-std::set<MeshFacetArray::_TConstIterator>
+MeshFacetArray::_TConstIterator
+MeshRefPointToFacets::getFacet (unsigned long index) const
+{
+    return _rclMesh.GetFacets().begin() + index;
+}
+
+const std::set<unsigned long>&
 MeshRefPointToFacets::operator[] (unsigned long pos) const
 {
-    const MeshFacetArray& rFacets = _rclMesh.GetFacets();
-    MeshFacetArray::_TConstIterator  pBegin = rFacets.begin();
-    std::set<MeshCore::MeshFacetArray::_TConstIterator> fts;
-
-    const std::set<unsigned long>& n = _map[pos];
-    for (std::set<unsigned long>::const_iterator it = n.begin(); it != n.end(); ++it) {
-        fts.insert(pBegin + *it);
-    }
-
-    return fts;
+    return _map[pos];
 }
 
 //----------------------------------------------------------------------------
@@ -1766,26 +1804,17 @@ void MeshRefFacetToFacets::Rebuild (void)
     MeshFacetArray::_TConstIterator pFBegin = rFacets.begin();
     for (MeshFacetArray::_TConstIterator pFIter = pFBegin; pFIter != rFacets.end(); ++pFIter) {
         for (int i = 0; i < 3; i++) {
-            std::set<MeshFacetArray::_TConstIterator> faces = vertexFace[pFIter->_aulPoints[i]];
-            for (std::set<MeshFacetArray::_TConstIterator>::iterator it = faces.begin(); it != faces.end(); ++it)
-                _map[pFIter - pFBegin].insert(*it - pFBegin);
+            const std::set<unsigned long>& faces = vertexFace[pFIter->_aulPoints[i]];
+            for (std::set<unsigned long>::const_iterator it = faces.begin(); it != faces.end(); ++it)
+                _map[pFIter - pFBegin].insert(*it);
         }
     }
 }
 
-std::set<MeshFacetArray::_TConstIterator>
+const std::set<unsigned long>&
 MeshRefFacetToFacets::operator[] (unsigned long pos) const
 {
-    const MeshFacetArray& rFacets = _rclMesh.GetFacets();
-    MeshFacetArray::_TConstIterator  pBegin = rFacets.begin();
-    std::set<MeshCore::MeshFacetArray::_TConstIterator> fts;
-
-    const std::set<unsigned long>& n = _map[pos];
-    for (std::set<unsigned long>::const_iterator it = n.begin(); it != n.end(); ++it) {
-        fts.insert(pBegin + *it);
-    }
-
-    return fts;
+    return _map[pos];
 }
 
 //----------------------------------------------------------------------------
@@ -1831,17 +1860,85 @@ Base::Vector3f MeshRefPointToPoints::GetNormal(unsigned long pos) const
     return normal;
 }
 
-std::set<MeshPointArray::_TConstIterator>
-MeshRefPointToPoints::operator[] (unsigned long pos) const
+float MeshRefPointToPoints::GetAverageEdgeLength(unsigned long index) const
 {
     const MeshPointArray& rPoints = _rclMesh.GetPoints();
-    MeshPointArray::_TConstIterator  pBegin = rPoints.begin();
-    std::set<MeshCore::MeshPointArray::_TConstIterator> pts;
-
-    const std::set<unsigned long>& n = _map[pos];
+    float len=0.0f;
+    const std::set<unsigned long>& n = (*this)[index];
+    const Base::Vector3f& p = rPoints[index];
     for (std::set<unsigned long>::const_iterator it = n.begin(); it != n.end(); ++it) {
-        pts.insert(pBegin + *it);
+        len += Base::Distance(p, rPoints[*it]);
     }
+    return (len/n.size());
+}
 
-    return pts;
+const std::set<unsigned long>&
+MeshRefPointToPoints::operator[] (unsigned long pos) const
+{
+    return _map[pos];
+}
+
+//----------------------------------------------------------------------------
+
+void MeshRefEdgeToFacets::Rebuild (void)
+{
+    _map.clear();
+
+    const MeshFacetArray& rFacets = _rclMesh.GetFacets();
+    unsigned long index = 0;
+    for (MeshFacetArray::_TConstIterator it = rFacets.begin(); it != rFacets.end(); ++it, ++index) {
+        for (int i=0; i<3; i++) {
+            MeshEdge e;
+            e.first = it->_aulPoints[i];
+            e.second = it->_aulPoints[(i+1)%3];
+            std::map<MeshEdge, MeshFacetPair, EdgeOrder>::iterator jt =
+                _map.find(e);
+            if (jt == _map.end()) {
+                _map[e].first = index;
+                _map[e].second = ULONG_MAX;
+            }
+            else {
+                _map[e].second = index;
+            }
+        }
+    }
+}
+
+const std::pair<unsigned long, unsigned long>&
+MeshRefEdgeToFacets::operator[] (const MeshEdge& edge) const
+{
+    return _map.find(edge)->second;
+}
+
+//----------------------------------------------------------------------------
+
+void MeshRefNormalToPoints::Rebuild (void)
+{
+    _norm.clear();
+
+    const MeshPointArray& rPoints = _rclMesh.GetPoints();
+    _norm.resize(rPoints.size());
+
+    const MeshFacetArray& rFacets = _rclMesh.GetFacets();
+    for (MeshFacetArray::_TConstIterator pF = rFacets.begin(); pF != rFacets.end(); ++pF) {
+        const MeshPoint &p0 = rPoints[pF->_aulPoints[0]];
+        const MeshPoint &p1 = rPoints[pF->_aulPoints[1]];
+        const MeshPoint &p2 = rPoints[pF->_aulPoints[2]];
+        float l2p01 = Base::DistanceP2(p0,p1);
+        float l2p12 = Base::DistanceP2(p1,p2);
+        float l2p20 = Base::DistanceP2(p2,p0);
+
+        Base::Vector3f facenormal = _rclMesh.GetFacet(*pF).GetNormal();
+        _norm[pF->_aulPoints[0]] += facenormal * (1.0f / (l2p01 * l2p20));
+        _norm[pF->_aulPoints[1]] += facenormal * (1.0f / (l2p12 * l2p01));
+        _norm[pF->_aulPoints[2]] += facenormal * (1.0f / (l2p20 * l2p12));
+    }
+    for (std::vector<Base::Vector3f>::iterator it = _norm.begin(); it != _norm.end(); ++it)
+        it->Normalize();
+}
+
+const Base::Vector3f&
+MeshRefNormalToPoints::operator[] (unsigned long pos) const
+{
+    return _norm[pos];
 }
