@@ -26,6 +26,7 @@
 
 #include <set>
 #include <vector>
+#include <map>
 
 #include "MeshKernel.h"
 #include "Elements.h"
@@ -170,6 +171,10 @@ public:
   void SetFacetsFlag (const std::vector<unsigned long> &raulInds, MeshFacet::TFlagType tF) const;
   /** Sets to all points in \a raulInds the flag \a tF. */
   void SetPointsFlag (const std::vector<unsigned long> &raulInds, MeshPoint::TFlagType tF) const;
+  /** Gets all facets in \a raulInds with the flag \a tF. */
+  void GetFacetsFlag (std::vector<unsigned long> &raulInds, MeshFacet::TFlagType tF) const;
+  /** Gets all points in \a raulInds with the flag \a tF. */
+  void GetPointsFlag (std::vector<unsigned long> &raulInds, MeshPoint::TFlagType tF) const;
   /** Resets from all facets in \a raulInds the flag \a tF. */
   void ResetFacetsFlag (const std::vector<unsigned long> &raulInds, MeshFacet::TFlagType tF) const;
   /** Resets from all points in \a raulInds the flag \a tF. */
@@ -223,6 +228,8 @@ public:
    * \note The mesh structure is not modified by this method. This is in the responsibility of the user.
    */
   void CutBorderFacets (std::vector<unsigned long> &raclFacetIndices, unsigned short usLevel = 1) const;
+  /** Returns the number of border edges */
+  unsigned long CountBorderEdges() const;
   /**
    * Determines all border points as indices of the facets in \a raclFacetIndices. The points are unsorted.
    */
@@ -306,7 +313,8 @@ public:
 
     /// Rebuilds up data structure
     void Rebuild (void);
-    std::set<MeshFacetArray::_TConstIterator> operator[] (unsigned long) const;
+    const std::set<unsigned long>& operator[] (unsigned long) const;
+    MeshFacetArray::_TConstIterator getFacet (unsigned long) const;
     std::set<unsigned long> NeighbourPoints(const std::vector<unsigned long>& , int level) const;
     void Neighbours (unsigned long ulFacetInd, float fMaxDist, std::vector<MeshFacetArray::_TConstIterator> &rclNb);
     Base::Vector3f GetNormal(unsigned long) const;
@@ -340,7 +348,7 @@ public:
 
     /// Returns a set of facets sharing one or more points with the facet with
     /// index \a ulFacetIndex.
-    std::set<MeshFacetArray::_TConstIterator> operator[] (unsigned long) const;
+    const std::set<unsigned long>& operator[] (unsigned long) const;
 
 protected:
     const MeshKernel  &_rclMesh; /**< The mesh kernel. */
@@ -365,14 +373,79 @@ public:
 
     /// Rebuilds up data structure
     void Rebuild (void);
-    std::set<MeshPointArray::_TConstIterator> operator[] (unsigned long) const;
+    const std::set<unsigned long>& operator[] (unsigned long) const;
     Base::Vector3f GetNormal(unsigned long) const;
+    float GetAverageEdgeLength(unsigned long) const;
 
 protected:
     const MeshKernel  &_rclMesh; /**< The mesh kernel. */
     std::vector<std::set<unsigned long> > _map;
 };
 
+/**
+ * The MeshRefEdgeToFacets builds up a structure to have access to all facets 
+ * of an edge. On a manifold mesh an edge has one or two facets associated.
+ * \note If the underlying mesh kernel gets changed this structure becomes invalid and must
+ * be rebuilt.
+ */
+class MeshExport MeshRefEdgeToFacets
+{
+public:
+    /// Construction
+    MeshRefEdgeToFacets (const MeshKernel &rclM) : _rclMesh(rclM) 
+    { Rebuild(); }
+    /// Destruction
+    ~MeshRefEdgeToFacets (void)
+    { }
+
+    /// Rebuilds up data structure
+    void Rebuild (void);
+    const std::pair<unsigned long, unsigned long>& operator[] (const MeshEdge&) const;
+
+protected:
+    class EdgeOrder {
+    public:
+        bool operator () (const MeshEdge &e1, const MeshEdge &e2) const
+        {
+            if (e1.first < e2.first)
+                return true;
+            else if (e1.first > e2.first)
+                return false;
+            else if (e1.second < e2.second)
+                return true;
+            else
+                return false;
+        }
+    };
+    typedef std::pair<unsigned long, unsigned long> MeshFacetPair;
+    const MeshKernel  &_rclMesh; /**< The mesh kernel. */
+    std::map<MeshEdge, MeshFacetPair, EdgeOrder> _map;
+};
+
+/**
+ * The MeshRefNormalToPoints builds up a structure to have access to the normal of a vertex.
+ * \note If the underlying mesh kernel gets changed this structure becomes invalid and must
+ * be rebuilt.
+ */
+class MeshExport MeshRefNormalToPoints
+{
+public:
+    /// Construction
+    MeshRefNormalToPoints (const MeshKernel &rclM) : _rclMesh(rclM) 
+    { Rebuild(); }
+    /// Destruction
+    ~MeshRefNormalToPoints (void)
+    { }
+
+    /// Rebuilds up data structure
+    void Rebuild (void);
+    const Base::Vector3f& operator[] (unsigned long) const;
+
+protected:
+    const MeshKernel  &_rclMesh; /**< The mesh kernel. */
+    std::vector<Base::Vector3f> _norm;
+};
+
 }; // namespace MeshCore 
 
 #endif  // MESH_ALGORITHM_H 
