@@ -67,17 +67,26 @@ public:
   unsigned short _ausCorner[2];  // corner point indices of the facet
 };
 
+/** MeshEdge just a pair of two point indices */
+typedef std::pair<unsigned long, unsigned long> MeshEdge;
+
 /**
  * The MeshPoint class represents a point in the mesh data structure. The class inherits from
  * Vector3f and provides some additional information such as flag state and property value.
  * The flags can be modified by the Set() and Reset() and queried by IsFlag().
  * A point can temporary be in an invalid state (e.g during deletion of several points), but
  * must not be set in general, i.e. always usable within a mesh-internal algorithm.
+ *
+ * Note: The status flag SEGMENT mark a point to be part of certain subset, a segment.
+ * This flag must not be set by any algorithm unless it adds or removes points to a segment.
+ *
+ * Note: The status flag SELECTED mark a point to be selected which is e.g. used in the GUI.
+ * This flag must not be set by any algorithm unless it adds or removes points to the selection.
  */
 class MeshExport MeshPoint: public Base::Vector3f
 {
 public:
-  enum TFlagType {INVALID=1, VISIT=2, SEGMENT=4, MARKED=8, REV1=16, REV2=32, TMP0=64, TMP1=128};
+  enum TFlagType {INVALID=1, VISIT=2, SEGMENT=4, MARKED=8, SELECTED=16, REV=32, TMP0=64, TMP1=128};
 
   /** @name Construction */
   //@{
@@ -152,11 +161,17 @@ public:
  * \li neighbour or edge number of 1 is defined by corner 1 and 2
  * \li neighbour or edge number of 2 is defined by corner 2 and 0
  * \li neighbour index is set to ULONG_MAX if there is no neighbour facet
+ *
+ * Note: The status flag SEGMENT mark a facet to be part of certain subset, a segment.
+ * This flag must not be set by any algorithm unless it adds or removes facets to a segment.
+ *
+ * Note: The status flag SELECTED mark a facet to be selected which is e.g. used in the GUI.
+ * This flag must not be set by any algorithm unless it adds or removes facets to the selection.
  */
 class MeshFacet
 {
 public:
-  enum TFlagType {INVALID=1, VISIT=2, SEGMENT=4, MARKED=8, REV1=16, REV2=32, TMP0=64, TMP1=128};
+  enum TFlagType {INVALID=1, VISIT=2, SEGMENT=4, MARKED=8, SELECTED=16, REV=32, TMP0=64, TMP1=128};
 
 public:
   /** @name Construction */
@@ -194,6 +209,8 @@ public:
 
   // Assignment
   inline MeshFacet& operator = (const MeshFacet &rclF);
+  inline void SetVertices(unsigned long,unsigned long,unsigned long);
+  inline void SetNeighbours(unsigned long,unsigned long,unsigned long);
 
   /**
    * Returns the indices of the corner points of the given edge number. 
@@ -713,8 +730,8 @@ inline MeshFacet::MeshFacet (void)
 : _ucFlag(0),
   _ulProp(0)
 {
-    memset(_aulNeighbours, ULONG_MAX, sizeof(ULONG_MAX) * 3);
-    memset(_aulPoints, ULONG_MAX, sizeof(ULONG_MAX) * 3);
+    memset(_aulNeighbours, 0xff, sizeof(ULONG_MAX) * 3);
+    memset(_aulPoints, 0xff, sizeof(ULONG_MAX) * 3);
 }
 
 inline MeshFacet::MeshFacet(const MeshFacet &rclF)
@@ -760,6 +777,20 @@ inline MeshFacet& MeshFacet::operator = (const MeshFacet &rclF)
     return *this;
 }
 
+void MeshFacet::SetVertices(unsigned long p1,unsigned long p2,unsigned long p3)
+{
+    _aulPoints[0] = p1;
+    _aulPoints[1] = p2;
+    _aulPoints[2] = p3;
+}
+
+void MeshFacet::SetNeighbours(unsigned long n1,unsigned long n2,unsigned long n3)
+{
+    _aulNeighbours[0] = n1;
+    _aulNeighbours[1] = n2;
+    _aulNeighbours[2] = n3;
+}
+
 inline void MeshFacet::GetEdge (unsigned short usSide, MeshHelpEdge &rclEdge) const
 {
     rclEdge._ulIndex[0] = _aulPoints[usSide];
