@@ -23,6 +23,7 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <queue>
 #endif
 
 #include "Triangulation.h"
@@ -651,3 +652,18 @@ bool ConstraintDelaunayTriangulator::Triangulate()
 
     return succeeded;
 }
+
+// -------------------------------------------------------------
+
+Triangulator::Triangulator(const MeshKernel& k) : _kernel(k)
+{
+}
+
+Triangulator::~Triangulator()
+{
+}
+
+bool Triangulator::Triangulate()
+{
+    return false;
+}
