@@ -270,7 +270,7 @@ void MeshObject::RestoreDocFile(Base::Reader &reader)
     load(reader);
 }
 
-void MeshObject::save(const char* file, MeshCore::MeshOutput::Format f) const
+void MeshObject::save(const char* file, MeshCore::MeshIO::Format f) const
 {
     MeshCore::MeshOutput aWriter(_kernel);
     aWriter.SaveAny(file, f);
@@ -289,7 +289,32 @@ bool MeshObject::load(const char* file)
         return false;
 
     _kernel.Swap(kernel);
+    // Some file formats define several objects per file (e.g. OBJ).
+    // Now we mark each object as an own segment so that we can break
+    // the object into its orriginal objects again.
     this->_segments.clear();
+    const MeshCore::MeshFacetArray& faces = _kernel.GetFacets();
+    MeshCore::MeshFacetArray::_TConstIterator it;
+    std::vector<unsigned long> segment;
+    segment.reserve(faces.size());
+    unsigned long prop = 0;
+    unsigned long index = 0;
+    for (it = faces.begin(); it != faces.end(); ++it) {
+        if (prop < it->_ulProp) {
+            prop = it->_ulProp;
+            if (!segment.empty()) {
+                this->_segments.push_back(Segment(this,segment));
+                segment.clear();
+            }
+        }
+
+        segment.push_back(index++);
+    }
+
+    // if the whole mesh is a single object then don't mark as segment
+    if (!segment.empty() && (segment.size() < faces.size())) {
+        this->_segments.push_back(Segment(this,segment));
+    }
 
 #ifndef FC_DEBUG
     try {
@@ -469,6 +494,50 @@ void MeshObject::deletedFacets(const std::vector<unsigned long>& remFacets)
     }
 }
 
+void MeshObject::deleteSelectedFacets()
+{
+    std::vector<unsigned long> facets;
+    MeshCore::MeshAlgorithm(this->_kernel).GetFacetsFlag(facets, MeshCore::MeshFacet::SELECTED);
+    deleteFacets(facets);
+}
+
+void MeshObject::deleteSelectedPoints()
+{
+    std::vector<unsigned long> points;
+    MeshCore::MeshAlgorithm(this->_kernel).GetPointsFlag(points, MeshCore::MeshPoint::SELECTED);
+    deletePoints(points);
+}
+
+void MeshObject::addFacetsToSelection(const std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).SetFacetsFlag(inds, MeshCore::MeshFacet::SELECTED);
+}
+
+void MeshObject::addPointsToSelection(const std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).SetPointsFlag(inds, MeshCore::MeshPoint::SELECTED);
+}
+
+void MeshObject::removeFacetsFromSelection(const std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).ResetFacetsFlag(inds, MeshCore::MeshFacet::SELECTED);
+}
+
+void MeshObject::removePointsFromSelection(const std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).ResetPointsFlag(inds, MeshCore::MeshPoint::SELECTED);
+}
+
+void MeshObject::getFacetsFromSelection(std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).GetFacetsFlag(inds, MeshCore::MeshFacet::SELECTED);
+}
+
+void MeshObject::getPointsFromSelection(std::vector<unsigned long>& inds) const
+{
+    MeshCore::MeshAlgorithm(this->_kernel).GetPointsFlag(inds, MeshCore::MeshPoint::SELECTED);
+}
+
 void MeshObject::updateMesh(const std::vector<unsigned long>& facets)
 {
     std::vector<unsigned long> points;
@@ -493,6 +562,14 @@ void MeshObject::updateMesh()
     }
 }
 
+std::vector<std::vector<unsigned long> > MeshObject::getComponents() const
+{
+    std::vector<std::vector<unsigned long> > segments;
+    MeshCore::MeshComponents comp(_kernel);
+    comp.SearchForComponents(MeshCore::MeshComponents::OverEdge,segments);
+    return segments;
+}
+
 unsigned long MeshObject::countComponents() const
 {
     std::vector<std::vector<unsigned long> > segments;
@@ -1170,6 +1247,7 @@ Segment& MeshObject::getSegment(unsigned long index)
 MeshObject* MeshObject::meshFromSegment(const std::vector<unsigned long>& indices) const
 {
     MeshCore::MeshFacetArray facets;
+    facets.reserve(indices.size());
     const MeshCore::MeshPointArray& kernel_p = _kernel.GetPoints();
     const MeshCore::MeshFacetArray& kernel_f = _kernel.GetFacets();
     for (std::vector<unsigned long>::const_iterator it = indices.begin(); it != indices.end(); ++it) {
