@@ -37,6 +37,7 @@
 #include "Core/Iterator.h"
 #include "Core/Elements.h"
 #include "Core/Grid.h"
+#include "Core/MeshKernel.h"
 #include "Core/Triangulation.h"
 
 using namespace Mesh;
@@ -56,19 +57,24 @@ int MeshPy::PyInit(PyObject* args, PyObject*)
         }
         else if (PyList_Check(pcObj)) {
             PyObject* ret = addFacets(args);
+            bool ok = (ret!=0);
             Py_XDECREF(ret);
-            if (!ret)
-                return -1;
+            if (!ok) return -1;
         }
         else if (PyTuple_Check(pcObj)) {
             PyObject* ret = addFacets(args);
+            bool ok = (ret!=0);
             Py_XDECREF(ret);
-            if (!ret)
-                return -1;
+            if (!ok) return -1;
         }
         else if (PyString_Check(pcObj)) {
             getMeshObjectPtr()->load(PyString_AsString(pcObj));
         }
+        else {
+            PyErr_Format(PyExc_TypeError, "Cannot create a mesh out of a '%s'",
+                pcObj->ob_type->tp_name);
+            return -1;
+        }
     }
     catch (const Base::Exception &e) {
         PyErr_SetString(PyExc_Exception,e.what());
@@ -123,20 +129,22 @@ PyObject*  MeshPy::write(PyObject *args)
     if (!PyArg_ParseTuple(args, "s|s",&Name,&Ext))
         return NULL;
 
-    MeshCore::MeshOutput::Format format = MeshCore::MeshOutput::Undefined;
+    MeshCore::MeshIO::Format format = MeshCore::MeshIO::Undefined;
     if (Ext) {
-        std::map<std::string, MeshCore::MeshOutput::Format> ext;
-        ext["BMS" ] = MeshCore::MeshOutput::BMS;
-        ext["STL" ] = MeshCore::MeshOutput::BSTL;
-        ext["AST" ] = MeshCore::MeshOutput::ASTL;
-        ext["OBJ" ] = MeshCore::MeshOutput::OBJ;
-        ext["IV"  ] = MeshCore::MeshOutput::IV;
-        ext["VRML"] = MeshCore::MeshOutput::VRML;
-        ext["WRL" ] = MeshCore::MeshOutput::VRML;
-        ext["WRZ" ] = MeshCore::MeshOutput::WRZ;
-        ext["NAS" ] = MeshCore::MeshOutput::NAS;
-        ext["BDF" ] = MeshCore::MeshOutput::NAS;
-        ext["PY"  ] = MeshCore::MeshOutput::PY;
+        std::map<std::string, MeshCore::MeshIO::Format> ext;
+        ext["BMS" ] = MeshCore::MeshIO::BMS;
+        ext["STL" ] = MeshCore::MeshIO::BSTL;
+        ext["AST" ] = MeshCore::MeshIO::ASTL;
+        ext["OBJ" ] = MeshCore::MeshIO::OBJ;
+        ext["OFF" ] = MeshCore::MeshIO::OFF;
+        ext["IV"  ] = MeshCore::MeshIO::IV;
+        ext["VRML"] = MeshCore::MeshIO::VRML;
+        ext["WRL" ] = MeshCore::MeshIO::VRML;
+        ext["WRZ" ] = MeshCore::MeshIO::WRZ;
+        ext["NAS" ] = MeshCore::MeshIO::NAS;
+        ext["BDF" ] = MeshCore::MeshIO::NAS;
+        ext["PLY" ] = MeshCore::MeshIO::PLY;
+        ext["PY"  ] = MeshCore::MeshIO::PY;
         if (ext.find(Ext) != ext.end())
             format = ext[Ext];
     };
@@ -148,6 +156,35 @@ PyObject*  MeshPy::write(PyObject *args)
     Py_Return; 
 }
 
+PyObject*  MeshPy::writeInventor(PyObject *args)
+{
+    float creaseangle=0.0f;
+    if (!PyArg_ParseTuple(args, "|f",&creaseangle))
+        return NULL;
+
+    MeshObject* mesh = getMeshObjectPtr();
+    const MeshCore::MeshFacetArray& faces = mesh->getKernel().GetFacets();
+    std::vector<int> indices;
+    std::vector<Base::Vector3f> coords;
+    coords.reserve(mesh->countPoints());
+    for (MeshObject::const_point_iterator it = mesh->points_begin(); it != mesh->points_end(); ++it)
+        coords.push_back(Base::Vector3f((float)it->x,(float)it->y,(float)it->z));
+    indices.reserve(4*faces.size());
+    for (MeshCore::MeshFacetArray::_TConstIterator it = faces.begin(); it != faces.end(); ++it) {
+        indices.push_back(it->_aulPoints[0]);
+        indices.push_back(it->_aulPoints[1]);
+        indices.push_back(it->_aulPoints[2]);
+        indices.push_back(-1);
+    }
+
+    std::stringstream result;
+    Base::InventorBuilder builder(result);
+    builder.addIndexedFaceSet(coords, indices, creaseangle);
+    builder.close();
+
+    return Py::new_reference_to(Py::String(result.str()));
+}
+
 PyObject*  MeshPy::offset(PyObject *args)
 {
     float Float;
@@ -551,6 +588,15 @@ PyObject*  MeshPy::setPoint(PyObject *args)
     Py_Return;
 }
 
+PyObject* MeshPy::countSegments(PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return 0;
+
+    unsigned long count = getMeshObjectPtr()->countSegments();
+    return Py_BuildValue("k",count);
+}
+
 PyObject* MeshPy::getSegment(PyObject *args)
 {
     unsigned long index;
@@ -572,6 +618,21 @@ PyObject* MeshPy::getSegment(PyObject *args)
     return Py::new_reference_to(ary);
 }
 
+PyObject* MeshPy::getSeparateComponents(PyObject *args)
+{
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
+    Py::List meshesList;
+    std::vector<std::vector<unsigned long> > segs;
+    segs = getMeshObjectPtr()->getComponents();
+    for (unsigned int i=0; i<segs.size(); i++) {
+        MeshObject* mesh = getMeshObjectPtr()->meshFromSegment(segs[i]);
+        meshesList.append(Py::Object(new MeshPy(mesh),true));
+    }
+    return Py::new_reference_to(meshesList);
+}
+
 PyObject* MeshPy::meshFromSegment(PyObject *args)
 {
     PyObject* list;
@@ -1263,3 +1324,5 @@ Py::Tuple MeshPy::getTopology(void) const
     tuple.setItem(1, facet);
     return tuple;
 }
+
+
