@@ -47,7 +47,7 @@
 #include <Base/Exception.h>
 #include <App/Document.h>
 #include <App/DocumentObjectGroup.h>
-#include <App/Feature.h>
+#include <App/DocumentObject.h>
 #include <Gui/Application.h>
 #include <Gui/BitmapFactory.h>
 #include <Gui/MainWindow.h>
@@ -61,6 +61,7 @@
 
 #include "DlgEvaluateMeshImp.h"
 #include "DlgRegularSolidImp.h"
+#include "RemoveComponents.h"
 #include "ViewProviderMeshFaceSet.h"
 #include "ViewProviderCurvature.h"
 
@@ -439,6 +440,7 @@ void CmdMeshImport::activated(int iMsg)
     filter << QObject::tr("Binary Mesh (*.bms)");
     filter << QObject::tr("Alias Mesh (*.obj)");
     filter << QObject::tr("Inventor V2.1 ascii (*.iv)");
+    filter << QObject::tr("Stanford Polygon (*.ply)");
     //filter << "Nastran (*.nas *.bdf)";
     filter << QObject::tr("All Files (*.*)");
 
@@ -496,7 +498,9 @@ void CmdMeshExport::activated(int iMsg)
     ext << qMakePair<QString, QByteArray>(QObject::tr("ASCII STL (*.ast)"), "AST");
     ext << qMakePair<QString, QByteArray>(QObject::tr("Binary Mesh (*.bms)"), "BMS");
     ext << qMakePair<QString, QByteArray>(QObject::tr("Alias Mesh (*.obj)"), "OBJ");
+    ext << qMakePair<QString, QByteArray>(QObject::tr("Object File Format (*.off)"), "OFF");
     ext << qMakePair<QString, QByteArray>(QObject::tr("Inventor V2.1 ascii (*.iv)"), "IV");
+    ext << qMakePair<QString, QByteArray>(QObject::tr("Standford Polygon (*.ply)"), "PLY");
     ext << qMakePair<QString, QByteArray>(QObject::tr("VRML V2.0 (*.wrl *.vrml)"), "VRML");
     ext << qMakePair<QString, QByteArray>(QObject::tr("Compressed VRML 2.0 (*.wrz)"), "WRZ");
     ext << qMakePair<QString, QByteArray>(QObject::tr("Nastran (*.nas *.bdf)"), "NAS");
@@ -950,31 +954,31 @@ DEF_STD_CMD_A(CmdMeshEvaluation);
 CmdMeshEvaluation::CmdMeshEvaluation()
   :Command("Mesh_Evaluation")
 {
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Evaluate mesh...");
-  sToolTipText  = QT_TR_NOOP("Opens a dialog to analyze and repair a mesh");
-  sWhatsThis    = "Mesh_Evaluation";
-  sStatusTip    = QT_TR_NOOP("Opens a dialog to analyze and repair a mesh");
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    // needs two ampersands to display one
+    sMenuText     = QT_TR_NOOP("Evaluate && Repair mesh...");
+    sToolTipText  = QT_TR_NOOP("Opens a dialog to analyze and repair a mesh");
+    sWhatsThis    = "Mesh_Evaluation";
+    sStatusTip    = QT_TR_NOOP("Opens a dialog to analyze and repair a mesh");
 }
 
 void CmdMeshEvaluation::activated(int iMsg)
 {
-  if (MeshGui::DockEvaluateMeshImp::hasInstance()) {
-    MeshGui::DockEvaluateMeshImp::instance()->show();
-    return;
-  }
+    if (MeshGui::DockEvaluateMeshImp::hasInstance()) {
+        MeshGui::DockEvaluateMeshImp::instance()->show();
+        return;
+    }
 
-  MeshGui::DlgEvaluateMeshImp* dlg = MeshGui::DockEvaluateMeshImp::instance();
-  dlg->setAttribute(Qt::WA_DeleteOnClose);
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    dlg->setMesh((Mesh::Feature*)(*it));
-    break;
-  }
+    MeshGui::DlgEvaluateMeshImp* dlg = MeshGui::DockEvaluateMeshImp::instance();
+    dlg->setAttribute(Qt::WA_DeleteOnClose);
+    std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
+    for (std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it) {
+        dlg->setMesh((Mesh::Feature*)(*it));
+        break;
+    }
 
-  dlg->show();
+    dlg->show();
 }
 
 bool CmdMeshEvaluation::isActive(void)
@@ -1030,6 +1034,36 @@ bool CmdMeshEvaluateFacet::isActive(void)
 
 //--------------------------------------------------------------------------------------
 
+DEF_STD_CMD_A(CmdMeshRemoveComponents);
+
+CmdMeshRemoveComponents::CmdMeshRemoveComponents()
+  : Command("Mesh_RemoveComponents")
+{
+    sAppModule    = "Mesh";
+    sGroup        = QT_TR_NOOP("Mesh");
+    sMenuText     = QT_TR_NOOP("Remove components...");
+    sToolTipText  = QT_TR_NOOP("Remove topologic independant components from the mesh");
+    sWhatsThis    = "Mesh_RemoveComponents";
+    sStatusTip    = QT_TR_NOOP("Remove topologic independant components from the mesh");
+}
+
+void CmdMeshRemoveComponents::activated(int iMsg)
+{
+    static QPointer<MeshGui::RemoveComponents> dlg=0;
+    if (!dlg)
+        dlg = new MeshGui::RemoveComponents(Gui::getMainWindow());
+    dlg->setAttribute(Qt::WA_DeleteOnClose);
+    dlg->show();
+}
+
+bool CmdMeshRemoveComponents::isActive(void)
+{
+    // Check for the selected mesh feature (all Mesh types)
+    return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
+}
+
+//--------------------------------------------------------------------------------------
+
 DEF_STD_CMD_A(CmdMeshRemoveCompByHand);
 
 CmdMeshRemoveCompByHand::CmdMeshRemoveCompByHand()
@@ -1489,50 +1523,6 @@ bool CmdMeshFillupHoles::isActive(void)
 
 //--------------------------------------------------------------------------------------
 
-DEF_STD_CMD_A(CmdMeshRemoveComponents);
-
-CmdMeshRemoveComponents::CmdMeshRemoveComponents()
-  :Command("Mesh_RemoveComponents")
-{
-  sAppModule    = "Mesh";
-  sGroup        = QT_TR_NOOP("Mesh");
-  sMenuText     = QT_TR_NOOP("Remove components...");
-  sToolTipText  = QT_TR_NOOP("Remove topologic independant components from the mesh");
-  sWhatsThis    = "Mesh_RemoveComponents";
-  sStatusTip    = QT_TR_NOOP("Remove topologic independant components from the mesh");
-//  sPixmap       = "curv_info";
-}
-
-void CmdMeshRemoveComponents::activated(int iMsg)
-{
-  std::vector<App::DocumentObject*> meshes = getSelection().getObjectsOfType(Mesh::Feature::getClassTypeId());
-  bool ok;
-  int RemoveCompOfSize = QInputDialog::getInteger( Gui::getMainWindow(), QObject::tr("Remove components"), QObject::tr("Removes components up to a maximum number of triangles:"), 
-                                                      3, 1, 10000000, 1, &ok );
-  if (!ok) return;
-
-  openCommand("Remove components");
-  for ( std::vector<App::DocumentObject*>::const_iterator it = meshes.begin(); it != meshes.end(); ++it )
-  {
-    std::string fName = (*it)->getNameInDocument();
-    fName += "_rem_comps";
-    fName = getUniqueObjectName(fName.c_str());
-    doCommand(Doc,"App.activeDocument().addObject(\"Mesh::RemoveComponents\",\"%s\")",fName.c_str());
-    doCommand(Doc,"App.activeDocument().%s.Source = App.activeDocument().%s",fName.c_str(),(*it)->getNameInDocument());
-    doCommand(Doc,"App.activeDocument().%s.RemoveCompOfSize = %d",fName.c_str(), RemoveCompOfSize);
-  }
-  commitCommand();
-  updateActive();
-}
-
-bool CmdMeshRemoveComponents::isActive(void)
-{
-  // Check for the selected mesh feature (all Mesh types)
-  return getSelection().countObjectsOfType(Mesh::Feature::getClassTypeId()) > 0;
-}
-
-//--------------------------------------------------------------------------------------
-
 DEF_STD_CMD_A(CmdMeshFillInteractiveHole);
 
 CmdMeshFillInteractiveHole::CmdMeshFillInteractiveHole()
