@@ -39,13 +39,14 @@
 # include <Inventor/nodes/SoOrthographicCamera.h>
 # include <Inventor/nodes/SoPolygonOffset.h>
 # include <Inventor/nodes/SoShapeHints.h>
+# include <Inventor/nodes/SoSeparator.h>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
 #include <Base/Console.h>
 #include <Base/Exception.h>
 #include <Base/Sequencer.h>
-#include <Base/Tools2D.h>
+#include <Base/Tools.h>
 #include <Base/ViewProj.h>
 
 #include <App/Document.h>
@@ -89,6 +90,57 @@ using MeshCore::MeshFacetIterator;
 using MeshCore::MeshGeomFacet;
 using MeshCore::MeshFacet;
 
+void ViewProviderMeshBuilder::buildNodes(const App::Property* prop, std::vector<SoNode*>& nodes) const
+{
+    SoCoordinate3 *pcPointsCoord=0;
+    SoIndexedFaceSet *pcFaces=0;
+
+    if (nodes.empty()) {
+        pcPointsCoord = new SoCoordinate3();
+        nodes.push_back(pcPointsCoord);
+        pcFaces = new SoIndexedFaceSet();
+        nodes.push_back(pcFaces);
+    }
+    else if (nodes.size() == 2) {
+        if (nodes[0]->getTypeId() == SoCoordinate3::getClassTypeId())
+            pcPointsCoord = static_cast<SoCoordinate3*>(nodes[0]);
+        if (nodes[1]->getTypeId() == SoIndexedFaceSet::getClassTypeId())
+            pcFaces = static_cast<SoIndexedFaceSet*>(nodes[1]);
+    }
+
+    if (pcPointsCoord && pcFaces)
+        createMesh(prop, pcPointsCoord, pcFaces);
+}
+
+void ViewProviderMeshBuilder::createMesh(const App::Property* prop, SoCoordinate3* coords, SoIndexedFaceSet* faces) const
+{
+    const Mesh::PropertyMeshKernel* mesh = static_cast<const Mesh::PropertyMeshKernel*>(prop);
+    const MeshCore::MeshKernel& rcMesh = mesh->getValue().getKernel();
+
+    // set the point coordinates
+    const MeshCore::MeshPointArray& cP = rcMesh.GetPoints();
+    coords->point.setNum(rcMesh.CountPoints());
+    SbVec3f* verts = coords->point.startEditing();
+    unsigned long i=0;
+    for (MeshCore::MeshPointArray::_TConstIterator it = cP.begin(); it != cP.end(); ++it, i++) {
+        verts[i].setValue(it->x, it->y, it->z);
+    }
+    coords->point.finishEditing();
+
+    // set the face indices
+    unsigned long j=0;
+    const MeshCore::MeshFacetArray& cF = rcMesh.GetFacets();
+    faces->coordIndex.setNum(4*rcMesh.CountFacets());
+    int32_t* indices = faces->coordIndex.startEditing();
+    for (MeshCore::MeshFacetArray::_TConstIterator it = cF.begin(); it != cF.end(); ++it, j++) {
+        for (int i=0; i<3; i++) {
+            indices[4*j+i] = it->_aulPoints[i];
+        }
+        indices[4*j+3] = SO_END_FACE_INDEX;
+    }
+    faces->coordIndex.finishEditing();
+}
+
 PROPERTY_SOURCE(MeshGui::ViewProviderExport, Gui::ViewProviderDocumentObject)
 
 ViewProviderExport::ViewProviderExport()
@@ -169,7 +221,7 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
     ADD_PROPERTY(OpenEdges,(false));
     ADD_PROPERTY(Lighting,(1));
     Lighting.setEnums(LightingEnums);
-    ADD_PROPERTY(LineColor,(ShapeColor.getValue()));
+    ADD_PROPERTY(LineColor,(0,0,0));
     //ADD_PROPERTY(BacksideColor,(0.8f,0.8f,0.8f));
 
     pOpenColor = new SoBaseColor();
@@ -208,6 +260,8 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
 
     // read the correct shape color from the preferences
     Base::Reference<ParameterGrp> hGrp = Gui::WindowParameter::getDefaultParameter()->GetGroup("Mod/Mesh");
+
+    // Mesh color
     App::Color color = ShapeColor.getValue();
     unsigned long current = color.getPackedValue();
     unsigned long setting = hGrp->GetUnsigned("MeshColor", current);
@@ -215,6 +269,17 @@ ViewProviderMesh::ViewProviderMesh() : pcOpenEdge(0), m_bEdit(false)
         color.setPackedValue((uint32_t)setting);
         ShapeColor.setValue(color);
     }
+    Transparency.setValue(hGrp->GetInt("MeshTransparency", 0));
+
+    // Line color
+    color = LineColor.getValue();
+    current = color.getPackedValue();
+    setting = hGrp->GetUnsigned("LineColor", current);
+    if (current != setting) {
+        color.setPackedValue((uint32_t)setting);
+        LineColor.setValue(color);
+    }
+    LineTransparency.setValue(hGrp->GetInt("LineTransparency", 0));
 
     bool twoside = hGrp->GetBool("TwoSideRendering", false);
     if (twoside) Lighting.setValue(1);
@@ -574,15 +639,20 @@ void ViewProviderMesh::clipMeshCallback(void * ud, SoEventCallback * n)
         clPoly.push_back(clPoly.front());
 
     std::vector<Gui::ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
-    for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
-        ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
-        if (that->m_bEdit) {
-            that->unsetEdit();
-            that->cutMesh(clPoly, *view, clip_inner);
+    if (!views.empty()) {
+        Gui::Application::Instance->activeDocument()->openCommand("Cut");
+        for (std::vector<Gui::ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
+            ViewProviderMesh* that = static_cast<ViewProviderMesh*>(*it);
+            if (that->m_bEdit) {
+                that->unsetEdit();
+                that->cutMesh(clPoly, *view, clip_inner);
+            }
         }
-    }
 
-    view->render();
+        Gui::Application::Instance->activeDocument()->commitCommand();
+
+        view->render();
+    }
 }
 
 void ViewProviderMesh::partMeshCallback(void * ud, SoEventCallback * cb)
@@ -751,18 +821,6 @@ void ViewProviderMesh::selectGLCallback(void * ud, SoEventCallback * n)
     view->render();
 }
 
-namespace MeshGui {
-template <class T>
-struct iotaGen {
-public:
-    T operator()() { return n++; }
-    iotaGen(T v) : n(v) {}
-
-private:
-    T n;
-};
-}
-
 void ViewProviderMesh::getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
                                             Gui::View3DInventorViewer &Viewer,
                                             SbBool inner,
@@ -793,7 +851,7 @@ void ViewProviderMesh::getFacetsFromPolygon(const std::vector<SbVec2f>& picked,
     if (!inner) {
         // get the indices that are completely outside
         std::vector<unsigned long> complete(meshProp.getValue().countFacets());
-        std::generate(complete.begin(), complete.end(), iotaGen<unsigned long>(0));
+        std::generate(complete.begin(), complete.end(), Base::iotaGen<unsigned long>(0));
         std::sort(indices.begin(), indices.end());
         std::vector<unsigned long> complementary;
         std::back_insert_iterator<std::vector<unsigned long> > biit(complementary);
@@ -816,10 +874,8 @@ void ViewProviderMesh::cutMesh(const std::vector<SbVec2f>& picked,
     Mesh::PropertyMeshKernel& meshProp = static_cast<Mesh::Feature*>(pcObject)->Mesh;
 
     //Remove the facets from the mesh and open a transaction object for the undo/redo stuff
-    Gui::Application::Instance->activeDocument()->openCommand("Cut");
-    meshProp.deleteFacetIndices( indices );
-    Gui::Application::Instance->activeDocument()->commitCommand();
-    ((Mesh::Feature*)pcObject)->purgeTouched();
+    meshProp.deleteFacetIndices(indices);
+    pcObject->purgeTouched();
 }
 
 void ViewProviderMesh::splitMesh(const MeshCore::MeshKernel& toolMesh, const Base::Vector3f& normal, SbBool clip_inner)
@@ -836,7 +892,7 @@ void ViewProviderMesh::splitMesh(const MeshCore::MeshKernel& toolMesh, const Bas
     if (!clip_inner) {
         // get the indices that are completely outside
         std::vector<unsigned long> complete(meshPropKernel.CountFacets());
-        std::generate(complete.begin(), complete.end(), iotaGen<unsigned long>(0));
+        std::generate(complete.begin(), complete.end(), Base::iotaGen<unsigned long>(0));
         std::sort(indices.begin(), indices.end());
         std::vector<unsigned long> complementary;
         std::back_insert_iterator<std::vector<unsigned long> > biit(complementary);
@@ -868,7 +924,7 @@ void ViewProviderMesh::segmentMesh(const MeshCore::MeshKernel& toolMesh, const B
     if (!clip_inner) {
         // get the indices that are completely outside
         std::vector<unsigned long> complete(meshPropKernel.CountFacets());
-        std::generate(complete.begin(), complete.end(), iotaGen<unsigned long>(0));
+        std::generate(complete.begin(), complete.end(), Base::iotaGen<unsigned long>(0));
         std::sort(indices.begin(), indices.end());
         std::vector<unsigned long> complementary;
         std::back_insert_iterator<std::vector<unsigned long> > biit(complementary);
@@ -984,6 +1040,8 @@ void ViewProviderMesh::markPartCallback(void * ud, SoEventCallback * n)
             // context-menu
             QMenu menu;
             QAction* cl = menu.addAction(QObject::tr("Leave removal mode"));
+            QAction* rm = menu.addAction(QObject::tr("Delete selected faces"));
+            QAction* cf = menu.addAction(QObject::tr("Clear selected faces"));
             QAction* id = menu.exec(QCursor::pos());
             if (cl == id) {
                 view->setEditing(false);
@@ -991,9 +1049,24 @@ void ViewProviderMesh::markPartCallback(void * ud, SoEventCallback * n)
 
                 std::vector<ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
                 for (std::vector<ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
-                    static_cast<ViewProviderMesh*>(*it)->unmarkParts();
+                    static_cast<ViewProviderMesh*>(*it)->clearSelection();
                 }
             }
+            else if (cf == id) {
+                std::vector<ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
+                for (std::vector<ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
+                    static_cast<ViewProviderMesh*>(*it)->clearSelection();
+                }
+            }
+            else if (rm == id) {
+                Gui::Application::Instance->activeDocument()->openCommand("Delete");
+                std::vector<ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
+                for (std::vector<ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
+                    static_cast<ViewProviderMesh*>(*it)->deleteSelection();
+                }
+                view->render();
+                Gui::Application::Instance->activeDocument()->commitCommand();
+            }
         }
         else if (mbe->getButton() == SoMouseButtonEvent::BUTTON1 && mbe->getState() == SoButtonEvent::DOWN) {
             const SoPickedPoint * point = n->getPickedPoint();
@@ -1013,42 +1086,10 @@ void ViewProviderMesh::markPartCallback(void * ud, SoEventCallback * n)
             const SoDetail* detail = point->getDetail(that->getShapeNode());
             if ( detail && detail->getTypeId() == SoFaceDetail::getClassTypeId() ) {
                 // get the boundary to the picked facet
-                unsigned long uFacet = ((SoFaceDetail*)detail)->getFaceIndex();
-                if (that->isMarked(uFacet)) {
-                    Gui::Application::Instance->activeDocument()->openCommand("Remove");
-                    that->removePart();
-                    view->render();
-                    Gui::Application::Instance->activeDocument()->commitCommand();
-                }
-                else {
-                    that->markPart(uFacet);
-                }
+                unsigned long uFacet = static_cast<const SoFaceDetail*>(detail)->getFaceIndex();
+                that->selectComponent(uFacet);
             }
         }
-#if 0
-        else if (mbe->getButton() == SoMouseButtonEvent::BUTTON3 && mbe->getState() == SoButtonEvent::DOWN) {
-            Gui::View3DInventorViewer* view  = reinterpret_cast<Gui::View3DInventorViewer*>(n->getUserData());
-            std::vector<ViewProvider*> views = view->getViewProvidersOfType(ViewProviderMesh::getClassTypeId());
-
-            // Check if facets are marked
-            unsigned long count = 0;
-            for (std::vector<ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
-                count += static_cast<ViewProviderMesh*>(*it)->countMarkedFacets();
-            }
-
-            if (count > 0) {
-                Gui::Application::Instance->activeDocument()->openCommand("Remove");
-                for (std::vector<ViewProvider*>::iterator it = views.begin(); it != views.end(); ++it) {
-                    unsigned long count = static_cast<ViewProviderMesh*>(*it)->countMarkedFacets();
-                    static_cast<ViewProviderMesh*>(*it)->removePart();
-                    if (count > 0)
-                        view->render();
-                }
-                
-                Gui::Application::Instance->activeDocument()->commitCommand();
-            }
-        }
-#endif
     }
 }
 
@@ -1119,7 +1160,7 @@ void ViewProviderMesh::fillHole(unsigned long uFacet)
             }
         }
     }
- 
+
     if (newFacets.empty())
         return; // nothing to do
  
@@ -1129,64 +1170,132 @@ void ViewProviderMesh::fillHole(unsigned long uFacet)
     Gui::Application::Instance->activeDocument()->commitCommand();
 }
 
-void ViewProviderMesh::markPart(unsigned long uFacet)
+void ViewProviderMesh::selectFacet(unsigned long facet)
 {
-    _markedFacets.push_back(uFacet);
-    MeshCore::MeshTopFacetVisitor clVisitor(_markedFacets);
-    const MeshCore::MeshKernel& rKernel = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue().getKernel();
-    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::VISIT);
-    rKernel.VisitNeighbourFacets(clVisitor, uFacet);
+    std::vector<unsigned long> selection;
+    selection.push_back(facet);
 
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.addFacetsToSelection(selection);
+
+    // Colorize the selection
     pcMatBinding->value = SoMaterialBinding::PER_FACE;
     App::Color c = ShapeColor.getValue();
-    unsigned long uCtFacets = rKernel.CountFacets();
-    pcShapeMaterial->diffuseColor.setNum(uCtFacets);
-    for (unsigned long i=0; i<uCtFacets; i++)
-        pcShapeMaterial->diffuseColor.set1Value(i, c.r,c.g,c.b);
-    for (std::vector<unsigned long>::iterator it = _markedFacets.begin(); it != _markedFacets.end(); ++it)
-        pcShapeMaterial->diffuseColor.set1Value(*it, 1.0f,0.0f,0.0f);
-}
+    int uCtFacets = (int)rMesh.countFacets();
 
-bool ViewProviderMesh::isMarked(unsigned long facet) const
-{
-    for (std::vector<unsigned long>::const_iterator it = _markedFacets.begin(); it != _markedFacets.end(); ++it) {
-        if (*it == facet)
-            return true;
+    if (uCtFacets != pcShapeMaterial->diffuseColor.getNum()) {
+        highlightSelection();
+    }
+    else {
+        pcShapeMaterial->diffuseColor.set1Value(facet,1.0f,0.0f,0.0f);
     }
-    return false;
 }
 
-void ViewProviderMesh::unmarkParts()
+void ViewProviderMesh::deselectFacet(unsigned long facet)
 {
-    _markedFacets.clear();
-    pcMatBinding->value = SoMaterialBinding::OVERALL;
+    std::vector<unsigned long> selection;
+    selection.push_back(facet);
+
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.removeFacetsFromSelection(selection);
+
+    // Colorize the selection
+    pcMatBinding->value = SoMaterialBinding::PER_FACE;
     App::Color c = ShapeColor.getValue();
-    pcShapeMaterial->diffuseColor.setNum(0);
-    pcShapeMaterial->diffuseColor.setValue(c.r,c.g,c.b);
-}
+    int uCtFacets = (int)rMesh.countFacets();
 
-void ViewProviderMesh::removePart()
-{
-    if (!_markedFacets.empty()) {
-        // Get the attached mesh property
-        Mesh::PropertyMeshKernel& meshProp = ((Mesh::Feature*)pcObject)->Mesh;
-        meshProp.deleteFacetIndices(_markedFacets);
-        ((Mesh::Feature*)pcObject)->purgeTouched();
-        
-        // notify the mesh shape node
-        const MeshCore::MeshKernel& rKernel = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue().getKernel();
+    if (uCtFacets != pcShapeMaterial->diffuseColor.getNum()) {
+        highlightSelection();
+    }
+    else {
         App::Color c = ShapeColor.getValue();
-        unsigned long uCtFacets = rKernel.CountFacets();
-        pcShapeMaterial->diffuseColor.setNum(uCtFacets);
-        for (unsigned long i=0; i<uCtFacets; i++)
-            pcShapeMaterial->diffuseColor.set1Value(i, c.r,c.g,c.b);
-        _markedFacets.clear();
+        pcShapeMaterial->diffuseColor.set1Value(facet,c.r,c.g,c.b);
     }
 }
 
-unsigned long ViewProviderMesh::countMarkedFacets() const
+void ViewProviderMesh::selectComponent(unsigned long uFacet)
 {
-    return _markedFacets.size();
+    std::vector<unsigned long> selection;
+    selection.push_back(uFacet);
+
+    MeshCore::MeshTopFacetVisitor clVisitor(selection);
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
+    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::VISIT);
+    rKernel.VisitNeighbourFacets(clVisitor, uFacet);
+    rMesh.addFacetsToSelection(selection);
+
+    // Colorize the selection
+    highlightSelection();
+}
+
+void ViewProviderMesh::deselectComponent(unsigned long uFacet)
+{
+    std::vector<unsigned long> selection;
+    selection.push_back(uFacet);
+
+    MeshCore::MeshTopFacetVisitor clVisitor(selection);
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
+    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::VISIT);
+    rKernel.VisitNeighbourFacets(clVisitor, uFacet);
+    rMesh.removeFacetsFromSelection(selection);
+
+    // Colorize the selection
+    highlightSelection();
+}
+
+void ViewProviderMesh::setSelection(const std::vector<unsigned long>& indices)
+{
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
+    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::SELECTED);
+    rMesh.addFacetsToSelection(indices);
+
+    // Colorize the selection
+    highlightSelection();
+}
+
+void ViewProviderMesh::addSelection(const std::vector<unsigned long>& indices)
+{
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.addFacetsToSelection(indices);
+
+    // Colorize the selection
+    highlightSelection();
+}
+
+void ViewProviderMesh::removeSelection(const std::vector<unsigned long>& indices)
+{
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.removeFacetsFromSelection(indices);
+
+    // Colorize the selection
+    highlightSelection();
+}
+
+void ViewProviderMesh::clearSelection()
+{
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    const MeshCore::MeshKernel& rKernel = rMesh.getKernel();
+    MeshCore::MeshAlgorithm(rKernel).ResetFacetFlag(MeshCore::MeshFacet::SELECTED);
+    unhighlightSelection();
+}
+
+void ViewProviderMesh::deleteSelection()
+{
+    std::vector<unsigned long> indices;
+    Mesh::PropertyMeshKernel& meshProp = static_cast<Mesh::Feature*>(pcObject)->Mesh;
+    const Mesh::MeshObject& rMesh = meshProp.getValue();
+    rMesh.getFacetsFromSelection(indices);
+    if (!indices.empty()) {
+        unhighlightSelection();
+
+        Mesh::MeshObject* pMesh = meshProp.startEditing();
+        pMesh->deleteFacets(indices);
+        meshProp.finishEditing();
+        pcObject->purgeTouched();
+    }
 }
 
 void ViewProviderMesh::selectArea(short x, short y, short w, short h)
@@ -1196,26 +1305,41 @@ void ViewProviderMesh::selectArea(short x, short y, short w, short h)
     gl.w = w; gl.h = h;
     gl.apply(this->pcHighlight);
 
-    _markedFacets.clear();
-    _markedFacets.reserve(gl.indices.size());
-    for (std::vector<unsigned int>::iterator it = gl.indices.begin(); it != gl.indices.end(); ++it)
-        _markedFacets.push_back(*it);
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.addFacetsToSelection(gl.indices);
 
     // Colorize the selected part
+    highlightSelection();
+}
+
+void ViewProviderMesh::highlightSelection()
+{
+    std::vector<unsigned long> selection;
+    const Mesh::MeshObject& rMesh = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue();
+    rMesh.getFacetsFromSelection(selection);
+
+    // Colorize the selection
     pcMatBinding->value = SoMaterialBinding::PER_FACE;
     App::Color c = ShapeColor.getValue();
-    const MeshCore::MeshKernel& rKernel = static_cast<Mesh::Feature*>(pcObject)->Mesh.getValue().getKernel();
-    unsigned long uCtFacets = rKernel.CountFacets();
+    int uCtFacets = (int)rMesh.countFacets();
     pcShapeMaterial->diffuseColor.setNum(uCtFacets);
 
     SbColor* cols = pcShapeMaterial->diffuseColor.startEditing();
-    for (unsigned long i=0; i<uCtFacets; i++)
+    for (int i=0; i<uCtFacets; i++)
         cols[i].setValue(c.r,c.g,c.b);
-    for (std::vector<unsigned long>::iterator it = _markedFacets.begin(); it != _markedFacets.end(); ++it)
+    for (std::vector<unsigned long>::iterator it = selection.begin(); it != selection.end(); ++it)
         cols[*it].setValue(1.0f,0.0f,0.0f);
     pcShapeMaterial->diffuseColor.finishEditing();
 }
 
+void ViewProviderMesh::unhighlightSelection()
+{
+    App::Color c = ShapeColor.getValue();
+    pcMatBinding->value = SoMaterialBinding::OVERALL;
+    pcShapeMaterial->diffuseColor.setNum(1);
+    pcShapeMaterial->diffuseColor.setValue(c.r,c.g,c.b);
+}
+
 // ------------------------------------------------------
 
 PROPERTY_SOURCE(MeshGui::ViewProviderIndexedFaceSet, MeshGui::ViewProviderMesh)
@@ -1228,32 +1352,6 @@ ViewProviderIndexedFaceSet::~ViewProviderIndexedFaceSet()
 {
 }
 
-void ViewProviderIndexedFaceSet::createMesh(const MeshCore::MeshKernel& rcMesh)
-{
-    // set the point coordinates
-    const MeshCore::MeshPointArray& cP = rcMesh.GetPoints();
-    pcMeshCoord->point.setNum(rcMesh.CountPoints());
-    SbVec3f* verts = pcMeshCoord->point.startEditing();
-    unsigned long i=0;
-    for (MeshCore::MeshPointArray::_TConstIterator it = cP.begin(); it != cP.end(); ++it, i++) {
-        verts[i].setValue(it->x, it->y, it->z);
-    }
-    pcMeshCoord->point.finishEditing();
-
-    // set the face indices
-    unsigned long j=0;
-    const MeshCore::MeshFacetArray& cF = rcMesh.GetFacets();
-    pcMeshFaces->coordIndex.setNum(4*rcMesh.CountFacets());
-    int32_t* indices = pcMeshFaces->coordIndex.startEditing();
-    for (MeshCore::MeshFacetArray::_TConstIterator it = cF.begin(); it != cF.end(); ++it, j++) {
-        for (int i=0; i<3; i++) {
-            indices[4*j+i] = it->_aulPoints[i];
-        }
-        indices[4*j+3] = SO_END_FACE_INDEX;
-    }
-    pcMeshFaces->coordIndex.finishEditing();
-}
-
 /** 
  * Extracts the mesh data from the feature \a pcFeature and creates
  * an Inventor node \a SoNode with these data. 
@@ -1278,8 +1376,8 @@ void ViewProviderIndexedFaceSet::updateData(const App::Property* prop)
 {
     Gui::ViewProviderGeometryObject::updateData(prop);
     if (prop->getTypeId() == Mesh::PropertyMeshKernel::getClassTypeId()) {
-        const Mesh::PropertyMeshKernel* mesh = static_cast<const Mesh::PropertyMeshKernel*>(prop);
-        createMesh(mesh->getValue().getKernel());
+        ViewProviderMeshBuilder builder;
+        builder.createMesh(prop, pcMeshCoord, pcMeshFaces);
         showOpenEdges(OpenEdges.getValue());
     }
 }
