@@ -30,6 +30,7 @@
 # include <Inventor/nodes/SoFaceSet.h>
 # include <Inventor/nodes/SoLineSet.h>
 # include <Inventor/nodes/SoMarkerSet.h>
+# include <Inventor/nodes/SoSeparator.h>
 # include <Inventor/nodes/SoShapeHints.h>
 #endif
 
@@ -39,7 +40,6 @@
 #include <Base/Exception.h>
 #include <Base/Sequencer.h>
 #include <App/Application.h>
-#include <App/Feature.h>
 #include <Gui/Selection.h>
 #include <Gui/SoFCSelection.h>
 
