@@ -22,6 +22,7 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <BRepAdaptor_Curve.hxx>
 # include <BRepCheck_Analyzer.hxx>
 # include <BRepPrimAPI_MakeBox.hxx>
 # include <BRepPrimAPI_MakeCone.hxx>
@@ -49,6 +50,7 @@
 # include <Geom_Line.hxx>
 # include <Geom_Plane.hxx>
 # include <Geom_BSplineSurface.hxx>
+# include <Geom_ConicalSurface.hxx>
 # include <Geom_CylindricalSurface.hxx>
 # include <Geom_OffsetSurface.hxx>
 # include <GeomAPI_PointsToBSplineSurface.hxx>
@@ -72,12 +74,13 @@
 # include <Precision.hxx>
 #endif
 
-#include <stdio.h>
-
-#if defined (_POSIX_C_SOURCE)
-#   undef  _POSIX_C_SOURCE
-#endif // (re-)defined in pyconfig.h
-#include <Python.h>
+#include <BRepOffsetAPI_ThruSections.hxx>
+#include <BSplCLib.hxx>
+#include <GeomFill_AppSurf.hxx>
+#include <GeomFill_Line.hxx>
+#include <GeomFill_Pipe.hxx>
+#include <GeomFill_SectionGenerator.hxx>
+#include <NCollection_List.hxx>
 
 #include <Base/Console.h>
 #include <Base/PyObjectBase.h>
@@ -101,12 +104,15 @@
 #include "TopoShapeShellPy.h"
 #include "TopoShapeVertexPy.h"
 #include "GeometryPy.h"
+#include "GeometryCurvePy.h"
+#include "BSplineSurfacePy.h"
 #include "FeaturePartBox.h"
 #include "FeaturePartCut.h"
 #include "FeaturePartImportStep.h"
 #include "FeaturePartImportIges.h"
 #include "FeaturePartImportBrep.h"
 #include "ImportStep.h"
+#include "edgecluster.h"
 
 using Base::Console;
 using namespace Part;
@@ -148,22 +154,21 @@ static PyObject * open(PyObject *self, PyObject *args)
 #endif 
             pcDoc->recompute();
         }
-        else if (file.hasExtension("igs") || file.hasExtension("iges")) {
-            // create new document and add Import feature
-            App::Document *pcDoc = App::GetApplication().newDocument(file.fileNamePure().c_str());
-            Part::ImportIges *pcFeature = (Part::ImportIges*) pcDoc->addObject("Part::ImportIges",file.fileNamePure().c_str());
-            pcFeature->FileName.setValue(Name);
-            pcDoc->recompute();
-        }
-        else if (file.hasExtension("brp") || file.hasExtension("brep")) {
-            // create new document and add Import feature
-            App::Document *pcDoc = App::GetApplication().newDocument(file.fileNamePure().c_str());
-            Part::ImportBrep *pcFeature = (Part::ImportBrep *)pcDoc->addObject("Part::ImportBrep",file.fileNamePure().c_str());
-            pcFeature->FileName.setValue(Name);
-            pcDoc->recompute();
-        }
         else {
-            Py_Error(PyExc_Exception,"unknown file ending");
+            try {
+                TopoShape shape;
+                shape.read(Name);
+
+                // create new document set loaded shape
+                App::Document *pcDoc = App::GetApplication().newDocument(file.fileNamePure().c_str());
+                Part::Feature *object = static_cast<Part::Feature *>(pcDoc->addObject
+                    ("Part::Feature",file.fileNamePure().c_str()));
+                object->Shape.setValue(shape);
+                pcDoc->recompute();
+            }
+            catch (const Base::Exception& e) {
+                Py_Error(PyExc_Exception, e.what());
+            }
         }
     } PY_CATCH;
 
@@ -187,8 +192,7 @@ static PyObject * insert(PyObject *self, PyObject *args)
             Py_Error(PyExc_Exception,"no file ending");
         App::Document *pcDoc = App::GetApplication().getDocument(DocName);
         if (!pcDoc) {
-            PyErr_Format(PyExc_Exception, "Import called to the non-existing document '%s'", DocName);
-            return NULL;
+            pcDoc = App::GetApplication().newDocument(DocName);
         }
 
         if (file.hasExtension("stp") || file.hasExtension("step")) {
@@ -201,18 +205,19 @@ static PyObject * insert(PyObject *self, PyObject *args)
 #endif 
             pcDoc->recompute();
         }
-        else if (file.hasExtension("igs") || file.hasExtension("iges")) {
-            Part::ImportIges *pcFeature = (Part::ImportIges *)pcDoc->addObject("Part::ImportIges",file.fileNamePure().c_str());
-            pcFeature->FileName.setValue(Name);
-            pcDoc->recompute();
-        }
-        else if (file.hasExtension("brp") || file.hasExtension("brep")) {
-            Part::ImportBrep *pcFeature = (Part::ImportBrep *) pcDoc->addObject("Part::ImportBrep",file.fileNamePure().c_str());
-            pcFeature->FileName.setValue(Name);
-            pcDoc->recompute();
-        }
         else {
-            Py_Error(PyExc_Exception,"unknown file ending");
+            try {
+                TopoShape shape;
+                shape.read(Name);
+
+                Part::Feature *object = static_cast<Part::Feature *>(pcDoc->addObject
+                    ("Part::Feature",file.fileNamePure().c_str()));
+                object->Shape.setValue(shape);
+                pcDoc->recompute();
+            }
+            catch (const Base::Exception& e) {
+                Py_Error(PyExc_Exception, e.what());
+            }
         }
     } PY_CATCH;
 
@@ -632,8 +637,8 @@ static PyObject * makeTorus(PyObject *self, PyObject *args)
 
 static PyObject * makeHelix(PyObject *self, PyObject *args)
 {
-    double pitch, height, radius;
-    if (!PyArg_ParseTuple(args, "ddd", &pitch, &height, &radius))
+    double pitch, height, radius, angle=-1.0;
+    if (!PyArg_ParseTuple(args, "ddd|d", &pitch, &height, &radius, &angle))
         return 0;
 
     try {
@@ -643,22 +648,30 @@ static PyObject * makeHelix(PyObject *self, PyObject *args)
         if (height < Precision::Confusion())
             Standard_Failure::Raise("Height of helix too small");
 
-        if (radius < Precision::Confusion())
-            Standard_Failure::Raise("Radius of helix too small");
-
         gp_Ax2 cylAx2(gp_Pnt(0.0,0.0,0.0) , gp::DZ());
-        Handle(Geom_CylindricalSurface) cyl = new Geom_CylindricalSurface(cylAx2 , radius);
+        Handle_Geom_Surface surf;
+        if (angle < 0) {
+            if (radius < Precision::Confusion())
+                Standard_Failure::Raise("Radius of helix too small");
+            surf = new Geom_CylindricalSurface(cylAx2, radius);
+        }
+        else {
+            angle = angle*(M_PI/180);
+            if (angle < Precision::Confusion())
+                Standard_Failure::Raise("Angle of helix too small");
+            surf = new Geom_ConicalSurface(gp_Ax3(cylAx2), angle, radius);
+        }
 
         gp_Pnt2d aPnt(0, 0);
         gp_Dir2d aDir(2. * PI, pitch);
         gp_Ax2d aAx2d(aPnt, aDir);
 
-        Handle(Geom2d_Line) line = new Geom2d_Line(aAx2d);
+        Handle_Geom2d_Line line = new Geom2d_Line(aAx2d);
         gp_Pnt2d beg = line->Value(0);
         gp_Pnt2d end = line->Value(2.0*PI*(height/pitch));
-        Handle(Geom2d_TrimmedCurve) segm = GCE2d_MakeSegment(beg , end);
+        Handle_Geom2d_TrimmedCurve segm = GCE2d_MakeSegment(beg , end);
 
-        TopoDS_Edge edgeOnSurf = BRepBuilderAPI_MakeEdge(segm , cyl);
+        TopoDS_Edge edgeOnSurf = BRepBuilderAPI_MakeEdge(segm , surf);
         TopoDS_Wire wire = BRepBuilderAPI_MakeWire(edgeOnSurf);
         BRepLib::BuildCurves3d(wire);
         return new TopoShapeWirePy(new TopoShape(wire));
@@ -863,6 +876,143 @@ static PyObject * makeRuledSurface(PyObject *self, PyObject *args)
     }
 }
 
+static PyObject * makeTube(PyObject *self, PyObject *args)
+{
+    PyObject *pshape;
+    double radius;
+    if (!PyArg_ParseTuple(args, "O!d", &(TopoShapePy::Type), &pshape, &radius))
+        return 0;
+
+    const TopoDS_Shape& shape = static_cast<TopoShapePy*>(pshape)->getTopoShapePtr()->_Shape;
+
+    try {
+        if (shape.ShapeType() == TopAbs_EDGE) {
+            const TopoDS_Edge& edge = TopoDS::Edge(shape);
+            BRepAdaptor_Curve adapt(edge);
+            double umin = adapt.FirstParameter();
+            double umax = adapt.LastParameter();
+
+            const Handle_Geom_Curve& hCurve = adapt.Curve().Curve();
+            if (hCurve.IsNull()) {
+                PyErr_SetString(PyExc_Exception, "invalid curve in edge");
+                return 0;
+            }
+            GeomFill_Pipe mkTube(hCurve, radius);
+            mkTube.Perform();
+
+            const Handle_Geom_Surface& surf = mkTube.Surface();
+            double u1,u2,v1,v2;
+            surf->Bounds(u1,u2,v1,v2);
+
+            BRepBuilderAPI_MakeFace mkBuilder(surf, umin, umax, v1, v2);
+            const TopoDS_Face& face = mkBuilder.Face();
+            return new TopoShapeFacePy(new TopoShape(face));
+        }
+        else {
+            PyErr_SetString(PyExc_Exception, "path must be an edge");
+            return 0;
+        }
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
+    }
+}
+
+static PyObject * makeLoft(PyObject *self, PyObject *args)
+{
+#if 0
+    PyObject *pcObj;
+    if (!PyArg_ParseTuple(args, "O!", &(PyList_Type), &pcObj))     // convert args: Python->C
+        return NULL;                             // NULL triggers exception
+
+    NCollection_List<Handle_Geom_Curve> theSections;
+    Py::List list(pcObj);
+    for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+        if (PyObject_TypeCheck((*it).ptr(), &(Part::GeometryCurvePy::Type))) {
+            Handle_Geom_Curve hCurve = Handle_Geom_Curve::DownCast(
+                static_cast<GeometryCurvePy*>((*it).ptr())->getGeomCurvePtr()->handle());
+            theSections.Append(hCurve);
+        }
+    }
+
+    //populate section generator
+    GeomFill_SectionGenerator aSecGenerator;
+    for (NCollection_List<Handle_Geom_Curve>::Iterator anIt(theSections); anIt.More(); anIt.Next()) {
+        const Handle_Geom_Curve& aCurve = anIt.Value();
+        aSecGenerator.AddCurve (aCurve);
+    }
+    aSecGenerator.Perform (Precision::PConfusion());
+
+    Handle_GeomFill_Line aLine = new GeomFill_Line (theSections.Size());
+
+    //parameters
+    const Standard_Integer aMinDeg = 1, aMaxDeg = BSplCLib::MaxDegree(), aNbIt = 0;
+    Standard_Real aTol3d = 1e-4, aTol2d = Precision::Parametric (aTol3d);
+
+    //algorithm
+    GeomFill_AppSurf anAlgo (aMinDeg, aMaxDeg, aTol3d, aTol2d, aNbIt);
+    anAlgo.Perform (aLine, aSecGenerator);
+
+    if (!anAlgo.IsDone()) {
+        PyErr_SetString(PyExc_Exception, "Failed to create loft surface");
+        return 0;
+    }
+
+    Handle_Geom_BSplineSurface aRes;
+    aRes = new Geom_BSplineSurface(anAlgo.SurfPoles(), anAlgo.SurfWeights(),
+        anAlgo.SurfUKnots(), anAlgo.SurfVKnots(), anAlgo.SurfUMults(), anAlgo.SurfVMults(),
+        anAlgo.UDegree(), anAlgo.VDegree());
+    return new BSplineSurfacePy(new GeomBSplineSurface(aRes));
+#else
+    PyObject *pcObj;
+    if (!PyArg_ParseTuple(args, "O!", &(PyList_Type), &pcObj))     // convert args: Python->C
+        return NULL;                             // NULL triggers exception
+
+    try {
+        Standard_Boolean anIsSolid = Standard_False;
+        Standard_Boolean anIsRuled = Standard_False;
+        BRepOffsetAPI_ThruSections aGenerator (anIsSolid,anIsRuled);
+
+        int countWires = 0;
+        Py::List list(pcObj);
+        for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+            if (PyObject_TypeCheck((*it).ptr(), &(Part::TopoShapePy::Type))) {
+                const TopoDS_Shape& sh = static_cast<TopoShapePy*>((*it).ptr())->
+                    getTopoShapePtr()->_Shape;
+                if (!sh.IsNull() && sh.ShapeType() == TopAbs_WIRE) {
+                    aGenerator.AddWire(TopoDS::Wire (sh));
+                    countWires++;
+                }
+            }
+        }
+
+        // need at least two wires
+        if (countWires < 2) {
+            PyErr_SetString(PyExc_Exception, "Need at least two wires to create loft face");
+            return 0;
+        }
+
+        Standard_Boolean anIsCheck = Standard_False;
+        aGenerator.CheckCompatibility (anIsCheck);
+        aGenerator.Build();
+        if (!aGenerator.IsDone()) {
+            PyErr_SetString(PyExc_Exception, "Failed to create loft face");
+            return 0;
+        }
+
+        const TopoDS_Shape& aResult = aGenerator.Shape();
+        return new TopoShapePy(new TopoShape(aResult));
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
+    }
+#endif
+}
+
 static PyObject * toPythonOCC(PyObject *self, PyObject *args)
 {
     PyObject *pcObj;
@@ -873,7 +1023,7 @@ static PyObject * toPythonOCC(PyObject *self, PyObject *args)
         TopoDS_Shape* shape = new TopoDS_Shape();
         (*shape) = static_cast<TopoShapePy*>(pcObj)->getTopoShapePtr()->_Shape;
         PyObject* proxy = 0;
-        proxy = Base::Interpreter().createSWIGPointerObj("TopoDS_Shape *", (void*)shape, 1);
+        proxy = Base::Interpreter().createSWIGPointerObj("OCC.TopoDS", "TopoDS_Shape *", (void*)shape, 1);
         return proxy;
     }
     catch (const Base::Exception& e) {
@@ -891,7 +1041,7 @@ static PyObject * fromPythonOCC(PyObject *self, PyObject *args)
     void* ptr;
     try {
         TopoShape* shape = new TopoShape();
-        Base::Interpreter().convertSWIGPointerObj("TopoDS_Shape *", proxy, &ptr, 0);
+        Base::Interpreter().convertSWIGPointerObj("OCC.TopoDS","TopoDS_Shape *", proxy, &ptr, 0);
         TopoDS_Shape* s = reinterpret_cast<TopoDS_Shape*>(ptr);
         shape->_Shape = (*s);
         return new TopoShapePy(shape);
@@ -984,7 +1134,7 @@ static std::list<TopoDS_Edge> sort_Edges(const std::vector<TopoDS_Edge>& edges)
 }
 }
 
-static PyObject * sortEdges(PyObject *self, PyObject *args)
+static PyObject * getSortedClusters(PyObject *self, PyObject *args)
 {
     PyObject *obj;
     if (!PyArg_ParseTuple(args, "O!", &(PyList_Type), &obj)) {
@@ -997,7 +1147,51 @@ static PyObject * sortEdges(PyObject *self, PyObject *args)
     for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
         PyObject* item = (*it).ptr();
         if (PyObject_TypeCheck(item, &(Part::TopoShapePy::Type))) {
-            TopoDS_Shape sh = static_cast<Part::TopoShapePy*>(item)->getTopoShapePtr()->_Shape;
+            const TopoDS_Shape& sh = static_cast<Part::TopoShapePy*>(item)->getTopoShapePtr()->_Shape;
+            if (sh.ShapeType() == TopAbs_EDGE)
+                edges.push_back(TopoDS::Edge(sh));
+            else {
+                PyErr_SetString(PyExc_TypeError, "shape is not an edge");
+                return 0;
+            }
+        }
+        else {
+            PyErr_SetString(PyExc_TypeError, "item is not a shape");
+            return 0;
+        }
+    }
+
+    Edgecluster acluster(edges);
+    tEdgeClusterVector aclusteroutput = acluster.GetClusters();
+
+    Py::List root_list;
+    for (tEdgeClusterVector::iterator it=aclusteroutput.begin(); it != aclusteroutput.end();++it) {
+        Py::List add_list;
+        for (tEdgeVector::iterator it1=(*it).begin();it1 != (*it).end();++it1) {
+            add_list.append(Py::Object(new TopoShapeEdgePy(new TopoShape(*it1)),true));
+        }
+        root_list.append(add_list);
+    }
+
+    return Py::new_reference_to(root_list);
+}
+
+
+static PyObject * sortEdges(PyObject *self, PyObject *args)
+{
+    PyObject *obj;
+    if (!PyArg_ParseTuple(args, "O!", &(PyList_Type), &obj)) {
+        PyErr_SetString(PyExc_Exception, "list of edges expected");
+        return 0;
+    }
+
+
+    Py::List list(obj);
+    std::vector<TopoDS_Edge> edges;
+    for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+        PyObject* item = (*it).ptr();
+        if (PyObject_TypeCheck(item, &(Part::TopoShapePy::Type))) {
+            const TopoDS_Shape& sh = static_cast<Part::TopoShapePy*>(item)->getTopoShapePtr()->_Shape;
             if (sh.ShapeType() == TopAbs_EDGE)
                 edges.push_back(TopoDS::Edge(sh));
             else {
@@ -1012,6 +1206,7 @@ static PyObject * sortEdges(PyObject *self, PyObject *args)
     }
 
     std::list<TopoDS_Edge> sorted = sort_Edges(edges);
+
     Py::List sorted_list;
     for (std::list<TopoDS_Edge>::iterator it = sorted.begin(); it != sorted.end(); ++it) {
         sorted_list.append(Py::Object(new TopoShapeEdgePy(new TopoShape(*it)),true));
@@ -1122,7 +1317,9 @@ struct PyMethodDef Part_methods[] = {
      "By default pnt=Vector(0,0,0),dir=Vector(0,0,1),angle1=0,angle1=360 and angle=360"},
 
     {"makeHelix" ,makeHelix,METH_VARARGS,
-     "makeHelix(pitch,height,radius) -- Make a helix with a given pitch, height and radius"},
+     "makeHelix(pitch,height,radius,[angle]) -- Make a helix with a given pitch, height and radius\n"
+     "By default a cylindrical surface is used to create the helix. If the fourth parameter is set\n"
+     "a conical surface is used instead"},
 
     {"makeRevolution" ,makeRevolution,METH_VARARGS,
      "makeRevolution(Curve,[vmin,vmax,angle,pnt,dir]) -- Make a revolved shape\n"
@@ -1135,9 +1332,18 @@ struct PyMethodDef Part_methods[] = {
      "Create a ruled surface out of two edges or wires. If wires are used then"
      "these must have the same number of edges."},
 
+    {"makeTube" ,makeTube,METH_VARARGS,
+     "makeTube(edge,float) -- Create a tube."},
+
+    {"makeLoft" ,makeLoft,METH_VARARGS,
+     "makeLoft() -- Create a loft shape."},
+
     {"cast_to_shape" ,cast_to_shape,METH_VARARGS,
      "cast_to_shape(shape) -- Cast to the actual shape type"},
 
+    {"getSortedClusters" ,getSortedClusters,METH_VARARGS,
+    "getSortedClusters(list of edges) -- Helper method to sort and cluster a variety of edges"},
+
     {"__sortEdges__" ,sortEdges,METH_VARARGS,
      "__sortEdges__(list of edges) -- Helper method to sort an unsorted list of edges so that afterwards\n"
      "two adjacent edges share a common vertex"},
