@@ -24,6 +24,7 @@
 #include "PreCompiled.h"
 #ifndef _PreComp_
 # include <sstream>
+# include <BRepMesh.hxx>
 # include <BRepBuilderAPI_Copy.hxx>
 # include <BRepCheck_Analyzer.hxx>
 # include <BRepCheck_ListIteratorOfListOfStatus.hxx>
@@ -44,6 +45,7 @@
 
 #include <BRepGProp.hxx>
 #include <GProp_GProps.hxx>
+#include <BRepAlgo_NormalProjection.hxx>
 
 
 #include <Base/GeometryPyCXX.h>
@@ -130,7 +132,6 @@ int TopoShapePy::PyInit(PyObject* args, PyObject*)
     return 0;
 }
 
-
 PyObject*  TopoShapePy::read(PyObject *args)
 {
     char* filename;
@@ -141,6 +142,27 @@ PyObject*  TopoShapePy::read(PyObject *args)
     Py_Return;
 }
 
+PyObject* TopoShapePy::writeInventor(PyObject * args)
+{
+    double dev=0.3, angle=0.4;
+    int mode=2;
+    if (!PyArg_ParseTuple(args, "|idd", &mode,&dev,&angle))
+        return NULL;
+
+    std::stringstream result;
+    BRepMesh::Mesh(getTopoShapePtr()->_Shape,dev);
+    if (mode == 0)
+        getTopoShapePtr()->exportFaceSet(dev, angle, result);
+    else if (mode == 1)
+        getTopoShapePtr()->exportLineSet(result);
+    else {
+        getTopoShapePtr()->exportFaceSet(dev, angle, result);
+        getTopoShapePtr()->exportLineSet(result);
+    }
+    BRepTools::Clean(getTopoShapePtr()->_Shape); // remove triangulation
+    return Py::new_reference_to(Py::String(result.str()));
+}
+
 PyObject*  TopoShapePy::exportIges(PyObject *args)
 {
     char* filename;
@@ -908,6 +930,38 @@ PyObject* TopoShapePy::tessellate(PyObject *args)
     return Py::new_reference_to(tuple);
 }
 
+PyObject* TopoShapePy::project(PyObject *args)
+{
+    PyObject *obj;
+
+    BRepAlgo_NormalProjection algo;
+    algo.Init(this->getTopoShapePtr()->_Shape);
+    if (PyArg_ParseTuple(args, "O!", &(PyList_Type), &obj)) {
+        try {
+            Py::List list(obj);
+            for (Py::List::iterator it = list.begin(); it != list.end(); ++it) {
+                if (PyObject_TypeCheck((*it).ptr(), &(Part::TopoShapePy::Type))) {
+                    const TopoDS_Shape& shape = static_cast<TopoShapePy*>((*it).ptr())->getTopoShapePtr()->_Shape;
+                    algo.Add(shape);
+                }
+            }
+
+            algo.Compute3d(Standard_True);
+            algo.SetLimit(Standard_True);
+            algo.SetParams(1.e-6, 1.e-6, GeomAbs_C1, 14, 16);
+            //algo.SetDefaultParams();
+            algo.Build();
+            return new TopoShapePy(new TopoShape(algo.Projection()));
+        }
+        catch (Standard_Failure) {
+            PyErr_SetString(PyExc_Exception, "Failed to project shape");
+            return NULL;
+        }
+    }
+
+    return 0;
+}
+
 PyObject* TopoShapePy::makeShapeFromMesh(PyObject *args)
 {
     PyObject *tup;
@@ -1073,6 +1127,32 @@ Py::String TopoShapePy::getOrientation(void) const
     return Py::String(name);
 }
 
+void TopoShapePy::setOrientation(Py::String arg)
+{
+    TopoDS_Shape& sh = getTopoShapePtr()->_Shape;
+    if (sh.IsNull())
+        throw Py::Exception(PyExc_Exception, "cannot determine orientation of null shape");
+    std::string name = (std::string)arg;
+    TopAbs_Orientation type;
+    if (name == "Forward") {
+        type = TopAbs_FORWARD;
+    }
+    else if (name == "Reversed") {
+        type = TopAbs_REVERSED;
+    }
+    else if (name == "Internal") {
+        type = TopAbs_INTERNAL;
+    }
+    else if (name == "External") {
+        type = TopAbs_EXTERNAL;
+    }
+    else {
+        throw Py::AttributeError("Invalid orientation type");
+    }
+
+    sh.Orientation(type);
+}
+
 Py::List TopoShapePy::getFaces(void) const
 {
     Py::List ret;
@@ -1264,38 +1344,32 @@ Py::Float TopoShapePy::getVolume(void) const
 
 PyObject *TopoShapePy::getCustomAttributes(const char* attr) const
 {
-    if ( attr[0]== 'F' && 
-         attr[1]== 'a' && 
-         attr[2]== 'c' && 
-         attr[3]== 'e' && 
-         attr[4]>=48 && attr[4]<=57 
-         ){
-        Part::ShapeSegment* s = dynamic_cast<Part::ShapeSegment*>(getTopoShapePtr()->getSubElementByName(attr));
-        TopoDS_Shape Shape = s->Shape;
-        delete s;
-        return new TopoShapeFacePy(new TopoShape(Shape));
-    }else if ( attr[0]== 'E' && 
-               attr[1]== 'd' && 
-               attr[2]== 'g' && 
-               attr[3]== 'e' && 
-               attr[4]>=48 && attr[4]<=57 
-         ){
-        Part::ShapeSegment* s = dynamic_cast<Part::ShapeSegment*>(getTopoShapePtr()->getSubElementByName(attr));
-        TopoDS_Shape Shape = s->Shape;
-        delete s;
-        return new TopoShapeEdgePy(new TopoShape(Shape));
-    }else if ( attr[0]== 'V' && 
-               attr[1]== 'e' && 
-               attr[2]== 'r' && 
-               attr[3]== 't' && 
-               attr[4]== 'e' && 
-               attr[5]== 'x' && 
-               attr[6]>=48 && attr[4]<=57 
-               ){
-        Part::ShapeSegment* s = dynamic_cast<Part::ShapeSegment*>(getTopoShapePtr()->getSubElementByName(attr));
-        TopoDS_Shape Shape = s->Shape;
-        delete s;
-        return new TopoShapeVertexPy(new TopoShape(Shape));
+    if (!attr) return 0;
+    std::string name(attr);
+    try {
+        if (name.size() > 4 && name.substr(0,4) == "Face" && name[4]>=48 && name[4]<=57) {
+            std::auto_ptr<Part::ShapeSegment> s(static_cast<Part::ShapeSegment*>
+                (getTopoShapePtr()->getSubElementByName(attr)));
+            TopoDS_Shape Shape = s->Shape;
+            return new TopoShapeFacePy(new TopoShape(Shape));
+        }
+        else if (name.size() > 4 && name.substr(0,4) == "Edge" && name[4]>=48 && name[4]<=57) {
+            std::auto_ptr<Part::ShapeSegment> s(static_cast<Part::ShapeSegment*>
+                (getTopoShapePtr()->getSubElementByName(attr)));
+            TopoDS_Shape Shape = s->Shape;
+            return new TopoShapeEdgePy(new TopoShape(Shape));
+        }
+        else if (name.size() > 6 && name.substr(0,6) == "Vertex" && name[6]>=48 && name[6]<=57) {
+            std::auto_ptr<Part::ShapeSegment> s(static_cast<Part::ShapeSegment*>
+                (getTopoShapePtr()->getSubElementByName(attr)));
+            TopoDS_Shape Shape = s->Shape;
+            return new TopoShapeVertexPy(new TopoShape(Shape));
+        }
+    }
+    catch (Standard_Failure) {
+        Handle_Standard_Failure e = Standard_Failure::Caught();
+        PyErr_SetString(PyExc_Exception, e->GetMessageString());
+        return 0;
     }
     return 0;
 }
