@@ -23,19 +23,23 @@
 
 #include "PreCompiled.h"
 #ifndef _PreComp_
+# include <QString>
 # include <QDir>
 # include <QFileInfo>
 # include <QLineEdit>
 # include <QPointer>
 # include <Standard_math.hxx>
+# include <TopoDS_Shape.hxx>
 # include <Inventor/events/SoMouseButtonEvent.h>
 #endif
 
+#include <Base/Console.h>
 #include <Base/Exception.h>
 #include <App/Document.h>
 #include <Gui/Application.h>
 #include <Gui/BitmapFactory.h>
 #include <Gui/Command.h>
+#include <Gui/Control.h>
 #include <Gui/Document.h>
 #include <Gui/FileDialog.h>
 #include <Gui/MainWindow.h>
@@ -497,6 +501,138 @@ bool CmdPartImportCurveNet::isActive(void)
         return false;
 }
 
+//===========================================================================
+// Part_MakeSolid
+//===========================================================================
+DEF_STD_CMD_A(CmdPartMakeSolid);
+
+CmdPartMakeSolid::CmdPartMakeSolid()
+  :Command("Part_MakeSolid")
+{
+    sAppModule    = "Part";
+    sGroup        = QT_TR_NOOP("Part");
+    sMenuText     = QT_TR_NOOP("Convert to solid");
+    sToolTipText  = QT_TR_NOOP("Create solid from a shell or compound");
+    sWhatsThis    = "Part_MakeSolid";
+    sStatusTip    = sToolTipText;
+    iAccel        = 0;
+}
+
+void CmdPartMakeSolid::activated(int iMsg)
+{
+    std::vector<App::DocumentObject*> objs = Gui::Selection().getObjectsOfType
+        (Part::Feature::getClassTypeId());
+    doCommand(Doc, "import Part");
+    for (std::vector<App::DocumentObject*>::iterator it = objs.begin(); it != objs.end(); ++it) {
+        const TopoDS_Shape& shape = static_cast<Part::Feature*>(*it)->Shape.getValue();
+        if (!shape.IsNull()) {
+            TopAbs_ShapeEnum type = shape.ShapeType();
+            QString str;
+            if (type == TopAbs_SOLID) {
+                Base::Console().Message("%s is ignored because it is already a solid.\n",
+                    (*it)->Label.getValue());
+            }
+            else if (type == TopAbs_COMPOUND || type == TopAbs_COMPSOLID) {
+                str = QString::fromAscii(
+                    "__s__=App.ActiveDocument.%1.Shape.Faces\n"
+                    "__s__=Part.Solid(Part.Shell(__s__))\n"
+                    "__o__=App.ActiveDocument.addObject(\"Part::Feature\",\"%1_solid\")\n"
+                    "__o__.Label=\"%2 (Solid)\"\n"
+                    "__o__.Shape=__s__\n"
+                    "del __s__, __o__"
+                    )
+                    .arg(QLatin1String((*it)->getNameInDocument()))
+                    .arg(QLatin1String((*it)->Label.getValue()));
+            }
+            else if (type == TopAbs_SHELL) {
+                str = QString::fromAscii(
+                    "__s__=App.ActiveDocument.%1.Shape\n"
+                    "__s__=Part.Solid(__s__)\n"
+                    "__o__=App.ActiveDocument.addObject(\"Part::Feature\",\"%1_solid\")\n"
+                    "__o__.Label=\"%2 (Solid)\"\n"
+                    "__o__.Shape=__s__\n"
+                    "del __s__, __o__"
+                    )
+                    .arg(QLatin1String((*it)->getNameInDocument()))
+                    .arg(QLatin1String((*it)->Label.getValue()));
+            }
+            else {
+                Base::Console().Message("%s is ignored because it is neither a shell nor a compound.\n",
+                    (*it)->Label.getValue());
+            }
+
+            try {
+                if (!str.isEmpty())
+                    doCommand(Doc, (const char*)str.toAscii());
+            }
+            catch (const Base::Exception& e) {
+                Base::Console().Error("Cannot convert %s because %s.\n",
+                    (*it)->Label.getValue(), e.what());
+            }
+        }
+    }
+}
+
+bool CmdPartMakeSolid::isActive(void)
+{
+    return Gui::Selection().countObjectsOfType
+        (Part::Feature::getClassTypeId()) > 0;
+}
+
+//===========================================================================
+// Part_ReverseShape
+//===========================================================================
+DEF_STD_CMD_A(CmdPartReverseShape);
+
+CmdPartReverseShape::CmdPartReverseShape()
+  :Command("Part_ReverseShape")
+{
+    sAppModule    = "Part";
+    sGroup        = QT_TR_NOOP("Part");
+    sMenuText     = QT_TR_NOOP("Reverse shapes");
+    sToolTipText  = QT_TR_NOOP("Reverse orientation of shapes");
+    sWhatsThis    = "Part_ReverseShape";
+    sStatusTip    = sToolTipText;
+    iAccel        = 0;
+}
+
+void CmdPartReverseShape::activated(int iMsg)
+{
+    std::vector<App::DocumentObject*> objs = Gui::Selection().getObjectsOfType
+        (Part::Feature::getClassTypeId());
+    doCommand(Doc, "import Part");
+    for (std::vector<App::DocumentObject*>::iterator it = objs.begin(); it != objs.end(); ++it) {
+        const TopoDS_Shape& shape = static_cast<Part::Feature*>(*it)->Shape.getValue();
+        if (!shape.IsNull()) {
+            QString str = QString::fromAscii(
+                "__s__=App.ActiveDocument.%1.Shape\n"
+                "__s__.reverse()\n"
+                "__o__=App.ActiveDocument.addObject(\"Part::Feature\",\"%1_rev\")\n"
+                "__o__.Label=\"%2 (Rev)\"\n"
+                "__o__.Shape=__s__\n"
+                "del __s__, __o__"
+                )
+                .arg(QLatin1String((*it)->getNameInDocument()))
+                .arg(QLatin1String((*it)->Label.getValue()));
+
+            try {
+                if (!str.isEmpty())
+                    doCommand(Doc, (const char*)str.toAscii());
+            }
+            catch (const Base::Exception& e) {
+                Base::Console().Error("Cannot convert %s because %s.\n",
+                    (*it)->Label.getValue(), e.what());
+            }
+        }
+    }
+}
+
+bool CmdPartReverseShape::isActive(void)
+{
+    return Gui::Selection().countObjectsOfType
+        (Part::Feature::getClassTypeId()) > 0;
+}
+
 //===========================================================================
 // Part_Boolean
 //===========================================================================
@@ -517,13 +653,15 @@ CmdPartBoolean::CmdPartBoolean()
 
 void CmdPartBoolean::activated(int iMsg)
 {
-    PartGui::DlgBooleanOperation dlg(Gui::getMainWindow());
-    dlg.exec();
+    Gui::TaskView::TaskDialog* dlg = Gui::Control().activeDialog();
+    if (!dlg)
+        dlg = new PartGui::TaskBooleanOperation();
+    Gui::Control().showDialog(dlg);
 }
 
 bool CmdPartBoolean::isActive(void)
 {
-    return hasActiveDocument();
+    return (hasActiveDocument() && !Gui::Control().activeDialog());
 }
 
 //===========================================================================
@@ -546,13 +684,12 @@ CmdPartExtrude::CmdPartExtrude()
 
 void CmdPartExtrude::activated(int iMsg)
 {
-    PartGui::DlgExtrusion dlg(Gui::getMainWindow());
-    dlg.exec();
+    Gui::Control().showDialog(new PartGui::TaskExtrusion());
 }
 
 bool CmdPartExtrude::isActive(void)
 {
-    return hasActiveDocument();
+    return (hasActiveDocument() && !Gui::Control().activeDialog());
 }
 
 //===========================================================================
@@ -575,13 +712,12 @@ CmdPartRevolve::CmdPartRevolve()
 
 void CmdPartRevolve::activated(int iMsg)
 {
-    PartGui::DlgRevolution dlg(Gui::getMainWindow());
-    dlg.exec();
+    Gui::Control().showDialog(new PartGui::TaskRevolution());
 }
 
 bool CmdPartRevolve::isActive(void)
 {
-    return hasActiveDocument();
+    return (hasActiveDocument() && !Gui::Control().activeDialog());
 }
 
 //===========================================================================
@@ -604,13 +740,12 @@ CmdPartFillet::CmdPartFillet()
 
 void CmdPartFillet::activated(int iMsg)
 {
-    PartGui::DlgFilletEdges dlg(Gui::getMainWindow());
-    dlg.exec();
+    Gui::Control().showDialog(new PartGui::TaskFilletEdges());
 }
 
 bool CmdPartFillet::isActive(void)
 {
-    return hasActiveDocument();
+    return (hasActiveDocument() && !Gui::Control().activeDialog());
 }
 
 //--------------------------------------------------------------------------------------
@@ -712,6 +847,8 @@ void CreatePartCommands(void)
 {
     Gui::CommandManager &rcCmdMgr = Gui::Application::Instance->commandManager();
 
+    rcCmdMgr.addCommand(new CmdPartMakeSolid());
+    rcCmdMgr.addCommand(new CmdPartReverseShape());
     rcCmdMgr.addCommand(new CmdPartBoolean());
     rcCmdMgr.addCommand(new CmdPartExtrude());
     rcCmdMgr.addCommand(new CmdPartRevolve());
