@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (c) 2004 Jrgen Riegel <juergen.riegel@web.de>              *
+ *   Copyright (c) 2004 Jrgen Riegel <juergen.riegel@web.de>               *
  *                                                                         *
  *   This file is part of the FreeCAD CAx development system.              *
  *                                                                         *
@@ -25,6 +25,7 @@
 
 #ifndef _PreComp_
 # include <Poly_Polygon3D.hxx>
+# include <BRepBndLib.hxx>
 # include <BRepMesh.hxx>
 # include <BRepMesh_IncrementalMesh.hxx>
 # include <BRep_Tool.hxx>
@@ -68,6 +69,9 @@
 # include <Inventor/nodes/SoPointSet.h>
 # include <Inventor/nodes/SoShapeHints.h>
 # include <Inventor/nodes/SoSwitch.h>
+# include <Inventor/nodes/SoGroup.h>
+# include <Inventor/nodes/SoSphere.h>
+# include <Inventor/nodes/SoScale.h>
 #endif
 
 /// Here the FreeCAD includes sorted by Base,App,Gui......
@@ -76,8 +80,6 @@
 #include <Base/Exception.h>
 #include <App/Application.h>
 #include <App/Document.h>
-#include <App/DocumentObject.h>
-#include <App/Feature.h>
 #include <Gui/SoFCSelection.h>
 #include <Gui/Selection.h>
 #include <Gui/View3DInventorViewer.h>
@@ -87,10 +89,20 @@
 #include "SoFCShapeObject.h"
 
 #include <Mod/Part/App/PartFeature.h>
+#include <Mod/Part/App/PrimitiveFeature.h>
 
 
 using namespace PartGui;
 
+void ViewProviderShapeBuilder::buildNodes(const App::Property* prop, std::vector<SoNode*>& nodes) const
+{
+}
+
+void ViewProviderShapeBuilder::createShape(const App::Property* prop, SoSeparator* coords) const
+{
+}
+
+
 PROPERTY_SOURCE(PartGui::ViewProviderPart, Gui::ViewProviderGeometryObject)
 
 
@@ -209,7 +221,9 @@ void ViewProviderPart::onChanged(const App::Property* prop)
         pcPointMaterial->transparency.setValue(Mat.transparency);
     }
     else if (prop == &ControlPoints) {
-        showControlPoints(ControlPoints.getValue());
+        App::DocumentObject* obj = this->pcObject;
+        App::Property* shape = obj->getPropertyByName("Shape");
+        showControlPoints(ControlPoints.getValue(), shape);
     }
     else if (prop == &Lighting) {
         if (Lighting.getValue() == 0)
@@ -240,7 +254,7 @@ void ViewProviderPart::attach(App::DocumentObject *pcFeat)
     pcNormalRoot->addChild(pShapeHints);
     pcNormalRoot->addChild(FaceRoot);
     pcNormalRoot->addChild(EdgeRoot);
-    //  pcNormalRoot->addChild(VertexRoot);
+    pcNormalRoot->addChild(VertexRoot);
 
     // just faces with no edges or points
     pcFlatRoot->addChild(pShapeHints);
@@ -248,7 +262,7 @@ void ViewProviderPart::attach(App::DocumentObject *pcFeat)
 
     // only edges
     pcWireframeRoot->addChild(EdgeRoot);
-    //  pcWireframeRoot->addChild(VertexRoot);
+    pcWireframeRoot->addChild(VertexRoot);
 
     // normal viewing with edges and points
     pcPointsRoot->addChild(VertexRoot);
@@ -374,24 +388,33 @@ void ViewProviderPart::updateData(const App::Property* prop)
     Gui::ViewProviderGeometryObject::updateData(prop);
     if (prop->getTypeId() == Part::PropertyPartShape::getClassTypeId()) {
         TopoDS_Shape cShape = static_cast<const Part::PropertyPartShape*>(prop)->getValue();
-        if (cShape.IsNull())
-            return;
 
         // clear anchor nodes
         vertexShapeMap.clear();
         EdgeRoot->removeAllChildren();
         FaceRoot->removeAllChildren();
         VertexRoot->removeAllChildren();
+        // do nothing if shape is empty
+        if (cShape.IsNull())
+            return;
 
         try {
             // creating the mesh on the data structure
-            BRepMesh::Mesh(cShape,this->meshDeviation);
+            Bnd_Box bounds;
+            BRepBndLib::Add(cShape, bounds);
+            bounds.SetGap(0.0);
+            Standard_Real xMin, yMin, zMin, xMax, yMax, zMax;
+            bounds.Get(xMin, yMin, zMin, xMax, yMax, zMax);
+            Standard_Real deflection = ((xMax-xMin)+(yMax-yMin)+(zMax-zMin))/300.0 *
+                this->meshDeviation;
+
+            BRepMesh::Mesh(cShape,deflection);
             //BRepMesh_IncrementalMesh MESH(cShape,meshDeviation);
             // We must reset the location here because the transformation data
             // are set in the placement property
             TopLoc_Location aLoc;
             cShape.Location(aLoc);
-            computeFaces   (FaceRoot,cShape);
+            computeFaces   (FaceRoot,cShape,deflection);
             computeEdges   (EdgeRoot,cShape);
             computeVertices(VertexRoot,cShape);
             BRepTools::Clean(cShape); // remove triangulation
@@ -399,7 +422,7 @@ void ViewProviderPart::updateData(const App::Property* prop)
             // update control points if there
             if (pcControlPoints) {
                 pcControlPoints->removeAllChildren();
-                showControlPoints(this->ControlPoints.getValue());
+                showControlPoints(this->ControlPoints.getValue(), prop);
             }
         }
         catch (...){
@@ -418,7 +441,7 @@ void ViewProviderPart::updateData(const App::Property* prop)
     }
 }
 
-Standard_Boolean ViewProviderPart::computeEdges (SoSeparator* EdgeRoot, const TopoDS_Shape &myShape)
+Standard_Boolean ViewProviderPart::computeEdges (SoGroup* EdgeRoot, const TopoDS_Shape &myShape)
 {
     //TopExp_Explorer ex;
 
@@ -529,8 +552,8 @@ Standard_Boolean ViewProviderPart::computeEdges (SoSeparator* EdgeRoot, const To
         sel->documentName = pcObject->getDocument()->getName();
         sel->subElementName = name;
         sel->style = Gui::SoFCSelection::EMISSIVE_DIFFUSE;
-        sel->highlightMode = Gui::SoFCSelection::AUTO;
-        sel->selectionMode = Gui::SoFCSelection::SEL_ON;
+        //sel->highlightMode = Gui::SoFCSelection::AUTO;
+        //sel->selectionMode = Gui::SoFCSelection::SEL_ON;
 
         SoLineSet * lineset = new SoLineSet;
         sel->addChild(lineset);
@@ -541,8 +564,9 @@ Standard_Boolean ViewProviderPart::computeEdges (SoSeparator* EdgeRoot, const To
     return true;
 }
 
-Standard_Boolean ViewProviderPart::computeVertices(SoSeparator* VertexRoot, const TopoDS_Shape &myShape)
+Standard_Boolean ViewProviderPart::computeVertices(SoGroup* VertexRoot, const TopoDS_Shape &myShape)
 {
+#if 0 // new implementation of computeVertice
     VertexRoot->addChild(pcPointMaterial);  
     VertexRoot->addChild(pcPointStyle);
 
@@ -574,24 +598,73 @@ Standard_Boolean ViewProviderPart::computeVertices(SoSeparator* VertexRoot, cons
     sel->documentName = pcObject->getDocument()->getName();
     sel->subElementName = name;
     sel->style = Gui::SoFCSelection::EMISSIVE_DIFFUSE;
-    sel->highlightMode = Gui::SoFCSelection::AUTO;
-    sel->selectionMode = Gui::SoFCSelection::SEL_ON;
+    //sel->highlightMode = Gui::SoFCSelection::AUTO;
+    //sel->selectionMode = Gui::SoFCSelection::SEL_ON;
 
     SoPointSet * pointset = new SoPointSet;
     sel->addChild(pointset);
     VertexRoot->addChild(sel);
 
     return true;
+#else
+    VertexRoot->addChild(pcPointMaterial);  
+    VertexRoot->addChild(pcPointStyle);
+
+
+    // get a indexed map of edges
+    TopTools_IndexedMapOfShape M;
+    TopExp::MapShapes(myShape, TopAbs_VERTEX, M);
+    
+
+    //int i=0;
+    //for (ex.Init(myShape, TopAbs_VERTEX); ex.More(); ex.Next()) {
+    for (int i=0; i<M.Extent(); i++)
+    {
+        const TopoDS_Vertex& aVertex = TopoDS::Vertex(M(i+1));
+
+        // each point has its own selection node
+        Gui::SoFCSelection* sel = createFromSettings();
+        SbString name("Point");
+        name += SbString(i+1);
+        sel->objectName = pcObject->getNameInDocument();
+        sel->documentName = pcObject->getDocument()->getName();
+        sel->subElementName = name;
+        sel->style = Gui::SoFCSelection::EMISSIVE_DIFFUSE;
+        //sel->highlightMode = Gui::SoFCSelection::AUTO;
+        //sel->selectionMode = Gui::SoFCSelection::SEL_ON;
+
+        // define the vertices
+        SoCoordinate3 * coords = new SoCoordinate3;
+        coords->point.setNum(1);
+        VertexRoot->addChild(coords);
+
+
+        // get the shape
+        //const TopoDS_Vertex& aVertex = TopoDS::Vertex(ex.Current());
+        gp_Pnt pnt = BRep_Tool::Pnt(aVertex);
+        coords->point.set1Value(0, (float)pnt.X(), (float)pnt.Y(), (float)pnt.Z());
+
+
+        SoPointSet * pointset = new SoPointSet;
+        sel->addChild(pointset);
+        VertexRoot->addChild(sel);
+        //i++;
+    }
+
+
+    return true;
+
+#endif
 }
 
-Standard_Boolean ViewProviderPart::computeFaces(SoSeparator* FaceRoot, const TopoDS_Shape &myShape)
+Standard_Boolean ViewProviderPart::computeFaces(SoGroup* FaceRoot, const TopoDS_Shape &myShape, double defl)
 {
     TopExp_Explorer ex;
 
     FaceRoot->addChild(pcShapeMaterial);
 
 //  BRepMesh::Mesh(myShape,1.0);
-    BRepMesh_IncrementalMesh MESH(myShape,this->meshDeviation);
+    BRepMesh_IncrementalMesh MESH(myShape,defl);
 
     int i = 1;
     for (ex.Init(myShape, TopAbs_FACE); ex.More(); ex.Next(),i++) {
@@ -650,8 +723,8 @@ Standard_Boolean ViewProviderPart::computeFaces(SoSeparator* FaceRoot, const Top
         sel->documentName = pcObject->getDocument()->getName();
         sel->subElementName = name;
         sel->style = Gui::SoFCSelection::EMISSIVE;
-        sel->highlightMode = Gui::SoFCSelection::AUTO;
-        sel->selectionMode = Gui::SoFCSelection::SEL_ON;
+        //sel->highlightMode = Gui::SoFCSelection::AUTO;
+        //sel->selectionMode = Gui::SoFCSelection::SEL_ON;
 
         SoIndexedFaceSet * faceset = new SoIndexedFaceSet;
         faceset->coordIndex.setValues(0,4*nbTriInFace,(const int32_t*) cons);
@@ -783,7 +856,7 @@ void ViewProviderPart::transferToArray(const TopoDS_Face& aFace,SbVec3f** vertic
     }
 }
 
-void ViewProviderPart::showControlPoints(bool show)
+void ViewProviderPart::showControlPoints(bool show, const App::Property* prop)
 {
     if (!pcControlPoints && show) {
         pcControlPoints = new SoSwitch();
@@ -798,27 +871,25 @@ void ViewProviderPart::showControlPoints(bool show)
         return;
 
     // ask for the property we are interested in
-    App::DocumentObject* obj = this->pcObject;
-    App::Property* prop = obj->getPropertyByName("Shape");
     if (prop && prop->getTypeId() == Part::PropertyPartShape::getClassTypeId()) {
-        TopoDS_Shape shape = static_cast<Part::PropertyPartShape*>(prop)->getValue();
+        const TopoDS_Shape& shape = static_cast<const Part::PropertyPartShape*>(prop)->getValue();
         if (shape.IsNull())
             return; // empty shape
         switch (shape.ShapeType())
         {
         case TopAbs_EDGE:
             {
-                TopoDS_Edge edge = TopoDS::Edge(shape);
+                const TopoDS_Edge& edge = TopoDS::Edge(shape);
                 showControlPointsOfEdge(edge);
             }   break;
         case TopAbs_WIRE:
             {
-                TopoDS_Wire wire = TopoDS::Wire(shape);
+                const TopoDS_Wire& wire = TopoDS::Wire(shape);
                 showControlPointsOfWire(wire);
             }   break;
         case TopAbs_FACE:
             {
-                TopoDS_Face face = TopoDS::Face(shape);
+                const TopoDS_Face& face = TopoDS::Face(shape);
                 showControlPointsOfFace(face);
             }   break;
         default:
@@ -862,7 +933,7 @@ void ViewProviderPart::showControlPointsOfWire(const TopoDS_Wire& wire)
     TopoDS_Iterator it;
     for (it.Initialize(wire); it.More(); it.Next()) {
         if (it.Value().ShapeType() == TopAbs_EDGE) {
-            TopoDS_Edge edge = TopoDS::Edge(it.Value());
+            const TopoDS_Edge& edge = TopoDS::Edge(it.Value());
             BRepAdaptor_Curve curve(edge);
 
             std::list<gp_Pnt> poles, knots; 
@@ -974,3 +1045,58 @@ void ViewProviderPart::showControlPointsOfFace(const TopoDS_Face& face)
 
     pcControlPoints->addChild(nodes);
 }
+
+// ----------------------------------------------------------------------------
+
+PROPERTY_SOURCE(PartGui::ViewProviderEllipsoid, PartGui::ViewProviderPart)
+
+ViewProviderEllipsoid::ViewProviderEllipsoid()
+{
+    pSphere = new SoSphere();
+    pSphere->ref();
+    pScaling = new SoScale();
+    pScaling->ref();
+}
+
+ViewProviderEllipsoid::~ViewProviderEllipsoid()
+{
+    pSphere->unref();
+    pScaling->unref();
+}
+
+void ViewProviderEllipsoid::updateData(const App::Property* prop)
+{
+    if (prop->getTypeId() == Part::PropertyPartShape::getClassTypeId()) {
+        const TopoDS_Shape& cShape = static_cast<const Part::PropertyPartShape*>(prop)->getValue();
+        // clear anchor nodes
+        //vertexShapeMap.clear();
+        EdgeRoot->removeAllChildren();
+        FaceRoot->removeAllChildren();
+        VertexRoot->removeAllChildren();
+        // do nothing if shape is empty
+        if (cShape.IsNull())
+            return;
+        App::DocumentObject* object = this->getObject();
+        if (object && object->isDerivedFrom(Part::Ellipsoid::getClassTypeId())) {
+            float angle1 = static_cast<Part::Ellipsoid*>(object)->Angle1.getValue();
+            float angle2 = static_cast<Part::Ellipsoid*>(object)->Angle2.getValue();
+            float angle3 = static_cast<Part::Ellipsoid*>(object)->Angle3.getValue();
+            float radius1 = static_cast<Part::Ellipsoid*>(object)->Radius1.getValue();
+            float radius2 = static_cast<Part::Ellipsoid*>(object)->Radius2.getValue();
+            if (angle1 == -90.0f && angle2 == 90.0f && angle3 == 360.0f) {
+                float scale = radius1/radius2;
+                pScaling->scaleFactor.setValue(1,1,scale);
+                pSphere->radius.setValue(radius2);
+                FaceRoot->addChild(pScaling);
+                FaceRoot->addChild(pSphere);
+                return; // ok, done
+            }
+        }
+
+        // if not a full ellipsoid do it the general way
+        ViewProviderPart::updateData(prop);
+    }
+    else {
+        Gui::ViewProviderGeometryObject::updateData(prop);
+    }
+}
